#include "gm.h"
#include <glade/glade.h>
#include "gm-plugin.h"
#include "csv.h"

#define CSV_PLUGIN_GLADE_FILE "csv.glade"
#define CUSTOM_INDEX 9

static gint csv_plugin_create_dialog   (GmPluginData *pd);
static gint csv_plugin_on_delete_event (GtkWidget *widget, CsvConfigPageData *data);
static gint csv_plugin_export_data     (CsvConfigPageData *config_data) ;
static void csv_plugin_collect_options (CsvConfigPageData *config_page_data);
static void csv_plugin_destroy         (GmPluginData *pd);
static void csv_plugin_cancel_button_pressed (void * ignore, CsvConfigPageData *data);
static void csv_plugin_ok_button_pressed (void * ignore,  gpointer data);
static void csv_plugin_separator_menu_deactivate (GtkMenuShell *shell,FormatData  *data);
static gint csv_plugin_create_dialog (GmPluginData *pd);




/* Init and destroy functions */


static void
csv_plugin_destroy (GmPluginData *pd)
{
}





gint
init_plugin (GmPluginData *pd)
{
	/* initialize */
    pd->destroy_plugin = csv_plugin_destroy;
    pd->name = _("CsvExporter");
    pd->desc = _("Text File Export (*.csv)");
    pd->long_desc = _("Exporting structured text , Export your Results in a text format \n"
                      "you have the ability to configure the output options\n");
    pd->author = "Mohamed Elassadi";
    pd->installed_by_default = TRUE;
    pd->progressbar = NULL;	
    pd->mini_buffer = NULL;
    pd->type = PLUGIN_EXPORTER;    
    return PLUGIN_OK;
}


gint
run_plugin (GmPluginData *pd)
{
    return csv_plugin_create_dialog (pd);
}






/*****************************************************************************************************
 *   Private Functions 
 *****************************************************************************************************/




/* start the wizard and collect the apropriate informations */

static void 
csv_plugin_cancel_button_pressed (void * ignore, CsvConfigPageData *data )
{
    
    data->canceled = TRUE;
    
    gtk_main_quit();
}


static void 
csv_plugin_ok_button_pressed (void * ignore,  gpointer data)
{
    gtk_main_quit();
}



static gint
csv_plugin_on_delete_event (GtkWidget *widget, CsvConfigPageData *data)
{

    
    data->canceled = TRUE;

    gtk_main_quit();
    return TRUE;    
}


static void
csv_plugin_separator_menu_deactivate (GtkMenuShell *shell,FormatData  *data)
{
    if (gm_utils_option_menu_get_selected_index (data->format_separator) == CUSTOM_INDEX) {
        gtk_widget_set_sensitive (GTK_WIDGET (data->format_custom), TRUE);
        gtk_widget_grab_focus      (GTK_WIDGET (data->format_custom));
        gtk_editable_select_region (GTK_EDITABLE (data->format_custom), 0, -1);
    }
    else {
        gtk_widget_set_sensitive (GTK_WIDGET (data->format_custom), FALSE);
        gtk_editable_select_region (GTK_EDITABLE (data->format_custom), 0, 0); /* If we don't use this the selection will remain blue */
    }
}



static gint
csv_plugin_create_dialog (GmPluginData *pd)
{
    GladeXML *gui;
    GtkWidget *dialog;
    GtkWidget *ok_button;
    GtkWidget *cancel_button;
    CsvConfigPageData cpd;
    GtkMenu *menu;


    gui = glade_xml_new (GMYCLIENT_GLADEDIR CSV_PLUGIN_GLADE_FILE, NULL);
    if (!gui){
        gdm ("Could not find %s", GMYCLIENT_GLADEDIR CSV_PLUGIN_GLADE_FILE);
        return PLUGIN_ERROR;
    }
    cpd.dialog   = dialog                 = glade_xml_get_widget (gui, "csv_export_dialog");
    cpd.canceled = FALSE;
    cpd.pd       = pd;

    cpd.format_data = g_new (FormatData, 1);


	/* Create get object and fill information struct */
    cpd.format_data->format_termination = GTK_OPTION_MENU (glade_xml_get_widget (gui, "format_termination"));
    cpd.format_data->format_separator   = GTK_OPTION_MENU (glade_xml_get_widget (gui, "format_separator"));
    cpd.format_data->format_custom      = GTK_ENTRY       (glade_xml_get_widget (gui, "format_custom"));
    menu = (GtkMenu *) gtk_option_menu_get_menu (cpd.format_data->format_separator);
    ok_button              = glade_xml_get_widget (gui, "ok_button");
    cancel_button          = glade_xml_get_widget (gui, "cancel_button");

        /* Connect the signals */

    gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
                        GTK_SIGNAL_FUNC (csv_plugin_cancel_button_pressed), 
                        &cpd);

    gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                        GTK_SIGNAL_FUNC (csv_plugin_ok_button_pressed), 
                        &cpd);

    gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
                        GTK_SIGNAL_FUNC (csv_plugin_on_delete_event),
                        &cpd);

    gtk_signal_connect (GTK_OBJECT (menu),"deactivate",
                        GTK_SIGNAL_FUNC (csv_plugin_separator_menu_deactivate),
                        cpd.format_data);

    gtk_widget_show_all(dialog);


        /* start the plugin loop */

    gtk_main();

    if (!cpd.canceled) {
        gint result;
        
        csv_plugin_collect_options (&cpd);
        result = csv_plugin_export_data (&cpd);
        gtk_widget_destroy (dialog);
        gtk_object_unref (GTK_OBJECT (gui));    

        if (result) 
            return PLUGIN_OK;
        else 
            return PLUGIN_ERROR;        
    }else {
        gtk_widget_destroy (dialog);
        gtk_object_unref (GTK_OBJECT (gui));
        return PLUGIN_CANCELED;        
    }
}








static void
csv_plugin_collect_options (CsvConfigPageData *config_page_data)
{
    CsvTerminatorType terminator;
    gchar *text;
    gchar separator;


    FormatData * data;
        
    data = config_page_data->format_data;
        

    switch (gm_utils_option_menu_get_selected_index (data->format_termination)) {
	case 0 : terminator = TERMINATOR_TYPE_LINEFEED; break;
	case 1 : terminator = TERMINATOR_TYPE_RETURN; break;
	case 2 : terminator = TERMINATOR_TYPE_RETURN_LINEFEED; break;
	default :
            terminator = TERMINATOR_TYPE_UNKNOWN;
            break;
    }
    config_page_data->terminator_type = terminator;
    separator = '\0';
    switch (gm_utils_option_menu_get_selected_index (data->format_separator)) {
	case 0 : separator = ' '; break;
	case 1 : separator = '\t'; break;
	case 2 : separator = '!'; break;
	case 3 : separator = ':'; break;
	case 4 : separator = ','; break;
	case 5 : separator = '-'; break;
	case 6 : separator = '|'; break;
	case 7 : separator = ';'; break;
	case 8 : separator = '/'; break;
	case 9 : {
            text = gtk_editable_get_chars (GTK_EDITABLE (data->format_custom), 0, -1);
            separator = text[0];
            g_free (text);
            break;
	}
	default :
            g_warning ("Unknown separator");
            break;
    }
    config_page_data->cell_separator = separator;
}



static gint
csv_plugin_export_data (CsvConfigPageData  *config_data) 
{

    GtkCList * clist;    
    GString * string ;
    FILE *file;    
    GmExporterPluginData *epd;    
    GmPluginData *pd;
    gint row = 0;
    GList * rows_selection;
    GList * columns_list;
    gint max_rows;
    gint row_counter=0;    
    gfloat tick;    
    gchar * text;
    
    string = g_string_new ("");    
    pd  = config_data->pd;
    epd = (GmExporterPluginData *) pd->private_data;    
    clist = epd->data_clist;
    file = gm_utils_open_file (epd->output_filename);
    if (!file)
        return FALSE;
    
    gdm(" %s " , epd->output_filename);

    rows_selection = epd->rows_selection;
    max_rows = g_list_length(rows_selection);
    text = g_strdup_printf (_("Exporting csv file %s "), epd->output_filename);    
    gtk_entry_set_text (GTK_ENTRY (pd->mini_buffer), text);
    g_free (text);
    
    gm_utils_progress_tick (pd->progressbar, 0.0);    
    while (rows_selection){

        if (epd->stype != ALL_ROWS) 
            row = GPOINTER_TO_INT (rows_selection->data);
        else 
            row = row_counter;

        if (!row && epd->ignore_first_row) {
                rows_selection = rows_selection->next;
                row_counter++;  
            continue;
        }
        
        string = g_string_new("");
        columns_list = epd->columns_list;
        while (columns_list){
            gint col = GPOINTER_TO_INT(columns_list->data);
            gchar *cdata = NULL;

            gtk_clist_get_text (clist, row, col, &cdata);

            if (cdata)
                g_string_sprintfa (string,"%s", cdata);
            g_string_append_c (string, config_data->cell_separator);
            columns_list = columns_list->next;        
        }
        
        switch (config_data->terminator_type) {
            case TERMINATOR_TYPE_LINEFEED :
                g_string_append (string, "\n");
                break;
            case TERMINATOR_TYPE_RETURN :
                g_string_append (string, "\r");
                break;
            case TERMINATOR_TYPE_RETURN_LINEFEED :
                g_string_append (string, "\r\n");
                break;
            default :
                g_warning ("CvsExporter : Unknown terminator type");
                break;
        }
        if (!gm_utils_write_to_file (file, string->str)){
            gtk_editable_delete_text (GTK_EDITABLE (pd->mini_buffer), 0, -1);
            gm_utils_progress_tick (pd->progressbar, 0.0);
            fclose(file);
            g_string_free (string, TRUE);
            return 0;
        }
        g_string_free (string, TRUE);
        rows_selection = rows_selection->next;
        tick = (gfloat) row_counter / (gfloat) max_rows;
        gm_utils_progress_tick (pd->progressbar, tick);
        row_counter++;        
    }
    gtk_editable_delete_text (GTK_EDITABLE (pd->mini_buffer), 0, -1);
    gm_utils_progress_tick (pd->progressbar, 0.0);
    fclose(file);    
    return 1;
}




