#include <glade/glade.h>
#include <math.h>
#include "gm.h"
#include "gm-plugin.h"
#include "gm-clist.h"
#include "html.h"

#define HTML_PLUGIN_GLADE_FILE "html.glade"
#define HTML_PLUGIN_PRESET_FILE "htmlPlugin.preset"
#define DEFAULT_GRID_WIDTH 1


static gint    html_plugin_show_wizard     (GmPluginData *pd);
static gint    html_plugin_export_data     (HtmlConfigPageData *config_data) ;
static gint    html_plugin_on_delete_event (GtkWidget *widget,  HtmlConfigPageData *data);
static void    html_plugin_collect_options (HtmlConfigPageData *cpd);
static void    html_plugin_destroy         (GmPluginData *pd);
static void    html_plugin_cancel          (GnomeDruidPage *dpage, gpointer arg1, HtmlConfigPageData *data);
static void    html_plugin_finish          (GnomeDruidPage *dpage, gpointer arg1, HtmlConfigPageData *data);
static void    html_plugin_save_preset     (HtmlConfigPageData *config_data);




static gchar * 
html_plugin_compose_key (GmPluginData *pd, gchar * key)
{
    gchar * confkey;

    g_return_val_if_fail (pd != NULL, NULL);
    g_return_val_if_fail (pd->preset_file != NULL, NULL);
    g_return_val_if_fail (key != NULL, NULL);
  
    confkey = g_strdup_printf ("=%s=%s", pd->preset_file, key);
    return confkey;
}


static void 
html_plugin_save_preset (HtmlConfigPageData *config_data)
{

    HtmlData * hdata ;
    GmPluginData *pd;
    gchar * confkey;


    g_return_if_fail (config_data != NULL);
    pd  = config_data->pd;
    g_return_if_fail (pd != NULL);
    hdata = config_data->hdata ;
    g_return_if_fail (hdata != NULL);

    confkey = html_plugin_compose_key (pd, "/Presets/RowsPerPage");
    gnome_config_set_int    (confkey, hdata->rows_per_page);
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/PageWidth");
    if (hdata->table_width)
        gnome_config_set_string    (confkey, hdata->table_width);
    else
        gnome_config_clean_key (confkey);
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/BodyBgcolor");
    if (hdata->body_bgcolor)
        gnome_config_set_string    (confkey, hdata->body_bgcolor);
    else
        gnome_config_clean_key (confkey);
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/TextColor");
    if (hdata->text_color)
        gnome_config_set_string    (confkey, hdata->text_color);
    else
        gnome_config_clean_key (confkey);
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/LinksColor");
    if (hdata->links_color)
        gnome_config_set_string    (confkey, hdata->links_color);
    else
        gnome_config_clean_key (confkey);

    g_free (confkey);


    confkey = html_plugin_compose_key (pd, "/Presets/HBgcolor");
    if (hdata->h_bgcolor)
        gnome_config_set_string    (confkey, hdata->h_bgcolor);
    else
        gnome_config_clean_key (confkey);

    g_free (confkey);


    confkey = html_plugin_compose_key (pd, "/Presets/HText");
    if (hdata->h_text)
        gnome_config_set_string    (confkey, hdata->h_text);
    else
        gnome_config_clean_key (confkey);

    g_free (confkey);


    confkey = html_plugin_compose_key (pd, "/Presets/RowsBgcolor");
    if (hdata->rows_bgcolor)
        gnome_config_set_string (confkey, hdata->rows_bgcolor);
    else
        gnome_config_clean_key (confkey);

    g_free (confkey);


    confkey = html_plugin_compose_key (pd, "/Presets/ARowsBgcolor");
    if (hdata->a_rows_bgcolor)
        gnome_config_set_string    (confkey, hdata->a_rows_bgcolor);
    else
        gnome_config_clean_key (confkey);

    g_free (confkey);


    confkey = html_plugin_compose_key (pd, "/Presets/GridColor");
    if (hdata->grid_color)
        gnome_config_set_string    (confkey, hdata->grid_color);
    else
        gnome_config_clean_key (confkey);

    g_free (confkey);


    confkey = html_plugin_compose_key (pd, "/Presets/HtmlHeader");
    if (hdata->html_header)
        gnome_config_set_string    (confkey, hdata->html_header);
    else
        gnome_config_clean_key (confkey);

    g_free (confkey);


    confkey = html_plugin_compose_key (pd, "/Presets/HtmlFooter");
    if (hdata->html_footer)
        gnome_config_set_string    (confkey, hdata->html_footer);
    else
        gnome_config_clean_key (confkey);
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/ShowHeader");
    if (hdata->show_header)
        gnome_config_set_int    (confkey, hdata->show_header);
    else
        gnome_config_clean_key (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/TableAlignment");
    if (hdata->table_alignment)
        gnome_config_set_int    (confkey, hdata->table_alignment);
    else
        gnome_config_clean_key (confkey);
    g_free (confkey);


    gnome_config_sync ();

}


static void 
html_plugin_load_preset (HtmlConfigPageData *config_data)
{

    HtmlData * hdata ;
    GmPluginData *pd;
    gchar * confkey;
    gchar * value;
    gint intvalue;


    g_return_if_fail (config_data != NULL);
    pd  = config_data->pd;
    g_return_if_fail (pd != NULL);
    hdata = config_data->hdata ;
    g_return_if_fail (hdata != NULL);


    confkey = html_plugin_compose_key (pd, "/Presets/RowsPerPage");
    value = gnome_config_get_string    (confkey);
    if (value )
    {
        gtk_entry_set_text (config_data->rows_per_page, value);
        g_free (value);
    }
    g_free (confkey);



    confkey = html_plugin_compose_key (pd, "/Presets/PageWidth");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text ( config_data->table_width, value);
        g_free (value);
    }
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/BodyBgcolor");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text (config_data->body_bgcolor, value);
        g_free (value);
    }
    else
    {
        gtk_signal_emit_by_name (GTK_OBJECT (config_data->body_bgcolor), "changed");
    }
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/TextColor");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text ( config_data->text_color, value);
        g_free (value);
    }
    else
    {
        gtk_signal_emit_by_name (GTK_OBJECT (config_data->text_color), "changed");
    }
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/LinksColor");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text ( config_data->links_color, value);
        g_free (value);
    }
    else
    {
        gtk_signal_emit_by_name (GTK_OBJECT (config_data->links_color), "changed");
    }
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/HBgcolor");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text ( config_data->h_bgcolor, value);
        g_free (value);
    }
    else
    {
        gtk_signal_emit_by_name (GTK_OBJECT (config_data->h_bgcolor), "changed");
    }
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/HText");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text ( config_data->h_text, value);
        g_free (value);
    }
    else
    {
        gtk_signal_emit_by_name (GTK_OBJECT (config_data->h_text), "changed");
    }
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/RowsBgcolor");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text ( config_data->rows_bgcolor, value);
        g_free (value);
    }
    else
    {
        gtk_signal_emit_by_name (GTK_OBJECT (config_data->rows_bgcolor), "changed");
    }

    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/ARowsBgcolor");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text ( config_data->a_rows_bgcolor, value);
        g_free (value);
    }
    else
    {
        gtk_signal_emit_by_name (GTK_OBJECT (config_data->a_rows_bgcolor), "changed");
    }

    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/GridColor");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gtk_entry_set_text ( config_data->grid_color, value);
        g_free (value);
    }
    else
    {
        gtk_signal_emit_by_name (GTK_OBJECT (config_data->grid_color), "changed");
    }

    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/HtmlHeader");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gint temp_pos = 0;
        gtk_editable_delete_text (GTK_EDITABLE (config_data->header_text_widget), 0, -1);
        gtk_editable_insert_text (GTK_EDITABLE (config_data->header_text_widget), value, strlen (value), &temp_pos);
        g_free (value);
    }
    g_free (confkey);


    confkey = html_plugin_compose_key (pd, "/Presets/HtmlFooter");
    value = gnome_config_get_string    (confkey);
    if (value)
    {
        gint temp_pos = 0;
        gtk_editable_delete_text (GTK_EDITABLE (config_data->footer_text_widget), 0, -1);
        gtk_editable_insert_text (GTK_EDITABLE (config_data->footer_text_widget), value, strlen (value), &temp_pos);
        g_free (value);
    }
    g_free (confkey);



    confkey = html_plugin_compose_key (pd, "/Presets/ShowHeader");
    intvalue = gnome_config_get_int    (confkey);
    switch (intvalue)
    {
        case NO_HEADER:
            gtk_toggle_button_set_active (config_data->rb_no_header, TRUE);
            break;
        case SHOW_HEADER_ON_FIRST_PAGE :
            gtk_toggle_button_set_active (config_data->rb_header_on_first, TRUE);
            break;
        case SHOW_HEADER_ON_EVERY_PAGE :
            gtk_toggle_button_set_active (config_data->rb_header_on_every, TRUE);
            break;
    }
    g_free (confkey);

    confkey = html_plugin_compose_key (pd, "/Presets/TableAlignment");
    intvalue = gnome_config_get_int    (confkey);
    if (intvalue < 0 || intvalue > TABLE_CENTER_ALIGN)
        intvalue = 0;  
    gtk_option_menu_set_history (config_data->table_alignment, intvalue);
    g_free (confkey);    
}



static void 
html_plugin_finish (GnomeDruidPage *dpage, gpointer arg1, HtmlConfigPageData *data)
{
  gtk_main_quit ();
}


static void 
html_plugin_cancel (GnomeDruidPage *dpage, gpointer arg1, HtmlConfigPageData *data)
{
  data->canceled = TRUE;
  gtk_main_quit ();
}







/* Init and destroy functions */


static void
html_plugin_destroy (GmPluginData *pd)
{
  g_free (pd->preset_file);
}


gint
init_plugin (GmPluginData *pd)
{
  /* initialize */
  gchar *home_dir;

  pd->destroy_plugin = html_plugin_destroy;
  pd->name = _("HtmlExporter");
  pd->desc = _("Html File Export (*.html)");
  pd->long_desc = _("Exporting Html 4.0 format, Export your Results in html-format \n"
                    "you have the ability to configure the output options\n");
  pd->author = "Mohamed Elassadi";
  pd->installed_by_default = TRUE;
  pd->progressbar = NULL;
  pd->mini_buffer = NULL;
  pd->type = PLUGIN_EXPORTER;
  home_dir = g_get_home_dir ();
  pd->preset_file = g_strdup_printf ("%s/%s/%s", home_dir, GM_CONFIG_DIR_NAME, HTML_PLUGIN_PRESET_FILE);
  return PLUGIN_OK;
}


gint
run_plugin (GmPluginData *pd)
{
  return html_plugin_show_wizard (pd);
}




static void
html_plugin_disable_back_button (GnomeDruidPage *dpage, gpointer arg1, HtmlConfigPageData *data)
{
  gnome_druid_set_buttons_sensitive  (data->html_druid, FALSE, TRUE, TRUE);
}








static gint
html_plugin_on_delete_event (GtkWidget *widget, HtmlConfigPageData *data)
{
  data->canceled = TRUE;
  gtk_main_quit ();
  return TRUE;
}

static void
html_plugin_color_entry_changed (GtkEditable     *editable, 
                                 GtkWidget       *button)
{

  gchar * color;

  color = gtk_entry_get_text (GTK_ENTRY (editable));

  if (color && strlen (color)==7)
    gm_utils_set_widget_background (button, NULL, color);

}


static void
html_plugin_choose_color_clicked (GtkWidget *button, GtkEntry *entry)
{
  gchar * hex_color;
  gchar * color;

  color = gtk_entry_get_text (entry);

  if (color && strlen (color)==7)
    hex_color = gm_dialogs_show_color_selection_dialog (color);
  else
    hex_color = gm_dialogs_show_color_selection_dialog ("#FFFFFF");

  if (hex_color)
    {
      gtk_entry_set_text (entry, hex_color);
      g_free (hex_color);
    }
}








static gint
html_plugin_show_wizard (GmPluginData *pd)
{
  GladeXML *gui;
  GtkWidget *dialog;
  HtmlConfigPageData cpd;
  GtkWidget *button;



  gui = glade_xml_new (GMYCLIENT_GLADEDIR HTML_PLUGIN_GLADE_FILE, NULL);

  if (!gui)
    {
      gdm ("Could not find %s", GMYCLIENT_GLADEDIR HTML_PLUGIN_GLADE_FILE);
      return PLUGIN_ERROR;
    }

  cpd.dialog   = dialog                 = glade_xml_get_widget (gui, "html_export_dialog");
  cpd.canceled = FALSE;
  cpd.pd       = pd;
  cpd.header_page  = glade_xml_get_widget (gui, "header_page");
  cpd.color_options_page = glade_xml_get_widget (gui, "color_options_page");
  cpd.last_page    = glade_xml_get_widget (gui, "last_page");
  cpd.html_druid = GNOME_DRUID (glade_xml_get_widget (gui, "html_druid"));
  gnome_druid_set_buttons_sensitive  (cpd.html_druid, FALSE, TRUE, TRUE);

  cpd.rb_no_header       =  GTK_TOGGLE_BUTTON (glade_xml_get_widget (gui, "rb_no_header"));
  cpd.rb_header_on_first =  GTK_TOGGLE_BUTTON (glade_xml_get_widget (gui, "rb_header_on_first"));
  cpd.rb_header_on_every =  GTK_TOGGLE_BUTTON (glade_xml_get_widget (gui, "rb_header_on_every"));
  cpd.table_alignment    = GTK_OPTION_MENU(glade_xml_get_widget (gui, "table_alignment"));


  cpd.header_text_widget = glade_xml_get_widget (gui, "header_text_widget");
  gm_utils_configure_text_widget (cpd.header_text_widget);
  gtk_widget_show (cpd.header_text_widget);
  gtk_text_set_editable (GTK_TEXT (cpd.header_text_widget), TRUE);

  cpd.footer_text_widget = glade_xml_get_widget (gui, "footer_text_widget");
  gm_utils_configure_text_widget (cpd.footer_text_widget);
  gtk_widget_show (cpd.footer_text_widget);
  gtk_text_set_editable (GTK_TEXT (cpd.footer_text_widget), TRUE);


  cpd.rows_per_page = GTK_ENTRY (glade_xml_get_widget (gui, "rows_per_page"));
  cpd.table_width = GTK_ENTRY (glade_xml_get_widget (gui, "table_width"));

  button = glade_xml_get_widget (gui, "choose_bgcolor_button");
  cpd.body_bgcolor =  GTK_ENTRY (glade_xml_get_widget (gui, "body_bgcolor"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
                      GTK_SIGNAL_FUNC (html_plugin_choose_color_clicked), 
                      GTK_ENTRY (cpd.body_bgcolor));
  gtk_signal_connect (GTK_OBJECT (cpd.body_bgcolor), "changed", 
                      GTK_SIGNAL_FUNC (html_plugin_color_entry_changed), 
                      button);

  button = glade_xml_get_widget (gui, "choose_textcolor_button");
  cpd.text_color =  GTK_ENTRY (glade_xml_get_widget (gui, "text_color"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
                      GTK_SIGNAL_FUNC (html_plugin_choose_color_clicked), 
                      GTK_ENTRY (cpd.text_color));
  gtk_signal_connect (GTK_OBJECT (cpd.text_color), "changed", 
                      GTK_SIGNAL_FUNC (html_plugin_color_entry_changed), 
                      button);

  button = glade_xml_get_widget (gui, "choose_linkscolor_button");
  cpd.links_color =  GTK_ENTRY (glade_xml_get_widget (gui, "links_color"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
                      GTK_SIGNAL_FUNC (html_plugin_choose_color_clicked), 
                      GTK_ENTRY (cpd.links_color));
  gtk_signal_connect (GTK_OBJECT (cpd.links_color), "changed", 
                      GTK_SIGNAL_FUNC (html_plugin_color_entry_changed), 
                      button);



  button = glade_xml_get_widget (gui, "choose_hbgcolor_button");
  cpd.h_bgcolor =  GTK_ENTRY (glade_xml_get_widget (gui, "h_bgcolor"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
                      GTK_SIGNAL_FUNC (html_plugin_choose_color_clicked), 
                      GTK_ENTRY (cpd.h_bgcolor));
  gtk_signal_connect (GTK_OBJECT (cpd.h_bgcolor), "changed", 
                      GTK_SIGNAL_FUNC (html_plugin_color_entry_changed), 
                      button);


  button = glade_xml_get_widget (gui, "choose_htext_color_button");
  cpd.h_text =  GTK_ENTRY (glade_xml_get_widget (gui, "h_text"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
                      GTK_SIGNAL_FUNC (html_plugin_choose_color_clicked), 
                      GTK_ENTRY (cpd.h_text));
  gtk_signal_connect (GTK_OBJECT (cpd.h_text), "changed", 
                      GTK_SIGNAL_FUNC (html_plugin_color_entry_changed), 
                      button);



  button = glade_xml_get_widget (gui, "choose_rows_bgcolor_button");
  cpd.rows_bgcolor =  GTK_ENTRY (glade_xml_get_widget (gui, "rows_bgcolor"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
                      GTK_SIGNAL_FUNC (html_plugin_choose_color_clicked), 
                      GTK_ENTRY (cpd.rows_bgcolor));
  gtk_signal_connect (GTK_OBJECT (cpd.rows_bgcolor), "changed", 
                      GTK_SIGNAL_FUNC (html_plugin_color_entry_changed), 
                      button);

  button = glade_xml_get_widget (gui, "choose_alternate_bgcolor_button");
  cpd.a_rows_bgcolor =  GTK_ENTRY (glade_xml_get_widget (gui, "a_rows_bgcolor"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
                      GTK_SIGNAL_FUNC (html_plugin_choose_color_clicked), 
                      GTK_ENTRY (cpd.a_rows_bgcolor));
  gtk_signal_connect (GTK_OBJECT (cpd.a_rows_bgcolor), "changed", 
                      GTK_SIGNAL_FUNC (html_plugin_color_entry_changed), 
                      button);

  button = glade_xml_get_widget (gui, "choose_grid_color_button");
  cpd.grid_color =  GTK_ENTRY (glade_xml_get_widget (gui, "grid_color"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
                      GTK_SIGNAL_FUNC (html_plugin_choose_color_clicked), 
                      GTK_ENTRY (cpd.grid_color));
  gtk_signal_connect (GTK_OBJECT (cpd.grid_color), "changed", 
                      GTK_SIGNAL_FUNC (html_plugin_color_entry_changed), 
                      button);

  /* Connect the signals */

  gtk_signal_connect (GTK_OBJECT (cpd.last_page), "finish", 
                      GTK_SIGNAL_FUNC (html_plugin_finish), 
                      &cpd);

  gtk_signal_connect (GTK_OBJECT (cpd.last_page), "cancel", 
                      GTK_SIGNAL_FUNC (html_plugin_cancel), 
                      &cpd);


  gtk_signal_connect (GTK_OBJECT (cpd.header_page), "cancel", 
                      GTK_SIGNAL_FUNC (html_plugin_cancel), 
                      &cpd);

  gtk_signal_connect (GTK_OBJECT (cpd.color_options_page), "cancel", 
                      GTK_SIGNAL_FUNC (html_plugin_cancel), 
                      &cpd);

  gtk_signal_connect (GTK_OBJECT (cpd.header_page), "prepare", 
                      GTK_SIGNAL_FUNC (html_plugin_disable_back_button), 
                      &cpd);


  gtk_signal_connect (GTK_OBJECT (dialog), "delete_event", 
                      GTK_SIGNAL_FUNC (html_plugin_on_delete_event), 
                      &cpd);

  html_plugin_load_preset (&cpd);

  gtk_widget_show_all (dialog);


  gtk_main ();

  if (!cpd.canceled)
    {
      gint result;
      HtmlData * hdata ;

      html_plugin_collect_options (&cpd);
      hdata = cpd.hdata;
      result = html_plugin_export_data (&cpd);
      if (result)
        html_plugin_save_preset (&cpd);
      if (hdata->html_header)
        g_free (hdata->html_header);
      if (hdata->html_footer)
        g_free (hdata->html_footer);
      g_free (hdata);

      gtk_widget_destroy (dialog);
      gtk_object_unref (GTK_OBJECT (gui));

      if (result)
        {
          return PLUGIN_OK;
        }
      else
        return PLUGIN_ERROR;
    }
  else
    {
      gtk_widget_destroy (dialog);
      gtk_object_unref (GTK_OBJECT (gui));
      return PLUGIN_CANCELED;
    }
}








static void
html_plugin_collect_options (HtmlConfigPageData *cpd)
{
  HtmlData * hdata ;

  gchar * text;

  hdata = g_new (HtmlData, 1);

  text = gtk_entry_get_text (cpd->body_bgcolor);
  if (text && strlen (text))
    hdata->body_bgcolor = text;
  else
    hdata->body_bgcolor = NULL;


  text = gtk_entry_get_text (cpd->text_color);
  if (text && strlen (text))
    hdata->text_color = text;
  else
    hdata->text_color = NULL;

  text = gtk_entry_get_text (cpd->links_color);
  if (text && strlen (text))
    hdata->links_color = text;
  else
    hdata->links_color = NULL;

  text = gtk_entry_get_text (cpd->h_bgcolor);
  if (text && strlen (text))
    hdata->h_bgcolor = text;
  else
    hdata->h_bgcolor = NULL;

  text = gtk_entry_get_text (cpd->h_text);
  if (text && strlen (text))
    hdata->h_text = text;
  else
    hdata->h_text = NULL;

  text = gtk_entry_get_text (cpd->rows_bgcolor);
  if (text && strlen (text))
    hdata->rows_bgcolor = text;
  else
    hdata->rows_bgcolor = NULL;

  text = gtk_entry_get_text (cpd->a_rows_bgcolor);
  if (text && strlen (text))
    hdata->a_rows_bgcolor = text;
  else
    hdata->a_rows_bgcolor = NULL;

  text = gtk_entry_get_text (cpd->grid_color);
  if (text && strlen (text))
    hdata->grid_color = text;
  else
    hdata->grid_color = NULL;

  if (!hdata->rows_bgcolor && hdata->grid_color)
    {
      hdata->rows_bgcolor = hdata->body_bgcolor;
    }

  if (!hdata->a_rows_bgcolor && hdata->grid_color)
    {
      hdata->a_rows_bgcolor = hdata->body_bgcolor;
    }

  text = gtk_entry_get_text (cpd->rows_per_page);
  if (text && strlen (text))
    hdata->rows_per_page = (gint) atoi (text);
  else
    hdata->rows_per_page = 0;


  text = gtk_entry_get_text (cpd->table_width);
  if (text && strlen (text))
    hdata->table_width = text;
  else
    hdata->table_width = NULL;

  text  = gtk_editable_get_chars (GTK_EDITABLE (cpd->header_text_widget), 0, -1);
  if (text && strlen (text))
    hdata->html_header = text;
  else
    hdata->html_header = NULL;


  text  = gtk_editable_get_chars (GTK_EDITABLE (cpd->footer_text_widget), 0, -1);
  if (text && strlen (text))
    hdata->html_footer = text;
  else
    hdata->html_footer = NULL;

  if (gtk_toggle_button_get_active (cpd->rb_no_header))
    hdata->show_header =NO_HEADER;

  if (gtk_toggle_button_get_active (cpd->rb_header_on_first))
    hdata->show_header = SHOW_HEADER_ON_FIRST_PAGE;

  if (gtk_toggle_button_get_active (cpd->rb_header_on_every))
    hdata->show_header = SHOW_HEADER_ON_EVERY_PAGE;

  hdata->table_alignment = gm_utils_option_menu_get_selected_index (cpd->table_alignment);

  cpd->hdata = hdata;

}

static gchar *
html_plugin_get_html_header (HtmlConfigPageData  *config_data)
{

  GString * header_string;
  HtmlData * hdata ;
  gchar *ret;


  hdata = config_data->hdata;


  header_string = g_string_new (hdata->html_header);
  g_string_append (header_string, "\n<BODY ");
  if (hdata->body_bgcolor)
    g_string_sprintfa (header_string, "BGCOLOR=\"%s\" ", hdata->body_bgcolor);

  if (hdata->text_color)
    g_string_sprintfa (header_string, "TEXT=\"%s\" ", hdata->text_color);

  if (hdata->links_color)
    g_string_sprintfa (header_string, "LINK=\"%s\" ", hdata->links_color);

  g_string_append (header_string, ">");
  ret = header_string->str;
  g_string_free (header_string, FALSE);

  return ret;
}

static gchar *
html_plugin_get_html_footer (HtmlConfigPageData  *config_data)
{

  GString * footer_string;
  HtmlData * hdata ;
  gchar *ret;


  hdata = config_data->hdata;


  footer_string = g_string_new (hdata->html_footer);
  ret = footer_string->str;
  g_string_free (footer_string, FALSE);
  return ret;
}



static gchar *
html_plugin_get_css_styles (HtmlConfigPageData  *config_data)
{

  HtmlData     *hdata ;
  gchar   *css_styles;
  GString *string;




  hdata = config_data->hdata;


  string = g_string_new ("\n<STYLE type=\"text/css\">\n");

  g_string_sprintfa (string, ".tr_header{background-color: %s; font-family: Arial, Helvetica, sans-serif;  color: %s;}\n", (hdata->h_bgcolor)?hdata->h_bgcolor:"", (hdata->h_text)?hdata->h_text:"");

  if (hdata->grid_color)
    {
      g_string_sprintfa (string, ".grid{background-color: %s;}\n", hdata->grid_color);
    }
  if (hdata->rows_bgcolor)
    g_string_sprintfa (string, ".first_row{background-color:%s}", hdata->rows_bgcolor);

  if (hdata->a_rows_bgcolor)
    g_string_sprintfa (string, ".second_row{background-color:%s}", hdata->a_rows_bgcolor);
  g_string_append (string, "</STYLE>\n");
  css_styles = string->str;
  g_string_free (string, FALSE);
  return css_styles;

}




static gchar *
html_plugin_get_table_header (HtmlConfigPageData  *config_data, gint page_number)
{
  GmExporterPluginData *epd;
  GmPluginData *pd;
  HtmlData     *hdata ;
  gchar   *table_header;
  GString *string;
  GList * columns_list;
  GtkCList * clist;
  gchar * alignment;
  


  hdata = config_data->hdata;
  pd    = config_data->pd;
  epd   = (GmExporterPluginData *) pd->private_data;
  clist = epd->data_clist;
  string = g_string_new ("");

  switch (hdata->table_alignment) {
      case TABLE_RIGHT_ALIGN:
          alignment = "right";
          break;
      case TABLE_CENTER_ALIGN:
          alignment = "center";
          break;          
      case TABLE_LEFT_ALIGN :
      default :          
          alignment = "left";
          break;
  }
  
  g_string_sprintfa (string, "\n<TABLE ALIGN=\"%s\" WIDTH=\"%s\" BORDER=\"0\" CLASS =\"grid\" CELLSPACING=\"%d\" CELLPADDING=\"3\">\n",alignment,
                      (hdata->table_width)?hdata->table_width:"", (hdata->grid_color)?DEFAULT_GRID_WIDTH:0);
  if (hdata->show_header)
    {
      if (hdata->show_header == SHOW_HEADER_ON_EVERY_PAGE || !page_number)
        {
          g_string_append (string, "<TR class=\"tr_header\">\n");
          columns_list = epd->columns_list;
          while (columns_list)
            {
              gint col = GPOINTER_TO_INT (columns_list->data);
              gchar *column_title;
              column_title = gm_clist_get_column_title (clist, col);
              g_string_append (string, "<TD class=\"td_header\" align =\"center\">\n");
              if (column_title)
                {
                  g_string_sprintfa (string, "%s\n", column_title);
                }
              g_string_append (string, "</TD>\n");
              columns_list = columns_list->next;
            }
        }
      g_string_append (string, "</TR>\n");
    }
  table_header = string->str;
  g_string_free (string, FALSE);
  return table_header;
}



static gint
html_plugin_export_data (HtmlConfigPageData  *config_data)
{
  GmExporterPluginData *epd;
  HtmlData *hdata;
  GmPluginData *pd;
  gboolean stoped = FALSE;
  gboolean written = FALSE;
  gboolean second_row = FALSE;
  GtkCList * clist;
  GString * string = NULL;
  FILE *file = NULL;
  GList * rows_selection;
  GList * columns_list;
  gint pages=0;
  gint p = 0;
  gint row = 0;
  gint max_rows;
  gint row_counter=0;
  gint absolut_row_counter=0;
  gfloat tick;
  gchar * base_name;
  gchar * base_dir;
  gchar * output_filename = NULL;
  gchar * table_header;
  gint rows_per_page;



  pd  = config_data->pd;
  epd = (GmExporterPluginData *) pd->private_data;
  hdata = config_data->hdata;

  rows_selection = epd->rows_selection;
  max_rows = g_list_length (rows_selection);
  clist = epd->data_clist;
  rows_per_page = hdata->rows_per_page;
  if (hdata->rows_per_page > 0 && hdata->rows_per_page < max_rows )
    pages = ceil ( (float) max_rows / (float) hdata->rows_per_page);
  else
    {
      rows_per_page = max_rows;
      pages = 1;
    }


  base_name = g_strdup (g_basename (epd->output_filename));
  base_dir = g_strdup (g_dirname (epd->output_filename));
  if (pages > 1)
    max_rows = rows_per_page;


  p = 0;

  while (rows_selection)
    {
      row = (epd->stype != ALL_ROWS)?GPOINTER_TO_INT (rows_selection->data):row = absolut_row_counter;
      if (!row && epd->ignore_first_row)
        {
          rows_selection = rows_selection->next;
          absolut_row_counter++;
          continue;
        }
      if (!row_counter)
        {
          gchar *message;
          gchar * header_string = NULL;
          gchar * css_styles = NULL;

          second_row = FALSE;
          if (pages > 1)
            output_filename = g_strdup_printf ("%s/page_%d_%s", base_dir, p+1, base_name);
          else
            output_filename = g_strdup_printf (epd->output_filename);

          file = gm_utils_open_file (output_filename);
          if (!file)
            {
              g_free (output_filename);
              stoped = TRUE;
              break;
            }
          message = g_strdup_printf (_("Exporting html file %s "), output_filename);
          g_free (output_filename);

          gtk_entry_set_text (GTK_ENTRY (pd->mini_buffer), message);
          gm_utils_progress_tick (pd->progressbar, 0.0);
          g_free (message);
          header_string = html_plugin_get_html_header (config_data);
          string = g_string_new (header_string);
          g_free (header_string);
          css_styles = html_plugin_get_css_styles (config_data);
          g_string_append (string, css_styles);
          g_free (css_styles);

          table_header = html_plugin_get_table_header (config_data, p);
          g_string_append (string, table_header);
          g_free (table_header);
          p++;
        }
      if (!second_row)
        {
          g_string_append (string, "<TR class=\"first_row\">\n");
          second_row = TRUE;
        }
      else
        {
          g_string_append (string, "<TR class=\"second_row\">\n");
          second_row = FALSE;
        }

      columns_list = epd->columns_list;
      while (columns_list)
        {
          gint col = GPOINTER_TO_INT (columns_list->data);
          gchar *column_data = NULL;

          gtk_clist_get_text (clist, row, col, &column_data);

          g_string_append (string, "<TD class=\"col_data\" align=\"left\">\n");
          if (column_data)
            {
              g_string_sprintfa (string, "%s\n", column_data);
            }
          g_string_append (string, "</TD>\n");
          columns_list = columns_list->next;
        }
      g_string_append (string, "</TR>\n");
      written = FALSE;
      if (row_counter == rows_per_page - 1)
        {
            gchar * footer;
            footer = html_plugin_get_html_footer (config_data);
            g_string_append (string, "</TABLE>\n");
            g_string_append (string, footer);
            g_free (footer);
          
            written = TRUE;
          if (!gm_utils_write_to_file (file, string->str))
            {
              g_string_free (string, TRUE);
              string = NULL;
              stoped = TRUE;
              break;
            }
          fclose (file);
          file = NULL;
          g_string_free (string, TRUE);
          string = NULL;
          row_counter = 0;
        }
      else
        {
          tick = (gfloat) row_counter / (gfloat) max_rows;
          gm_utils_progress_tick (pd->progressbar, tick);
          row_counter++;
        }
      rows_selection = rows_selection->next;
      absolut_row_counter++;
    }

  if (!written && !stoped)
    {
        gchar * footer;
        footer = html_plugin_get_html_footer (config_data);
        g_string_append (string, "</TABLE>\n");
        g_string_append (string, footer);
        g_free (footer);
      if (!gm_utils_write_to_file (file, string->str))
        stoped = TRUE;
    }
  if (string)
    g_string_free (string, TRUE);
  gtk_editable_delete_text (GTK_EDITABLE (pd->mini_buffer), 0, -1);
  gm_utils_progress_tick (pd->progressbar, 0.0);
  if (file)
    fclose (file);
  g_free (base_name);
  g_free (base_dir);

  if (!stoped)
    return TRUE;
  else
    return FALSE;
}
