/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gm-app.h"
#include "glade_interface.h"
#include "gm-control-center.h"
#include "gm-interface.h"
#include "gm-bpanel.h"
#include "gm-utils.h"
#include "gm-ccci.h"
#include "gm-connection.h"
#include "gm-plugin.h"
#include <glade/glade.h>





#define DEFAULT_COMPLETION_FILE_NAME "gmyclient-completion"

static guint app_signals[GM_APP_LAST_SIGNAL] = { 0 };

GmApp *global_app;



static void           app_emit_database_changed        (GmApp *app,gchar *path);
static void           app_emit_options_changed         (GmApp *app);
static void           app_emit_start                   (GmApp *app);
static void           gm_app_class_init (GmAppClass *klass);
static void           gm_app_init       (GmApp      *app);
static gboolean       gm_app_start (GmApp *app);
static void           gm_app_end (GmApp *app);
static void           gm_app_destroy            (GtkObject *object);
static GCompletion    *app_load_keys_funcs_completion_list (void);




static GtkDataClass *parent_class = NULL;




static void
gm_app_destroy (GtkObject * object){

    GmApp * app;



    app = GM_APP (object);
    gm_app_end (app);
    if (GTK_OBJECT_CLASS (parent_class)->destroy)
        (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}





GtkType
gm_app_get_type (void)
{
    static GtkType app_type = 0;

    if (!app_type)
    {
        static const GtkTypeInfo app_info =
            {
                "GmApp",
                sizeof (GmApp),
                sizeof (GmAppClass),
                (GtkClassInitFunc) gm_app_class_init,
                (GtkObjectInitFunc) gm_app_init,
                /* reserved_1 */ NULL,
                /* reserved_2 */ NULL,
                (GtkClassInitFunc) NULL,
            };

        app_type = gtk_type_unique (GTK_TYPE_DATA, &app_info);
    }

    return app_type;
}

static void gm_app_class_init (GmAppClass *class){

    GtkObjectClass *object_class;

    object_class = (GtkObjectClass*) class;

    app_signals[GM_SIGNAL_START] =
        gtk_signal_new ("start",
                        GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
                        object_class->type,
                        GTK_SIGNAL_OFFSET (GmAppClass, start),
                        gtk_marshal_NONE__NONE,
                        GTK_TYPE_NONE, 0);

    app_signals[GM_SIGNAL_END] =
        gtk_signal_new ("end",
                        GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
                        object_class->type,
                        GTK_SIGNAL_OFFSET (GmAppClass, end),
                        gtk_marshal_NONE__NONE,
                        GTK_TYPE_NONE, 0);

    app_signals[GM_SIGNAL_DATABASE_CHANGED] =
        gtk_signal_new ("database_changed",
                        GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
                        object_class->type,
                        GTK_SIGNAL_OFFSET (GmAppClass, database_changed),
                        gtk_marshal_NONE__NONE,
                        GTK_TYPE_NONE, 0);

    app_signals[GM_SIGNAL_OPTIONS_CHANGED] =
        gtk_signal_new ("options_changed",
                        GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
                        object_class->type,
                        GTK_SIGNAL_OFFSET (GmAppClass, options_changed),
                        gtk_marshal_NONE__NONE,
                        GTK_TYPE_NONE, 0);

    app_signals[GM_SIGNAL_SQL_MODE_COLORS_CHANGED] =
        gtk_signal_new ("sql_mode_colors_changed",
                        GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
                        object_class->type,
                        GTK_SIGNAL_OFFSET (GmAppClass, sql_mode_colors_changed),
                        gtk_marshal_NONE__NONE,
                        GTK_TYPE_NONE, 0);

    gtk_object_class_add_signals (object_class, app_signals, GM_APP_LAST_SIGNAL);
    class->start = NULL;
    class->database_changed = NULL;
    class->options_changed = NULL;
    class->sql_mode_colors_changed = NULL;
    class->end = NULL;
    object_class->destroy = gm_app_destroy;
    parent_class = gtk_type_class (gtk_data_get_type ());
}



static void gm_app_init (GmApp *app){

    GmAppMethodsdatabase * m_dbbase;

    app->cc = NULL;
    app->frozen = FALSE;
    app->logfilehandler = NULL;
    app->find_dialog       = NULL;
    app->clip_window       = NULL;
    app->flush_options_dialog = NULL;
    app->dumpout_dialog    = NULL;
    app->open_filesel      = NULL;
    app->save_filesel      = NULL;
    app->connections       = NULL;
    app->connection_dialog = NULL;
    app->keys_funcs_completion_list = app_load_keys_funcs_completion_list();    
    app->char_clipboard    = NULL;
    app->plugins_list      = NULL;
    app->regex_cache       = NULL;
    app->tooltips = gtk_tooltips_new ();
    gtk_tooltips_enable (app->tooltips);

    m_dbbase =  app->methods_dbase = g_new0 (GmAppMethodsdatabase, 1);
    m_dbbase->methods_hash = g_hash_table_new (g_str_hash, g_str_equal);
}



GmApp* gm_app_new (void){
    GmApp *app;

    app = gtk_type_new (gm_app_get_type ());
    global_app = app;
    app->cc = GTK_OBJECT(gm_cc_new(app));
    gm_cc_read_configuration(GM_CCENTER(app->cc));

    return app;
}




static void gm_app_end (GmApp *app){

    GmCcenterGeneral * ge_config;

    ge_config = GM_CCENTER(app->cc)->ge_config;
    if (ge_config->logfilehandler)
        fclose (ge_config->logfilehandler);
    app->frozen = TRUE;
    gnome_config_sync();
}




static gboolean gm_app_start (GmApp *app){

    glade_gnome_init();
    gm_ccci_init   ();
    gm_plugins_init ();

    app->regex_cache = gm_regex_cache_new();
    if (!gm_app_new_connection ())
        return FALSE;
    return TRUE;
};










static void app_emit_database_changed (GmApp *app,gchar *new_database){

    g_return_if_fail (app != NULL);
    g_return_if_fail (GM_IS_APP (app));
    gtk_signal_emit (GTK_OBJECT (app), app_signals[GM_SIGNAL_DATABASE_CHANGED]);
}


void app_emit_options_changed(GmApp *app){

    g_return_if_fail (app != NULL);
    g_return_if_fail (GM_IS_APP (app));


    gtk_signal_emit (GTK_OBJECT (app), app_signals[GM_SIGNAL_OPTIONS_CHANGED]);
}

void gm_app_emit_sql_mode_colors_changed(GmApp *app){

    g_return_if_fail (app != NULL);
    g_return_if_fail (GM_IS_APP (app));


    gtk_signal_emit (GTK_OBJECT (app), app_signals[GM_SIGNAL_SQL_MODE_COLORS_CHANGED]);
}


void app_emit_start(GmApp *app){
    g_return_if_fail (app != NULL);
    g_return_if_fail (GM_IS_APP (app));

    gtk_signal_emit (GTK_OBJECT (app), app_signals[GM_SIGNAL_START]);
}

void gm_app_emit_end(GmApp *app){
    g_return_if_fail (app != NULL);
    g_return_if_fail (GM_IS_APP (app));


    gtk_signal_emit (GTK_OBJECT (app), app_signals[GM_SIGNAL_END]);
}






void gm_app_emit_signal (GmApp * app, gint signal, gchar *chardata, gint gintdata){

    switch (signal){
        case GM_SIGNAL_START:{
            app_emit_start(app);
            break;
        }
        case GM_SIGNAL_DATABASE_CHANGED:{
            app_emit_database_changed (app, chardata);
            break;
        }
        case GM_SIGNAL_OPTIONS_CHANGED:{
            app_emit_options_changed(app);
            break;
        }
        default:
            break;
    }
}





void gm_application_exit (GtkWidget * egal, gpointer user_data){

    static gboolean was_hier = FALSE;
    GList * c;




    if (!was_hier) {
        if (gm_utils_yes_no_dialog("Do you realy want to Quit Gmyclient?"))
        {
            was_hier = TRUE;
            gm_cc_update_configuration () ;
            gm_app_end (global_app);

            for (c = global_app->connections; c ; c = c->next)
                gtk_object_destroy (GTK_OBJECT(c->data));

            gtk_main_quit();
        }
    }
}

void gm_application_exit_immidiatly (GtkWidget * egal, gpointer user_data){


    GList * c;

    GmConnection * con;

    gm_cc_update_configuration () ;
    gm_app_end (global_app);


    for (c = global_app->connections; c ; c = c->next){
        con = c->data;
        gtk_object_destroy (GTK_OBJECT(con));
    }
    gtk_main_quit();
}


void gm_application_close_connection (GtkWidget * egal, GmConnection * connection){






    if (!GM_IS_CONNECTION(connection))
        return;

    gtk_object_destroy (GTK_OBJECT(connection));
    global_app->connections = g_list_remove (global_app->connections, connection);

    if (!global_app->connections) {
        gm_application_exit_immidiatly (NULL,NULL);
    }


}


void gm_app_main (void){
    GmApp * app;
    app = gm_app_new();
    if (gm_app_start(app))
        gtk_main();
}




GmNpageData * gm_app_new_npage_data (void)
{
    GmNpageData *npage_data;

    npage_data = g_new0 (GmNpageData, 1);

    npage_data->data_list_sw = NULL;
    npage_data->data_list = NULL;
    npage_data->type = GM_NPAGE_USER_QUERY;
    npage_data->parent = NULL;
    npage_data->label = NULL;
    npage_data->query = NULL;    
    return npage_data;
}


void gm_app_free_npage_data (GmNpageData * npage_data)
{
    g_free (npage_data->query);    
    g_free (npage_data);
}



GmConnection * gm_app_new_connection (void)
{
    GmConnection * connection;

    connection = gm_connection_new();
        /* the connection module popup the connection-dialog and create the connection object only if the connection
           could be established , so we can add it to our list */

    if (connection)
        global_app->connections = g_list_append (global_app->connections, connection);




    return connection;
}

static GCompletion * 
app_load_keys_funcs_completion_list (void)
{

    GCompletion * completion = NULL;   
    gchar * home_dir;
    gchar * filename;
    GList * list = NULL;

    home_dir = g_get_home_dir();
    filename = g_strdup_printf ("%s/misc/%s",PACKAGE_DATA_DIR,DEFAULT_COMPLETION_FILE_NAME);
    list = gm_utils_load_file_as_list(filename);
    g_free (filename);
    if (list){        
        completion = g_completion_new (NULL);    
        g_completion_add_items (completion, list);
    }    
    return completion;
}
