#ifndef _GM_APP_H_
#define _GM_APP_H_
#include <gnome.h>
#include "gm-connection.h"
#include "gm-regex.h"
#define GM_TYPE_APP                  (gm_app_get_type ())
#define GM_APP(obj)                  (GTK_CHECK_CAST ((obj), GM_TYPE_APP, GmApp))
#define GM_APP_CLASS(klass)          (GTK_CHECK_CLASS_CAST ((klass), GM_TYPE_APP, GmAppClass))
#define GM_IS_APP(obj)               (GTK_CHECK_TYPE ((obj), GM_TYPE_APP))
#define GM_IS_APP_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GM_TYPE_APP))





typedef struct _GmApp	    GmApp;
typedef struct _GmAppClass  GmAppClass ;



typedef struct _GmAppMethodsdatabase GmAppMethodsdatabase ;
typedef struct _GmAppMethod GmAppMethod ;
typedef struct _GmNpageData GmNpageData;

typedef struct _GmClipBoard    GmClipBoard;
typedef struct _GmClipBoardData    GmClipBoardData;






struct _GmApp
{
    GtkObject object;

    gboolean frozen;
    GtkObject * cc;
    FILE * logfilehandler;


    GtkWidget * connection_dialog;
    GtkWidget * prop_window;
    GtkWidget * clip_window;
    GtkWidget * find_dialog;
    GtkWidget * open_filesel;
    GtkWidget * save_filesel;
    GtkWidget * dumpout_dialog;
    GtkWidget * flush_options_dialog;
    GList * connections;

    GmClipBoard * clipboard;

    GtkTooltips *tooltips;
    GList	*plugins_list;
    GmRegexCache *regex_cache;
    /* Methods are still under development */

    GmAppMethodsdatabase *methods_dbase;

    /* Data used globaly */
    gchar *char_clipboard;
    GCompletion *keys_funcs_completion_list;
};

struct _GmClipBoard
{
    GmClipBoardData * databases;
    GmClipBoardData * tables;
    GmClipBoardData * rows;
};




struct _GmClipBoardData
{
    gint    type;
    gchar * host;
    gchar * database;
    gchar * table;
    GList * list;  /* list pointer to the data stored in the clipboard
                    * depends on type                    
                    */
    GSList * src_fields_list ; /* list holding the structure of the source table
                                *in case of copying rows from table 
                                */
};



enum {
    APP_DEFAULT_MODE=0,
    APP_COMMAND_MODE,
};





struct _GmAppClass
{
    GtkObjectClass parent_class;

    void (* start)	                 (GmApp *app);
    void (* end)                         (GmApp *app);
    void (* database_changed)	         (GmApp *app);
    void (* options_changed)	         (GmApp *app);
    void (* sql_mode_colors_changed)	 (GmApp *app);
};


typedef gint (*GmMethodFunc)	(gpointer   data);

#define GM_METHOD_FUNC(X) (GmMethodFunc)(X)
#define MAX_METHODS_QUERY_LEN 255

struct _GmAppMethod {
    gchar *name;

    GmMethodFunc func;
};


struct _GmAppMethodsdatabase {
    GHashTable *methods_hash;
};

struct _GmNpageData
{
    GtkWidget *parent;
    GtkWidget *data_list_sw;
    GtkWidget *data_list;
    GtkWidget *label;
    gchar *query;    
    gint type;
};

enum {
    GM_NPAGE_BROWSER=0,
    GM_NPAGE_ROWEDIT,
    GM_NPAGE_TABLE_PROP,
    GM_NPAGE_USER_QUERY,
    GM_NPAGE_VARIABLES
};





enum {
    GM_SIGNAL_START,
    GM_SIGNAL_DATABASE_CHANGED,
    GM_SIGNAL_OPTIONS_CHANGED,
    GM_SIGNAL_SQL_MODE_COLORS_CHANGED,
    GM_SIGNAL_END,
    GM_APP_LAST_SIGNAL
};

enum {
    GM_APP_ARRANGE_HORIZONTALY =0,
    GM_APP_ARRANGE_VERTICALY
};


enum {
    GM_SQL_LOG_PANEL_VISIBLE = 1 << 1,
    GM_DBTREE_VISIBLE = 1 << 2
};



GmApp       * new_gmapp(void);
GtkType	      gm_app_get_type		(void);
GmApp       * gm_app_new (void);
void          gm_app_emit_signal                  (GmApp * app, gint signal, gchar *chardata, gint gintdata);
void          gm_app_emit_end                     (GmApp *app);
void          gm_app_emit_sql_mode_colors_changed (GmApp *app);
void          gm_app_main (void);
void          gm_application_exit (GtkWidget * egal, gpointer user_data);
void          gm_application_exit_immidiatly (GtkWidget * egal, gpointer user_data);
void          gm_application_close_connection (GtkWidget * egal, GmConnection * connection);
void          gm_app_free_npage_data (GmNpageData * npage_data);
GmNpageData * gm_app_new_npage_data (void);




GmConnection *gm_app_new_connection (void) ;



/*
 *  MDatabase  routines
 *
 */


void       gm_app_register_method (GmApp *app, GmAppMethod *method);
gint       gm_app_call_method (GmApp * app, gchar * method_key );

#endif /* __GM_APP_H__ */







