/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gm.h"
#include "gm-ccci.h"
#include "glade_interface.h"
#include "gm-table-properties.h"
#include "gm-entry.h"
#include "gm-bpanel.h"
#include "gm-dbapi.h"
#include "gm-clist.h"
#include "dotdot.xpm"
#include "gm-formular.h"
#include "gm-dumpout.h"
#include "gm-filter.h"
#include "gm-interface.h"






static void             bpanel_browse_by_query (GmConnection *connection, gchar * query, BpanelViewInfo * view_info);
static void             bpanel_show_status_information (GmConnection * connection, DbQuery * q, BpanelViewInfo * view_info);
static void             bpanel_data_list_row_activated (GmClist * mlist, gint row, GmConnection * connection);
static void             bpanel_data_list_resorted (GmClist * mlist, gint col, gint direction, DbBrowserNodeInfo * info) ;
static BpanelViewInfo * bpanel_clone_view_info (BpanelViewInfo * src);
static BpanelViewInfo * bpanel_new_view_info (void);
static void             bpanel_edit_selected_row (GmClist * mlist, GmConnection * connection);
static gint             bpanel_key_press_event (GtkWidget *widget, GdkEventKey *event, GmConnection * connection);
static void             bpanel_new_table_properties_window (GmClist * mlist, GmConnection * connection);
static void             bpanel_new_edit_notebook_page (GmConnection * connection, BpanelViewInfo * view_info , GSList * old_data_list);
static void             bpanel_popup_up_activate (GtkMenuItem     *menuitem, gpointer         user_data);
static void             bpanel_configure_bpanel_context_menu ( GmConnection * connection) ;
static void             on_filter_sbutton_toggled (GtkButton *filter_sbutton, GmConnection * connection);


static DbBrowserNodeInfo * new_dbbrowsernodeinfo  (void);
static void                free_dbbrowsernodeinfo (DbBrowserNodeInfo *info);



static DbBrowserNodeInfo * new_dbbrowsernodeinfo  (void)
{
    DbBrowserNodeInfo * info;
    info = g_new0 (DbBrowserNodeInfo, 1);
    info->name = NULL;
    info->type = -1;
    info->sorted_by_column = -1;
    info->selected_row = 0;
    info->sort_direction = SORT_NONE;
    info->filter_query = NULL;
    info->filter_enabled = FALSE;
    info->text_fields = NULL;
    return info;
}

static void                free_dbbrowsernodeinfo (DbBrowserNodeInfo *info)
{
    g_return_if_fail (info != NULL);
    g_free (info->name);
    g_free (info->filter_query);
    g_free (info);
    if (info->text_fields){
        g_list_g_free (info->text_fields);
        g_free (info->text_fields);
    }
}



static BpanelViewInfo * bpanel_new_view_info (void)

{
    BpanelViewInfo * view_info;

    view_info = g_new0 (BpanelViewInfo, 1);
    view_info->view_type = 0;
    view_info->database_in_use = NULL;
    view_info->table_in_use = NULL;
    view_info->last_view_row_pos = 0;
    view_info->row_pos = 0;
    view_info->rows_count = 0;
    view_info->fields_list = NULL;
    view_info->table_info = NULL;
    return view_info;
}



static BpanelViewInfo * bpanel_clone_view_info (BpanelViewInfo * src)

{
    BpanelViewInfo * view_info;

    view_info = g_new0 (BpanelViewInfo, 1);
    view_info->view_type = src->view_type;
    view_info->database_in_use = NULL;
    if (src->database_in_use)
        view_info->database_in_use = g_strdup (src->database_in_use);
    view_info->table_in_use = NULL;
    if (src->table_in_use)
        view_info->table_in_use = g_strdup (src->table_in_use);
    view_info->last_view_row_pos = src->last_view_row_pos;
    view_info->row_pos = src->row_pos;
    view_info->rows_count = src->rows_count;
    view_info->fields_list = gm_bpanel_clone_fields_list (src->fields_list);
    return view_info;
}

void gm_bpanel_free_fields_list (GSList *fields_list)
{
    GSList * l = NULL;

    if (fields_list)
    {
        l = g_slist_nth (fields_list, 0);
        while (l)
        {
            MYSQL_FIELD *field = (MYSQL_FIELD *) l->data;
            g_free (field->name);
            g_free (field->table);
            g_free (field->def);
            l = l->next;
        }
        g_slist_free (fields_list);
    }
}


void  gm_bpanel_free_view_info (BpanelViewInfo * view_info)

{


    g_free(view_info->database_in_use);
    g_free(view_info->table_in_use);
    gm_bpanel_free_fields_list (view_info->fields_list);
    if (view_info->table_info)
        gm_db_free_table_info (view_info->table_info);
    g_free (view_info);
}





GSList * gm_bpanel_clone_fields_list (GSList *src)

{
    GSList * slist = NULL;
    GSList * l ;

    if (!src)
        return NULL;
    l = g_slist_nth (src, 0);
    while (l)
    {
        MYSQL_FIELD *f;
        MYSQL_FIELD *field = (MYSQL_FIELD *) l->data;
        f = g_new0(MYSQL_FIELD, 1);
        f->name   = g_strdup (field->name);
        f->table  = g_strdup (field->table);
        f->def    = g_strdup (field->def);
        f->type   = field->type;
        f->length = field->length;
        f->max_length = field->max_length;
        f->flags = field->flags;
        f->decimals = field->decimals;
        slist = g_slist_append (slist, (gpointer) f);
        l = l->next;
    }
    return slist;
}


void gm_bpanel_edit_selected_row (GtkButton *button, GmConnection *connection)

{
    GmNpageData * npage_data;
    GtkWidget * data_list;
    GmClist * mlist;
    GtkWidget * npage;

    g_return_if_fail ( connection != NULL);
    npage = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");

    npage_data = (GmNpageData *) gm_utils_get_data (GTK_OBJECT(npage),"npage_data");



    data_list = npage_data->data_list;
    mlist = GM_CLIST (data_list);
    bpanel_edit_selected_row (mlist, connection);
}


void gm_bpanel_save_under (void * ignore, GmConnection * connection)
{


    GmNpageData * npage_data;
    GtkWidget * npage;

    g_return_if_fail ( connection != NULL);
    npage = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");

    npage_data = (GmNpageData *) gm_utils_get_data (GTK_OBJECT(npage),"npage_data");

    gm_utils_clist_save_under_activated (NULL, GTK_CLIST(npage_data->data_list));

}













void gm_bpanel_delete_selected_row (void *ignore, GmConnection *connection)

{
    GmNpageData * npage_data;
    GtkWidget * data_list;
    BpanelViewInfo * view_info;
    GtkCList * clist;
    GtkWidget * npage;
    GList * list ;

    g_return_if_fail ( connection != NULL);


    npage = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");
    npage_data = (GmNpageData *) gm_utils_get_data (GTK_OBJECT(npage),"npage_data");



    data_list = npage_data->data_list;
    view_info = gm_utils_get_data (GTK_OBJECT (data_list), "view_info");
    clist = GTK_CLIST (data_list);
    list = clist->selection;
    if (!list)
        return;
    switch (view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            GList * remove_list = NULL;
            while (list)
            {

                gint  sel_row = GPOINTER_TO_INT (list->data);
                gchar * database = NULL;

                if (gtk_clist_get_row_data (clist, sel_row))
                {
                    /* up row */
                    list = list->next;
                    continue;
                }
                gtk_clist_get_text              (clist, sel_row, 0,&database);
                if (database)
                    remove_list = g_list_append (remove_list, g_strdup (database));
                list = list->next;
            }
            if (remove_list){
                gm_interface_delete_databases (connection, remove_list);
                g_list_g_free (remove_list);
            }
            break;
        }
    case TABLES_VIEW :
        {
            GList * remove_list = NULL;
            if (!list)
                break;
            if (g_list_length (list) ==1 && GPOINTER_TO_INT (list->data) == 0)
                break;
            while (list)
            {

                gint  sel_row = GPOINTER_TO_INT (list->data);
                gchar * database = NULL;
                if (gtk_clist_get_row_data (clist, sel_row))
                {
                    /* up row */
                    list = list->next;
                    continue;
                }
                gtk_clist_get_text              (clist, sel_row, 0,&database);
                if (database)
                    remove_list = g_list_append (remove_list, g_strdup (database));
                list = list->next;
            }
            if (remove_list){
                gm_interface_delete_tables (connection, view_info->database_in_use, remove_list);
                g_list_g_free (remove_list);
            }
            break;
        }
    case ROWS_VIEW :
        {
            gchar * database = view_info->database_in_use;
            if (!list)
                break;
            if (g_list_length (list) ==1 && GPOINTER_TO_INT (list->data) == 0)
                break;


            if (gm_utils_yes_no_dialog("Do you realy want to drop the selected datarow(s) ?"))
            {
                DbTableInfo *table_info;
                gboolean has_unique_keys = FALSE;

                table_info = gm_db_has_unique_keys(connection, view_info->database_in_use, view_info->table_in_use, &has_unique_keys);
                gtk_clist_freeze (clist);
                while (clist->selection)
                {
                    DbQuery * q;
                    gint col = 0;
                    gint  sel_row ;
                    GSList * l;
                    GString * dquery;

                    sel_row =  GPOINTER_TO_INT (clist->selection->data);
                    if (!sel_row)
                    {
                        gtk_clist_unselect_row (clist, 0, 0);
                        continue;
                    }

                    dquery = g_string_new ("delete from ");
                    g_string_append (dquery, view_info->table_in_use);
                    g_string_append (dquery, " where ");
                    l = g_slist_nth (view_info->fields_list, 0);

                    while (l)
                    {
                        gchar * field_data = NULL;
                        gboolean set_where_query = FALSE;
                        MYSQL_FIELD *field = (MYSQL_FIELD *) l->data;
                        gtk_clist_get_text              (clist, sel_row, col,&field_data);

                        /***** ROW DELETE ***/
                        if (has_unique_keys)
                        {
                            GList *keys_info_list = table_info->keys_info_list;
                            while (keys_info_list){
                                DbKeyInfo * key_info = (DbKeyInfo *) keys_info_list->data;
                                if (key_info->is_unique || key_info->is_primary_key)
                                {
                                    GList *l = key_info->affected_column_list;
                                    while (l) {
                                        gchar *name = (gchar * )l->data;
                                        if (name && !strcmp (field->name, name)){
                                            set_where_query = TRUE;
                                            break;
                                        }
                                        l=l->next;
                                    }
                                    if (set_where_query)
                                        break;
                                }
                                keys_info_list = keys_info_list->next;
                            }
                        }
                        if (!has_unique_keys || (has_unique_keys && set_where_query))
                        {
                            if (field_data)
                            {
                                field_data = gm_utils_strescape (field_data, NULL);
                            }

                            if (col)
                            {
                                if (!field_data)
                                    g_string_sprintfa (dquery, "AND %s IS NULL ",field->name);
                                else
                                    g_string_sprintfa (dquery, "AND %s='%s' ",field->name, field_data);
                            }
                            else
                            {
                                if (!field_data)
                                    g_string_sprintfa (dquery, " %s IS NULL ",field->name);
                                else
                                    g_string_sprintfa (dquery, " %s='%s' ",field->name, field_data);
                            }
                            g_free (field_data);
                        }
                        l = l->next;
                        col++;
                    }


                    g_string_append (dquery, "limit 1");
                    q = gm_db_new_query (dquery->str);
                    q->database = database;
                    if (!gm_db_request(connection, q))
                    {
                        gm_utils_write_log_message (connection,q->error,  ERROR_LOGMODE);
                        gm_db_free_query (q);
                        g_string_free (dquery, TRUE);
                        gtk_clist_thaw (clist);
                        return;
                    }
                    gm_db_free_query (q);
                    g_string_free (dquery, TRUE);
                    gtk_clist_remove (clist, sel_row);
                }
                clist->focus_row = 0;
                gtk_clist_thaw (clist);
                gm_clist_select_row (clist, 0 ,0);
            }
            break;
        }
    }
}


gchar * gm_bpanel_get_view_query (GmConnection * connection, BpanelViewInfo * view_info)
{

    gchar * query = NULL;
    DbBrowserNodeInfo * info = NULL;

    info = gm_bpanel_get_node_info (connection, view_info);

    switch (view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            if (!info || !info->filter_query || !info->filter_enabled)
                query = g_strdup(GE_CONFIG(global_app)->default_databases_query);
            else
                query = g_strdup(info->filter_query);
            break;
        }
    case TABLES_VIEW :
        {
            if (!info || !info->filter_query || !info->filter_enabled){
                gchar * default_tables_query;
                GmCcenter * cc;
                GmCcenterGeneral * ge_config ;

                cc = GM_CCENTER (global_app->cc);

                ge_config = cc->ge_config;
                default_tables_query = ge_config->default_tables_query;

                query = g_strdup(default_tables_query);
            }
            else {
                query = g_strdup(info->filter_query);
            }

            break;
        }
    case ROWS_VIEW :
        {
            gchar * temp_query;

            if (!info || !info->filter_query || !info->filter_enabled){
                temp_query = g_strdup(GE_CONFIG(global_app)->default_rows_query);
                query = g_str_replace (temp_query, "&table_in_use", view_info->table_in_use);
                g_free(temp_query);
            }
            else{
                query = g_strdup(info->filter_query);
            }
            break;
        }

    }
    return query;

}



void gm_bpanel_reload (void * ignore, GmConnection * connection)

{

    GtkWidget * data_list;
    BpanelViewInfo * view_info;
    BpanelViewInfo * cloned_view_info;
    GtkWidget * npage;
    GmNpageData * npage_data;
    gchar * query= NULL;


    g_return_if_fail ( connection != NULL);


    npage = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");
    npage_data = gm_utils_get_data (GTK_OBJECT (npage), "npage_data");

    data_list = npage_data->data_list;

    view_info = gm_utils_get_data (GTK_OBJECT (data_list), "view_info");


    cloned_view_info = bpanel_clone_view_info (view_info);



    query = gm_bpanel_get_view_query (connection, view_info);

    if (query)
    {
        bpanel_browse_by_query (connection, query, cloned_view_info);
        data_list = npage_data->data_list;
        g_free (query);
    }else
    {
        gm_bpanel_free_view_info (cloned_view_info);
    }


}



static gint bpanel_key_press_event      (GtkWidget       *widget,
        GdkEventKey     *event,
        GmConnection * connection)

{


    if (!(event->state & GDK_CONTROL_MASK) && !(event->state & GDK_MOD1_MASK))
    {
        switch (event->keyval)
        {
        case GDK_F7:
            {
                gm_bpanel_new_data_list_row (GM_CLIST (widget), connection);
                break;
            }
        case GDK_F4:
            {
                bpanel_edit_selected_row (GM_CLIST (widget), connection);
                break;
            }

        case GDK_BackSpace :
            {
                gm_bpanel_go_up (NULL, connection);
                return TRUE;
                break;
            }
        case GDK_Delete :
            {
                gm_bpanel_delete_selected_row (NULL, connection);
                break;
            }
        }
    }

    if (event->state & GDK_CONTROL_MASK)
    {
        switch (event->keyval)
        {
        case GDK_r :
            {
                gm_bpanel_reload (NULL, connection);
                return TRUE;
            }
        }
    }


    return FALSE;
}







void gm_bpanel_new_data_list_row (GmClist * mlist, GmConnection * connection)

{
    BpanelViewInfo * old_view_info;




    g_return_if_fail (mlist != NULL || connection != NULL);
    old_view_info = gm_utils_get_data (GTK_OBJECT (mlist), "view_info");

    if (!old_view_info)
        return;


    switch (old_view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            gm_interface_create_new_database (NULL, connection);
            break;
        }

    case TABLES_VIEW :
        {
            bpanel_new_table_properties_window (mlist, connection);
            break;
        }
    case ROWS_VIEW :
        {
            if (old_view_info->fields_list)
            {
                BpanelViewInfo * cloned_view_info;
                cloned_view_info = bpanel_clone_view_info (old_view_info);
                cloned_view_info->table_info = gm_db_get_table_info (connection, cloned_view_info->table_in_use, cloned_view_info->database_in_use);
                bpanel_new_edit_notebook_page (connection, cloned_view_info, NULL);
            }
            break;
        }

    }
}


static void bpanel_data_list_resorted (GmClist * mlist, gint col, gint direction, DbBrowserNodeInfo * info)
{

    g_return_if_fail (info != NULL);
    info->sorted_by_column = col;
    info->sort_direction = direction;

}


void gm_bpanel_go_to_table (gchar * database, gchar * table_name, GmConnection * connection)
{

    BpanelViewInfo * view_info = NULL;
    gchar * query;

    view_info = bpanel_new_view_info ();
    view_info->last_view_row_pos = 0;
    view_info->view_type =   ROWS_VIEW ;

    view_info->database_in_use = g_strdup (database);
    view_info->table_in_use = g_strdup (table_name);
    query = gm_bpanel_get_view_query (connection, view_info);
    bpanel_browse_by_query (connection, query, view_info);

    g_free (query);

}


void gm_bpanel_show_tables (GmConnection *connection, gchar * database)
{


    BpanelViewInfo * view_info = NULL;
    gchar * query;

    view_info = bpanel_new_view_info ();
    view_info->last_view_row_pos = 0;
    view_info->view_type =   TABLES_VIEW;

    view_info->database_in_use = g_strdup (database);
    query = gm_bpanel_get_view_query (connection, view_info);
    bpanel_browse_by_query (connection, query, view_info);
    g_free (query);
}





static void bpanel_data_list_row_activated (GmClist * mlist, gint row, GmConnection * connection)

{
    BpanelViewInfo * view_info = NULL;
    BpanelViewInfo * old_view_info;
    DbBrowserNodeInfo * info;
    gchar *cell_data = NULL;
    gchar * query = NULL;
    gboolean  up_row = FALSE;
    gint i = 0;

    old_view_info = gm_utils_get_data (GTK_OBJECT (mlist), "view_info");
    info = gm_utils_get_data (GTK_OBJECT (mlist), "info");
    info->selected_row = row;

    g_return_if_fail (old_view_info != NULL);

    if (!gtk_clist_get_row_data (GTK_CLIST (mlist), row)){
        gtk_clist_get_text (GTK_CLIST (mlist), row, 0,&cell_data);
        up_row = FALSE;
    }else
        up_row = TRUE;

    view_info = bpanel_new_view_info ();
    view_info->last_view_row_pos = row;

    switch (old_view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            if (cell_data)
            {
                view_info->view_type =   TABLES_VIEW ;
                view_info->database_in_use = g_strdup (cell_data);
                query = gm_bpanel_get_view_query (connection, view_info);
            }
            break;
        }
    case TABLES_VIEW :
        {
            if (!up_row)
            {
                view_info->view_type =   ROWS_VIEW ;
                view_info->database_in_use = g_strdup (old_view_info->database_in_use);
                view_info->table_in_use = g_strdup (cell_data);
                query = gm_bpanel_get_view_query (connection, view_info);
            }else
            {
                view_info->view_type = DATABASES_VIEW;
                view_info->row_pos = old_view_info->last_view_row_pos;
                query = gm_bpanel_get_view_query (connection, view_info);
            }
            break;
        }

    case ROWS_VIEW :
        {
            if (!up_row)
            {
                if (old_view_info->fields_list)
                {
                    GSList * l;
                    BpanelViewInfo * cloned_view_info;

                    GSList * old_fields_data=NULL;
                    l = old_view_info->fields_list;
                    i = 0;
                    while (l)
                    {
                        gchar * value=NULL;
                        gtk_clist_get_text (GTK_CLIST (mlist), row, i,&value);
                        if (!value)
                            old_fields_data = g_slist_append (old_fields_data, NULL);
                        else
                            old_fields_data = g_slist_append (old_fields_data, g_strdup(value));
                        l = l->next;
                        i++;
                    }
                    cloned_view_info = bpanel_clone_view_info (old_view_info);
                    cloned_view_info->table_info = gm_db_get_table_info (connection, cloned_view_info->table_in_use, cloned_view_info->database_in_use);
                    bpanel_new_edit_notebook_page (connection, cloned_view_info, old_fields_data);
                }
            }else
            {
                view_info->view_type = TABLES_VIEW;
                view_info->row_pos = old_view_info->last_view_row_pos;
                view_info->database_in_use = g_strdup (old_view_info->database_in_use);
                query = gm_bpanel_get_view_query (connection, view_info);
            }
            break;
        }
    }
    if (query)
    {
        bpanel_browse_by_query (connection, query, view_info);
        g_free (query);
    }else
    {
        gm_bpanel_free_view_info (view_info);
    }
}









static GtkWidget * bpanel_create_clist_scrolled_window (void)

{
    GtkWidget * data_list_sw;

    data_list_sw = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_set_border_width (GTK_CONTAINER (data_list_sw), 2);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (data_list_sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);    
    return data_list_sw;
}


static gboolean
on_dbbrowser_clist_focus_in_event        (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data)
{

    return FALSE;
}


static gboolean
on_dbbrowser_clist_focus_out_event       (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data)
{

    return FALSE;
}

static GtkWidget * bpanel_create_clist (gint columns, BpanelViewInfo * view_info)

{
    GtkWidget * data_list;

    data_list = gm_clist_new (columns);
    gtk_clist_set_selection_mode (GTK_CLIST (data_list), GTK_SELECTION_EXTENDED);
    gtk_object_set_data_full (GTK_OBJECT(data_list), "view_info", (gpointer) view_info,(GtkDestroyNotify) gm_bpanel_free_view_info);
    gtk_signal_connect (GTK_OBJECT (data_list), "focus_in_event",
                        GTK_SIGNAL_FUNC (on_dbbrowser_clist_focus_in_event),
                        NULL);
    gtk_signal_connect (GTK_OBJECT (data_list), "focus_out_event",
                        GTK_SIGNAL_FUNC (on_dbbrowser_clist_focus_out_event),
                        NULL);

    return data_list;
}



static GString * bpanel_compose_key_string (GmConnection * connection, BpanelViewInfo * view_info)
{
    GmConnectionAccessData * access_data;
    GString * key_string;


    g_return_val_if_fail (connection != NULL || view_info != NULL , NULL);


    access_data = connection->access_data;
    key_string = g_string_new ("");

    g_string_append (key_string, access_data->host);

    switch (view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            break;
        }
    case TABLES_VIEW :
        {
            g_string_sprintfa (key_string,":%s", view_info->database_in_use);
            break;
        }
    case ROWS_VIEW :
        {
            g_string_sprintfa (key_string,":%s:%s", view_info->database_in_use,view_info->table_in_use);
            break;
        }
    }
    return key_string;
}


DbBrowserNodeInfo *  gm_bpanel_get_node_info (GmConnection * connection, BpanelViewInfo * view_info)
{
    GString * key_string;
    DbBrowserNodeInfo * info = NULL;

    g_return_val_if_fail (connection != NULL || view_info != NULL , NULL);

    key_string = bpanel_compose_key_string (connection, view_info);

    if (key_string)
        info = g_hash_table_lookup (connection->info_hash, key_string->str);

    g_string_free (key_string, TRUE);

    return info;
}

DbBrowserNodeInfo * gm_bpanel_create_node_info (GmConnection * connection, BpanelViewInfo * view_info)
{

    GString * key_string;
    DbBrowserNodeInfo * info = NULL;

    g_return_val_if_fail (connection != NULL || view_info != NULL , NULL);

    info = gm_bpanel_get_node_info (connection, view_info);

    if (info)
        free_dbbrowsernodeinfo (info);

    key_string = bpanel_compose_key_string (connection, view_info);

    info = new_dbbrowsernodeinfo ();
    info->type = view_info->view_type;
    info->name = g_strdup (key_string->str);
    if (view_info->view_type == TABLES_VIEW)
        info->sorted_by_column = 3;

    g_hash_table_insert (connection->info_hash, info->name, (gpointer) info);
    g_string_free (key_string, TRUE);

    return info;

}


static void gm_bpanel_restore_view_session (GmConnection * connection, GtkCList * clist)
{

    DbBrowserNodeInfo * info;
    BpanelViewInfo * view_info = NULL;

    view_info = gm_utils_get_data (GTK_OBJECT (clist), "view_info");

    g_return_if_fail (view_info != NULL);
    g_return_if_fail (connection != NULL);

    info = gm_bpanel_get_node_info (connection, view_info);


    if (!info)
        info = gm_bpanel_create_node_info (connection, view_info);




    gtk_signal_connect (GTK_OBJECT (clist), "resorted",
                        GTK_SIGNAL_FUNC (bpanel_data_list_resorted),
                        info);
    gtk_object_set_data (GTK_OBJECT (clist), "info",info);

    clist->focus_row = info->selected_row;
    if (info->sorted_by_column != -1)
        gm_clist_sort_by_column (clist, info->sorted_by_column, info->sort_direction);

    gm_clist_select_row (clist, info->selected_row, 0);
}






static void bpanel_show_result (DbQuery * q, GtkWidget *data_list)

{
    BpanelViewInfo * view_info = NULL;
    MYSQL_RES *result;
    MYSQL_ROW row;
    GSList *f = NULL;
    GtkWidget * label;
    GtkCList * clist;
    gchar *empty_row[100];
    gint i =0;
    gboolean alternate;
    GdkColor color;
    GmConnection * connection;
    static GdkPixmap * pix=NULL;
    static GdkPixmap * mask=NULL;


    g_return_if_fail (q != NULL);
    result = q->mysql_result;
    clist = GTK_CLIST (data_list);

    if (!pix)
        pix = gdk_pixmap_create_from_xpm_d    (data_list->window,&mask, NULL,dotdot_xpm);

    view_info = gm_utils_get_data (GTK_OBJECT (clist), "view_info");
    connection = gm_utils_get_data (GTK_OBJECT (clist), "connection");
    gtk_clist_freeze (clist);
    gtk_clist_column_titles_show (clist);
    f = g_slist_nth(q->fields_list, 0);
    while (f)
    {
        MYSQL_FIELD * field;
        gchar * fname;
        field = (MYSQL_FIELD *) f->data;
        gtk_clist_set_column_auto_resize (clist, i, TRUE);
        fname = g_strdup_printf (" %s ", field->name);
        label = gtk_label_new (fname);
        g_free (fname);
        gtk_widget_show (label);
        gm_clist_set_column_widget (GM_CLIST (data_list), i, label);
        gm_clist_set_column_sortdata_type (GM_CLIST (data_list), i, field->type);
        i++;
        f = f->next;
    }
    gdk_color_parse                 ("#ededed", &color);
    mysql_data_seek(result,0);
    alternate = FALSE;
    for (i=0;i<100;i++)
    {
        empty_row[i] = NULL;
    }
    empty_row[0]="..";
    i = 0;
    if (view_info->view_type != DATABASES_VIEW){
        gtk_clist_append (clist,empty_row);
        gtk_clist_set_row_data (clist, 0, "up_row");
        gtk_clist_set_pixmap (clist, 0, 0, pix, mask);
        i++;
    }

    while ((row = mysql_fetch_row(result)))
    {

#if 0


        unsigned long *lengths;
        gint s;
        lengths = mysql_fetch_lengths(result);
        for (s=0; s<clist->columns;s++){
            if (row[s])
                printf("[%.*s] \t", (int) lengths[s], row[s]);
            else
                printf("[NULL]\t");
        }
#endif            



        i = gtk_clist_append (clist,row);
        if (alternate )
        {
            //            gtk_clist_set_background (clist, i, &color);
        }
        alternate = !alternate;
    }
    view_info->rows_count = i;
    gtk_clist_thaw (clist);
    gtk_widget_grab_focus(data_list);
    gm_bpanel_restore_view_session (connection, clist);

}







void gm_bpanel_init (GmConnection *connection)

{
    BpanelViewInfo * view_info = NULL;
    gchar * query;
    GtkWidget * data_list_sw;
    GtkWidget *bpanel_frame;
    GtkWidget * npage;
    GmNpageData * npage_data;

    GmConnectionAccessData * access_data;

    DbBrowserNodeInfo * info;


    access_data = connection->access_data;

    info = new_dbbrowsernodeinfo ();
    info->type = DATABASES_VIEW;



    info->name = g_strdup (access_data->host);

    g_hash_table_insert (connection->info_hash, info->name, (gpointer) info);


    query= "show databases";
    npage_data = gm_app_new_npage_data ();
    npage_data->type = GM_NPAGE_BROWSER;



    bpanel_frame = gm_utils_get_data (GTK_OBJECT (connection->connection_window),  "bpanel_frame");

    data_list_sw = bpanel_create_clist_scrolled_window ();
    gtk_container_add (GTK_CONTAINER (bpanel_frame), data_list_sw);
    gtk_widget_show (data_list_sw);

    npage_data->data_list_sw = data_list_sw;
    npage_data->parent = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"main_note_book");
    npage = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");

    gtk_object_set_data_full (GTK_OBJECT (npage), "npage_data", npage_data,
                              (GtkDestroyNotify) gm_app_free_npage_data);

    gtk_object_set_data_full (GTK_OBJECT (gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_hbox")), "npage_data", npage_data,
                              (GtkDestroyNotify) gm_app_free_npage_data);



    view_info = bpanel_new_view_info ();
    view_info->view_type = DATABASES_VIEW;
    bpanel_browse_by_query (connection, query, view_info);

    gm_db_update_mini_buffer_completion_list (connection);
}




static void bpanel_browse_by_query (GmConnection *connection, gchar * query, BpanelViewInfo * view_info)

{
    GtkWidget * data_list;
    GtkWidget * data_list_sw;
    GtkWidget * npage;
    GmNpageData * npage_data;
    GtkAdjustment *adjustment;
    
    DbQuery * q;

    g_return_if_fail (connection != NULL);
    g_return_if_fail (query != NULL);


    q = gm_db_new_query (query);
    if (view_info->database_in_use)
        q->database = view_info->database_in_use;


    npage = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");
    npage_data = (GmNpageData *) gm_utils_get_data (GTK_OBJECT(npage),"npage_data");


    if (!npage_data)
    {
        gm_debug_warning (GM_BPANEL_CHANEL, "npage_data Pointer is Null bpanel_browse_by_query");
        return;
    }
    data_list_sw = npage_data->data_list_sw;
    data_list = npage_data->data_list;


    if (gm_db_request(connection, q))
    {



        if (data_list)
            gtk_widget_destroy (data_list);

        connection->data_list = data_list = bpanel_create_clist (q->fields_count, view_info);
        gm_clist_set_hint_widget (GM_CLIST (data_list), connection->mini_buffer);

        bpanel_configure_bpanel_context_menu (connection);




        gtk_signal_connect (GTK_OBJECT (data_list), "row_activated",
                            GTK_SIGNAL_FUNC (bpanel_data_list_row_activated),
                            connection);



        gtk_signal_connect (GTK_OBJECT (data_list), "key_press_event",
                            GTK_SIGNAL_FUNC (bpanel_key_press_event),
                            connection);
        gtk_widget_show (data_list);
        npage_data->data_list = data_list;
        gtk_container_add (GTK_CONTAINER (data_list_sw), data_list);
        if (view_info->view_type == ROWS_VIEW)
        {
            view_info->fields_list = gm_bpanel_clone_fields_list (q->fields_list);
        }
        gtk_object_set_data (GTK_OBJECT (data_list), "connection",connection);

        bpanel_show_result (q, data_list);
        bpanel_show_status_information (connection, q, view_info);
    }else
    {
        gm_utils_write_log_message (connection, q->error, ERROR_LOGMODE);
    }

/* work around a bug in the scrolled window widget */



    

    adjustment = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (data_list_sw));
    gtk_adjustment_changed (adjustment);
    gtk_adjustment_value_changed (adjustment);
    gdm ("value %f ", adjustment->value);
    adjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (data_list_sw));
    gdm ("value %f ", adjustment->value);

    gm_db_free_query (q);
}


static void bpanel_show_status_information (GmConnection * connection, DbQuery * q, BpanelViewInfo * view_info)

{
    GmConnectionAccessData * access_data;
    GString * message_string;
    DbBrowserNodeInfo * info = NULL;
    GtkWidget * filter_sbutton;
    gboolean updating = FALSE;


    g_return_if_fail (connection != NULL || view_info != NULL);



    info = gm_bpanel_get_node_info (connection, view_info);


    access_data = connection->access_data;
    message_string = g_string_new(" ");
    g_string_sprintfa (message_string, "%s@",access_data->user);
    g_string_append (message_string, access_data->host);


    switch (view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            g_string_sprintfa (message_string, ":/  --%d Databases-- ",view_info->rows_count);
            break;
        }
    case TABLES_VIEW :
        {
            g_string_sprintfa (message_string, ":/%s --%d Table(s)--",view_info->database_in_use, view_info->rows_count);
            break;
        }
    case ROWS_VIEW :
        {
            g_string_sprintfa (message_string,":/%s/%s --%d Row(s)--" , view_info->database_in_use, view_info->table_in_use ,view_info->rows_count);
            break;
        }
    }
    filter_sbutton = $pointer (connection->connection_window, "filter_sbutton");
    updating = GPOINTER_TO_INT ($pointer (filter_sbutton, "updating"));
    gtk_object_set_data (GTK_OBJECT (filter_sbutton), "view_info", view_info);
    if (!updating) {
        if (GTK_WIDGET_VISIBLE (filter_sbutton))
            gtk_signal_disconnect_by_data (GTK_OBJECT (filter_sbutton), connection);
        gtk_widget_hide (filter_sbutton);
        gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON (filter_sbutton), FALSE);
        if (info->filter_query) {
            gtk_widget_show (filter_sbutton);
            gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON (filter_sbutton), info->filter_enabled);
            gtk_signal_connect (GTK_OBJECT (filter_sbutton), "toggled",
                                GTK_SIGNAL_FUNC (on_filter_sbutton_toggled),
                                connection);
        }
    }


    gm_bpanel_show_status_information (connection, message_string->str);

    g_string_free (message_string, TRUE);

}











void gm_bpanel_select_db_browser_page ( GmConnection * connection)
{

    GtkWidget * main_note_book;
    main_note_book = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"main_note_book");
    gtk_notebook_set_page (GTK_NOTEBOOK (main_note_book), 0);
}


static void bpanel_new_table_properties_window (GmClist * mlist, GmConnection * connection)
{

    BpanelViewInfo * view_info;
    DbTableInfo * table_info;
    GmNpageData * npage_data;
    gchar * t_label;
    GtkWidget *npage;
    GtkWidget * main_note_book;
    GtkWidget * fields_properties_window;

    g_return_if_fail (connection != NULL);


    view_info = gm_utils_get_data (GTK_OBJECT (mlist), "view_info");
    g_return_if_fail (view_info != NULL);

    if (view_info->view_type != TABLES_VIEW)
        return;
    table_info = db_new_table_info ();
    table_info->database = g_strdup (view_info->database_in_use);


    main_note_book = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"main_note_book");

    fields_properties_window =  gm_table_properties_new (connection, table_info);
    npage = gm_utils_get_data (GTK_OBJECT(fields_properties_window),"properties_box");


    gtk_object_set_data (GTK_OBJECT (npage), "fields_properties_window", fields_properties_window);
    npage_data = gm_app_new_npage_data ();
    npage_data->type = GM_NPAGE_TABLE_PROP;
    gtk_object_set_data_full (GTK_OBJECT (npage), "npage_data", npage_data,
                              (GtkDestroyNotify) gm_app_free_npage_data);


    gtk_widget_unparent (npage);

    gtk_object_set_data (GTK_OBJECT (npage), "main_note_book", main_note_book);
    gtk_widget_show (npage);
    gtk_notebook_append_page        (GTK_NOTEBOOK (main_note_book),
                                     npage, NULL);
    t_label = g_strdup_printf ("/%s/New Table", view_info->database_in_use);
    bpanel_append_notebook_page_with_icon (GTK_NOTEBOOK (main_note_book), npage,t_label, "new_table" );
    g_free (t_label);
    gtk_notebook_set_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_page_num (GTK_NOTEBOOK (main_note_book), npage));
}



static void bpanel_edit_selected_row (GmClist * mlist, GmConnection * connection)
{

    BpanelViewInfo * view_info;
    GList * list ;
    GtkCList * clist;
    gchar * table = NULL;
    gint sel_row =0;





    g_return_if_fail (connection != NULL);


    view_info = gm_utils_get_data (GTK_OBJECT (mlist), "view_info");
    g_return_if_fail (view_info != NULL);



    clist = GTK_CLIST (mlist);
    list = clist->selection;

    if (!list)
        return;

    if (g_list_length (list) == 1 && GPOINTER_TO_INT (list->data) == 0 && view_info->view_type != DATABASES_VIEW)
        return;

    sel_row = GPOINTER_TO_INT (list->data);

    if (view_info->view_type == TABLES_VIEW)
    {

        gtk_clist_get_text              (clist, sel_row, 0,&table);
        gm_interface_edit_table (connection, view_info->database_in_use, table);
    }


    if (view_info->view_type == ROWS_VIEW)
    {
        bpanel_data_list_row_activated (mlist, sel_row, connection);
    }

    if (view_info->view_type == DATABASES_VIEW)
    {
        gm_utils_message (_("sorry Mysql don't support database renaming yet !!"));

#if 0      
        gchar * database_name = NULL;
        gchar * old_database_name = NULL;
        gchar *query = NULL;
        gtk_clist_get_text              (clist, sel_row, 0,&old_database_name);
        database_name = bpanel_new_database_dialog (old_database_name);
        if (!database_name || strlen (database_name)<=0)
        {
            if (database_name)
                g_free (database_name);

            return ;
        }
        move_database (old_database_name, database_name);
#endif 

    }

}


























static void bpanel_new_edit_notebook_page (GmConnection * connection, BpanelViewInfo * view_info , GSList * old_data_list)
{

    GtkWidget *npage;
    GtkWidget * main_note_book;
    GmNpageData * npage_data;

    main_note_book = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"main_note_book");

    npage = gm_formular_new (connection, view_info, old_data_list);
    npage_data = gm_app_new_npage_data ();
    npage_data->type = GM_NPAGE_ROWEDIT;
    gtk_object_set_data_full (GTK_OBJECT (npage), "npage_data", npage_data,
                              (GtkDestroyNotify) gm_app_free_npage_data);


    gtk_object_set_data (GTK_OBJECT (npage), "main_note_book", main_note_book);
    gtk_widget_show (npage);
    gtk_notebook_append_page        (GTK_NOTEBOOK (main_note_book),
                                     npage, NULL);
    if (old_data_list)
        bpanel_append_notebook_page_with_icon (GTK_NOTEBOOK (main_note_book), npage," Editor","row-edit");
    else
        bpanel_append_notebook_page_with_icon (GTK_NOTEBOOK (main_note_book), npage," Editor","row-edit-new");

    gtk_notebook_set_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_page_num (GTK_NOTEBOOK (main_note_book), npage));
}


void
bpanel_append_notebook_page_with_icon (GtkNotebook *note_book, GtkWidget * note_book_page, gchar * titel, gchar * icon_name)

{
    GtkWidget * hbox;
    GtkWidget * label;
    GtkWidget * pixmap = NULL ;


    if (icon_name)
    {
        pixmap = gm_stock_pixmap_widget (GTK_WIDGET (note_book), icon_name);
        gtk_widget_show (pixmap);
    }

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox);
    if (pixmap)
        gtk_box_pack_start (GTK_BOX (hbox), pixmap, TRUE, TRUE, 0);

    if (titel)
    {
        label = gtk_label_new (titel);
        gtk_widget_show (label);
        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
    }
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 1);
    gtk_notebook_set_tab_label (note_book, note_book_page, hbox);
}





static void
bpanel_popup_up_activate                         (GtkMenuItem     *menuitem,
        gpointer         user_data)
{

    GmConnection * connection ;
    GtkWidget * data_list;
    GtkWidget * npage;
    GmNpageData * npage_data;

    connection = (GmConnection *) user_data,

                 npage = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");
    npage_data = (GmNpageData *) gm_utils_get_data (GTK_OBJECT(npage),"npage_data");


    data_list = npage_data->data_list;

    bpanel_data_list_row_activated (GM_CLIST (data_list), 0, connection);
}


void gm_bpanel_show_filter_dialog (void *ignore, GmConnection * connection)
{

    GtkWidget * data_list;
    GtkWidget * npage;
    BpanelViewInfo * view_info;
    GmNpageData * npage_data;

    npage      = gm_utils_get_data (GTK_OBJECT (connection->connection_window), "dbbrowser_page_vbox");
    npage_data = gm_utils_get_data (GTK_OBJECT (npage), "npage_data");

    data_list = npage_data->data_list;
    view_info = gm_utils_get_data (GTK_OBJECT (data_list), "view_info");
    gm_filter_show_dialog (connection, view_info);
}



void gm_bpanel_go_up (void *ignore, GmConnection * connection)
{

    GtkWidget * data_list;
    GtkWidget * npage;
    BpanelViewInfo * view_info;
    GmNpageData * npage_data;

    npage      = gm_utils_get_data (GTK_OBJECT (connection->connection_window), "dbbrowser_page_vbox");
    npage_data = gm_utils_get_data (GTK_OBJECT (npage), "npage_data");

    data_list = npage_data->data_list;
    view_info = gm_utils_get_data (GTK_OBJECT (data_list), "view_info");

    if (view_info && view_info->view_type != DATABASES_VIEW)
        bpanel_data_list_row_activated (GM_CLIST (data_list), 0, connection);
}




static void free_the_hash_data  (gpointer key,
                                 gpointer value,
                                 gpointer user_data)
{
    DbBrowserNodeInfo * info = value;

    if (info)
        free_dbbrowsernodeinfo (info);
}

void gm_bpanel_clean_up (GmConnection * connection )
{
    /* free the hash data */

    g_hash_table_foreach            (connection->info_hash,
                                     free_the_hash_data,NULL);
    g_hash_table_destroy (connection->info_hash);
}


static void on_filter_sbutton_toggled (GtkButton *filter_sbutton, GmConnection * connection)

{

    DbBrowserNodeInfo * info;
    BpanelViewInfo * view_info = NULL;



    view_info = gm_utils_get_data (GTK_OBJECT (filter_sbutton), "view_info");

    g_return_if_fail (view_info != NULL);
    g_return_if_fail (connection != NULL);

    info = gm_bpanel_get_node_info (connection, view_info);
    g_return_if_fail (info != NULL);
    info->filter_enabled = ! info->filter_enabled ;
    gtk_object_set_data (GTK_OBJECT (filter_sbutton), "updating", GINT_TO_POINTER(1));
    gm_bpanel_reload (NULL, connection);
    gtk_object_set_data (GTK_OBJECT (filter_sbutton), "updating", GINT_TO_POINTER(0));
}








void gm_bpanel_show_status_information (GmConnection *connection, gchar * message)
{
    GtkWidget * label;
    label = gm_utils_get_data (connection->connection_window, "database_path_label");
    gtk_label_set_text (GTK_LABEL (label), message);
}



void gm_bpanel_select_all (void *ignore, GmConnection *connection)
{

    gtk_clist_select_all (GTK_CLIST(connection->data_list));
}


void gm_bpanel_unselect_all (void *ignore, GmConnection *connection)
{

    gtk_clist_unselect_all (GTK_CLIST(connection->data_list));
}


void gm_bpanel_invert_selection (void *ignore, GmConnection *connection)
{
    GtkCList * clist;

    clist = GTK_CLIST(connection->data_list);
    gm_clist_invert_selection (clist);

}

void gm_bpanel_new_row (GtkButton *button, GmConnection * connection)
{

    GtkWidget * npage;
    GmNpageData * npage_data;
    GtkWidget * data_list;


    npage = gm_utils_get_data  (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");
    npage_data = (GmNpageData *) gm_utils_get_data  (GTK_OBJECT(npage),"npage_data");

    data_list = npage_data->data_list;

    if (data_list)
        gm_bpanel_new_data_list_row (GM_CLIST(data_list), connection);
}



static void bpanel_configure_bpanel_context_menu ( GmConnection * connection)
{

    GtkWidget * popup_menu= NULL;


    connection->dbrowser_popup_menu = create_dbrowser_popup_menu ();
    gnome_popup_menu_attach (connection->dbrowser_popup_menu, connection->data_list, connection);

    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "up");
    gtk_object_set_data (GTK_OBJECT (popup_menu), "data_list",connection->data_list);
    gtk_object_set_data (GTK_OBJECT (popup_menu), "connection",connection);
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (bpanel_popup_up_activate),
                        connection);


    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "reload_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_bpanel_reload),
                        connection);


    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "edit_row_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_bpanel_edit_selected_row),
                        connection);

    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "new_row_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_bpanel_new_row),
                        connection);

    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "delete_row_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_bpanel_delete_selected_row),
                        connection);


    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "select_all_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_bpanel_select_all),
                        connection);


    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "unselect_all_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_bpanel_unselect_all),
                        connection);



    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "invert_selection_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_bpanel_invert_selection),
                        connection);

    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "copy_row_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_ccci_copy),
                        connection);

    popup_menu = gm_utils_get_data (GTK_OBJECT (connection->dbrowser_popup_menu), "paste_row_popup_menu");
    gtk_signal_connect (GTK_OBJECT (popup_menu), "activate",
                        GTK_SIGNAL_FUNC (gm_ccci_paste),
                        connection);





}












