#include "gm.h"
#include "gm-bpanel.h"
#include "gm-ccci.h"
#include "glade_interface.h"

static void ccci_real_copy (GmConnection *connection) ;
static GmClipBoardData * ccci_get_clipboard_data (GmConnection * connection);
static void ccci_set_clipboard_data (GmConnection * connection, BpanelViewInfo * view_info, GmClipBoardData * cb_data) ;
static GList * ccci_collect_cb_data (BpanelViewInfo *view_info, GtkCList * clist) ;
static void ccci_free_clipboard_data (GmClipBoardData * cb_data) ;
static GmClipBoardData * ccci_new_clipboard_data (gint type, gchar * host, BpanelViewInfo * view_info);
static GtkWidget * ccci_get_data_list (GmConnection * connection) ;
static BpanelViewInfo * ccci_get_view_info (GmConnection * connection) ;


static void ccci_paste_rows (GmConnection * connection) ;
static void ccci_paste_tables (GmConnection *connection);
static void ccci_paste_databases (GmConnection *connection);
static GtkWidget *ccci_show_paste_dialog (gint * answer) ;
static gchar * ccci_prompt_for_table_name (gchar *old_name) ;
static void ccci_clip_window_destroyed (void *ignore, GmConnection * connection) ;








static void ccci_paste_databases (GmConnection *connection)
{
    GmClipBoardData * cb_data;
    GList * selection;

    cb_data = ccci_get_clipboard_data (connection);
    selection = cb_data->list;
    while (selection) {
        gdm ("NOT IMPLEMETED YET >%s< ", (gchar *) selection->data);
        selection = selection->next;
    }
}













static BpanelViewInfo * ccci_get_view_info (GmConnection * connection)
{
    GtkWidget * npage;
    GmNpageData * npage_data;

    g_return_val_if_fail (connection != NULL, NULL);

    npage      = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");
    npage_data = (GmNpageData *) gm_utils_get_data (GTK_OBJECT(npage),"npage_data");

    g_return_val_if_fail ( npage_data != NULL, NULL);
    return gm_utils_get_data (GTK_OBJECT (npage_data->data_list), "view_info");

}

static GtkWidget * ccci_get_data_list (GmConnection * connection)
{
    GtkWidget * npage;
    GmNpageData * npage_data;

    g_return_val_if_fail (connection != NULL, NULL);

    npage      = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"dbbrowser_page_vbox");
    npage_data = (GmNpageData *) gm_utils_get_data (GTK_OBJECT(npage),"npage_data");

    g_return_val_if_fail ( npage_data != NULL, NULL);

    return npage_data->data_list;
}




static GmClipBoardData * ccci_new_clipboard_data (gint type, gchar * host, BpanelViewInfo * view_info)
{
    GmClipBoardData * cb_data;
    cb_data = g_new0 (GmClipBoardData, 1);
    cb_data->type = type;
    cb_data->host = g_strdup (host);
    cb_data->database = g_strdup (view_info->database_in_use);
    cb_data->table    = g_strdup (view_info->table_in_use);
    cb_data->list     = NULL;
    cb_data->src_fields_list = NULL;
    return cb_data;
}




static void ccci_free_clipboard_data (GmClipBoardData * cb_data)
{
    if (!cb_data )
        return;

    g_free (cb_data->host);
    g_free (cb_data->database);
    g_free (cb_data->table);



    if (cb_data->list) {
        switch (cb_data->type)
        {
        case DATABASES_VIEW :
        case TABLES_VIEW :
            {
                g_list_g_free (cb_data->list);
                break;
            }
        case ROWS_VIEW :
            {
                GList * l;
                for (l=cb_data->list; l; l=l->next){
                    if (l->data)
                        g_list_g_free(l->data);
                    g_list_free (l->data);
                }
                g_list_free (cb_data->list);
                break;
            }

        }
    }
    if (cb_data->src_fields_list)
        gm_bpanel_free_fields_list (cb_data->src_fields_list);
    g_free (cb_data);
}



static GList * ccci_collect_cb_data (BpanelViewInfo *view_info, GtkCList * clist)
{
    GList * selection;
    GList * list = NULL;


    switch (view_info->view_type)
    {
    case DATABASES_VIEW :
    case TABLES_VIEW :
        {
            selection = clist->selection;
            while (selection) {
                gchar * text = NULL;
                gint   sel_row = GPOINTER_TO_INT (selection->data);
                gtk_clist_get_text              (clist, sel_row, 0,&text);
                if (text)
                    list = g_list_append (list, g_strdup (text));
                selection = selection->next;
            }
            break;
        }
    case ROWS_VIEW :
        {
            selection = clist->selection;
            while (selection) {
                GList * lrow = NULL;
                gint i = 0;
                gint   sel_row = GPOINTER_TO_INT (selection->data);
                for (i=0; i<clist->columns; i++){
                    gchar * text = NULL;
                    gtk_clist_get_text              (clist, sel_row, i,&text);
                    if (text)
                        lrow = g_list_append (lrow, g_strdup (text));
                    else {

                        lrow = g_list_append (lrow, NULL);

                    }

                }
                list = g_list_append (list, lrow);
                selection = selection->next;
            }
            break;
        }
    default:
        {
            g_assert("Should never happend gm-ccci.c \n");
        }
    }
    return list;
}




static void ccci_set_clipboard_data (GmConnection * connection, BpanelViewInfo * view_info, GmClipBoardData * cb_data)
{

    GmClipBoardData * old_cb_data = NULL;
    GmClipBoard * clipboard   = NULL;



    g_return_if_fail (view_info != NULL);
    g_return_if_fail (connection != NULL);
    g_return_if_fail (cb_data != NULL);

    clipboard = global_app->clipboard;

    g_return_if_fail (clipboard != NULL);


    switch (view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            old_cb_data = clipboard->databases;
            clipboard->databases = cb_data;
            break;
        }
    case TABLES_VIEW :
        {
            old_cb_data = clipboard->tables;
            clipboard->tables = cb_data;
            break;
        }
    case ROWS_VIEW :
        {
            old_cb_data = clipboard->rows;
            clipboard->rows = cb_data;
            break;
        }
    default:
        {
            g_assert("Should never happend gm-ccci.c \n");
        }
    }
    gm_ccci_update_clip_window (connection);
    ccci_free_clipboard_data (old_cb_data);
}



static GmClipBoardData * ccci_get_clipboard_data (GmConnection * connection)
{

    GmClipBoard * clipboard   = NULL;
    GmClipBoardData * cb_data = NULL;
    BpanelViewInfo * view_info = NULL;


    clipboard = global_app->clipboard;
    view_info = ccci_get_view_info (connection);

    g_return_val_if_fail (view_info != NULL, NULL);
    g_return_val_if_fail (clipboard != NULL, NULL);

    switch (view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            cb_data = clipboard->databases;
            break;
        }
    case TABLES_VIEW :
        {
            cb_data = clipboard->tables;
            break;
        }
    case ROWS_VIEW :
        {
            cb_data = clipboard->rows;
            break;
        }
    default:
        {
            g_assert("Should never happend gm-ccci.c \n");
        }
    }
    return cb_data;
}






static void ccci_real_copy (GmConnection *connection)
{

    BpanelViewInfo * view_info;
    GmClipBoard * clipboard=NULL;
    GmClipBoardData * cb_data=NULL;
    GmConnectionAccessData * access_data;
    GtkWidget * data_list;


    g_return_if_fail ( connection != NULL);
    g_return_if_fail ( global_app->clipboard != NULL);

    access_data = connection->access_data;

    clipboard = global_app->clipboard;

    data_list = ccci_get_data_list (connection);
    view_info = ccci_get_view_info (connection);
    cb_data = ccci_new_clipboard_data (view_info->view_type, access_data->host, view_info);
    cb_data->list  = ccci_collect_cb_data (view_info, GTK_CLIST (data_list));
    if (view_info->fields_list)
        cb_data->src_fields_list = gm_bpanel_clone_fields_list (view_info->fields_list);
    ccci_set_clipboard_data (connection, view_info, cb_data);
}













void gm_ccci_init  (void)
{
    global_app->clipboard = g_new0 (GmClipBoard, 1);
    global_app->clipboard->databases = NULL;
    global_app->clipboard->tables    = NULL;
    global_app->clipboard->rows      = NULL;
}




void gm_ccci_copy (void *ignore, GmConnection *connection)
{
    BpanelViewInfo * view_info;
    GtkWidget * data_list;
    GtkCList  * clist;
    GList * selection;

    g_return_if_fail ( connection != NULL);
    g_return_if_fail ( global_app->clipboard != NULL);


    data_list = ccci_get_data_list (connection);
    g_return_if_fail ( data_list != NULL);
    clist = GTK_CLIST (data_list);

    /* return if no selections are made so we could'nt copy any thing */
    selection = clist->selection;
    if (!selection)
        return;


    view_info = gm_utils_get_data (GTK_OBJECT (data_list), "view_info");
    g_return_if_fail ( view_info != NULL);

    /* return if the only selection is the up_row */
    if (g_list_length (selection) ==1 && GPOINTER_TO_INT (selection->data) == 0 && view_info->view_type != DATABASES_VIEW)
        return;

    ccci_real_copy (connection);
}




void gm_ccci_paste (void *ignore, GmConnection *connection)
{
    GmClipBoard * clipboard=NULL;
    GmClipBoardData * cb_data=NULL;
    BpanelViewInfo * view_info;

    g_return_if_fail ( connection != NULL);
    g_return_if_fail ( global_app->clipboard != NULL);
    clipboard = global_app->clipboard;
    cb_data = ccci_get_clipboard_data (connection);
    if (!cb_data) {
        gdm ("No data in the clipboard \n");
        return;
    }

    view_info = ccci_get_view_info (connection);
    switch (view_info->view_type)
    {

    case DATABASES_VIEW :
        {
            ccci_paste_databases (connection);
            break;
        }
    case TABLES_VIEW :
        {
            ccci_paste_tables (connection);
            break;
        }

    case ROWS_VIEW :
        {
            ccci_paste_rows (connection);
            break;
        }
    default:
        {
            g_assert("Should never happend gm-ccci.c \n");
        }
    }

}

static gboolean ccci_check_src_and_target (BpanelViewInfo *view_info, GmClipBoardData *cb_data)
{
    GSList * src_fields_list;
    GSList * dest_fields_list;

    src_fields_list  = cb_data->src_fields_list;
    dest_fields_list = view_info->fields_list;
    if (g_slist_length (src_fields_list) != g_slist_length (dest_fields_list)) {
        return FALSE;
    }

    /* should we test if the tables have exactly the same field-names ? */
    return TRUE;
}



static void ccci_paste_rows (GmConnection * connection)
{
    GList * selection;
    GmClipBoardData * cb_data;
    BpanelViewInfo * view_info;
    GString * query_string ;
    gchar * dest_table;
    gchar * dest_database;
    gboolean  stop_warning=FALSE;
    gboolean  reload = FALSE;


    view_info = ccci_get_view_info (connection);
    cb_data = ccci_get_clipboard_data (connection);

    if (!ccci_check_src_and_target (view_info, cb_data)){
        gm_utils_warning ("Source table and target don't have the same structure !");
        return;
    }

    dest_database = view_info->database_in_use ;
    dest_table    = view_info->table_in_use;
    selection = cb_data->list;
    while (selection) {
        GList * l = NULL;
        gboolean first_time = TRUE;
        DbQuery * q;
        query_string = g_string_new ("insert into ");
        g_string_append (query_string,dest_table);
        g_string_append (query_string," VALUES (");
        for (l = selection->data; l; l = l->next) {
            gchar * field_value = (gchar *) l->data;

            if (!first_time)
                g_string_append (query_string," , ");
            if (field_value) {
                gchar * escaped_value;
                escaped_value = gm_utils_strescape (field_value, NULL);
                g_string_sprintfa (query_string," '%s' ",escaped_value);
                g_free (escaped_value);
            }

            else
                g_string_append (query_string," NULL");
            first_time = FALSE;
        }
        g_string_append (query_string," )");
        q = gm_db_new_query (query_string->str);
        q->database = view_info->database_in_use;
        if (!gm_db_request (connection, q))
        {
            if (!stop_warning) {
                gint answer;
                gchar * error_message;
                error_message = g_strdup_printf ( "Mysql Error :%s\n",  q->error);
                gm_utils_write_log_message (connection, error_message,  ERROR_LOGMODE);
                answer = gm_utils_ignore_dialog(error_message);
                g_free (error_message);
                if (answer == IGNORE_DIALOG_CANCELD)
                    break;
                if (answer == IGNORE_DIALOG_IGNORE_ALL)
                    stop_warning = TRUE;
            }
        }else {
            reload = TRUE;
        }
        gm_db_free_query (q);
        g_string_free (query_string, TRUE);
        selection = selection->next;
    }
    if (reload)
        gm_bpanel_reload (NULL, connection);
}




static gint ccci_paste_dialog_key_pressed     (GtkWidget       *widget,
        GdkEventKey     *event,
        gint *answer){

    if  (event->keyval == GDK_Escape) {
        *answer = 0;
        gtk_main_quit ();
        return TRUE;
    }
    return FALSE;

}


static void ccci_paste_dialog_ok_button_clicked               (GtkButton  *button,
        gint *answer){
    GtkWidget * paste_dialog;
    gboolean  prefixed;
    GtkWidget * prefix_entry;
    gchar *prefix;


    paste_dialog   = $pointer (button ,"paste_dialog");
    prefixed       = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON ($pointer (paste_dialog, "prefix_rb")));
    if (prefixed) {
        prefix_entry = $pointer (paste_dialog, "prefix_entry");
        prefix = gtk_entry_get_text (GTK_ENTRY (prefix_entry) );
        if (!prefix || !strlen (prefix)) {
            gm_utils_warning ("please type in a prefix ");
            return;
        }
    }

    *answer = 1;
    gtk_main_quit ();
}


static void ccci_paste_dialog_cancel_button_clicked               (GtkButton  *button,
        gint *answer){
    *answer = 0;
    gtk_main_quit ();
}


static GtkWidget *ccci_show_paste_dialog (gint * answer)
{

    GtkWidget * paste_dialog;
    GtkWidget * button;


    paste_dialog = create_paste_dialog ();
    gtk_widget_show (paste_dialog);

    gtk_signal_connect (GTK_OBJECT (paste_dialog), "key_press_event",
                        GTK_SIGNAL_FUNC (ccci_paste_dialog_key_pressed),
                        answer);


    button = gm_utils_get_data (GTK_OBJECT(paste_dialog),"ok_button");
    gtk_object_set_data (GTK_OBJECT (button) ,"paste_dialog", paste_dialog);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (ccci_paste_dialog_ok_button_clicked),
                        answer);

    button = gm_utils_get_data (GTK_OBJECT(paste_dialog),"cancel_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (ccci_paste_dialog_cancel_button_clicked),
                        answer);

    gtk_signal_connect (GTK_OBJECT (paste_dialog), "destroy",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);


    gtk_main();
    return paste_dialog;
}


/* table name dialog */


static gint ccci_rename_table_dialog_key_pressed     (GtkWidget       *widget,
        GdkEventKey     *event,
        gint *answer){

    if  (event->keyval == GDK_Escape) {
        *answer = 0;
        gtk_main_quit ();
        return TRUE;
    }
    return FALSE;

}


static void ccci_rename_table_dialog_ok_button_clicked               (GtkButton  *button,
        GtkWidget *dialog){
    gchar * old_name;
    gchar * new_name;
    GtkWidget * table_name_entry;

    table_name_entry = $pointer (dialog, "table_name_entry");
    old_name         = $pointer (dialog, "old_name");

    new_name = gtk_entry_get_text (GTK_ENTRY (table_name_entry));
    if (!new_name || !strlen (new_name) || !strcmp (new_name, old_name)) {
        gm_utils_warning ("please type in a new table name to be used for the new table ");
        return;
    }
    gtk_main_quit ();
}


static void ccci_rename_table_dialog_cancel_button_clicked               (GtkButton  *button,
        gint *answer){
    *answer = 0;
    gtk_main_quit ();
}


static gchar * ccci_prompt_for_table_name (gchar *old_name)
{

    GtkWidget * rename_table_dialog;
    GtkWidget * button;
    GtkWidget * table_name_entry;
    gchar * new_name;
    gint answer = 1;


    rename_table_dialog = create_rename_table_dialog ();
    gtk_widget_show (rename_table_dialog);

    table_name_entry = $pointer (rename_table_dialog, "table_name_entry");

    gtk_object_set_data (GTK_OBJECT (rename_table_dialog), "old_name", old_name);

    gtk_entry_set_text (GTK_ENTRY (table_name_entry), old_name);

    gtk_signal_connect (GTK_OBJECT (rename_table_dialog), "key_press_event",
                        GTK_SIGNAL_FUNC (ccci_rename_table_dialog_key_pressed),
                        &answer);


    button = gm_utils_get_data (GTK_OBJECT(rename_table_dialog),"ok_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (ccci_rename_table_dialog_ok_button_clicked),
                        rename_table_dialog);

    button = gm_utils_get_data (GTK_OBJECT(rename_table_dialog),"cancel_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (ccci_rename_table_dialog_cancel_button_clicked),
                        &answer);

    gtk_signal_connect (GTK_OBJECT (rename_table_dialog), "destroy",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);
    gtk_main();

    if (!GTK_IS_WIDGET(rename_table_dialog))
        return NULL;

    new_name = gtk_entry_get_text (GTK_ENTRY (table_name_entry));
    if (new_name)
        new_name = g_strdup (new_name);

    gtk_signal_disconnect_by_func(GTK_OBJECT (rename_table_dialog), gtk_main_quit, NULL);
    gtk_widget_destroy (rename_table_dialog);

    if (!answer)
        return NULL;

    return new_name;
}





















static void ccci_paste_tables (GmConnection *connection)
{
    GmClipBoardData * cb_data;
    GList * selection;
    GtkWidget * paste_dialog;
    GtkWidget * prefix_entry;
    BpanelViewInfo * view_info;
    gchar * target_database;
    gchar * target_table = NULL;
    gchar * src_table;
    gchar * src_database;
    gboolean  reload = FALSE;
    gboolean structure_only;
    gboolean prefixed ;
    gchar * prefix = NULL ;
    gboolean stop_warning = FALSE;


    gint weiter = 0;

    paste_dialog = ccci_show_paste_dialog (&weiter);

    if (!GTK_IS_WIDGET(paste_dialog))
        return;

    structure_only = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON ($pointer (paste_dialog, "structure_only_rb")));
    prefixed       = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON ($pointer (paste_dialog, "prefix_rb")));
    if (prefixed) {
        prefix_entry = $pointer (paste_dialog, "prefix_entry");
        prefix = gtk_entry_get_text (GTK_ENTRY (prefix_entry) );
        if (!prefix || !strlen (prefix)) {
            gm_utils_warning ("please type in a prefix ");
            return;
        }
    }


    gtk_signal_disconnect_by_func(GTK_OBJECT (paste_dialog), gtk_main_quit, NULL);
    gtk_widget_destroy (paste_dialog);

    if (!weiter)
        return;




    view_info = ccci_get_view_info (connection);
    target_database = view_info->database_in_use ;

    cb_data      = ccci_get_clipboard_data (connection);
    selection    = cb_data->list;


    src_database = cb_data->database;

    while (selection) {
        src_table = (gchar * ) selection->data;
        if (gm_db_table_exists (connection, target_database, src_table)) {
            if (prefixed)
                target_table = g_strdup_printf ("%s%s", src_table, prefix);
            else {
                target_table = ccci_prompt_for_table_name (src_table);
                if (!target_table)
                    return;
                while (gm_db_table_exists (connection, target_database, target_table)) {
                    g_free (target_table);
                    target_table = ccci_prompt_for_table_name (src_table);
                    if (!target_table)
                        return;
                }
            }
        }else {
            target_table = g_strdup (src_table);
        }

        if (!gm_db_clone_table (connection,src_database,src_table,target_database,target_table, structure_only)){
            if (!stop_warning) {
                gint answer;
                answer = gm_utils_ignore_dialog("An error occur how should we proceed ");
                if (answer == IGNORE_DIALOG_CANCELD)
                    break;
                if (answer == IGNORE_DIALOG_IGNORE_ALL)
                    stop_warning = TRUE;
            }
        }else
            reload = TRUE;
        g_free (target_table);
        selection = selection->next;
    }
    if (reload)
        gm_bpanel_reload (NULL, connection);
}


/* Clip window */


void        gm_ccci_update_clip_window (GmConnection * connection)
{

    GtkWidget * clip_window ;
    GtkCList * db_clist;
    GtkCList * table_clist;
    GtkCList * row_clist = NULL;
    GtkWidget * row_sw;
    GList * selection;
    gchar *db_append_row[2];
    gchar *table_append_row[3];
    gchar **row_array;


    GmClipBoard * clipboard   = NULL;
    GmClipBoardData * cb_data = NULL;

    clipboard   = global_app->clipboard;
    clip_window = global_app->clip_window;
    g_return_if_fail (clipboard != NULL);

    if (!clip_window || !GTK_IS_WIDGET (clip_window))
        return;

    db_clist    = GTK_CLIST ($pointer (clip_window, "db_clist"));
    table_clist = GTK_CLIST ($pointer (clip_window, "table_clist"));
    if ($pointer (clip_window, "row_clist"))
        row_clist = GTK_CLIST ($pointer (clip_window, "row_clist"));
    row_sw     = $pointer (clip_window, "row_sw");

    /* databases */

    gtk_clist_clear (db_clist);
    cb_data = clipboard->databases;
    if (cb_data ) {
        db_append_row[0] = cb_data->host;
        selection = cb_data->list;
        while (selection) {
            db_append_row[1] = (gchar *) selection->data;
            selection = selection->next;
            gtk_clist_append (db_clist, db_append_row);
        }
    }

    gtk_clist_clear (table_clist);
    cb_data = clipboard->tables;
    if (cb_data ) {
        table_append_row[0] = cb_data->host;
        table_append_row[1] = cb_data->database;
        selection = cb_data->list;
        while (selection) {
            table_append_row[2] = (gchar *) selection->data;
            selection = selection->next;
            gtk_clist_append (table_clist, table_append_row);
        }
    }

    /* data-rows */

    cb_data = clipboard->rows;
    if (row_clist) {
        gtk_widget_destroy (GTK_WIDGET(row_clist));
        gtk_object_remove_data (GTK_OBJECT(clip_window), "row_clist");
    }
    if (cb_data ) {
        GSList * src_fields_list;
        GSList * f;
        GtkWidget * widget, * label;
        gint i ;

        gint columns;
        selection = cb_data->list;
        src_fields_list = cb_data->src_fields_list;
        if (selection && src_fields_list) {
            columns = g_slist_length (src_fields_list);
            widget = gtk_clist_new (columns);
            row_clist = GTK_CLIST (widget);
            gtk_widget_ref (widget);
            gtk_object_set_data_full (GTK_OBJECT (clip_window), "row_clist", widget,
                                      (GtkDestroyNotify) gtk_widget_unref);
            gtk_widget_show (widget);
            gtk_container_add (GTK_CONTAINER (row_sw), widget);
            gtk_clist_column_titles_show (row_clist);
            i = 0;
            for (f = src_fields_list; f; f = f->next) {
                MYSQL_FIELD *field = (MYSQL_FIELD *) f->data;
                gtk_clist_set_column_auto_resize (row_clist, i, TRUE);
                label = gtk_label_new (field->name);
                gtk_widget_show (label);
                gtk_clist_set_column_widget (row_clist, i, label);
                i++;
            }

            row_array = g_new (gchar*, columns );
            row_array[columns] = NULL;
            for (i=0;i<columns; i++)
                row_array[i] = NULL;

            selection = g_list_first (selection);
            while (selection) {
                GList * l;
                i=0;

                for (l = selection->data; l; l = l->next) {
                    gchar * field_value ;
                    field_value = (gchar *) l->data;


                    row_array[i] = field_value;
                    i++;
                }
                //                gtk_clist_append (row_clist, row_array);
                selection = selection->next;
            }
            /* FIXME: this cause segfault */
            /* we should free row_array */
        }
    }

    return;
}

static void ccci_clip_window_destroyed (void *ignore, GmConnection * connection)
{

    GtkWidget * button;

    global_app->clip_window = NULL;
    button = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"clip_win_on_off");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), FALSE);

    //    gtk_signal_handler_block_by_data(GTK_OBJECT(button), connection);
    /* we don't need to block this signal */

}



GtkWidget * gm_ccci_create_clip_window (GmConnection *connection)
{


    GtkWidget * clip_window ;
    GtkCList * clist;

    clip_window = global_app->clip_window;

    if (clip_window)
        return clip_window;

    global_app->clip_window = clip_window = create_clip_window ();

    gtk_signal_connect (GTK_OBJECT (clip_window), "destroy",
                        GTK_SIGNAL_FUNC (ccci_clip_window_destroyed),
                        connection);

    clist    = GTK_CLIST ($pointer (clip_window, "db_clist"));
    gtk_clist_set_column_auto_resize (clist, 0, TRUE);
    gtk_clist_set_column_auto_resize (clist, 1, TRUE);

    clist    = GTK_CLIST ($pointer (clip_window, "table_clist"));
    gtk_clist_set_column_auto_resize (clist, 0, TRUE);
    gtk_clist_set_column_auto_resize (clist, 1, TRUE);
    gtk_clist_set_column_auto_resize (clist, 2, TRUE);


    gm_ccci_update_clip_window (connection);
    return clip_window;

}













