#include "gm.h"
#include "gm-interface.h"
#include "gm-connection.h"
#include "glade_interface.h"
#include "gm-bpanel.h"
#include "gm-qbuffer.h"
static void       connection_class_init       (GmConnectionClass *klass);
static void       connection_init             (GmConnection *connection);
static void       connection_destroy          (GtkObject *obj);
static void       connection_finalize         (GtkObject *obj);
static void          connection_build_interface (GmConnection * connection);



static GtkObjectClass *parent_class = NULL;

guint gm_connection_get_type ()
{
    static guint connection_type = 0;

    if (!connection_type) {
        GtkTypeInfo connection_info = {
            "GmConnection",
            sizeof (GmConnection),
            sizeof (GmConnectionClass),
            (GtkClassInitFunc) connection_class_init,
            (GtkObjectInitFunc) connection_init,
            (GtkArgSetFunc) NULL,
            (GtkArgGetFunc) NULL,
        };

        connection_type = gtk_type_unique (gtk_object_get_type (), &connection_info);
    }

    return connection_type;
}

static void connection_class_init (GmConnectionClass *klass)
{
    GtkObjectClass *object_class;

    object_class = (GtkObjectClass*)klass;

    object_class->destroy  = connection_destroy;
    object_class->finalize = connection_finalize;

    parent_class = gtk_type_class (gtk_object_get_type ());
}

static void connection_init (GmConnection *connection)
{

    connection->dbrowser_popup_menu = NULL;
    connection->name = NULL;
    connection->key_mode = APP_DEFAULT_MODE;
    connection->info_hash = g_hash_table_new (g_str_hash, g_str_equal);
    connection->data_list = NULL;
    connection->remove_messages_timer_tag = 0;
    connection->progressbar = NULL;
}




static void connection_finalize (GtkObject *obj)
{
    GmConnection *connection;




    connection = GM_CONNECTION(obj);

    if(connection->name)
        g_free(connection->name);

    if(GTK_OBJECT_CLASS(parent_class)->finalize)
        (* GTK_OBJECT_CLASS(parent_class)->finalize)(obj);
}

static void connection_destroy (GtkObject *obj)
{
    GmConnection *connection;

    connection = GM_CONNECTION(obj);
    gtk_widget_destroy (connection->connection_window);
    if(GTK_OBJECT_CLASS(parent_class)->destroy)
        (* GTK_OBJECT_CLASS(parent_class)->destroy)(obj);
}


GmConnection * gm_connection_new (void){

    GmConnection *connection;
    GmConnectionAccessData *access_data;


    access_data = g_new0(GmConnectionAccessData, 1);
    access_data->host = NULL;
    access_data->user = NULL;
    access_data->database = NULL;
    access_data->password = NULL;
    access_data->port = -1;
    if (gm_dialogs_connection (access_data)) {
        connection = gtk_type_new (gm_connection_get_type ());
        connection->access_data = access_data;
        connection_build_interface(connection);
        return connection;
    }else
        return NULL;
}





static void connection_build_interface (GmConnection * connection)
{

    gm_interface_create_connection_window (connection);
    gm_bpanel_init (connection);
}





