/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <sys/stat.h> /* mkdir */
#include "gm-utils.h"
#include "gm-dbapi.h"
#include "glade_interface.h"
#include "gm-control-center.h"
#include "gm-debug.h"
#include "gm-print.h"

extern GmApp * global_app;



static void       cc_class_init (GmCcenterClass *klass);
static void       cc_init       (GmCcenter      *cc);








GtkType
gm_cc_get_type (void)

{
    static GtkType cc_type = 0;
    if (!cc_type)
    {
        static const GtkTypeInfo cc_info =
        {
            "GmCcenter",
            sizeof (GmCcenter),
            sizeof (GmCcenterClass),
            (GtkClassInitFunc) cc_class_init,
            (GtkObjectInitFunc) cc_init,
                /* reserved_1 */ NULL,
                /* reserved_2 */ NULL,
            (GtkClassInitFunc) NULL,
        };
        cc_type = gtk_type_unique (GTK_TYPE_OBJECT, &cc_info);
    }

    return cc_type;
}

static void
cc_class_init (GmCcenterClass *class)
{
    GtkObjectClass *object_class;
    object_class = (GtkObjectClass*) class;
}


static void
cc_create_config_dir (GmCcenter *cc)
{

    gchar * home_dir;
    gchar * dir;

    home_dir = g_get_home_dir ();
    dir = g_strdup_printf ("%s/%s",home_dir,GM_CONFIG_DIR_NAME);

    if (!is_directory (dir)){
        if (mkdir (dir, 0755) == -1){
            gm_debug_warning ( -1, "Warning *** Counld not create the config directory %s \n", dir);
        }
    }
    g_free (dir);
}


static void
cc_init (GmCcenter *cc)
{
    static gboolean first_time = TRUE;

    cc->options_dialog = NULL;
    cc->ge_config = NULL;
    cc->pr_config = NULL;
    cc->sql_editor_config = NULL;
    first_time = FALSE;
}



GmCcenter*
gm_cc_new (GmApp * app)
{

    GmCcenter * cc;
    cc = gtk_type_new (gm_cc_get_type ());
    cc_create_config_dir (cc);
    gtk_object_set_data (GTK_OBJECT (cc), "app", app);
    return cc;
}

static GmCcenterGeneral *
new_ge_config (void)
{
    GmCcenterGeneral * ge_config;


    ge_config = g_new (GmCcenterGeneral, 1);

    ge_config->logfilename = NULL;
    ge_config->show_fields = FALSE;
    ge_config->logfilehandler = NULL;
    ge_config->text_editor = NULL;
    ge_config->browser = NULL;
    ge_config->default_databases_query = NULL;
    ge_config->default_tables_query = NULL;
    ge_config->default_rows_query = NULL;
    ge_config->dump_file_browser_path = NULL;
    ge_config->view_state = 0;
    ge_config->horizontal_paned_position = 0;
    ge_config->vertical_paned_position = 0;
    ge_config->host_list = NULL;
    ge_config->last_used_exporter = NULL;

    return ge_config;
}


static GmCcenterSqlEditor *
new_sql_editor_config (void)
{
    GmCcenterSqlEditor *sql_editor_config;


    sql_editor_config = g_new (GmCcenterSqlEditor, 1);
    sql_editor_config->font_name = NULL;
    sql_editor_config->comments_color = NULL;
    sql_editor_config->keywords_color = NULL;
    sql_editor_config->functions_color = NULL;
    sql_editor_config->errors_color = NULL;
    sql_editor_config->default_color = NULL;
    sql_editor_config->strings_color = NULL;
    sql_editor_config->bgcolor = NULL;

    return sql_editor_config;

}


static GmCcenterPrinting *
new_pr_config (void)
{
    GmCcenterPrinting * pr_config;


    pr_config = g_new (GmCcenterPrinting, 1);
    pr_config->font_size = 10;
    pr_config->font_name = NULL;
    pr_config->paper_size = NULL;
    pr_config->unit_name = NULL;
    

    return pr_config;
}


static void
cc_read_sql_editor_config (GmCcenterSqlEditor * sql_editor_config)
{
    sql_editor_config->comments_color = gnome_config_get_string ("/gmyclient/SqlEditor/comments_color=#7E7E7B");
    sql_editor_config->keywords_color = gnome_config_get_string ("/gmyclient/SqlEditor/keywords_color=#0000FF");
    sql_editor_config->functions_color = gnome_config_get_string ("/gmyclient/SqlEditor/functions_color=#960000");
    sql_editor_config->errors_color = gnome_config_get_string ("/gmyclient/SqlEditor/errors_color=#FF0000");
    sql_editor_config->default_color = gnome_config_get_string ("/gmyclient/SqlEditor/default_color=#000000");
    sql_editor_config->strings_color = gnome_config_get_string ("/gmyclient/SqlEditor/strings_color=#0F9B00");
    sql_editor_config->font_name     = gnome_config_get_string ("/gmyclient/SqlEditor/FontName");
    sql_editor_config->bgcolor     = gnome_config_get_string ("/gmyclient/SqlEditor/BgColor=#FFFFFF");
}


static void
cc_read_printer_config (GmCcenterPrinting * pr_config)
{
    pr_config->paper_size = gnome_config_get_string ("/gmyclient/Printer/PaperSize");
    if (!pr_config->paper_size)
        pr_config->paper_size = g_strdup (gnome_paper_name_default ());
    pr_config->unit_name = gnome_config_get_string ("/gmyclient/Printer/UnitName");
    pr_config->print_grid = gnome_config_get_int ("/gmyclient/Printer/PrintGrid=1");
    pr_config->print_orientation = gnome_config_get_int ("/gmyclient/Printer/PrintOrientation=0");
}


static void
cc_write_printer_config (GmCcenterPrinting * pr_config)
{
    if (pr_config->paper_size)
        gnome_config_set_string ("/gmyclient/Printer/PaperSize",pr_config->paper_size);

    if (pr_config->unit_name)
        gnome_config_set_string ("/gmyclient/Printer/UnitName",pr_config->unit_name);


    gnome_config_set_int ("/gmyclient/Printer/PrintGrid",pr_config->print_grid);
    gnome_config_set_int ("/gmyclient/Printer/PrintOrientation",pr_config->print_orientation);
}



static void
cc_write_sql_editor_config (GmCcenterSqlEditor *sql_editor_config)
{
    if (sql_editor_config->comments_color)
        gnome_config_set_string ("/gmyclient/SqlEditor/comments_color",sql_editor_config->comments_color );
    else
        gnome_config_clean_key ("/gmyclient/SqlEditor/comments_color");

    if (sql_editor_config->keywords_color)
        gnome_config_set_string ("/gmyclient/SqlEditor/keywords_color",sql_editor_config->keywords_color );
    else
        gnome_config_clean_key ("/gmyclient/SqlEditor/keywords_color");

    if (sql_editor_config->functions_color )
        gnome_config_set_string ("/gmyclient/SqlEditor/functions_color",sql_editor_config->functions_color );
    else
        gnome_config_clean_key ("/gmyclient/SqlEditor/functions_color");

    if (sql_editor_config->errors_color )
        gnome_config_set_string ("/gmyclient/SqlEditor/errors_color",sql_editor_config->errors_color );
    else
        gnome_config_clean_key ("/gmyclient/SqlEditor/errors_color");

    if (sql_editor_config->default_color )
        gnome_config_set_string ("/gmyclient/SqlEditor/default_color",sql_editor_config->default_color );
    else
        gnome_config_clean_key ("/gmyclient/SqlEditor/default_color");

    if (sql_editor_config->strings_color )
        gnome_config_set_string ("/gmyclient/SqlEditor/strings_color",sql_editor_config->strings_color );
    else
        gnome_config_clean_key ("/gmyclient/SqlEditor/strings_color");

    gdm ("------- %s ", sql_editor_config->font_name);
    if (sql_editor_config->font_name )
        gnome_config_set_string ("/gmyclient/SqlEditor/FontName",sql_editor_config->font_name);
    else
        gnome_config_clean_key ("/gmyclient/SqlEditor/FontName");


    if (sql_editor_config->bgcolor )
        gnome_config_set_string ("/gmyclient/SqlEditor/BgColor",sql_editor_config->bgcolor);
    else
        gnome_config_clean_key ("/gmyclient/SqlEditor/BgColor");

}



static void
cc_read_ge_config (GmCcenterGeneral *ge_config)
{
    ge_config->date_time_on       = gnome_config_get_bool ("/gmyclient/General/DateTimeOn=TRUE");
    ge_config->show_hints       = gnome_config_get_bool ("/gmyclient/General/ShowHints=TRUE");
    ge_config->show_fields       = gnome_config_get_bool ("/gmyclient/General/ShowFields=FALSE");
    ge_config->view_state       = gnome_config_get_int ("/gmyclient/General/ViewState=14"); /* all flags on */
    ge_config->vertical_paned_position       = gnome_config_get_int ("/gmyclient/General/VerticalPanedPosition=200");
    ge_config->horizontal_paned_position       = gnome_config_get_int ("/gmyclient/General/HorizontalPanedPosition=250");
    ge_config->logfilename               = gnome_config_get_string ("/gmyclient/General/LogFileName");
    ge_config->dump_file_browser_path    = gnome_config_get_string ("/gmyclient/General/DumpFileBrowserPath");
    ge_config->text_editor               = gnome_config_get_string ("/gmyclient/General/TextEditor");
    ge_config->browser                   = gnome_config_get_string ("/gmyclient/General/Browser");
    ge_config->last_used_exporter        = gnome_config_get_string ("/gmyclient/General/LastUsedExporter");
    ge_config->default_rows_query        = gnome_config_get_string    ("/gmyclient/General/DefaultRowsQuery=SELECT * FROM &table_in_use LIMIT 50000");
    ge_config->default_tables_query      = gnome_config_get_string ("/gmyclient/General/DefaultTablesQuery=SHOW TABLES");
    ge_config->default_databases_query   = gnome_config_get_string ("/gmyclient/General/DefaultDatabasesQuery=SHOW DATABASES LIKE '%'");
}


void
gm_cc_read_configuration (GmCcenter * cc)
{


    GmCcenterGeneral  *ge_config;
    GmCcenterPrinting *pr_config;
    GmCcenterSqlEditor *sql_editor_config;


    gint i = 0;


    if (!cc)
        cc = (GM_CCENTER (global_app->cc));

        /* General */


    if (!cc->ge_config)
        ge_config = cc->ge_config = new_ge_config ();
    else
        ge_config = cc->ge_config;
    cc_read_ge_config (ge_config);

    if (!cc->pr_config)
        pr_config = cc->pr_config = new_pr_config ();
    else
        pr_config = cc->pr_config;
    cc_read_printer_config (pr_config);

    if (!cc->sql_editor_config)
        sql_editor_config = cc->sql_editor_config = new_sql_editor_config ();
    else
        sql_editor_config = cc->sql_editor_config;
    cc_read_sql_editor_config (sql_editor_config);



        /* connection Host-List */
    i = 0;
    while (1) {
        gchar * host = NULL;
        gint port = 0;
        GmHostPort * host_port;
        gint j = 0;

        gchar * host_key = g_strdup_printf ("/gmyclient/Hosts/Host_%d", i);
        gchar * port_key = g_strdup_printf ("/gmyclient/Hosts/Port_%d", i);
        gchar * last_user_key ;
        gchar * host_section ;

        host = gnome_config_get_string (host_key);
        port = gnome_config_get_int (port_key);
        if (!host) {
            g_free (host_key);
            g_free (port_key);
            break;
        }
        host_port = g_new (GmHostPort, 1);
        host_port->host = host;
        host_port->port = port;
        host_port->user_list = NULL;

        ge_config->host_list = g_list_append (ge_config->host_list, host_port);
        g_free (host_key);
        g_free (port_key);


        host_section  = g_strdup_printf ("/gmyclient/%s_%d", host, port);
        last_user_key = g_strdup_printf ("%s/LastUser", host_section);

        host_port->last_user =  gnome_config_get_string (last_user_key);
        g_free (last_user_key);
            /* loop to collect user and password data */
        while (1) {
            gchar * user_key = g_strdup_printf ("%s/User_%d",host_section, j);
            gchar * pass_key = g_strdup_printf ("%s/Pass_%d",host_section, j);
            gchar * user, *pass;
            GmUserPass * user_pass;

            user = gnome_config_get_string (user_key);
            pass = gnome_config_get_string (pass_key);

            if (!user){
                g_free (user_key);
                g_free (pass_key);
                break;
            }
            user_pass = g_new (GmUserPass, 1);
            user_pass->user = user;
            user_pass->pass = pass;
            host_port->user_list = g_list_append (host_port->user_list, user_pass);
            j++;
        }
        g_free (host_section);
        i++;
    }
}



static void
cc_write_configuration (GmCcenter * cc)
{


    GmCcenterGeneral   *ge_config;
    GmCcenterSqlEditor *sql_editor_config;
    GmCcenterPrinting *pr_config;

    GList * list;
    gint i = 0;

    g_return_if_fail (cc != NULL);

    ge_config = cc->ge_config;
    g_return_if_fail (ge_config != NULL);

    sql_editor_config = cc->sql_editor_config;
    pr_config = cc->pr_config;

    gnome_config_clean_section ("/gmyclient/Hosts");
    gnome_config_set_int    ("/gmyclient/General/ViewState",ge_config->view_state);
    gnome_config_set_int ("/gmyclient/General/HorizontalPanedPosition", ge_config->horizontal_paned_position);
    gnome_config_set_int ("/gmyclient/General/VerticalPanedPosition",ge_config->vertical_paned_position);

    gnome_config_set_bool ("/gmyclient/General/DateTimeOn", ge_config->date_time_on);
    gnome_config_set_bool ("/gmyclient/General/ShowHints", ge_config->show_hints);
    gnome_config_set_bool ("/gmyclient/General/ShowFields", ge_config->show_fields);

    if (ge_config->logfilename)
        gnome_config_set_string ("/gmyclient/General/LogFileName",ge_config->logfilename);
    else
        gnome_config_clean_key ("/gmyclient/General/LogFileName");

    if (ge_config->text_editor)
        gnome_config_set_string ("/gmyclient/General/TextEditor",ge_config->text_editor);
    else
        gnome_config_clean_key ("/gmyclient/General/TextEditor");

    if (ge_config->browser)
        gnome_config_set_string ("/gmyclient/General/Browser",ge_config->browser);
    else
        gnome_config_clean_key ("/gmyclient/General/Browser");

    if (ge_config->last_used_exporter)
        gnome_config_set_string ("/gmyclient/General/LastUsedExporter",ge_config->last_used_exporter);
    else
        gnome_config_clean_key ("/gmyclient/General/LastUsedExporter");

    if (ge_config->dump_file_browser_path)
        gnome_config_set_string ("/gmyclient/General/DumpFileBrowserPath",ge_config->dump_file_browser_path);
    else
        gnome_config_clean_key ("/gmyclient/General/DumpFileBrowserPath");



    if (ge_config->default_rows_query)
        gnome_config_set_string ("/gmyclient/General/DefaultRowsQuery",ge_config->default_rows_query);
    else
        gnome_config_clean_key ("/gmyclient/General/DefaultRowsQuery");


    if (ge_config->default_tables_query)
        gnome_config_set_string ("/gmyclient/General/DefaultTablesQuery",ge_config->default_tables_query);
    else
        gnome_config_clean_key ("/gmyclient/General/DefaultTablesQuery");

    if (ge_config->default_databases_query)
        gnome_config_set_string ("/gmyclient/General/DefaultDatabasesQuery",ge_config->default_databases_query);
    else
        gnome_config_clean_key ("/gmyclient/General/DefaultDatabasesQuery");

    cc_write_sql_editor_config (sql_editor_config);
    cc_write_printer_config (pr_config);


    list = ge_config->host_list;

    while (list) {
        gchar * host_section;
        GmHostPort * host_port;
        GList * user_list;
        gchar * host_key = g_strdup_printf ("/gmyclient/Hosts/Host_%d", i);
        gchar * port_key = g_strdup_printf ("/gmyclient/Hosts/Port_%d", i);
        gchar * temp ;
        gint j = 0;
        host_port = list->data;

        gnome_config_set_string (host_key, host_port->host);
        gnome_config_set_int (port_key, host_port->port);

        g_free (host_key);
        g_free (port_key);
        host_section = g_strdup_printf ("/gmyclient/%s_%d", host_port->host, host_port->port);
        gnome_config_clean_section (host_section);

        if (host_port->last_user) {
            temp = g_strdup_printf ("%s/LastUser", host_section);
            gnome_config_set_string (temp, host_port->last_user);
            g_free (temp);
        }

        for (user_list = host_port->user_list;user_list; user_list=user_list->next) {

            gchar * user_key = g_strdup_printf ("%s/User_%d",host_section, j);
            gchar * pass_key = g_strdup_printf ("%s/Pass_%d",host_section, j);
            GmUserPass * user_pass = user_list->data;


            gnome_config_set_string (user_key, user_pass->user);
            if (user_pass->pass) {
                gnome_config_set_string (pass_key, user_pass->pass);
            }
            j++;
        }
        i++;
        list = list->next;
        g_free (host_section);
    }
    gnome_config_sync ();
}

void
gm_cc_update_configuration (void)
{
    cc_write_configuration (GM_CCENTER (global_app->cc));
}




static void
cc_sql_editor_font_changed (GnomeFontPicker *fontpicker,
                            gchar *font_name,
                            GmSyntaxHighlighting *sh)
{

    if (sh->gdk_font)
        gdk_font_unref (sh->gdk_font);
    sh->gdk_font = gdk_font_load (font_name);
    gm_utils_text_colorize (sh->editable, 0, -1);
}


static void
cc_fill_printer_config_page (GmCcenter *cc)
{
    GtkWidget * options_dialog;
    GmCcenterPrinting * pr_config;
    GnomePaperSelector *paperselector;
    GtkToggleButton *tb;


    options_dialog = cc->options_dialog;
    pr_config = cc->pr_config;
    paperselector = GNOME_PAPER_SELECTOR (gm_utils_get_data (options_dialog, "paperselector"));

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (options_dialog, "print_orientation_portrait_radio_button"));

    if (pr_config->print_orientation == PRINT_ORIENT_PORTRAIT)
        gtk_toggle_button_set_active (tb, TRUE);
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (options_dialog, "print_orientation_landscape_radio_button"));
    if (pr_config->print_orientation == PRINT_ORIENT_LANDSCAPE)
        gtk_toggle_button_set_active (tb, TRUE);

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (options_dialog, "print_grid"));
    gtk_toggle_button_set_active (tb, pr_config->print_grid);

    if (pr_config->paper_size)
        gnome_paper_selector_set_name (paperselector, pr_config->paper_size);
    if (pr_config->unit_name)
        gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(paperselector->unit)->entry), pr_config->unit_name);
}




static void
cc_fill_sql_editor_config_page (GmCcenter *cc)
{

    GtkEntry  * entry;
    GtkWidget * options_dialog;
    GmCcenterSqlEditor * sql_editor_config;
    GtkWidget *button;
    GtkWidget * text_widget;
    GmSyntaxHighlighting * sh;
    GnomeFontPicker * sql_editor_font;








    sql_editor_config  = cc->sql_editor_config;
    options_dialog = cc->options_dialog;

    text_widget = gm_utils_get_data (options_dialog, "highlighting_preview");
    gm_utils_text_configure_sql_syntax_highlighting (GTK_EDITABLE (text_widget)) ;

    sh = gm_utils_get_data (text_widget, "syntax_highlighting");


    button = gm_utils_get_data (options_dialog, "sql_editor_bgcolor_button");
    gtk_object_set_data (GTK_OBJECT (button), "highlighting_preview", text_widget);
    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "sql_editor_bgcolor"));
    gtk_object_set_data (GTK_OBJECT (entry), "bgcolor",GINT_TO_POINTER (1));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_utils_choose_color_button_clicked),
                        entry);
    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (gm_utils_color_entry_changed),
                        button);
    gtk_entry_set_text (entry, sql_editor_config->bgcolor);


    button = gm_utils_get_data (options_dialog, "comments_color_button");
    gtk_object_set_data (GTK_OBJECT (button), "highlighting_preview", text_widget);
    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "comments_color"));
    gtk_object_set_data (GTK_OBJECT (entry), "gdk_color", g_hash_table_lookup (sh->colors_table, "comments"));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_utils_choose_color_button_clicked),
                        entry);
    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (gm_utils_color_entry_changed),
                        button);
    gtk_entry_set_text (entry, sql_editor_config->comments_color);



    button = gm_utils_get_data (options_dialog, "keywords_color_button");
    gtk_object_set_data (GTK_OBJECT (button), "highlighting_preview", text_widget);
    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "keywords_color"));
    gtk_object_set_data (GTK_OBJECT (entry), "gdk_color", g_hash_table_lookup (sh->colors_table, "keywords"));


    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_utils_choose_color_button_clicked),
                        entry);
    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (gm_utils_color_entry_changed),
                        button);
    gtk_entry_set_text (entry, sql_editor_config->keywords_color);


    button = gm_utils_get_data (options_dialog, "functions_color_button");
    gtk_object_set_data (GTK_OBJECT (button), "highlighting_preview", text_widget);
    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "functions_color"));
    gtk_object_set_data (GTK_OBJECT (entry), "gdk_color", g_hash_table_lookup (sh->colors_table, "functions"));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_utils_choose_color_button_clicked),
                        entry);
    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (gm_utils_color_entry_changed),
                        button);
    gtk_entry_set_text (entry, sql_editor_config->functions_color);


    button = gm_utils_get_data (options_dialog, "errors_color_button");
    gtk_object_set_data (GTK_OBJECT (button), "highlighting_preview", text_widget);
    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "errors_color"));
    gtk_object_set_data (GTK_OBJECT (entry), "gdk_color", g_hash_table_lookup (sh->colors_table, "errors"));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_utils_choose_color_button_clicked),
                        entry);
    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (gm_utils_color_entry_changed),
                        button);
    gtk_entry_set_text (entry, sql_editor_config->errors_color);



    button = gm_utils_get_data (options_dialog, "strings_color_button");
    gtk_object_set_data (GTK_OBJECT (button), "highlighting_preview", text_widget);
    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "strings_color"));
    gtk_object_set_data (GTK_OBJECT (entry), "gdk_color", g_hash_table_lookup (sh->colors_table, "strings"));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_utils_choose_color_button_clicked),
                        entry);
    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (gm_utils_color_entry_changed),
                        button);
    gtk_entry_set_text (entry, sql_editor_config->strings_color);



    button = gm_utils_get_data (options_dialog, "default_color_button");
    gtk_object_set_data (GTK_OBJECT (button), "highlighting_preview", text_widget);

    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "default_color"));
    gtk_object_set_data (GTK_OBJECT (entry), "gdk_color", g_hash_table_lookup (sh->colors_table, "default"));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_utils_choose_color_button_clicked),
                        entry);
    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (gm_utils_color_entry_changed),
                        button);
    gtk_entry_set_text (entry, sql_editor_config->default_color);

    sql_editor_font = GNOME_FONT_PICKER (gm_utils_get_data (options_dialog, "sql_editor_font"));
    if (sql_editor_config->font_name)
        gnome_font_picker_set_font_name (sql_editor_font, sql_editor_config->font_name);

    gtk_signal_connect (GTK_OBJECT (sql_editor_font), "font-set",
                        GTK_SIGNAL_FUNC (cc_sql_editor_font_changed),
                        sh);

    gm_utils_text_colorize (GTK_EDITABLE (text_widget), 0, -1);
}


static void
cc_fill_ge_config_page (GmCcenter *cc)
{
    GtkEntry  * entry;
    GtkWidget * options_dialog;

    GmCcenterGeneral * ge_config;
    GtkToggleButton  * tb;

    ge_config = cc->ge_config;
    options_dialog = cc->options_dialog;

    entry = GTK_ENTRY (gm_utils_get_data (GTK_OBJECT (options_dialog),"logfilename_entry"));
    if (ge_config->logfilename)
        gtk_entry_set_text (entry, ge_config->logfilename);

    entry = GTK_ENTRY (gm_utils_get_data (GTK_OBJECT (options_dialog),"text_editor"));
    if (ge_config->text_editor)
        gtk_entry_set_text (entry, ge_config->text_editor);

    entry = GTK_ENTRY (gm_utils_get_data (GTK_OBJECT (options_dialog),"browser"));
    if (ge_config->browser)
        gtk_entry_set_text (entry, ge_config->browser);


    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (options_dialog),"show_dt"));
    if (ge_config->date_time_on)
        gtk_toggle_button_set_active (tb, TRUE);

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (options_dialog),"show_hints"));
    if (ge_config->show_hints)
        gtk_toggle_button_set_active (tb, TRUE);

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (options_dialog),"show_fields"));
    if (ge_config->show_fields)
        gtk_toggle_button_set_active (tb, TRUE);
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (options_dialog),"hide_dt"));
    if (!ge_config->date_time_on)
        gtk_toggle_button_set_active (tb, TRUE);
}





static void
cc_options_cancel_button_clicked              (GtkButton       *button,
        GmCcenter * cc)
{
    GtkObject * obj;
    obj = GTK_OBJECT (cc);
    gtk_object_set_data (obj, "options_dialog_cancled", "true");
    gtk_widget_destroy (cc->options_dialog);
    cc->options_dialog = NULL;
    gtk_main_quit ();
}


static gboolean cc_collect_sql_editor_page_info (GmCcenter * cc)
{

    GtkWidget * options_dialog;
    GmCcenterSqlEditor * sql_editor_config;
    GtkEntry * entry;
    gchar * color;
    gboolean update_highlighting = FALSE;
    GnomeFontPicker * sql_editor_font;
    gchar * font_name;


    options_dialog = cc->options_dialog;
    sql_editor_config = cc->sql_editor_config;

    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "comments_color"));
    color = gtk_entry_get_text (entry);
    if (color && strlen (color)==7){
        if (sql_editor_config->comments_color && strcmp (sql_editor_config->comments_color, color)){
            g_free (sql_editor_config->comments_color);
            sql_editor_config->comments_color = g_strdup (color);
            update_highlighting = TRUE;
        }else if (!sql_editor_config->comments_color ){
            update_highlighting = TRUE;
            sql_editor_config->comments_color = g_strdup (color);
        }

    }

    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "keywords_color"));
    color = gtk_entry_get_text (entry);
    if (color && strlen (color)==7){
        if (sql_editor_config->keywords_color && strcmp (sql_editor_config->keywords_color, color)){
            g_free (sql_editor_config->keywords_color);
            sql_editor_config->keywords_color = g_strdup (color);
            update_highlighting = TRUE;
        }else if (!sql_editor_config->keywords_color ){
            update_highlighting = TRUE;
            sql_editor_config->keywords_color = g_strdup (color);
        }

    }

    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "functions_color"));
    color = gtk_entry_get_text (entry);
    if (color && strlen (color)==7){
        if (sql_editor_config->functions_color && strcmp (sql_editor_config->functions_color, color)){
            g_free (sql_editor_config->functions_color);
            sql_editor_config->functions_color = g_strdup (color);
            update_highlighting = TRUE;
        }else  if (!sql_editor_config->functions_color ){
            update_highlighting = TRUE;
            sql_editor_config->functions_color = g_strdup (color);
        }

    }

    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "errors_color"));
    color = gtk_entry_get_text (entry);
    if (color && strlen (color)==7){
        if (sql_editor_config->errors_color && strcmp (sql_editor_config->errors_color, color)){
            g_free (sql_editor_config->errors_color);
            sql_editor_config->errors_color = g_strdup (color);
            update_highlighting = TRUE;
        }else  if (!sql_editor_config->errors_color ){
            update_highlighting = TRUE;
            sql_editor_config->errors_color = g_strdup (color);
        }
    }

    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "strings_color"));
    color = gtk_entry_get_text (entry);
    if (color && strlen (color)==7){
        if (sql_editor_config->strings_color && strcmp (sql_editor_config->strings_color, color)){
            g_free (sql_editor_config->strings_color);
            sql_editor_config->strings_color = g_strdup (color);
            update_highlighting = TRUE;
        }else if (!sql_editor_config->strings_color ){
            update_highlighting = TRUE;
            sql_editor_config->strings_color = g_strdup (color);
        }
    }

    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "default_color"));
    color = gtk_entry_get_text (entry);
    if (color && strlen (color)==7){
        if (sql_editor_config->default_color && strcmp (sql_editor_config->default_color, color)){
            g_free (sql_editor_config->default_color);
            sql_editor_config->default_color = g_strdup (color);
            update_highlighting = TRUE;
        }else if (!sql_editor_config->default_color ){
            update_highlighting = TRUE;
            sql_editor_config->default_color = g_strdup (color);
        }
    }

    entry  = GTK_ENTRY (gm_utils_get_data (options_dialog, "sql_editor_bgcolor"));
    color = gtk_entry_get_text (entry);
    if (color && strlen (color)==7){
        if (sql_editor_config->bgcolor && strcmp (sql_editor_config->bgcolor, color)){
            g_free (sql_editor_config->bgcolor);
            sql_editor_config->bgcolor = g_strdup (color);
            update_highlighting = TRUE;
        }else if (!sql_editor_config->bgcolor ){
            update_highlighting = TRUE;
            sql_editor_config->bgcolor = g_strdup (color);
        }

    }

    sql_editor_font = GNOME_FONT_PICKER (gm_utils_get_data (options_dialog, "sql_editor_font"));


    font_name = gnome_font_picker_get_font_name (sql_editor_font);


    if (font_name) {
        if (sql_editor_config->font_name && strcmp (sql_editor_config->font_name, font_name)){
            g_free (sql_editor_config->font_name);
            sql_editor_config->font_name = g_strdup (font_name);
            update_highlighting = TRUE;
        }else if (!sql_editor_config->font_name) {
            sql_editor_config->font_name  = g_strdup (font_name);
            update_highlighting = TRUE;
        }
    }







    return update_highlighting;
}



static gboolean cc_collect_printing_page_info (GmCcenter * cc)
{

    GtkWidget * options_dialog;
    GmCcenterPrinting * pr_config;

    
    GnomePaperSelector *paperselector;
    GtkToggleButton *tb;


    options_dialog = cc->options_dialog;
    pr_config = cc->pr_config;
    paperselector = GNOME_PAPER_SELECTOR (gm_utils_get_data (options_dialog, "paperselector"));

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (options_dialog, "print_orientation_portrait_radio_button"));
    if (gtk_toggle_button_get_active (tb))
        pr_config->print_orientation = PRINT_ORIENT_PORTRAIT;
    else
        pr_config->print_orientation = PRINT_ORIENT_LANDSCAPE;

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (options_dialog, "print_grid"));
    pr_config->print_grid = gtk_toggle_button_get_active (tb);
    g_free (pr_config->paper_size);
    if (strcmp ( "custom", gnome_paper_selector_get_name ( paperselector))==0)
    {
        g_warning ("Custom paper sizes not yet supported. Setting paper size to default\n");
        pr_config->paper_size = g_strdup (gnome_paper_name_default ());
    }
    else
        pr_config->paper_size = g_strdup (gnome_paper_selector_get_name ( GNOME_PAPER_SELECTOR (paperselector)));

    g_free (pr_config->unit_name);
    pr_config->unit_name = g_strdup (gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(paperselector->unit)->entry)));
    return TRUE;
}


static gboolean cc_collect_general_page_info (GmCcenter * cc)
{

    GtkWidget * options_dialog;
    GtkEntry * entry;
    GmCcenterGeneral * ge_config;
    gchar * logfilename=NULL;
    gchar * text_editor = NULL;
    gchar *browser = NULL;
    GtkToggleButton * tb;

    options_dialog = cc->options_dialog;
    ge_config = cc->ge_config;

    entry = GTK_ENTRY (gm_utils_get_data (GTK_OBJECT (options_dialog),"logfilename_entry"));
    logfilename = gtk_entry_get_text (entry);
    if (logfilename && strlen (logfilename) > 1)
    {
        FILE *tmp;
        if ( (tmp = fopen (logfilename,"a+"))!=NULL)
        {
            g_free (ge_config->logfilename);
            ge_config->logfilename = g_strdup (logfilename);
            if (ge_config->logfilehandler)
            {
                fclose (ge_config->logfilehandler);
            }
            ge_config->logfilehandler = tmp;
        }else
        {
            gm_utils_warning ("could not open your selected logfile ");
            return FALSE;
        }
    }else
    {
        if (ge_config->logfilehandler)
            fclose (ge_config->logfilehandler);
        ge_config->logfilehandler = 0;
        g_free (ge_config->logfilename);
        ge_config->logfilename = NULL;
    }
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (options_dialog),"show_dt"));
    ge_config->date_time_on = gtk_toggle_button_get_active (tb);
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (options_dialog),"show_hints"));
    ge_config->show_hints = gtk_toggle_button_get_active (tb);

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (options_dialog),"show_fields"));
    ge_config->show_fields = gtk_toggle_button_get_active (tb);


    entry = GTK_ENTRY (gm_utils_get_data (GTK_OBJECT (options_dialog),"text_editor"));
    text_editor = gtk_entry_get_text (entry);
    g_free (ge_config->text_editor);
    if (text_editor)
        ge_config->text_editor = g_strdup (text_editor);
    else
        ge_config->text_editor = NULL;



    entry = GTK_ENTRY (gm_utils_get_data (GTK_OBJECT (options_dialog),"browser"));
    browser = gtk_entry_get_text (entry);
    g_free (ge_config->browser);
    if (browser)
        ge_config->browser = g_strdup (browser);
    else
        ge_config->browser = NULL;


    return TRUE;
}








static void
cc_options_ok_button_clicked              (void       *ignore,
        GmCcenter * cc)
{

    gboolean update_highlighting ;

    /* First Page General configuration */
    if (!cc_collect_general_page_info (cc))
        return ;
    cc_collect_printing_page_info (cc);
    update_highlighting = cc_collect_sql_editor_page_info (cc);
    cc_write_configuration (cc);
    gtk_widget_destroy (cc->options_dialog);
    cc->options_dialog = NULL;
    gtk_main_quit ();
    if (update_highlighting)
        gm_app_emit_sql_mode_colors_changed (global_app);


}

static gint
gm_cc_option_dialog_key_pressed     (GtkWidget       *widget,
                                     GdkEventKey     *event,
                                     GmCcenter * cc)
{

    if  (event->keyval == GDK_Escape) {
        cc_options_cancel_button_clicked (NULL, cc);
        return TRUE;
    }
    return FALSE;

}

static void
on_options_dialog_destroy (GtkWidget *options_dialog,
                           GmCcenter * cc)
{
    cc->options_dialog = NULL;
}






void gm_cc_show_option_dialog (GtkObject *obj, gint page )
{

    GtkWidget *options_dialog;
    GtkWidget *options_ok_button;
    GtkWidget *options_cancel_button;
    GtkWidget *note_book;


    GmCcenter * cc;



    g_return_if_fail (obj != NULL);
    cc = GM_CCENTER (obj);

    if (cc->options_dialog)
        return;

    options_dialog =  cc->options_dialog = create_options_window ();
    note_book = gm_utils_get_data (options_dialog, "options_notebook");

    gtk_signal_connect (GTK_OBJECT (options_dialog), "key_press_event",
                        GTK_SIGNAL_FUNC (gm_cc_option_dialog_key_pressed),
                        cc);

    options_ok_button     = gm_utils_get_data (GTK_OBJECT (options_dialog),"options_ok_button");
    options_cancel_button = gm_utils_get_data (GTK_OBJECT (options_dialog), "options_cancel_button");

    gtk_signal_connect (GTK_OBJECT (options_ok_button), "clicked",
                        GTK_SIGNAL_FUNC (cc_options_ok_button_clicked),
                        cc);

    gtk_signal_connect (GTK_OBJECT (options_cancel_button), "clicked",
                        GTK_SIGNAL_FUNC (cc_options_cancel_button_clicked),
                        cc);

    gtk_signal_connect (GTK_OBJECT (options_dialog), "destroy",
                        on_options_dialog_destroy,
                        cc);


    cc_fill_ge_config_page (cc);
    cc_fill_sql_editor_config_page (cc);
    cc_fill_printer_config_page (cc);
    gtk_notebook_set_page (GTK_NOTEBOOK (note_book), page);
    gtk_widget_show (cc->options_dialog);
    gtk_main ();
}
