/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <ctype.h> /* isdigit */
//#include <libgnomeui/gnome-window-icon.h>
#include "gm-date-entry.h"
#include "gm-enums.h"
#include "gm-utils.h"

enum {
    DATE_CHANGED,
    TIME_CHANGED,
    LAST_SIGNAL
};

static gint date_entry_signals [LAST_SIGNAL] = { 0 };


static void gm_date_entry_init         (GmDateEntry      *gde);
static void gm_date_entry_class_init   (GmDateEntryClass *class);
static void gm_date_entry_destroy      (GtkObject          *object);
static void gm_date_entry_forall       (GtkContainer       *container,
                                        gboolean	      include_internals,
                                        GtkCallback	      callback,
                                        gpointer	      callbabck_data);

static GtkHBoxClass *parent_class;

/**
 * gm_date_entry_get_type:
 *
 * Returns the GtkType for the GmDateEntry widget
 */

guint
gm_date_entry_get_type (void)
{
    static guint date_entry_type = 0;

    if (!date_entry_type){
        GtkTypeInfo date_entry_info = {
            "GmDateEntry",
            sizeof (GmDateEntry),
            sizeof (GmDateEntryClass),
            (GtkClassInitFunc) gm_date_entry_class_init,
            (GtkObjectInitFunc) gm_date_entry_init,
            NULL,
            NULL,
        };

        date_entry_type = gtk_type_unique (gtk_hbox_get_type (), &date_entry_info);
    }

    return date_entry_type;
}

static void
hide_popup (GmDateEntry *gde)
{
    gtk_widget_hide (gde->cal_popup);
    gtk_grab_remove (gde->cal_popup);
    gdk_pointer_ungrab (GDK_CURRENT_TIME);
}

static void
day_selected (GtkCalendar *calendar, GmDateEntry *gde)
{

    gint year, month, day;
    GString * sbuffer;

    gtk_calendar_get_date (calendar, &year, &month, &day);

    sbuffer = g_string_new("");
    switch (gde->date_format) {

    case GM_MYSQL_TIMESTAMP:
        {
            g_string_sprintfa (sbuffer, "%d",year);
            if (month + 1 <10)
                g_string_sprintfa (sbuffer, "0%d" ,month + 1);
            else
                g_string_sprintfa (sbuffer, "%d",month + 1);

            if (day <10)
                g_string_sprintfa (sbuffer, "0%d", day);
            else
                g_string_sprintfa (sbuffer, "%d", day);
            if (gde->hms)
                g_string_sprintfa (sbuffer, "%s", gde->hms);

            break;
        }
    case GM_MYSQL_YEAR:
        {
            g_string_sprintfa (sbuffer, "%d",year);

            break;
        }
    case GM_MYSQL_DATE :
        {
            g_string_sprintfa (sbuffer, "%d",year);

            if (month + 1 <10)
                g_string_sprintfa (sbuffer, "-0%d" , month + 1);
            else
                g_string_sprintfa (sbuffer, "-%d",month + 1);

            if (day <10)
                g_string_sprintfa (sbuffer, "-0%d", day);
            else
                g_string_sprintfa (sbuffer, "-%d", day);
            break;
        }
    case GM_MYSQL_DATETIME :
    default:
        {
            g_string_sprintfa (sbuffer, "%d",year);

            if (month + 1 <10)
                g_string_sprintfa (sbuffer, "-0%d" , month + 1);
            else
                g_string_sprintfa (sbuffer, "-%d",month + 1);

            if (day <10)
                g_string_sprintfa (sbuffer, "-0%d", day);
            else
                g_string_sprintfa (sbuffer, "-%d", day);

            if (gde->hms)
                g_string_sprintfa (sbuffer, " %s", gde->hms);
            break;
        }

    }
    gtk_entry_set_text (GTK_ENTRY (gde->date_entry), sbuffer->str);
    g_string_free (sbuffer, TRUE);

    gtk_signal_emit (GTK_OBJECT (gde), date_entry_signals [DATE_CHANGED]);
}

static void
day_selected_double_click (GtkCalendar *calendar, GmDateEntry *gde)
{
    hide_popup (gde);
}

static gint
delete_popup (GtkWidget *widget, gpointer data)
{
    GmDateEntry *gde;

    gde = data;
    hide_popup (gde);

    return TRUE;
}

static gint
key_press_popup (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
    GmDateEntry *gde;

    if (event->keyval != GDK_Escape)
        return FALSE;

    gde = data;
    gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
    hide_popup (gde);

    return TRUE;
}


static gint
button_press_popup (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
    GmDateEntry *gde;
    GtkWidget *child;

    gde = data;

    child = gtk_get_event_widget ((GdkEvent *) event);

    /* We don't ask for button press events on the grab widget, so
     *  if an event is reported directly to the grab widget, it must
     *  be on a window outside the application (and thus we remove
     *  the popup window). Otherwise, we check if the widget is a child
     *  of the grab widget, and only remove the popup window if it
     *  is not.
     */
    if (child != widget) {
        while (child) {
            if (child == widget)
                return FALSE;
            child = child->parent;
        }
    }

    hide_popup (gde);

    return TRUE;
}

static void
position_popup (GmDateEntry *gde)
{
    gint x, y;
    gint bwidth, bheight;


    gtk_widget_set_usize (gde->cal_popup, -1, -1);
    gtk_widget_size_request (gde->cal_popup, &gde->cal_popup->requisition);
    gdk_window_get_origin (gde->date_button->window, &x, &y);
    gdk_window_get_size (gde->date_button->window, &bwidth, &bheight);


    x += bwidth - gde->cal_popup->requisition.width;
    y += bheight;


    if (x < 0)
        x = 0;

    if (y < 0)
        y = 0;

    gtk_widget_set_uposition (gde->cal_popup, x, y);
}


static void
select_clicked (GtkWidget *widget, GmDateEntry *gde)
{
    struct tm mtm;
    GdkCursor *cursor;
    gchar buffer[9];
    gint i ;

    for (i=0;i<9;i++)
        buffer[i] = '\0';

    switch (gde->date_format) {

    case GM_MYSQL_TIMESTAMP:
        {
            sscanf (gtk_entry_get_text (GTK_ENTRY (gde->date_entry)), "%4d%2d%2d%6s",
                    &mtm.tm_year, &mtm.tm_mon, &mtm.tm_mday, buffer);
            g_free(gde->hms);
            gde->hms = g_strdup (buffer);

            break;
        }
    case GM_MYSQL_YEAR:
        {
            sscanf (gtk_entry_get_text (GTK_ENTRY (gde->date_entry)), "%4d",
                    &mtm.tm_year);
            break;
        }
    case GM_MYSQL_DATE :
        {
            sscanf (gtk_entry_get_text (GTK_ENTRY (gde->date_entry)), "%4d-%2d-%2d",
                    &mtm.tm_year, &mtm.tm_mon, &mtm.tm_mday);
        }

    case GM_MYSQL_DATETIME :
    default:
        {

            sscanf (gtk_entry_get_text (GTK_ENTRY (gde->date_entry)), "%4d-%2d-%2d %8s",
                    &mtm.tm_year, &mtm.tm_mon, &mtm.tm_mday, buffer);
            g_free(gde->hms);
            gde->hms = g_strdup (buffer);
            break;
        }

    }


    mtm.tm_mon--;

    /* Hope the user does not actually mean years early in the A.D. days...
     * This date widget will obviously not work for a history program :-)
     */
    if (mtm.tm_year >= 1900)
        mtm.tm_year -= 1900;
    if (mtm.tm_mday > 31 || mtm.tm_mday <1) mtm.tm_mday = 1;

    if (mtm.tm_mon < 0) mtm.tm_mon = 0;

    if (mtm.tm_mon > 11) mtm.tm_mon = 11;

    gtk_calendar_select_month (GTK_CALENDAR (gde->calendar), mtm.tm_mon, 1900 + mtm.tm_year);
    gtk_calendar_select_day (GTK_CALENDAR (gde->calendar), mtm.tm_mday);

    position_popup (gde);

    gtk_widget_show (gde->cal_popup);
    gtk_widget_grab_focus (gde->cal_popup);
    gtk_grab_add (gde->cal_popup);


    cursor = gdk_cursor_new (GDK_HAND1);

    gdk_pointer_grab (gde->cal_popup->window, TRUE,
                      (GDK_BUTTON_PRESS_MASK
                       | GDK_BUTTON_RELEASE_MASK
                       | GDK_POINTER_MOTION_MASK),
                      NULL, cursor, GDK_CURRENT_TIME);

    gdk_cursor_destroy (cursor);
}

typedef struct {
    char *hour;
    GmDateEntry *gde;
} hour_info_t;





static void
set_widget_cursors (GtkWidget *widget, GmDateEntry *gde)
{
    gm_utils_set_widget_cursor (gde->date_button, GDK_HAND1);
}





static void
gm_date_entry_class_init (GmDateEntryClass *class)
{
    GtkObjectClass *object_class = (GtkObjectClass *) class;
    GtkContainerClass *container_class = (GtkContainerClass *) class;

    object_class = (GtkObjectClass*) class;

    parent_class = gtk_type_class (gtk_hbox_get_type ());

    date_entry_signals [TIME_CHANGED] =
        gtk_signal_new ("time_changed",
                        GTK_RUN_FIRST, object_class->type,
                        GTK_SIGNAL_OFFSET (GmDateEntryClass, time_changed),
                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

    date_entry_signals [DATE_CHANGED] =
        gtk_signal_new ("date_changed",
                        GTK_RUN_FIRST, object_class->type,
                        GTK_SIGNAL_OFFSET (GmDateEntryClass, date_changed),
                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

    gtk_object_class_add_signals (object_class, date_entry_signals, LAST_SIGNAL);

    container_class->forall = gm_date_entry_forall;

    object_class->destroy = gm_date_entry_destroy;

    class->date_changed = NULL;
    class->time_changed = NULL;
}

static void
gm_date_entry_init (GmDateEntry *gde)
{
    gde->lower_hour = 7;
    gde->upper_hour = 19;
    gde->flags = GM_DATE_ENTRY_SHOW_TIME;
    gde->date_format = GM_MYSQL_DATETIME;
    gde->hms = NULL;
}

static void
gm_date_entry_destroy (GtkObject *object)
{
    GmDateEntry *gde;

    g_return_if_fail (object != NULL);
    g_return_if_fail (GNOME_IS_DATE_ENTRY (object));



    gde = GM_DATE_ENTRY (object);
    g_free (gde->hms);
    gtk_widget_destroy (gde->cal_popup);

    if (GTK_OBJECT_CLASS (parent_class)->destroy)
        (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
gm_date_entry_forall (GtkContainer *container, gboolean include_internals,
                      GtkCallback callback, gpointer callback_data)
{
    g_return_if_fail (container != NULL);
    g_return_if_fail (GNOME_IS_DATE_ENTRY (container));
    g_return_if_fail (callback != NULL);

    /* Let GtkBox handle things only if the internal widgets need to be
     * poked.
     */
    if (include_internals)
        if (GTK_CONTAINER_CLASS (parent_class)->forall)
            (* GTK_CONTAINER_CLASS (parent_class)->forall) (container,
                    include_internals,
                    callback,
                    callback_data);
}



/**
 * gm_date_entry_set_date_format:
 * @gde: the GmDateEntry widget
 * @date_format: The date format that should be set on the widget
 *
 * chage the format of the date-time presentation in the entry
 */

void
gm_date_entry_set_date_format (GmDateEntry *gde, gint date_format)

{
    g_return_if_fail(gde != NULL);
    gde->date_format = date_format;

}



/**
 * gm_date_entry_set_time:
 * @gde: the GmDateEntry widget
 * @the_time: The time and date that should be set on the widget
 *
 * Changes the displayed date and time in the GmDateEntry widget
 * to be the one represented by @the_time.
 */

void
gm_date_entry_set_time (GmDateEntry *gde, time_t the_time)
{
    struct tm *mytm;
    char buffer [40];
    GString *sbuffer;

    g_return_if_fail(gde != NULL);


    sbuffer = g_string_new("");

    if (the_time == 0)
        the_time = time (NULL);

    gde->initial_time = the_time;

    mytm = localtime (&the_time);

    /* Set the date */


    switch (gde->date_format) {

    case GM_MYSQL_TIMESTAMP:
        {
            gint t;

            g_string_sprintfa (sbuffer, "%d",1900 + mytm->tm_year);

            if (mytm->tm_mon + 1 <10)
                g_string_sprintfa (sbuffer, "0%d",mytm->tm_mon + 1);
            else
                g_string_sprintfa (sbuffer, "%d",mytm->tm_mon + 1);

            if (mytm->tm_mday  <10)
                g_string_sprintfa (sbuffer, "0%d",mytm->tm_mday);
            else
                g_string_sprintfa (sbuffer, "%d",mytm->tm_mday);

            t = mytm->tm_hour;
            if (t  < 10)
                g_string_sprintfa (sbuffer, "0%d", t);
            else
                g_string_sprintfa (sbuffer, "%d", t);

            t = mytm->tm_min;
            if (t  < 10)
                g_string_sprintfa (sbuffer, "0%d", t);
            else
                g_string_sprintfa (sbuffer, "%d", t);

            t = mytm->tm_sec;
            if (t  < 10)
                g_string_sprintfa (sbuffer, "0%d", t);
            else
                g_string_sprintfa (sbuffer, "%d", t);

            break;
        }
    case GM_MYSQL_YEAR:
        {
            g_string_sprintfa (sbuffer, "%d",1900 + mytm->tm_year);
            break;
        }
    case GM_MYSQL_DATE :
        {
            g_string_sprintfa (sbuffer, "%d",1900 + mytm->tm_year);

            if (mytm->tm_mon + 1 <10)
                g_string_sprintfa (sbuffer, "-0%d",mytm->tm_mon + 1);
            else
                g_string_sprintfa (sbuffer, "-%d",mytm->tm_mon + 1);

            if (mytm->tm_mday  <10)
                g_string_sprintfa (sbuffer, "-0%d",mytm->tm_mday);
            else
                g_string_sprintfa (sbuffer, "-%d",mytm->tm_mday);
            break;
        }
    case GM_MYSQL_DATETIME :
    default:
        {
            gint t;

            g_string_sprintfa (sbuffer, "%d",1900 + mytm->tm_year);

            if (mytm->tm_mon + 1 <10)
                g_string_sprintfa (sbuffer, "-0%d",mytm->tm_mon + 1);
            else
                g_string_sprintfa (sbuffer, "-%d",mytm->tm_mon + 1);

            if (mytm->tm_mday  <10)
                g_string_sprintfa (sbuffer, "-0%d",mytm->tm_mday);
            else
                g_string_sprintfa (sbuffer, "-%d",mytm->tm_mday);

            t = mytm->tm_hour;
            if (t  < 10)
                g_string_sprintfa (sbuffer, " 0%d", t);
            else
                g_string_sprintfa (sbuffer, " %d", t);

            t = mytm->tm_min;
            if (t  < 10)
                g_string_sprintfa (sbuffer, ":0%d", t);
            else
                g_string_sprintfa (sbuffer, ":%d", t);

            t = mytm->tm_sec;
            if (t  < 10)
                g_string_sprintfa (sbuffer, ":0%d", t);
            else
                g_string_sprintfa (sbuffer, ":%d", t);
            break;
        }

    }




    gtk_entry_set_text (GTK_ENTRY (gde->date_entry), sbuffer->str);
    g_string_free (sbuffer, TRUE);

    /* Set the time */
    if (gde->flags & GM_DATE_ENTRY_24_HR)
        strftime (buffer, sizeof (buffer), "%H:%M", mytm);
    else
        strftime (buffer, sizeof (buffer), "%I:%M %p", mytm);

}



static void
create_children (GmDateEntry *gde)
{
    GtkWidget *frame;


    GtkWidget *pixmap;


    gde->date_entry  = gtk_entry_new ();
    gtk_box_pack_start (GTK_BOX (gde), gde->date_entry, TRUE, TRUE, 0);
    gtk_widget_show (gde->date_entry);

    gde->date_button = gtk_button_new ();
    gtk_widget_set_usize (gde->date_button, 40, 22);
    gtk_signal_connect (GTK_OBJECT (gde->date_button), "clicked",
                        GTK_SIGNAL_FUNC (select_clicked), gde);
    gtk_box_pack_start (GTK_BOX (gde), gde->date_button, FALSE, FALSE, 0);



    /* Calendar label, only shown if the date editor has a time field */

    gde->cal_label = gtk_label_new (_("Calendar"));
    gtk_misc_set_alignment (GTK_MISC (gde->cal_label), 0.0, 0.5);


    pixmap = gm_stock_pixmap_widget (GTK_WIDGET (gde->date_button), "calender_button");
    gtk_container_add (GTK_CONTAINER (gde->date_button), pixmap);
    gtk_widget_show (pixmap);

    gtk_widget_show (gde->date_button);
    gtk_signal_connect (GTK_OBJECT (gde), "realize",
                        GTK_SIGNAL_FUNC (set_widget_cursors), gde);



    gde->cal_popup = gtk_window_new (GTK_WINDOW_POPUP);
    //gnome_window_icon_set_from_default (GTK_WINDOW (gde->cal_popup));
    gtk_widget_set_events (gde->cal_popup,
                           gtk_widget_get_events (gde->cal_popup) | GDK_KEY_PRESS_MASK);
    gtk_signal_connect (GTK_OBJECT (gde->cal_popup), "delete_event",
                        (GtkSignalFunc) delete_popup,
                        gde);
    gtk_signal_connect (GTK_OBJECT (gde->cal_popup), "key_press_event",
                        (GtkSignalFunc) key_press_popup,
                        gde);
    gtk_signal_connect (GTK_OBJECT (gde->cal_popup), "button_press_event",
                        (GtkSignalFunc) button_press_popup,
                        gde);
    gtk_window_set_policy (GTK_WINDOW (gde->cal_popup), FALSE, FALSE, TRUE);

    frame = gtk_frame_new (NULL);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
    gtk_container_add (GTK_CONTAINER (gde->cal_popup), frame);
    gtk_widget_show (frame);

    gde->calendar = gtk_calendar_new ();

    gtk_calendar_display_options (GTK_CALENDAR (gde->calendar),
                                  GTK_CALENDAR_SHOW_HEADING
                                  | GTK_CALENDAR_SHOW_DAY_NAMES
                                  | GTK_CALENDAR_SHOW_WEEK_NUMBERS
                                  | GTK_CALENDAR_WEEK_START_MONDAY);

    gtk_signal_connect (GTK_OBJECT (gde->calendar), "day_selected",
                        GTK_SIGNAL_FUNC (day_selected), gde);
    gtk_signal_connect (GTK_OBJECT (gde->calendar), "day_selected_double_click",
                        GTK_SIGNAL_FUNC (day_selected_double_click), gde);
    gtk_container_add (GTK_CONTAINER (frame), gde->calendar);
    gtk_widget_show (gde->calendar);
}

/**
 * gm_date_entry_new:
 * @the_time: date and time to be displayed on the widget
 * @show_time: whether time should be displayed
 * @use_24_format: whether 24-hour format is desired for the time display.
 *
 * Creates a new GmDateEntry widget which can be used to provide
 * an easy to use way for entering dates and times.
 * 
 * Returns a GmDateEntry widget.
 */
GtkWidget *
gm_date_entry_new (time_t the_time, int show_time, int use_24_format, gint date_format)
{
    return gm_date_entry_new_flags (the_time,
                                    ((show_time ? GM_DATE_ENTRY_SHOW_TIME : 0)
                                     | (use_24_format ? GM_DATE_ENTRY_24_HR : 0)), date_format);
}

/**
 * gm_date_entry_new_flags:
 * @the_time: The initial time for the date editor.
 * @flags: A bitmask of GmDateEntryFlags values.
 * 
 * Creates a new GmDateEntry widget with the specified flags.
 * 
 * Return value: the newly-created date editor widget.
 **/

GtkWidget *
gm_date_entry_new_flags (time_t the_time, GmDateEntryFlags flags, gint date_format)
{
    GmDateEntry *gde;

    gde = gtk_type_new (gm_date_entry_get_type ());

    gde->flags = flags;
    create_children (gde);
    gm_date_entry_set_date_format (gde,date_format);
    gm_date_entry_set_time (gde, the_time);

    return GTK_WIDGET (gde);
}

/**
 * gm_date_entry_get_date:
 * @gde: The GmDateEntry widget
 *
 * Returns the time entered in the GmDateEntry widget
 */
time_t
gm_date_entry_get_date (GmDateEntry *gde)
{
    struct tm tm = {0};


    /* Assert, because we're just hosed if it's NULL */
    g_assert(gde != NULL);
    g_assert(GNOME_IS_DATE_ENTRY(gde));

    sscanf (gtk_entry_get_text (GTK_ENTRY (gde->date_entry)), "%d/%d/%d",
            &tm.tm_mon, &tm.tm_mday, &tm.tm_year); /* FIXME: internationalize this - strptime()*/

    tm.tm_mon--;

    /* Hope the user does not actually mean years early in the A.D. days...
     * This date widget will obviously not work for a history program :-)
     */
    if (tm.tm_year >= 1900)
        tm.tm_year -= 1900;
    tm.tm_isdst = -1;

    return mktime (&tm);
}

/**
 * gm_date_entry_set_flags:
 * @gde: The date editor widget whose flags should be changed.
 * @flags: The new bitmask of GmDateEntryFlags values.
 * 
 * Changes the display flags on an existing date editor widget.
 **/
void
gm_date_entry_set_flags (GmDateEntry *gde, GmDateEntryFlags flags)
{
    GmDateEntryFlags old_flags;

    g_return_if_fail (gde != NULL);
    g_return_if_fail (GNOME_IS_DATE_ENTRY (gde));

    old_flags = gde->flags;
    gde->flags = flags;

    if ((flags & GM_DATE_ENTRY_SHOW_TIME) != (old_flags & GM_DATE_ENTRY_SHOW_TIME)) {
        if (flags & GM_DATE_ENTRY_SHOW_TIME) {
            gtk_widget_show (gde->cal_label);
        } else {
            gtk_widget_hide (gde->cal_label);
        }
    }



    if ((flags & GM_DATE_ENTRY_WEEK_STARTS_ON_MONDAY)
            != (old_flags & GM_DATE_ENTRY_WEEK_STARTS_ON_MONDAY)) {
        if (flags & GM_DATE_ENTRY_WEEK_STARTS_ON_MONDAY)
            gtk_calendar_display_options (GTK_CALENDAR (gde->calendar),
                                          (GTK_CALENDAR (gde->calendar)->display_flags
                                           | GTK_CALENDAR_WEEK_START_MONDAY));
        else
            gtk_calendar_display_options (GTK_CALENDAR (gde->calendar),
                                          (GTK_CALENDAR (gde->calendar)->display_flags
                                           & ~GTK_CALENDAR_WEEK_START_MONDAY));
    }
}

/**
 * gm_date_entry_get_flags:
 * @gde: The date editor whose flags should be queried.
 * 
 * Queries the display flags on a date editor widget.
 * 
 * Return value: The current display flags for the specified date editor widget.
 **/
int
gm_date_entry_get_flags (GmDateEntry *gde)
{
    g_return_val_if_fail (gde != NULL, 0);
    g_return_val_if_fail (GNOME_IS_DATE_ENTRY (gde), 0);

    return gde->flags;
}
