/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gm.h"
#include <mysql/mysql.h>
#include "gm-control-center.h"
#include "gm-dbapi.h"
#include "gm-bpanel.h"

gchar * mysql_attributes[]=
    {
        "binary",
        "unsigned",
        "unsigned zerofill"
    };

static gchar * mysql_types[]=
    {
        "tinyint",
        "smallint",
        "mediumint",
        "int",
        "bigint",
        "float",
        "double",
        "decimal",
        "date",
        "datetime",
        "timestamp",
        "time",
        "year",
        "char",
        "varchar",
        "tinyblob",
        "tinytext",
        "text",
        "blob",
        "mediumblob",
        "mediumtext",
        "longblob",
        "longtext",
        "enum",
        "set"
    };



gchar * gm_db_get_mysql_type_string (gint i)
{
    if (i>=0 && i< GM_MYSQL_LAST_TYPE)
        return mysql_types[i];
    return NULL;
}


DbQuery *gm_db_new_query(gchar * query)
{

    DbQuery * q;

    q = g_new(DbQuery, 1);
    q->query = g_strdup(query);
    q->error = NULL;
    q->fields_count = 0;
    q->affected_rows = 0;
    q->error_number = 0;
    q->mysql_result = NULL;
    q->fields_list = NULL;
    q->database = NULL;
    q->row = NULL;
    return q;
}



void gm_db_free_query(DbQuery * q)
{

    g_free (q->query);
    g_free (q->error);

    if (q->fields_list)
        g_slist_free(q->fields_list);
    if (q->mysql_result)
        mysql_free_result(q->mysql_result);
    g_free(q);
}



static void db_get_fields (DbQuery * q, MYSQL_RES *result)
{
    MYSQL_FIELD * field=NULL;
    field = mysql_fetch_field(result);
    while(field)
    {
        q->fields_list = g_slist_append (q->fields_list, field);
        field = mysql_fetch_field(result);
    }
}



/**
 *
 * gm_db_try_to_connect 
 * @hn : hostname 
 * @p  : Port 
 * @db : database 
 * @un : username
 * @pw : password 
 *
 * returns TRUE if the connection could be established
 * set error to a new allocated char pointer that point to the mysql error message 
 * must be freed by the caller 
 *
 */


gboolean gm_db_try_to_connect (gchar *hn, gint p, gchar *db, gchar *un, gchar *pw, gchar **error)
{

    MYSQL mysql;



    mysql_init(&mysql);
    mysql_options(&mysql,MYSQL_OPT_CONNECT_TIMEOUT,DEFAULT_TIMEOUT);
    if (!mysql_real_connect(&mysql,hn, un, pw, db,p,NULL,0))
    {
        *error = g_strdup (mysql_error (&mysql));
        mysql_close(&mysql);
        return FALSE;
    }
    mysql_close(&mysql);
    return TRUE;
}

gboolean gm_db_ping_host (GmConnectionAccessData *connection_access_data, gchar **error)
{
    return gm_db_try_to_connect (connection_access_data->host,
                                 connection_access_data->port,
                                 connection_access_data->database,
                                 connection_access_data->user,
                                 connection_access_data->password,
                                 error);
}


static GList * 
db_get_table_names_and_fields (GmConnection *connection)
{
    GList *l=NULL;
    DbQuery * tq;
    DbQuery * fq;
    gchar * tquery;
    gchar * fquery;
    MYSQL_ROW row;
    MYSQL_RES *tresult;
    MYSQL_RES *fresult;
    gchar * table_name;

    tquery = g_strdup ("show tables");
    tq = gm_db_new_query (tquery);
    if (gm_db_request (connection, tq))
    {
        tresult = tq->mysql_result;
        mysql_data_seek(tresult,0);
        while ((row = mysql_fetch_row(tresult)))
        {
            table_name = g_strdup_printf("%s",row[0]);
            l = g_list_append (l, table_name);
            fquery = g_strdup_printf("show fields from %s", table_name);
            fq = gm_db_new_query (fquery);
            if (gm_db_request (connection, fq))
            {
                fresult = fq->mysql_result;
                mysql_data_seek(fresult,0);
                while ((row = mysql_fetch_row(fresult)))
                {
                    l = g_list_append (l, g_strdup_printf("%s",row[0]));
                    l = g_list_append (l, g_strdup_printf("%s.%s", table_name, row[0]));
                }
            }
            gm_db_free_query (fq);
            g_free (fquery);
        }
        l = g_list_first(l);
    }
    gm_db_free_query (tq);
    g_free (tquery);
    return l;
}



void          gm_db_update_mini_buffer_completion_list (GmConnection * connection)
{

    /*    GtkWidget * mini_buffer_label;
        gchar * label;    
        GmCcenterDb * db_config;
        GList * new_completion_items;
        static GList * old_completion_items = NULL;
        gchar * database;
        GtkWidget *  entry;
        


        db_config = GM_CCENTER(app->cc)->db_config;
        database = db_config->database;
        

        mini_buffer_label = $pointer (app->connection_window, "mini_buffer_label");
        entry = app->mini_buffer_entry;


        label = g_strdup_printf ("mysql>:%s/", database);
        gtk_label_set_text (GTK_LABEL(mini_buffer_label), label);
        g_free (label);

        new_completion_items = db_get_table_names_and_fields(app);

        if (new_completion_items)
        {

            if (old_completion_items)
            {
                g_list_g_free (old_completion_items);
                g_completion_remove_items (GM_MINI_BUFFER(entry)->completion, old_completion_items);
            }
            if (entry)
            {
                old_completion_items = new_completion_items;
                g_completion_add_items (GM_MINI_BUFFER(entry)->completion, new_completion_items);
            }	
            }*/
}

void gm_db_change_database (GmConnection * connection, gchar * new_data_base)
{


    GmConnectionAccessData * access_data;

    if (!new_data_base)
    {
        gm_debug_warning (GM_DBAPI_CHANEL, "the new database is empty");
        return;
    }
    access_data = connection->access_data;
    g_free (access_data->database);
    access_data->database = g_strdup (new_data_base);
    gm_db_update_mini_buffer_completion_list (connection);
}


gboolean gm_db_request(GmConnection * connection, DbQuery * dbquery)
{


    MYSQL mysql;
    MYSQL_RES *r=NULL;
    gchar * error=NULL;
    glong affected_rows=0;

    GmConnectionAccessData * access_data;
    gchar * user;
    gchar * password;
    gchar * database;
    gchar * hostname;
    gint    port;

    g_return_val_if_fail (connection != NULL, FALSE);
    g_return_val_if_fail (connection->access_data != NULL, FALSE);
    g_return_val_if_fail (dbquery != NULL, FALSE);
    g_return_val_if_fail (dbquery->query != NULL, FALSE);

    access_data = connection->access_data;

    mysql_init(&mysql);
    mysql_options(&mysql,MYSQL_OPT_CONNECT_TIMEOUT,DEFAULT_TIMEOUT);

    user = access_data->user;
    password = access_data->password ;

    if (dbquery->database)
        database = dbquery->database;
    else
        database = access_data->database;

    /* try to get the current database from the browser panel view */

    if (!database && connection->data_list){
        BpanelViewInfo * view_info;
        view_info = gm_utils_get_data_ww (connection->data_list, "view_info");
        if (view_info)
            database = view_info->database_in_use;
    }



    hostname = access_data->host;
    port     = access_data->port;

    if (!mysql_real_connect(&mysql,hostname, user, password, database, port, NULL, 0))
    {
        if (!mysql_real_connect(&mysql,hostname, user, password, NULL, port, NULL, 0))
        {
            error = (gchar *) g_strdup_printf("Failed to connect to database: %s %d",mysql_error(&mysql),(int) mysql_errno(&mysql));
            dbquery->error = error;
            dbquery->error_number = (int) mysql_errno(&mysql);
            return FALSE;
        }
    }
    gm_debug_message (GM_QUERY_CHANEL,"%s", dbquery->query);
    if(mysql_query(&mysql, dbquery->query))
    {
        error = (gchar *) g_strdup_printf("%s  (ERRNO: %d)\n",mysql_error(&mysql),(int)mysql_errno(&mysql));
        dbquery->error= error;
        dbquery->error_number = (int) mysql_errno(&mysql);
        return 0;
    }

    affected_rows = (glong) mysql_affected_rows(&mysql);
    if(!(r = (MYSQL_RES *) mysql_store_result(&mysql)))
    {
        r= NULL;
    }

    if (r)
    {
        dbquery->affected_rows = (glong) mysql_num_rows(r);
        dbquery->mysql_result  = r;
        dbquery->fields_count   = mysql_num_fields(r);
        db_get_fields (dbquery, r);
    }else
    {
        if (affected_rows) dbquery->affected_rows= affected_rows;
        dbquery->mysql_result = NULL;
    }
    mysql_close(&mysql);
    return TRUE;
}


static gboolean gm_db_next_record (DbQuery * q)
{

    g_return_val_if_fail (q!=NULL , FALSE);
    g_return_val_if_fail (q->mysql_result != NULL , FALSE);


    q->row = mysql_fetch_row(q->mysql_result);
    if (q->row)
        return TRUE;
    return FALSE;
}


gchar * gm_db_f (DbQuery *q, gchar * field_name)
{

    MYSQL_FIELD * field=NULL;
    GSList * list;
    gint i = 0;
    gboolean found = FALSE;

    g_return_val_if_fail (q!=NULL , NULL);
    g_return_val_if_fail (field_name!=NULL , NULL);
    g_return_val_if_fail (q->mysql_result != NULL , NULL);
    g_return_val_if_fail (q->row != NULL , NULL);


    list = q->fields_list;
    i = 0;
    while (list)
    {
        field = (MYSQL_FIELD *) list->data;
        if (!strcmp (field->name, field_name))
        {
            found = TRUE;
            break;
        }
        i++;
        list = list->next;
    }
    if (found)
        return q->row[i];
    return NULL;
}



static gint field_parse_type (gchar * line)
{

    gchar * s;
    gchar * type_name = NULL;
    gint i = 0;

    g_return_val_if_fail (line != NULL, 0);
    s = line;
    while (*s!='\0' && *s !=' ' && *s !='(')
        s++;
    if (s - line)
    {
        type_name = g_strndup (line, s - line);
    }
    if (type_name)
    {
        for (i=0; i<GM_MYSQL_LAST_TYPE; i++)
        {
            if (!strcmp (type_name, mysql_types[i]))
                return i;
        }
    }
    return GM_MYSQL_UNKNOWN_TYPE;
}


static gint field_parse_attribute (gchar * line)
{

    gchar * s, * t;
    gchar * attrib = NULL;
    gint i;

    g_return_val_if_fail (line != NULL, GM_MYSQL_UNKNOWN_TYPE);
    s = line;
    while (*s!='\0' && *s !=' ')
        s++;
    if (*s != ' ')
        return GM_MYSQL_UNKNOWN_TYPE;
    s++;
    t = s;
    while (*s!='\0')
        s++;

    if (s - t)
    {
        attrib = g_strndup (t, s - t);
    }
    if (attrib)
    {
        for (i=0; i<GM_MYSQL_LAST_ATTRIBUTE;i++)
        {
            if (!strcmp (attrib, mysql_attributes[i]))
            {
                return i;
            }
        }
    }
    return GM_MYSQL_UNKNOWN_TYPE;
}



static gchar * field_parse_length (gchar * line)
{

    gchar * s;
    gchar * t;

    g_return_val_if_fail (line != NULL, 0);
    s = line;
    while (*s!='\0' && *s !=' ' && *s !='(')
        s++;
    if (*s != '(')
        return 0;
    s++;
    t = s;
    while (*s!='\0' && *s !=' ' && *s !=')')
        s++;

    if (*s != ')')
        return 0;
    if (s - t)
        return g_strndup (t, s - t);
    return NULL;
}


static GList * field_parse_enum (gchar * line)
{
    gchar * s;
    gchar * t;
    GList * enum_list = NULL;
    gchar **enum_values;
    gint i=0;

    g_return_val_if_fail (line != NULL, NULL);



    s = line;
    while (*s!='\0'  && *s !='(')
        s++;
    if (*s != '(')
        return NULL;
    s++;
    t = s;
    while (*s!='\0'  && *s !=')')
        s++;
    if (*s != ')')
        return NULL;

    if (s - t)
    {
        gchar *enum_line;
        enum_line = g_strndup (t, s - t);
        g_strdelimit (enum_line, "'",' ');
        enum_values = g_strsplit (enum_line, ",", 0);
        while (enum_values[i])
        {
            g_strstrip (enum_values[i]);
            enum_list = g_list_append (enum_list, g_strdup(enum_values[i]));
            i++;
        }
        g_free (enum_line);
        g_strfreev (enum_values);
        return enum_list;
    }
    return NULL;
}

DbKeyInfo * db_new_key_info (void)
{
    DbKeyInfo * key_info;
    key_info =g_new0 (DbKeyInfo, 1);
    key_info->name  = NULL;
    key_info->is_primary_key = FALSE;
    key_info->is_unique = FALSE;
    key_info->key_exists = FALSE;
    key_info->affected_column_list = NULL;
    return key_info;
}


void  db_free_key_info (DbKeyInfo * key_info)
{
    g_return_if_fail (key_info != NULL);
    g_free (key_info->name);
    g_list_g_free (key_info->affected_column_list);
    g_free (key_info);
}


static DbFieldInfo * db_new_field_info (void)
{
    DbFieldInfo * field_info  = NULL;

    field_info = g_new0(DbFieldInfo, 1);
    field_info->name = NULL;
    field_info->default_value = NULL;
    field_info->enum_list = NULL;
    field_info->extra = 0;
    field_info->type = 0;
    field_info->length = NULL;
    field_info->attributes = 0;
    field_info->not_null = TRUE;
    field_info->keys_list = NULL;
    field_info->drop_it = TRUE;
    field_info->is_unique = FALSE;
    field_info->is_index = FALSE;
    return field_info;
}


void db_free_field_info (DbFieldInfo * field_info)
{

    g_return_if_fail (field_info);

    g_free (field_info->name);
    g_free (field_info->default_value );
    g_free (field_info->length );
    g_list_g_free (field_info->enum_list);
    g_free (field_info);
}





GList * gm_db_get_keys_info (GmConnection * connection, gchar *table, gchar * database)
{

    DbQuery * keys_q;
    gchar * query = NULL;
    GList * list = NULL;


    query = g_strdup_printf ("show keys from %s ", table);
    keys_q = gm_db_new_query (query);
    keys_q->database = database;
    if (!gm_db_request(connection, keys_q))
    {
        gm_utils_write_log_message (connection,keys_q->error,  ERROR_LOGMODE);
        gm_db_free_query (keys_q);
        g_free (query);
        return NULL;
    }
    g_free (query);

    list = NULL;

    while(gm_db_next_record (keys_q))
    {
        gchar * column_name;
        DbKeyInfo * key_info = NULL;
        gchar * key_name;
        GList * l ;
        gboolean is_primary_key = FALSE;
        gboolean is_unique = FALSE;

        column_name = g_strdup(gm_db_f (keys_q, "Column_name"));
        key_name    = gm_db_f (keys_q, "Key_name");
        if (list)
        {
            l = g_list_first (list);
            while (l)
            {
                DbKeyInfo * k;
                k = l->data;
                if (!strcmp (k->name, key_name))
                {
                    key_info = k;
                    break;
                }
                l = l->next;
            }
        }
        if (!key_info)
        {
            key_info = db_new_key_info ();
            key_info->key_exists = TRUE;
            if (!strcmp ("PRIMARY", key_name))
            {
                key_info->name = g_strdup("PRIMARY");
                key_info->is_primary_key = is_primary_key = TRUE;
            }else
            {
                key_info->is_primary_key = is_primary_key = FALSE;
                key_info->name = g_strdup(key_name);
            }
            if (!strcmp ("0", gm_db_f (keys_q, "Non_unique")))
                key_info->is_unique = is_unique = TRUE;
            list = g_list_append (list, key_info);
        }
        key_info->affected_column_list = g_list_append (key_info->affected_column_list, column_name);
    }
    gm_db_free_query (keys_q);
    return list;
}

DbTableInfo *
gm_db_has_unique_keys (GmConnection *connection, gchar *database, gchar *table, gint *has_unique_keys)
{
    DbTableInfo *table_info;
    GList       *keys_info_list;
    gboolean yes = FALSE;


    table_info = gm_db_get_table_info (connection, table, database);
    keys_info_list = table_info->keys_info_list ;
    while (keys_info_list){
        DbKeyInfo * key_info = keys_info_list->data;
        if (key_info->is_unique || key_info->is_primary_key){
            yes = TRUE;
            break;
        }
        keys_info_list = keys_info_list->next;
    }
    *has_unique_keys = yes;
    return table_info;

}



GList * gm_db_get_fields_info (GmConnection * connection, gchar * database, gchar *table)
{

    DbQuery * q;
    gchar * query = NULL;
    GList * list = NULL;


    g_return_val_if_fail (connection != NULL || table != NULL || database != NULL, NULL);


    query = g_strdup_printf ("desc %s.%s ", database,table);
    q = gm_db_new_query (query);
    q->database = database;
    if (!gm_db_request(connection, q))
    {
        gm_utils_write_log_message (connection,q->error,  ERROR_LOGMODE);
        gm_db_free_query (q);
        g_free (query);
        return NULL;
    }
    g_free (query);
    while (gm_db_next_record (q))
    {
        gchar * type_length_atr;
        DbFieldInfo * field_info;

        field_info = db_new_field_info ();
        field_info->name = g_strdup (gm_db_f (q, "Field"));
        field_info->default_value = g_strdup (gm_db_f (q, "Default"));
        field_info->not_null = (!strcmp(gm_db_f (q, "Null"),"YES"))?FALSE:TRUE;
        field_info->extra = (!strcmp(gm_db_f (q, "Extra"),"auto_increment"))?1:0;
        field_info->is_unique = (!strcmp(gm_db_f (q, "Key"),"PRI")||!strcmp(gm_db_f (q, "Key"),"UNI"))?1:0;
        field_info->is_index  = (!strcmp(gm_db_f (q, "Key"),"MUL"))?1:0;
        type_length_atr  = gm_db_f (q, "Type");
        field_info->type = field_parse_type      (type_length_atr);
        if (field_info->type == GM_MYSQL_ENUM)
        {
            field_info->enum_list = field_parse_enum (type_length_atr);
        }else
        {
            field_info->length = field_parse_length    (type_length_atr);
        }
        field_info->attributes = field_parse_attribute (type_length_atr);
        list = g_list_append (list, (gpointer) field_info);
    }
    gm_db_free_query (q);

    return list;
}

DbTableInfo * db_new_table_info (void)
{

    DbTableInfo * table_info = NULL;
    table_info = g_new0(DbTableInfo, 1);
    table_info->name = NULL;
    table_info->database = NULL;
    table_info->fields_count = 0;
    table_info->comment = NULL;
    table_info->fields_info_list = NULL;
    table_info->keys_info_list = NULL;
    table_info->type = NULL;
    return table_info;
}

void gm_db_free_table_info (DbTableInfo * table_info)
{

    GList * list = NULL;
    g_free (table_info->name);
    g_free (table_info->database);
    g_free (table_info->comment);
    g_free (table_info->type);

    for (list = table_info->fields_info_list; list; list = list->next)
        db_free_field_info ((DbFieldInfo *) list->data);

    g_list_free (table_info->fields_info_list);

    for (list = table_info->keys_info_list; list; list = list->next)
        db_free_key_info ((DbKeyInfo *) list->data);

    g_list_free (table_info->keys_info_list);
    g_free (table_info);
}

DbTableInfo * gm_db_get_table_info (GmConnection * connection, gchar * table, gchar *database)
{

    DbQuery * q;
    DbTableInfo * table_info = NULL;
    gchar * query = NULL;
    g_return_val_if_fail (connection != NULL || table != NULL || database != NULL, NULL);


    query = g_strdup_printf("show table status from %s like '%s' ", database, table);
    q = gm_db_new_query (query);

    if (!gm_db_request(connection, q))
    {
        gm_utils_write_log_message (connection, q->error,  ERROR_LOGMODE);
        gm_db_free_query (q);
        g_free (query);
        return NULL;
    }
    table_info = db_new_table_info ();

    if  (gm_db_next_record (q))
    {
        table_info->comment = g_strdup (gm_db_f(q,"Comment"));
        table_info->type    = g_strdup (gm_db_f(q,"Type"));
    }
    gm_db_free_query (q);
    g_free (query);
    table_info->name             = g_strdup (table);
    table_info->database         = g_strdup (database);
    table_info->fields_info_list = gm_db_get_fields_info (connection, database, table);
    table_info->fields_count     = g_list_length (table_info->fields_info_list);
    table_info->keys_info_list   = gm_db_get_keys_info (connection, table, database);

    return table_info;
}





#if 0
static gm_dbapi_query_scanner (gchar * line, gboolean rest)
{
    gchar * s, * result = NULL;
    static gboolean quote_open = FALSE;
    static gboolean double_quote_open = FALSE;
    static gboolean escaped = FALSE;
    static GString *ret_string = NULL;

    if (rest){
        if (ret_string){
            result = ret_string->str;
            g_string_free (ret_string, FALSE);
            ret_string = NULL;
            return result;
        }else {
            return NULL;
        }
    }

    s = line;
    while (*s)
    {
        if (!escaped && *s == '\\')
        {
            /* ignore the next charachter */
            s++;
            escaped = TRUE;
        }else if (!escaped && *s == '\'')
        {
            quote_open = !quote_open;
        } else if (!quote_open && !escaped && *s == '"')
        {
            double_quote_open = !double_quote_open;
        } else if (!escaped && !quote_open  && !double_quote_open)
        {
            if (*s == ';')
            {
                if (!ret_string)
                    ret_string = g_string_new ("");
                g_string_append_len (ret_string, line, (s - line));
                result = ret_string->str;
                g_string_free (ret_string, FALSE);
                ret_string = NULL;
                s++;
                if (s) {
                    ret_string = g_string_new (s);
                }
                return result;
            }
        }
        escaped = FALSE;
        s++;
    }

    if (!ret_string)
        ret_string = g_string_new ("");
    g_string_append(ret_string, line);
    return NULL;
}
#endif

gboolean gm_db_dump_in (GmConnection *connection, gchar *databasename, gchar * filename)
{
    GmConnectionAccessData * access_data;

    GString * dump_command;
    gint ret;

    g_return_val_if_fail (databasename != NULL || filename != NULL, FALSE);
    access_data = connection->access_data;
    dump_command = g_string_new ("");
    g_string_sprintfa (dump_command,"mysql -u%s -p%s  %s < %s ", access_data->user, access_data->password, databasename, filename);
    gm_debug_message (GM_DBAPI_CHANEL, "%s ", dump_command->str);

    ret = system (dump_command->str);
    g_string_free (dump_command, TRUE);
    if (ret) {
        return FALSE;
    }
    return TRUE;
}





gchar * gm_db_get_table_def (GmConnection * connection ,gchar *table, gchar * db, gboolean drop_line)
{
    DbTableInfo * table_info = NULL;
    GString * schema_create;
    GList *l;
    gchar * query, * def;
    DbQuery * q;
    gint i;





    query = g_strdup_printf ("SHOW FIELDS FROM %s", table);
    q = gm_db_new_query (query);
    q->database = db;
    g_free (query);

    if (!gm_db_request (connection, q))
    {
        gm_utils_write_log_message (connection, q->error,  ERROR_LOGMODE);
        gm_db_free_query (q);
        return NULL;
    }

    schema_create = g_string_new ("");
    table_info = gm_db_get_table_info (connection , table, db);

    if (drop_line)
        g_string_sprintfa (schema_create, "DROP TABLE IF EXISTS %s ;\n", table);

    g_string_append (schema_create, "CREATE TABLE ");
    g_string_sprintfa (schema_create,"%s ( \n", table );
    i = 0;
    while (gm_db_next_record (q)){
        gchar * tmp;
        if (i)
            g_string_append (schema_create," ,\n");
        g_string_sprintfa (schema_create," %s ", gm_db_f(q,"Field"));
        g_string_sprintfa (schema_create," %s ", gm_db_f(q,"Type"));

        tmp = gm_db_f(q,"Default");
        if (tmp && strlen(tmp)>=1)
            g_string_sprintfa (schema_create," DEFAULT %s ", tmp);

        tmp = gm_db_f(q,"Null");
        if (tmp && strcmp(tmp, "YES"))
            g_string_append (schema_create," NOT NULL ");

        tmp = gm_db_f(q,"Extra");
        if (tmp && strlen(tmp)>=1)
            g_string_sprintfa (schema_create," %s", tmp);
        i++;
    }
    gm_db_free_query (q);

    l = table_info->keys_info_list;
    while (l) {
        DbKeyInfo * key_info ;
        GList * col_list ;

        key_info = (DbKeyInfo *) l->data;
        col_list = key_info->affected_column_list;

        if (key_info->is_primary_key)
            g_string_append (schema_create, ", PRIMARY KEY (");
        else if (key_info->is_unique)
            g_string_append (schema_create, ", UNIQUE (");
        else
            g_string_append (schema_create, ", KEY (");

        i = 0;
        while (col_list)
        {
            if (i)
                g_string_sprintfa (schema_create,",%s",(gchar*) col_list->data);
            else
                g_string_sprintfa (schema_create," %s ",(gchar *) col_list->data);
            col_list = col_list->next;
            i++;
        }
        l = l->next;
        g_string_append (schema_create, ") \n" );
    }
    g_string_append (schema_create, ")" );

    g_string_sprintfa (schema_create, " TYPE = %s ",table_info->type);
    if (table_info->comment)
        g_string_sprintfa (schema_create, " COMMENT = '%s' ",table_info->comment);

    gm_db_free_table_info (table_info);
    def = g_strdup (schema_create->str);
    g_string_free (schema_create, TRUE);
    return def;
}



gboolean gm_db_table_exists (GmConnection *connection, gchar *database, gchar *table)
{
    DbQuery * q;
    gchar * query;
    gboolean table_exist = FALSE;


    query = g_strdup_printf ("select * from %s.%s limit 1", database, table);
    q = gm_db_new_query (query);
    q->database = database;


    if (gm_db_request (connection, q))
    {
        table_exist = TRUE;
    }
    g_free (query);
    gm_db_free_query (q);

    return table_exist;
}


gboolean gm_db_clone_table (GmConnection * connection ,
                            gchar * src_database,
                            gchar * src_table,
                            gchar * target_database,
                            gchar * target_table,
                            gboolean structure_only)
{

    DbQuery * q;
    gchar * query;
    gboolean ret = TRUE;


    if (structure_only)
        query = g_strdup_printf ("CREATE TABLE %s SELECT * FROM %s.%s limit 1", target_table, src_database,src_table);
    else
        query = g_strdup_printf ("CREATE TABLE %s SELECT * FROM %s.%s", target_table, src_database,src_table);

    /*      table_def = gm_db_get_table_def (app, src_table, src_database, FALSE); */


    /*      g_free (table_def); */


    q = gm_db_new_query (query);
    q->database = target_database;

    if (!gm_db_request (connection, q))
    {
        gm_utils_write_log_message (connection, q->error,  ERROR_LOGMODE);
        ret  = FALSE;
    }


    g_free (query);
    gm_db_free_query (q);

    if (!ret)
        return ret;

    if (structure_only ) {
        query = g_strdup_printf ("DELETE FROM %s LIMIT 1", target_table);
        q = gm_db_new_query (query);
        q->database = target_database;
        if (!gm_db_request (connection, q))
        {
            gm_utils_write_log_message (connection, q->error,  ERROR_LOGMODE);
            ret  = FALSE;
        }

        g_free (query);
        gm_db_free_query (q);
    }

    return ret;
}



gboolean gm_db_create_database (GmConnection *connection, gchar * databasename)
{
    DbQuery * q;
    gchar * query;


    g_return_val_if_fail (databasename != NULL, FALSE);
    query = g_strdup_printf ("create database %s ", databasename);
    q = gm_db_new_query (query);
    if (!gm_db_request (connection, q))
    {
        gm_utils_write_log_message (connection,q->error,  ERROR_LOGMODE);
        g_free (query);
        gm_db_free_query (q);
        return FALSE;
    }
    gm_db_free_query (q);
    return TRUE;
}



gboolean gm_db_delete_database (GmConnection *connection, gchar * databasename)
{

    DbQuery * q;
    gchar * query;


    g_return_val_if_fail (databasename != NULL, FALSE);
    query = g_strdup_printf ("drop database %s ", databasename);
    q = gm_db_new_query (query);
    if (!gm_db_request (connection, q))
    {
        gm_utils_write_log_message (connection,q->error,  ERROR_LOGMODE);
        g_free (query);
        gm_db_free_query (q);
        return FALSE;
    }
    gm_db_free_query (q);
    return TRUE;
}

gboolean
gm_db_delete_table (GmConnection *connection, gchar * databasename, gchar * table)
{

    DbQuery * q;
    gchar * query;


    g_return_val_if_fail (databasename != NULL || table != NULL, FALSE);
    query = g_strdup_printf ("drop table %s.%s", databasename, table);
    q = gm_db_new_query (query);
    if (!gm_db_request (connection, q))
    {
        gm_utils_write_log_message (connection,q->error,  ERROR_LOGMODE);
        g_free (query);
        gm_db_free_query (q);
        return FALSE;
    }
    gm_db_free_query (q);
    return TRUE;
}

