#ifndef _GM_DBAPI_H_
#define _GM_DBAPI_H_
#include <mysql/mysql.h>
#include "gm-app.h"
#include "gm-enums.h"


#define DEFAULT_TIMEOUT "5"
#define GM_NULL_VALUE "Null"

typedef struct _DbQuery DbQuery;
typedef struct _DbTableInfo DbTableInfo;
typedef struct _DbFieldInfo DbFieldInfo;
typedef struct _DbKeyInfo DbKeyInfo;



struct _DbQuery {
    gchar     * query;
    MYSQL_RES * mysql_result;
    gint        fields_count;
    GSList    * fields_list;
    glong       affected_rows;
    gchar     * error;
    gint        error_number;
    gchar * database;		/* will not hold a copy  */
    MYSQL_ROW row;
};


struct _DbTableInfo {

    gchar * name;
    gchar * database;
    gint  fields_count;
    gchar * comment;
    gchar * type;
    GList * fields_info_list;
    GList * keys_info_list;
};

struct _DbFieldInfo {
    gchar    * name ;
    gchar    * default_value;
    gchar    *length ;
    GList    * enum_list;
    gint       extra;
    gint       type;
    gint       attributes;
    gboolean   not_null;
    gboolean drop_it;
    gboolean is_unique ;
    gboolean is_index ;
    GList * keys_list;
};


struct _DbKeyInfo {
    gchar * name  ;
    gboolean is_primary_key;
    gboolean is_unique;
    gboolean key_exists;
    GList * affected_column_list;
};



gboolean      gm_db_delete_table (GmConnection *connection, gchar * databasename, gchar * table);
gboolean      gm_db_delete_database (GmConnection *connection, gchar * databasename);
gboolean      gm_db_ping_host (GmConnectionAccessData *connection_access_data, gchar **error);
gboolean      gm_db_try_to_connect (gchar *hn, gint p, gchar *db, gchar *un, gchar *pw, gchar **error);
DbQuery     * gm_db_new_query  (gchar   * query);
void          gm_db_free_query (DbQuery * q);
gboolean      gm_db_request(GmConnection * connection, DbQuery * dbquery);
gchar       * gm_db_f (DbQuery *q, gchar * field_name);
GList       * gm_db_get_fields_info (GmConnection * connection, gchar * database, gchar *table);
DbTableInfo * gm_db_get_table_info (GmConnection * connection, gchar * table, gchar *database);
DbTableInfo * db_new_table_info (void);
void          gm_db_free_table_info (DbTableInfo * table_info);
gchar       * gm_db_get_mysql_type_string (gint i);
void          gm_db_free_key_info (DbKeyInfo * key_info);
GList       * gm_db_get_keys_info (GmConnection * connection, gchar *table, gchar * database);
DbKeyInfo   * db_new_key_info (void);
void          db_free_key_info (DbKeyInfo * key_info);
void          db_free_field_info (DbFieldInfo * field_info);
void          gm_db_change_database (GmConnection * connection, gchar * new_data_base);
gboolean      gm_db_create_database (GmConnection *connection, gchar * databasename);
gboolean      gm_db_dump_in (GmConnection *connection, gchar *databasename, gchar * filename);
gchar       * gm_db_get_table_def (GmConnection * connection ,gchar *table, gchar * db, gboolean drop_line);
void          gm_db_update_mini_buffer_completion_list (GmConnection * connection);
gboolean      gm_db_table_exists (GmConnection *connection, gchar *database, gchar *table) ;
DbTableInfo  *gm_db_has_unique_keys (GmConnection *connection, gchar *database, gchar *table, gint *has_unique_keys);

gboolean gm_db_clone_table (GmConnection * connection ,
                            gchar * src_database,
                            gchar * src_table,
                            gchar * target_database,
                            gchar * target_table,
                            gboolean structure_only);



#endif
