/*
 *GmDbtree: A database tree view 
 *
 *Original version by Daniel Lacroix (LACROIX@wanadoo.fr)
 *
 *Adapted to the Midnight Commander by Miguel.
 *
 *Adapted to gmyclient   by Mohamed.
 */


#include "gm.h"
#include "glade_interface.h"
#include "gm-dbtree.h"
#include "gm-bpanel.h"
#include "gm-interface.h"
#include "gm-mysql-admin.h"
#define TREE_SPACING 3
#define MAX_DBTREE_MENUS 18

static GtkCTreeClass *parent_class = NULL;

static gchar *open_folders_file[LAST_NODE_TYPE] = {
    "default",
    "host_node",
    "db_node",
    "table_node",
    "field_node",
    "key_node",
    "pkey_node",
    "default",
};


static gint dbtree_key_press                 (GtkWidget   *widget, GdkEventKey   *event);
static void dbtree_realize                   (GtkWidget   *widget);
static void dbtree_size_allocate             (GtkWidget   *widget, GtkAllocation *allocation);
static void dbtree_select_row                (GtkCTree    *ctree, GtkCTreeNode *row, gint column);
static DbTreeNodeData *dbtree_new_node_data (gint        type, gchar *name);
static gint dbtree_button_press              (GtkWidget   *widget, GdkEventButton *event);
static void dbtree_configure_popup_menu      (GmConnection *connection);
static gint dbtree_on_popup_menu_show        (GtkWidget *widget, GdkEventButton *event, GmConnection *connection) ;




static void
dbtree_select_row (GtkCTree *ctree, GtkCTreeNode *row, gint column)
{
    GmDbtree *dbtree = GM_DBTREE (ctree);
    parent_class->tree_select_row (ctree, row, column);
    dbtree = NULL;

}


static void
dbtree_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
{
    GmDbtree *dbtree = GM_DBTREE (widget);
    GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
    dbtree = NULL;
}



static void
dbtree_expand (GtkCTree *ctree, GtkCTreeNode *node)
{

    parent_class->tree_expand (ctree, node);
}

static void
dbtree_destroy (GtkObject *object)
{

    GmDbtree *dbtree = GM_DBTREE (object);
    (GTK_OBJECT_CLASS (parent_class))->destroy (object);
    dbtree = NULL;
}

static void
dbtree_class_init (GmDbtreeClass *klass)
{
    GtkObjectClass *object_class = (GtkObjectClass *) klass;
    GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
    GtkCTreeClass  *ctree_class  = (GtkCTreeClass *) klass;

    parent_class = gtk_type_class (GTK_TYPE_CTREE);

    object_class->destroy         = dbtree_destroy;
    ctree_class->tree_select_row  = dbtree_select_row;
    ctree_class->tree_expand      = dbtree_expand;

    widget_class->realize         = dbtree_realize;
    widget_class->size_allocate   = dbtree_size_allocate;
    widget_class->key_press_event = dbtree_key_press;
    widget_class->button_press_event = dbtree_button_press;
}

static void dbtree_init_pixmaps (GmDbtree *dbtree )
{
    gint i = 0;
    GtkWidget *pixmap;
    
    for (i=0; i<LAST_NODE_TYPE; i++) {
        pixmap = gm_stock_pixmap_widget (GTK_WIDGET (dbtree), open_folders_file[i]);
        if (!pixmap)
            pixmap = gm_stock_pixmap_widget (GTK_WIDGET (dbtree), "default");
        dbtree->gpixmap_open[i]= pixmap;
    }

}





static void
dbtree_init (GmDbtree *dbtree)
{
    DbTreeNodeData *ndata;

    dbtree->intern_call = FALSE;
    ndata = dbtree_new_node_data (ROOT_TYPE, NULL);
    dbtree->data_root_node = g_node_new (ndata);
    dbtree_init_pixmaps (dbtree);
}



void
gm_dbtree_construct (GmDbtree *dbtree)
{
    GtkCList *clist;
    GtkCTree *ctree;

    g_return_if_fail (dbtree != NULL);
    g_return_if_fail (GM_IS_DBTREE (dbtree));

    clist = GTK_CLIST (dbtree);
    ctree = GTK_CTREE (dbtree);
    gtk_ctree_construct (ctree, 1, 0, NULL);

    gtk_clist_set_selection_mode (clist, GTK_SELECTION_BROWSE);
    gtk_clist_set_auto_sort (clist, TRUE);
    gtk_clist_set_sort_type (clist, GTK_SORT_ASCENDING);
    gtk_clist_set_selection_mode (clist, GTK_SELECTION_BROWSE);
    gtk_clist_set_column_auto_resize (clist, 0, TRUE);
    gtk_clist_columns_autosize (clist);

    gtk_ctree_set_line_style (ctree, GTK_CTREE_LINES_DOTTED);
    gtk_clist_set_reorderable (GTK_CLIST (ctree), FALSE);


}



GtkWidget *
gm_dbtree_new (GmConnection *connection)
{
    GtkWidget *widget;


    widget = gtk_type_new (GM_TYPE_DBTREE);
    gtk_object_set_data (GTK_OBJECT (widget), "connection", connection);
    gm_dbtree_construct (GM_DBTREE (widget));
    connection->dbtree = widget;
    dbtree_configure_popup_menu (connection);
    return widget;
}

GtkType
gm_dbtree_get_type (void)
{
    static GtkType dbtree_type = 0;

    if (!dbtree_type)
    {
        GtkTypeInfo dbtree_info =
        {
            "GmDbtree",
            sizeof (GmDbtree),
            sizeof (GmDbtreeClass),
            (GtkClassInitFunc) dbtree_class_init,
            (GtkObjectInitFunc) dbtree_init,
                /*reserved_1 */ NULL,
                /*reserved_2 */ NULL,
            (GtkClassInitFunc) NULL,
        };

        dbtree_type = gtk_type_unique (GTK_TYPE_CTREE, &dbtree_info);
    }

    return dbtree_type;
}


static void
dbtree_go_to_table ( GtkWidget *widget,
                     GtkCTreeNode *work,
                     GmConnection * connection ,
                     gboolean expandit)
{

    DbTreeNodeData *ndata = NULL;
    gchar *database;
    gchar *table_name;

    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (widget), work);

    if (ndata && ndata->type == TABLE_TYPE) {
        GNode *node;
        DbTreeNodeData *pdata = NULL;
        node = g_node_find (GM_DBTREE (widget)->data_root_node, G_IN_ORDER, G_TRAVERSE_ALL, ndata);


        node = node->parent;
        pdata = node->data;

        table_name = ndata->name;
        database = pdata->name;
        gm_bpanel_go_to_table (database, table_name, connection) ;

    }else {
        if (expandit)
            gtk_ctree_toggle_expansion (GTK_CTREE (widget), work);
    }

}

#if 0
static
DbTreeNodeData *dbtree_get_selected_node_data (GmDbtree *dbtree )
{
    GtkCTreeNode *ctree_node;
    GList *list;
    GtkCList *clist;
    gint row;

    clist = GTK_CLIST (dbtree);

    list = clist->selection;

    if (!list)
        return NULL;

    row = GPOINTER_TO_INT (list->data);

    ctree_node = GTK_CTREE_NODE (g_list_nth (clist->row_list, row));
    g_return_val_if_fail (ctree_node != NULL, NULL);
    return gtk_ctree_node_get_row_data (GTK_CTREE (dbtree), ctree_node);
}
#endif



static gint dbtree_key_press       (GtkWidget        *widget,
                                    GdkEventKey      *event){

    GmDbtree *dbtree = GM_DBTREE (widget);
    GtkCList *clist;
    gint row;
    GList *list;


    clist = GTK_CLIST (widget) ;


    list = clist->selection;
    row = GPOINTER_TO_INT (list->data);

    if (GTK_WIDGET_CLASS (parent_class)->key_press_event &&
        GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event))
        return TRUE;
    switch (event->keyval)
    {
        case GDK_Return:
        {

            GtkCTreeNode *work;
            GmConnection *connection;

            connection = gm_utils_get_data (widget, "connection");
            row = 0;

            for (list = clist->row_list; list; list = list->next)
            {
                if (GTK_CLIST_ROW (list)->state == GTK_STATE_SELECTED){
                    work = GTK_CTREE_NODE (g_list_nth (clist->row_list, row));
                    dbtree_go_to_table (widget, work, connection, TRUE);
                    break;
                }
                row++;
            }





            break;
        }

        default:
            break;
    }
    dbtree = NULL;

    return FALSE;
}


static gint dbtree_button_press      (GtkWidget      *widget,
                                      GdkEventButton *event)
{
        /*GmDbtree *dbtree = GM_DBTREE (widget); */
    gboolean double_click;
    GmConnection *connection;
    GtkCList *clist;


    connection = gm_utils_get_data (widget, "connection");
    clist = GTK_CLIST (widget) ;

    double_click = (event->type == GDK_2BUTTON_PRESS);
    if (double_click){
        gint x, y;
        gint row, column;
        GtkCTreeNode *work;
        x = event->x;
        y = event->y;

        if (!gtk_clist_get_selection_info (clist, x, y, &row, &column))
            return FALSE;

        work = GTK_CTREE_NODE (g_list_nth (clist->row_list, row));

        dbtree_go_to_table (widget, work, connection, FALSE);
    }





    if (GTK_WIDGET_CLASS (parent_class)->button_press_event &&
        GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event))
        return TRUE;

    return FALSE;
}




static void
dbtree_realize (GtkWidget *widget)
{
    GmDbtree *dbtree ;



    g_return_if_fail (widget != NULL);
    dbtree =  GM_DBTREE (widget);

    GTK_WIDGET_CLASS (parent_class)->realize (widget);
}



static DbTreeNodeData *dbtree_new_node_data (gint type, gchar *name)
{
    DbTreeNodeData *ndata;

    ndata = g_new0 (DbTreeNodeData, 1);
    ndata->name = NULL;
    if (name)
        ndata->name = name;    
    ndata->type = type;
    ndata->tree_node = NULL;
    return ndata;
}











static void dbtree_build_data_nodes (GmConnection *connection, GNode *parent){

    DbTreeNodeData *parent_ndata;
    DbTreeNodeData *ndata;
    MYSQL_ROW row;
    DbQuery *q;
    gchar *query;


    parent_ndata = parent->data;
    g_return_if_fail (parent_ndata != NULL);
    switch (parent_ndata->type) {
        case HOST_TYPE:
        {
            query = g_strdup_printf ("show databases");
            q = gm_db_new_query (query);
            if (!gm_db_request (connection, q))
            {
                gm_utils_write_log_message (connection, q->error,  ERROR_LOGMODE);
                gm_db_free_query (q);
                g_free (query);
                return;
            }
            while ( (row = mysql_fetch_row (q->mysql_result)))
            {
                gchar *name;
                GNode *new_node;

                name     = g_strdup (row[0]);
                ndata    = dbtree_new_node_data (DATABASE_TYPE, name);
                new_node = g_node_append_data (parent, ndata);
                dbtree_build_data_nodes (connection, new_node);
            }
            g_free (query);
            gm_db_free_query (q);
            break;
        }
        case DATABASE_TYPE :
        {

            query = g_strdup_printf ("show tables");
            q = gm_db_new_query (query);
            q->database = parent_ndata->name;
            if (!gm_db_request (connection, q))
            {
                    /*in case of restricted use  ride of the warning messages */
                    //gm_utils_write_log_message (connection, q->error,  ERROR_LOGMODE);
                gm_db_free_query (q);
                g_free (query);
                return;
            }
            while ( (row = mysql_fetch_row (q->mysql_result)))
            {
                gchar *name;
                GNode *new_node;

                name     = g_strdup (row[0]);
                ndata    = dbtree_new_node_data (TABLE_TYPE, name);
                new_node = g_node_append_data (parent, ndata);
                if (GE_CONFIG (global_app)->show_fields)
                    dbtree_build_data_nodes (connection, new_node);
            }
            g_free (query);
            gm_db_free_query (q);
            break;
        }
        case TABLE_TYPE :
        {
            GList *l;
            DbTreeNodeData *db_tree_ndata;
            
            db_tree_ndata = (parent->parent)->data;
            l = gm_db_get_fields_info (connection, db_tree_ndata->name, parent_ndata->name);
            while (l)
            {
                gchar *name;
                GNode *new_node;
                DbFieldInfo *field_info;

                field_info = (DbFieldInfo *) l->data;                
                name     = g_strdup (field_info->name);
                if (field_info->is_unique)
                    ndata    = dbtree_new_node_data (PRIMARY_KEY_FIELD_TYPE, name);
                else if (field_info->is_index)
                    ndata    = dbtree_new_node_data (KEY_FIELD_TYPE, name);
                else 
                    ndata    = dbtree_new_node_data (FIELD_TYPE, name);

                new_node = g_node_append_data (parent, ndata);
                dbtree_build_data_nodes (connection, new_node);
                db_free_field_info (field_info);
                l = l->next;
            }
            break;
        }
    }

}




static void dbtree_insert_nodes (GNode *node, GmDbtree *dbtree){


    DbTreeNodeData *ndata = NULL;
    DbTreeNodeData *parent_ndata = NULL;
    GtkCTreeNode *parent_tree_node = NULL;
    gboolean expanded = FALSE;
    gboolean is_leaf = FALSE;
    GnomePixmap *gpixmap;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    gchar *name;


    ndata = node->data;

    if (node->parent) {
        parent_ndata = (node->parent)->data;
        parent_tree_node = parent_ndata->tree_node;
    }




    switch (ndata->type) {

    case HOST_TYPE:
        {
            expanded = TRUE;
            break;
        }

    case DATABASE_TYPE :
        {
            break;
        }

    case TABLE_TYPE :
        {
            break;
        }

    }
    gpixmap = GNOME_PIXMAP (dbtree->gpixmap_open[ndata->type]);
    pixmap = gpixmap->pixmap;
    mask   = gpixmap->mask;
    name = ndata->name;
    ndata->tree_node =  gtk_ctree_insert_node (GTK_CTREE (dbtree), parent_tree_node, NULL, &name, 5 , pixmap, mask, pixmap, mask, is_leaf, expanded);
    gtk_ctree_node_set_row_data (GTK_CTREE (dbtree), ndata->tree_node, ndata);
}


void
gm_dbtree_build_tree (GmConnection *connection){

    GNode *parent = NULL;
    DbTreeNodeData *ndata;
    GmConnectionAccessData *access_data;
    GmDbtree *dbtree;

    dbtree = GM_DBTREE (connection->dbtree);


    access_data = connection->access_data;

    /*Warning the tree is expected to be virgin */

    /*create the parent Node type host */
    ndata = dbtree_new_node_data (HOST_TYPE, g_strdup_printf ("%s@%s", access_data->user, access_data->host));

    parent = g_node_append_data (dbtree->data_root_node, ndata);


    /*build the tree in the memory as Node Struct */
    dbtree_build_data_nodes (connection, parent);

    /*insert them to the tree list */
    g_node_traverse (parent, G_PRE_ORDER, G_TRAVERSE_ALL, -1, (GNodeTraverseFunc) dbtree_insert_nodes, dbtree);
}

static gboolean
dbtree_free_node_data (GNode *node, GmDbtree *dbtree)
{


    DbTreeNodeData *ndata;

    ndata = node->data;
    if (ndata) {
        g_free (ndata->name);
        g_free (ndata);
    }
    return FALSE;
}


void
gm_dbtree_rebuild_tree (void *ignore, GmConnection *connection)
{

    DbTreeNodeData *ndata;
    GtkCTreeNode *parent_tree_node = NULL;
    GmDbtree *dbtree;

    dbtree = GM_DBTREE (connection->dbtree);
    ndata = (dbtree->data_root_node)->data;
    gtk_clist_freeze (GTK_CLIST (dbtree));
    parent_tree_node = ndata->tree_node;
    g_node_traverse (dbtree->data_root_node, G_PRE_ORDER, G_TRAVERSE_ALL, -1, (GNodeTraverseFunc) dbtree_free_node_data, dbtree);
    gtk_ctree_remove_node (GTK_CTREE (dbtree), parent_tree_node);
    ndata = dbtree_new_node_data (ROOT_TYPE, NULL);
    dbtree->data_root_node = g_node_new (ndata);
    gm_dbtree_build_tree (connection);
    gtk_clist_thaw (GTK_CLIST (dbtree));

}

/*popup menu */



static gint
dbtree_on_popup_menu_show (GtkWidget *widget, GdkEventButton *event, GmConnection *connection)
{
    gint x, y, i;
    gint row, column;
    GtkCTreeNode *selected_node;
    GtkCList *clist;
    DbTreeNodeData *ndata = NULL;
    GtkWidget *dbtree_popup_menu;
    GtkWidget *repair_table;
    GtkWidget *analyze_table;
    GtkWidget *check_table;
    GtkWidget *optimize_table;
    GtkWidget *edit_table;
    GtkWidget *empty_table;
    GtkWidget *delete_table;
    GtkWidget *new_table;
    GtkWidget *show_tables;
    GtkWidget *delete_database;
    GtkWidget *new_database;
    GtkWidget *host_status;
    GtkWidget *host_processlist;
    GtkWidget *host_variables;
    GtkWidget *host_flush;
    GtkWidget *dbtree_reload;
    GtkWidget *copy;
    GtkWidget *paste;
    GtkWidget *popup_menus[MAX_DBTREE_MENUS];
    GtkWidget *trennlinie[4];
    GmClipBoard * clipboard   = NULL;

    clipboard = global_app->clipboard;
    if (event->button != 3)
        return FALSE;

    clist = GTK_CLIST (widget);


    x = event->x;
    y = event->y;

    if (!gtk_clist_get_selection_info (clist, x, y, &row, &column))
        return FALSE;

    selected_node = GTK_CTREE_NODE (g_list_nth (clist->row_list, row));
    if (!selected_node)
        return FALSE;
    clist->focus_row = row;
    gtk_ctree_select (GTK_CTREE (widget), selected_node);

    gtk_object_set_data (GTK_OBJECT (widget), "selected_node", selected_node);


    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (widget), selected_node);
    if (!ndata)
        return FALSE;
    dbtree_popup_menu = gm_utils_get_data (connection->dbtree, "popup_menu");
    g_return_val_if_fail (dbtree_popup_menu != NULL, FALSE);


    popup_menus[0] = repair_table = gm_utils_get_data (dbtree_popup_menu, "repair_table");
    popup_menus[1] = analyze_table = gm_utils_get_data (dbtree_popup_menu, "analyze_table");
    popup_menus[2] = check_table = gm_utils_get_data (dbtree_popup_menu, "check_table");
    popup_menus[3] = optimize_table = gm_utils_get_data (dbtree_popup_menu, "optimize_table");
    popup_menus[4] = edit_table = gm_utils_get_data (dbtree_popup_menu, "edit_table");
    popup_menus[5] = delete_table = gm_utils_get_data (dbtree_popup_menu, "delete_table");
    popup_menus[6] = new_table = gm_utils_get_data (dbtree_popup_menu, "new_table");
    popup_menus[7] = show_tables = gm_utils_get_data (dbtree_popup_menu, "show_tables");
    popup_menus[8] = delete_database = gm_utils_get_data (dbtree_popup_menu, "delete_database");
    popup_menus[9] = new_database = gm_utils_get_data (dbtree_popup_menu, "new_database");
    popup_menus[10] = host_status = gm_utils_get_data (dbtree_popup_menu, "host_status");
    popup_menus[11] = host_processlist = gm_utils_get_data (dbtree_popup_menu, "host_processlist");
    popup_menus[12] = host_variables = gm_utils_get_data (dbtree_popup_menu, "host_variables");
    popup_menus[13] = host_flush = gm_utils_get_data (dbtree_popup_menu, "host_flush");
    popup_menus[14] = dbtree_reload = gm_utils_get_data (dbtree_popup_menu, "dbtree_reload");
    popup_menus[15] = empty_table = gm_utils_get_data (dbtree_popup_menu, "empty_table");
    popup_menus[16] = copy = gm_utils_get_data (dbtree_popup_menu, "copy");
    popup_menus[17] = paste = gm_utils_get_data (dbtree_popup_menu, "paste");

    trennlinie[0] =  gm_utils_get_data (dbtree_popup_menu, "trennlinie_reload_host");
    trennlinie[1] =  gm_utils_get_data (dbtree_popup_menu, "trennlinie_host_db");
    trennlinie[2] =  gm_utils_get_data (dbtree_popup_menu, "trennlinie_db_table");
    trennlinie[3] =  gm_utils_get_data (dbtree_popup_menu, "trennlinie_table_optimize");





    for (i=0 ; i<MAX_DBTREE_MENUS; i++)
        gtk_widget_show (popup_menus[i]);

    /* copy/paste will be implented in version 1.0 becaue we have to rewrite the clibboard comand center bad designed */
    gtk_widget_hide (copy);
    gtk_widget_hide (paste);

    for (i=0 ; i<4; i++)
        gtk_widget_show (trennlinie[i]);


    switch (ndata->type) {
    case HOST_TYPE:
        {
            gtk_widget_hide (copy);
            gtk_widget_hide (paste);
            gtk_widget_hide (delete_database);
            gtk_widget_hide (show_tables);
            gtk_widget_hide (new_table);
            gtk_widget_hide (empty_table);
            gtk_widget_hide (delete_table);
            gtk_widget_hide (edit_table);
            gtk_widget_hide (optimize_table);
            gtk_widget_hide (repair_table);
            gtk_widget_hide (check_table);
            gtk_widget_hide (analyze_table);
            gtk_widget_hide (trennlinie[2]);
            gtk_widget_hide (trennlinie[3]);
            break;
        }
    case DATABASE_TYPE:
        {
            gtk_widget_hide (copy);
            if (!clipboard->tables)
                gtk_widget_hide (paste);
            gtk_widget_hide (host_flush);
            gtk_widget_hide (host_variables);
            gtk_widget_hide (host_processlist);
            gtk_widget_hide (host_status);
            gtk_widget_hide (empty_table);
            gtk_widget_hide (delete_table);
            gtk_widget_hide (edit_table);
            gtk_widget_hide (optimize_table);
            gtk_widget_hide (repair_table);
            gtk_widget_hide (check_table);
            gtk_widget_hide (analyze_table);
            gtk_widget_hide (trennlinie[1]);
            gtk_widget_hide (trennlinie[3]);
            break;
        }
    case TABLE_TYPE :
        {
            if (!clipboard->tables)
                gtk_widget_hide (paste);
            gtk_widget_hide (host_flush);
            gtk_widget_hide (host_variables);
            gtk_widget_hide (host_processlist);
            gtk_widget_hide (host_status);
            gtk_widget_hide (delete_database);
            gtk_widget_hide (new_database);
            gtk_widget_hide (show_tables);
            gtk_widget_hide (trennlinie[1]);
            gtk_widget_hide (trennlinie[2]);
            break;
        }
    }
    return TRUE;
}


static GList *
dbtree_get_nodename_as_glist (GmConnection *connection, gchar ** database)
{
    DbTreeNodeData *ndata;
    DbTreeNodeData *pdata = NULL;
    GList *l = NULL;
    GtkCTreeNode *selected_node;
    GNode *node;



    selected_node = gm_utils_get_data (connection->dbtree, "selected_node");
    g_return_val_if_fail (selected_node != NULL, NULL);


    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (connection->dbtree), selected_node);

    g_return_val_if_fail (ndata != NULL || ndata->type != TABLE_TYPE, NULL);

    node = g_node_find (GM_DBTREE (connection->dbtree)->data_root_node, G_IN_ORDER, G_TRAVERSE_ALL, ndata);
    pdata = (DbTreeNodeData *) (node->parent)->data;
    *database = pdata->name;
    l = g_list_append (l, g_strdup (ndata->name));
    return l;
}


static void
dbtree_delete_database         (GtkMenuItem *menuitem, GmConnection *connection)
{
    DbTreeNodeData *ndata;
    GList *l = NULL;
    GtkCTreeNode *selected_node;






    selected_node = gm_utils_get_data (connection->dbtree, "selected_node");
    g_return_if_fail (selected_node != NULL);


    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (connection->dbtree), selected_node);
    g_return_if_fail (ndata != NULL || ndata->type != DATABASE_TYPE);

    l = g_list_append (l, g_strdup (ndata->name));
    gm_interface_delete_databases (connection, l);
    g_list_g_free (l);
}

static void
dbtree_delete_table         (GtkMenuItem *menuitem, GmConnection *connection)
{
    DbTreeNodeData *ndata;
    DbTreeNodeData *pdata = NULL;
    GList *l = NULL;
    GtkCTreeNode *selected_node;
    GNode *node;
    gchar * database;


    selected_node = gm_utils_get_data (connection->dbtree, "selected_node");
    g_return_if_fail (selected_node != NULL);


    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (connection->dbtree), selected_node);

    g_return_if_fail (ndata != NULL || ndata->type != TABLE_TYPE);
    node = g_node_find (GM_DBTREE (connection->dbtree)->data_root_node, G_IN_ORDER, G_TRAVERSE_ALL, ndata);
    pdata = (DbTreeNodeData *) (node->parent)->data;

    l = g_list_append (l, g_strdup (ndata->name));
    database = pdata->name;
    gm_interface_delete_tables (connection, database, l);
    g_list_g_free (l);
}


static void
dbtree_check_table         (GtkMenuItem *menuitem, GmConnection *connection)
{
    DbTreeNodeData *ndata;
    DbTreeNodeData *pdata = NULL;
    GList *l = NULL;
    GtkCTreeNode *selected_node;
    GNode *node;
    gchar * database;


    selected_node = gm_utils_get_data (connection->dbtree, "selected_node");
    g_return_if_fail (selected_node != NULL);


    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (connection->dbtree), selected_node);

    g_return_if_fail (ndata != NULL || ndata->type != TABLE_TYPE);
    node = g_node_find (GM_DBTREE (connection->dbtree)->data_root_node, G_IN_ORDER, G_TRAVERSE_ALL, ndata);
    pdata = (DbTreeNodeData *) (node->parent)->data;

    l = g_list_append (l, g_strdup (ndata->name));
    database = pdata->name;
    gm_mysql_admin_check_tables (connection, database, l);
    g_list_g_free (l);
}

static void
dbtree_repair_table         (GtkMenuItem *menuitem, GmConnection *connection)
{

    GList *l;
    gchar *database;
    l = dbtree_get_nodename_as_glist (connection, &database);
    gm_mysql_admin_repair_tables (connection, database, l);
    g_list_g_free (l);
}

static void
dbtree_optimize_table        (GtkMenuItem *menuitem, GmConnection *connection)
{

    GList *l;
    gchar *database;
    l = dbtree_get_nodename_as_glist (connection, &database);
    gm_mysql_admin_optimize_tables (connection, database, l);
    g_list_g_free (l);
}

static void
dbtree_analyze_table         (GtkMenuItem *menuitem, GmConnection *connection)
{

    GList *l;
    gchar *database;

    l = dbtree_get_nodename_as_glist (connection, &database);
    gm_mysql_admin_analyze_tables (connection, database, l);
    g_list_g_free (l);
}



static void
dbtree_truncate_table         (GtkMenuItem *menuitem, GmConnection *connection)
{

    GList *l;
    gchar *database;


    l = dbtree_get_nodename_as_glist (connection, &database);
    gm_mysql_admin_truncate_tables (connection, database, l);
    g_list_g_free (l);
}



static void
dbtree_new_table         (GtkMenuItem *menuitem, GmConnection *connection)
{
    DbTreeNodeData *ndata;
    DbTreeNodeData *pdata = NULL;
    GtkCTreeNode *selected_node;
    GNode *node;
    gchar * database;


    selected_node = gm_utils_get_data (connection->dbtree, "selected_node");
    g_return_if_fail (selected_node != NULL);


    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (connection->dbtree), selected_node);

    g_return_if_fail (ndata != NULL);
    database = ndata->name;
    if (ndata->type == TABLE_TYPE) {
        node = g_node_find (GM_DBTREE (connection->dbtree)->data_root_node, G_IN_ORDER, G_TRAVERSE_ALL, ndata);
        pdata = (DbTreeNodeData *) (node->parent)->data;
        database = pdata->name;
    }
    gm_interface_new_table_properties_window (connection, database);
}


static void
dbtree_edit_table         (GtkMenuItem *menuitem, GmConnection *connection)
{
    DbTreeNodeData *ndata;
    DbTreeNodeData *pdata = NULL;
    GtkCTreeNode *selected_node;
    GNode *node;
    gchar *database;
    gchar *table;


    selected_node = gm_utils_get_data (connection->dbtree, "selected_node");
    g_return_if_fail (selected_node != NULL);


    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (connection->dbtree), selected_node);

    g_return_if_fail (ndata != NULL);
    if (ndata->type == TABLE_TYPE) {
        node = g_node_find (GM_DBTREE (connection->dbtree)->data_root_node, G_IN_ORDER, G_TRAVERSE_ALL, ndata);
        table = ( (DbTreeNodeData *) (node->data))->name;        
        pdata = (DbTreeNodeData *) (node->parent)->data;
        database = pdata->name;
        gm_interface_edit_table (connection, database, table);
    }
}









static void
dbtree_show_tables         (GtkMenuItem *menuitem, GmConnection *connection)
{
    DbTreeNodeData *ndata;
    GtkCTreeNode *selected_node;


    selected_node = gm_utils_get_data (connection->dbtree, "selected_node");
    g_return_if_fail (selected_node != NULL);


    ndata = gtk_ctree_node_get_row_data (GTK_CTREE (connection->dbtree), selected_node);
    g_return_if_fail (ndata != NULL || ndata->type != DATABASE_TYPE);

    gm_interface_show_tables (connection, ndata->name);
    dbtree_expand (GTK_CTREE (connection->dbtree), selected_node);
}







static void
dbtree_configure_popup_menu (GmConnection *connection)
{

    GtkWidget *dbtree_popup_menu;
    GtkWidget *repair_table;
    GtkWidget *analyze_table;
    GtkWidget *check_table;
    GtkWidget *optimize_table;
    GtkWidget *edit_table;
    GtkWidget *delete_table;
    GtkWidget *empty_table;
    GtkWidget *new_table;
    GtkWidget *show_tables;
    GtkWidget *delete_database;
    GtkWidget *new_database;
    GtkWidget *host_status;
    GtkWidget *host_processlist;
    GtkWidget *host_variables;
    GtkWidget *host_flush;
    GtkWidget *dbtree_reload;
    GtkWidget *copy;
    GtkWidget *paste;
    GtkWidget *popup_menus[MAX_DBTREE_MENUS];

    gtk_signal_connect (GTK_OBJECT (connection->dbtree), "button_press_event", 
                        (GtkSignalFunc) dbtree_on_popup_menu_show, 
                        connection);

    dbtree_popup_menu = create_dbtree_popup_menu ();
    gnome_popup_menu_attach (dbtree_popup_menu, connection->dbtree, connection);
    gtk_object_set_data (GTK_OBJECT (connection->dbtree), "popup_menu", dbtree_popup_menu);


    popup_menus[0] = repair_table = gm_utils_get_data (dbtree_popup_menu, "repair_table");
    popup_menus[1] = analyze_table = gm_utils_get_data (dbtree_popup_menu, "analyze_table");
    popup_menus[2] = check_table = gm_utils_get_data (dbtree_popup_menu, "check_table");
    popup_menus[3] = optimize_table = gm_utils_get_data (dbtree_popup_menu, "optimize_table");
    popup_menus[4] = edit_table = gm_utils_get_data (dbtree_popup_menu, "edit_table");
    popup_menus[5] = delete_table = gm_utils_get_data (dbtree_popup_menu, "delete_table");
    popup_menus[6] = new_table = gm_utils_get_data (dbtree_popup_menu, "new_table");
    popup_menus[7] = show_tables = gm_utils_get_data (dbtree_popup_menu, "show_tables");
    popup_menus[8] = delete_database = gm_utils_get_data (dbtree_popup_menu, "delete_database");
    popup_menus[9] = new_database = gm_utils_get_data (dbtree_popup_menu, "new_database");
    popup_menus[10] = host_status = gm_utils_get_data (dbtree_popup_menu, "host_status");
    popup_menus[11] = host_processlist = gm_utils_get_data (dbtree_popup_menu, "host_processlist");
    popup_menus[12] = host_variables = gm_utils_get_data (dbtree_popup_menu, "host_variables");
    popup_menus[13] = host_flush = gm_utils_get_data (dbtree_popup_menu, "host_flush");
    popup_menus[14] = dbtree_reload = gm_utils_get_data (dbtree_popup_menu, "dbtree_reload");
    popup_menus[15] = empty_table = gm_utils_get_data (dbtree_popup_menu, "empty_table");
    popup_menus[16] = copy = gm_utils_get_data (dbtree_popup_menu, "copy");
    popup_menus[17] = paste = gm_utils_get_data (dbtree_popup_menu, "paste");

    gtk_signal_connect (GTK_OBJECT (host_variables), "activate", 
                        (GtkSignalFunc) gm_mysql_admin_show_variables , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (host_processlist), "activate", 
                        (GtkSignalFunc) gm_mysql_admin_show_processlist , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (host_status), "activate", 
                        (GtkSignalFunc) gm_mysql_admin_show_status , 
                        connection);
    gtk_signal_connect (GTK_OBJECT (host_flush), "activate", 
                        (GtkSignalFunc) gm_mysql_admin_flush , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (dbtree_reload), "activate", 
                        (GtkSignalFunc) gm_dbtree_rebuild_tree , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (new_database), "activate", 
                        (GtkSignalFunc) gm_interface_create_new_database , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (delete_database), "activate", 
                        (GtkSignalFunc) dbtree_delete_database , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (show_tables), "activate", 
                        (GtkSignalFunc) dbtree_show_tables, 
                        connection);

    gtk_signal_connect (GTK_OBJECT (delete_table), "activate", 
                        (GtkSignalFunc) dbtree_delete_table , 
                        connection);
    gtk_signal_connect (GTK_OBJECT (empty_table), "activate", 
                        (GtkSignalFunc) dbtree_truncate_table , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (new_table), "activate", 
                        (GtkSignalFunc) dbtree_new_table , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (edit_table), "activate", 
                        (GtkSignalFunc) dbtree_edit_table , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (check_table), "activate", 
                        (GtkSignalFunc) dbtree_check_table , 
                        connection);
    gtk_signal_connect (GTK_OBJECT (repair_table), "activate", 
                        (GtkSignalFunc) dbtree_repair_table , 
                        connection);

    gtk_signal_connect (GTK_OBJECT (optimize_table), "activate", 
                        (GtkSignalFunc) dbtree_optimize_table , 
                        connection);
    gtk_signal_connect (GTK_OBJECT (analyze_table), "activate", 
                        (GtkSignalFunc) dbtree_analyze_table , 
                        connection);
}




