/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gm.h"






static DebugChanel DEBUG_CHANEL[]={

    {ON,GM_BPANEL_CHANEL,"gm-bpanel"},
    {ON,GM_TABLE_PROPERTIES_CHANEL, "table properties "},
    {ON,GM_DBAPI_CHANEL, "gm-dbapi"},
    {ON,GM_QUERY_CHANEL, "gm-dbapi"},
    {ON,GM_UTILS_CHANEL, "gm-utils"},
    {ON,GM_DUMPOUT_CHANEL, "gm-dumpout"},
    {ON,GM_CLIST_CHANEL, "gm-clist"},
    {ON,GM_FORMULAR_CHANEL, "gm-formular"},
    {ON,GM_PRINTER_CHANEL, "gm-print"}
};


#if GM_DEBUG_ON
void
gm_debug_toggle_chanel (GtkCheckMenuItem *item, gpointer chanel_pointer)
{
    gint chanel;

    chanel = GPOINTER_TO_INT (chanel_pointer);
    DEBUG_CHANEL[chanel].status = item->active;
}
#endif


static void
debug_output_message (gint level, gint chanel, const char *fmt, va_list ap)
{
    gchar	buffer[1000];
    GString * debug_string;


    g_vsnprintf(buffer, sizeof(buffer), fmt, ap);	/* this is safe ? */
    debug_string = g_string_new("");


    switch (level){
    case MESSAGE_LEVEL:
        g_string_append (debug_string, "Message ");
        break;
    case WARNING_LEVEL:
        g_string_append (debug_string, "**Warning** ");
        break;
    }

    if (chanel > GM_LAST_CHANEL){
        return;
    }

    if (chanel>=0)
        g_string_sprintfa (debug_string,"Module[%s]: %s", DEBUG_CHANEL[chanel].chanel_description, buffer);
    else
        g_string_append (debug_string, buffer);

    g_print("%s\n",  debug_string->str);
    g_string_free (debug_string, TRUE);
    return;
}



void
gm_debug_message (gint chanel , const char *format, ...)
{
    va_list		ap;

    if (!GM_DEBUG_ON)
        return;


    if (chanel >=0){
        if (DEBUG_CHANEL[chanel].status == OFF)
            return;
    }
    va_start(ap, format);
    debug_output_message (MESSAGE_LEVEL, chanel, format, ap);
    va_end(ap);
}


void
gdm (const char *format, ...)
{
    va_list		ap;
    if (!GM_DEBUG_ON)
        return;
    va_start(ap, format);
    debug_output_message (MESSAGE_LEVEL, -1, format, ap);
    va_end(ap);
}



void
gdw (const char *format, ...)
{
    va_list		ap;

    if (!GM_DEBUG_ON)
        return;
    va_start(ap, format);
    debug_output_message (WARNING_LEVEL, -1, format, ap);
    va_end(ap);
}




void
gm_debug_warning (gint chanel, const char *format, ...)
{
    va_list		ap;



    va_start(ap, format);
    debug_output_message (WARNING_LEVEL, chanel, format, ap);
    va_end(ap);
}






