#include <math.h>
#include "gm.h"
#include "gm-dialogs.h"
#include "glade_interface.h"



#define SCALE(i) (gdouble)(i / 65535.)
#define UNSCALE(d) ((guint16)(d * 65535 + 0.5))

enum
{
    FLUSH_PAGE=0,
    CHECK_PAGE,
    REPAIR_PAGE
};



static void
dialogs_connection_user_entry_changed                      (GtkEditable     *editable,
        GtkWidget       *connection_dialog)
{

    GList * l;
    GmUserPass * user_pass;
    GList * user_list;
    GtkEntry * pass_entry;
    GtkCList * clist;
    gchar * user;
    gboolean found = FALSE;

    clist = GTK_CLIST (gm_utils_get_data (connection_dialog,"host_list"));

    if (!clist->selection) {
        gm_utils_warning (_("Please select a host, or add a new one, before you type in your login "));
    }
    user_list  = gm_utils_get_data_ww (editable, "user_list");
    pass_entry = gm_utils_get_data (connection_dialog, "password_entry");

    user = gtk_entry_get_text (GTK_ENTRY(editable));
    if (user && strlen(user)) {
        for (l = user_list; l; l = l->next) {
            user_pass = l->data;
            if (user_pass->pass && !strcmp (user_pass->user, user)) {
                gtk_entry_set_text (pass_entry, user_pass->pass);
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gm_utils_get_data (connection_dialog, "save_password")), TRUE);
                found = TRUE;
                break;
            }
        }
        if (!found){
            gtk_editable_delete_text (GTK_EDITABLE (pass_entry), 0, -1);
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gm_utils_get_data (connection_dialog, "save_password")), FALSE);
        }


    }


}



static void
dialogs_connection_hosts_clist_select_row                   (GtkCList        *clist,
        gint             row,
        gint             column,
        GdkEvent        *event,
        GtkWidget       *connection_dialog)
{

    GList * l;
    GmUserPass * user_pass;
    GmHostPort * host_port;
    GtkCombo * user_combo;
    GtkEditable * editable, * user_entry;
    GList * users = NULL;

    user_combo = GTK_COMBO (gm_utils_get_data (connection_dialog, "user_combo"));



    editable = GTK_EDITABLE (gm_utils_get_data (connection_dialog,"password_entry"));
    gtk_editable_delete_text(editable, 0, -1);
    user_entry = GTK_EDITABLE (gm_utils_get_data (connection_dialog,"user_entry"));
    gtk_editable_delete_text(user_entry, 0, -1);


    host_port = gtk_clist_get_row_data (clist, row);
    g_assert (host_port);
    l = host_port->user_list;



    for (l = host_port->user_list; l; l = l->next) {
        user_pass = l->data;
        users = g_list_append (users, user_pass->user);
    }


    gtk_object_set_data (GTK_OBJECT (user_entry), "user_list", host_port->user_list);
    if (users)
        gtk_combo_set_popdown_strings (user_combo, users);
    else
        gtk_list_clear_items (GTK_LIST (user_combo->list), 0, -1);

    if (host_port->last_user)
        gtk_entry_set_text (GTK_ENTRY (user_entry), host_port->last_user);


}









static gboolean
dialogs_delete_event           (GtkWidget       *widget,
                                GdkEvent        *event,
                                gint *canceld)
{


    gtk_widget_hide (gtk_widget_get_toplevel (widget));
    gtk_main_quit();
    *canceld = 1;
    return TRUE;
}


static void dialogs_connection_cancel_button_clicked ( void * ignore, gint * canceld)
{
    *canceld = 1;
    gtk_main_quit ();
}


static void dialogs_connection_add_host_button_clicked ( void * ignore, GtkWidget *connection_dialog)
{

    gchar * host;
    gchar * port;
    GtkEntry * entry;
    gchar * clist_row[2];
    GtkCList * clist;

    clist = GTK_CLIST (gm_utils_get_data (connection_dialog,"host_list"));

    entry = GTK_ENTRY (gm_utils_get_data (connection_dialog,"host_entry"));
    host  = gtk_entry_get_text(entry);

    entry = GTK_ENTRY (gm_utils_get_data (connection_dialog,"port_entry"));
    port = gtk_entry_get_text(entry);

    if (host && strlen(host)) {
        GmHostPort * host_port;
        GmCcenterGeneral * ge_config;
        GmCcenter * cc;


        cc = (GM_CCENTER (global_app->cc));
        ge_config = cc->ge_config;

        host_port = g_new (GmHostPort, 1);
        host_port->host = g_strdup (host) ;
        host_port->port = (port && strlen(port))?atoi(gtk_entry_get_text(entry)):0;
        host_port->user_list = NULL;
        host_port->last_user = NULL;

        clist_row[0] = host;
        clist_row[1] = port;
        gtk_clist_set_row_data (clist, gtk_clist_prepend (clist, clist_row), host_port);
        gtk_clist_select_row (clist, 0,0);
        ge_config->host_list = g_list_append (ge_config->host_list, host_port);
        gtk_editable_delete_text (GTK_EDITABLE(gm_utils_get_data (connection_dialog,"port_entry")), 0, -1);
        gtk_editable_delete_text (GTK_EDITABLE(gm_utils_get_data (connection_dialog,"host_entry")), 0, -1);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gm_utils_get_data (connection_dialog, "save_password")), FALSE);
    }
}

static void dialogs_connection_remove_host_button_clicked (void * ignore, GtkWidget *connection_dialog)
{

    GtkCList * clist;
    GList * selection = NULL;
    gint row;
    GmHostPort * host_port;
    GmCcenterGeneral * ge_config;
    GmCcenter * cc;
    GtkCombo * user_combo;
    gchar * host_section;




    cc = (GM_CCENTER (global_app->cc));
    ge_config = cc->ge_config;

    clist = GTK_CLIST (gm_utils_get_data (connection_dialog,"host_list"));
    selection = clist->selection;
    if (!selection)
        return;
    row = GPOINTER_TO_INT (selection->data);
    host_port = gtk_clist_get_row_data (clist, row);
    gtk_clist_remove (clist, row);
    g_free(host_port->host);

    /* free the user_combo */


    user_combo = GTK_COMBO (gm_utils_get_data (connection_dialog, "user_combo"));
    gtk_list_clear_items (GTK_LIST (user_combo->list), 0, -1);

    /* free the user list (user / password) */



    if (host_port->user_list) {
        GList * l;
        GmUserPass * user_pass;
        for (l = host_port->user_list; l; l = l->next) {
            user_pass = l->data;
            g_free (user_pass->user);
            g_free (user_pass->pass);
        }
        g_list_free (host_port->user_list);
    }


    host_section = g_strdup_printf ("/gmyclient/%s_%d", host_port->host, host_port->port);
    gnome_config_clean_section (host_section);
    g_free (host_section);
    gnome_config_sync();

    ge_config->host_list = g_list_remove (ge_config->host_list, host_port);

    if (row - 1 >= 0 )
        gtk_clist_select_row (clist, row -1 ,0);
    else
        gtk_clist_select_row (clist, 0,0);
    g_free (host_port);
}



static void dialogs_connection_ok_button_clicked (void *ignore, GmConnectionAccessData *connection_access_data)
{

    gchar * user;
    gchar * host;
    gchar * password;
    gchar * database;
    gint    port ;
    GtkEntry * entry;
    gchar *error ;
    GtkWidget * connection_dialog;
    GtkCList  * clist;
    GList * selection;
    GmHostPort * host_port;
    gboolean save_password = FALSE;
    GmUserPass * user_pass = NULL;
    gboolean user_exist = FALSE;
    GmCcenterGeneral * ge_config;
    GmCcenter * cc;
    GList * host_list;


    cc = (GM_CCENTER (global_app->cc));
    ge_config = cc->ge_config;


    connection_dialog = global_app->connection_dialog;


    entry = GTK_ENTRY (gm_utils_get_data (connection_dialog,"user_entry"));
    user = gtk_entry_get_text(entry);

    entry = GTK_ENTRY (gm_utils_get_data (connection_dialog,"password_entry"));
    password = gtk_entry_get_text(entry);

    entry = GTK_ENTRY (gm_utils_get_data (connection_dialog,"database_entry"));
    database = gtk_entry_get_text(entry);

    clist = GTK_CLIST (gm_utils_get_data (connection_dialog, "host_list"));
    // port =(gtk_entry_get_text(entry))?atoi(gtk_entry_get_text(entry)):0;
    selection = clist->selection;

    if (!selection) {
        gm_utils_warning ("Please select the host you want to connect to ");
        return ;
    }

    host_port = gtk_clist_get_row_data(clist, GPOINTER_TO_INT(selection->data));
    g_assert (host_port != NULL);






    host = host_port->host;
    port = host_port->port;

    if (!user || strlen(user)<1)
    {
        gm_utils_warning ("please provide your user and password to connect to the database");
        return ;
    }

    if (!gm_db_try_to_connect (host, port, database, user, password, &error))
    {
        gchar * message;
        message = g_strdup_printf ("Could not connect to the database \n Mysql said :%s\n", error);
        gm_utils_warning (message);
        g_free (error);
        g_free (message);
        return ;
    }

    g_free(connection_access_data->host);
    if (host && strlen(host)>1 )
        connection_access_data->host=g_strdup(host);
    else
        connection_access_data->host=g_strdup("localhost");

    g_free(connection_access_data->database);

    if (database && strlen(database))
        connection_access_data->database = g_strdup(database);
    else
        connection_access_data->database = NULL;

    g_free(connection_access_data->user);

    if (user && strlen (user))
        connection_access_data->user = g_strdup (user);
    else
        connection_access_data->user =  NULL;

    g_free(connection_access_data->password);

    if (password && strlen (password))
        connection_access_data->password=g_strdup(password);
    else
        connection_access_data->password = NULL;
    connection_access_data->port = port;



    /* change the order of the selected host in the list*/
    host_list = ge_config->host_list;
    if (host_list && g_list_index (host_list, host_port)) {
        ge_config->host_list = g_list_remove (ge_config->host_list, host_port);
        ge_config->host_list = g_list_prepend (ge_config->host_list, host_port);
    }


    save_password = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (gm_utils_get_data (connection_dialog, "save_password")));

    if (host_port->user_list) {
        GList * user_list;
        for (user_list = host_port->user_list;user_list; user_list=user_list->next) {
            user_pass = user_list->data;
            if (user_pass->user && !strcmp (user_pass->user, user)){
                if (save_password ) {
                    if (user_pass->pass)
                        g_free (user_pass->pass);
                    if (password)
                        user_pass->pass = g_strdup (password);
                    else
                        user_pass->pass = NULL;
                }else {
                    g_free (user_pass->pass);
                    user_pass->pass = NULL;
                }
                user_exist = TRUE;
                break;
            }
        }
    }

    if (!user_exist) {
        user_pass = g_new(GmUserPass, 1);
        user_pass->user = g_strdup (user);
        if (save_password)
            user_pass->pass = g_strdup (password);
        else
            user_pass->pass = NULL;
        host_port->user_list = g_list_append (host_port->user_list, user_pass);
    }
    host_port->last_user = user_pass->user;

    gtk_widget_hide (connection_dialog);
    gtk_main_quit();
}




gboolean gm_dialogs_connection (GmConnectionAccessData *access_data)
{

    GtkWidget * connection_dialog;
    GtkWidget * button ;
    gint canceld = 0;
    GmCcenterGeneral * ge_config;
    GmCcenter * cc;
    GtkCList * clist;
    GtkWidget * entry;



    cc = (GM_CCENTER (global_app->cc));
    ge_config = cc->ge_config;



    global_app->connection_dialog =  connection_dialog = create_connection_dialog ();


    button = gm_utils_get_data (connection_dialog, "ok_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (dialogs_connection_ok_button_clicked),
                        access_data);


    button = gm_utils_get_data (connection_dialog, "cancel_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (dialogs_connection_cancel_button_clicked),
                        &canceld);

    gtk_signal_connect (GTK_OBJECT (connection_dialog), "delete_event",
                        GTK_SIGNAL_FUNC (dialogs_delete_event),
                        &canceld);



    button = gm_utils_get_data (connection_dialog, "add_host_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (dialogs_connection_add_host_button_clicked),
                        connection_dialog);


    button = gm_utils_get_data (connection_dialog, "remove_host_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (dialogs_connection_remove_host_button_clicked),
                        connection_dialog);

    clist = GTK_CLIST (gm_utils_get_data (connection_dialog,"host_list"));
    gtk_signal_connect (GTK_OBJECT (clist), "select_row",
                        GTK_SIGNAL_FUNC (dialogs_connection_hosts_clist_select_row),
                        connection_dialog);
    entry = gm_utils_get_data (connection_dialog, "user_entry");
    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (dialogs_connection_user_entry_changed),
                        connection_dialog);

    entry = gm_utils_get_data (connection_dialog, "password_entry");
    gtk_signal_connect (GTK_OBJECT (entry), "activate",
                        GTK_SIGNAL_FUNC (dialogs_connection_ok_button_clicked),
                        connection_dialog);

    entry = gm_utils_get_data (connection_dialog, "port_entry");
    gtk_signal_connect (GTK_OBJECT (entry), "activate",
                        GTK_SIGNAL_FUNC (dialogs_connection_ok_button_clicked),
                        connection_dialog);

    entry = gm_utils_get_data (connection_dialog, "host_entry");
    gtk_signal_connect (GTK_OBJECT (entry), "activate",
                        GTK_SIGNAL_FUNC (dialogs_connection_add_host_button_clicked),
                        connection_dialog);




    if (ge_config->host_list) {
        GList * list;
        list = ge_config->host_list;
        while (list) {
            gchar * clist_vector[2];
            gchar * port;

            GmHostPort * host_port;
            host_port = list->data;
            if (host_port->port)
                port = g_strdup_printf ("%d", host_port->port);
            else
                port = g_strdup("");

            clist_vector[1] = port;
            clist_vector[0] = host_port->host;
            gtk_clist_set_row_data (clist, gtk_clist_append (clist, clist_vector), host_port);
            g_free (port);
            list = list->next;
        }
        gtk_clist_select_row (clist, 0,0);
    }





    gtk_widget_show (connection_dialog);
    gtk_main();
    button = gm_utils_get_data (connection_dialog, "ok_button");
    gtk_signal_disconnect_by_func (GTK_OBJECT (button), dialogs_connection_ok_button_clicked, access_data);
    button = gm_utils_get_data (connection_dialog, "cancel_button");
    gtk_signal_disconnect_by_func (GTK_OBJECT (button), dialogs_connection_cancel_button_clicked,&canceld);
    gtk_signal_disconnect_by_func (GTK_OBJECT (connection_dialog), GTK_SIGNAL_FUNC (dialogs_delete_event), &canceld);

    gtk_widget_destroy (connection_dialog);

    if (!canceld) {
        return TRUE;
    }
    return FALSE;
}




#if 0
gchar * gm_dialogs_save_file (GmApp * app, gchar * path) ;




void
dialogs_filesel_ok_button_clicked           (GtkButton       *button,
        gpointer         user_data)
{
    GtkWidget *filesel, *connection_window;
    gchar *filename;

    filesel = gtk_widget_get_toplevel (GTK_WIDGET (button));
    connection_window = gm_utils_get_data (GTK_OBJECT (filesel), MainWindowKey);
    gtk_widget_hide (filesel);
    filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
    real_open_file (connection_window, filename);
}


void
dialogs_filesel_cancel_button_clicked       (GtkButton       *button,
        gpointer         user_data)
{
    gtk_widget_hide (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}



gboolean
dialogs_filesel_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{


    gtk_widget_hide (gtk_widget_get_toplevel (widget));
    gtk_main_quit();
    return TRUE;
}


GtkWidget*
dialogs_create_file_selection (gchar *title)
{
    GtkWidget *file_selection;
    GtkWidget *ok_button;
    GtkWidget *cancel_button;
    gint canceld = 0;


    file_selection = gtk_file_selection_new (title);
    gtk_widget_ref (file_selection);
    gtk_object_set_data_full (GTK_OBJECT (file_selection), "file_selection", file_selection,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_container_set_border_width (GTK_CONTAINER (file_selection), 10);
    gtk_window_set_position (GTK_WINDOW (file_selection), GTK_WIN_POS_MOUSE);

    ok_button = GTK_FILE_SELECTION (file_selection)->ok_button;
    gtk_widget_ref (ok_button);
    gtk_object_set_data_full (GTK_OBJECT (file_selection), "ok_button", ok_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (ok_button);
    GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);

    cancel_button = GTK_FILE_SELECTION (file_selection)->cancel_button;
    gtk_widget_ref (cancel_button);
    gtk_object_set_data_full (GTK_OBJECT (file_selection), "cancel_button", cancel_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (cancel_button);
    GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);

    gtk_signal_connect (GTK_OBJECT (file_selection), "delete_event",
                        GTK_SIGNAL_FUNC (dialogs_filesel_delete_event),
                        &canceld);
    gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                        GTK_SIGNAL_FUNC (dialogs_filesel_ok_button_clicked),
                        NULL);
    gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
                        GTK_SIGNAL_FUNC (dialogs_filesel_cancel_button_clicked),
                        NULL);

    return file_selection;
}





gchar * gm_dialogs_open_file (GmApp * app, gchar * path)
{
    gchar * title = _("Open File");
    g_return_val_if_fail (app != NULL);

    if (!app->open_filesel)
        app->open_filesel = dialogs_create_file_selection (title);

    gtk_widget_show  (app->open_filesel);
    gdk_window_raise (ap->open_filesel->window);
    gtk_main ();
}
#endif 


/* GtkColorSelection */


typedef struct
{
    gboolean canceled;
    GtkWidget * label;

}ColorSelectionDialogData;

static double
scale_round (double val, double factor)
{
    val = floor (val * factor + 0.5);
    val = MAX (val, 0);
    val = MIN (val, factor);
    return val;
}


static gchar *
gm_dialog_color_selection_get_hex_color (GtkColorSelection *col_sel)
{
    gdouble   *color;
    gchar hex_string[12];
    gchar *ptr;


    color  = g_new (double, 4);

    gtk_color_selection_get_color (col_sel, color);
    g_snprintf (hex_string, 11, "#%2X%2X%2X",
                (guint) (scale_round (color[0], 255)),
                (guint) (scale_round (color[1], 255)),
                (guint) (scale_round (color[2], 255)));

    for (ptr = hex_string; *ptr; ptr++)
        if (*ptr == ' ')
            *ptr = '0';
    return g_strdup (hex_string);
}


static void
gm_dialogs_color_selection_color_changed (GtkWidget * col_sel, ColorSelectionDialogData *data)
{

    gchar * color;


    color = gm_dialog_color_selection_get_hex_color (GTK_COLOR_SELECTION(col_sel));
    gtk_label_set_text (GTK_LABEL (data->label), color);
    g_free (color);
}




static void
gm_dialogs_color_selection_canceled (void * ignore,ColorSelectionDialogData *data )
{
    data->canceled = TRUE;
    gtk_main_quit();
}

/***
 * return the selected color in hexformat , you must free the returned string 
 * return NULL if the dialog is canceled 
 *
 */


gchar *
gm_dialogs_show_color_selection_dialog (gchar *color_name)
{
    GtkWidget *dialog ;
    GtkWidget *ok_button;
    GtkWidget *cancel_button;
    GtkColorSelectionDialog *col_sel_dialog;
    gdouble   *color;
    GtkWidget * label ;
    gchar * hex_color;



    GdkColor gdk_color;
    ColorSelectionDialogData  data;

    data.canceled = FALSE;
    color  = g_new (double, 4);
    dialog = create_colorselectiondialog ();
    col_sel_dialog = GTK_COLOR_SELECTION_DIALOG (dialog);

    if (color_name)
        label = gtk_label_new(color_name);
    else
        label = gtk_label_new("");
    data.label = label;
    gtk_widget_show (label);
    gtk_box_pack_end (GTK_BOX (col_sel_dialog->main_vbox), label, FALSE, FALSE, 0);
    if (color_name) {
        if (gdk_color_parse (color_name, &gdk_color))
        {
            color[0] = SCALE(gdk_color.red);
            color[1] = SCALE(gdk_color.green);
            color[2] = SCALE(gdk_color.blue);
            color[3] = 0.0;
            gtk_color_selection_set_color (GTK_COLOR_SELECTION (col_sel_dialog->colorsel), color);
        }
    }
    g_free (color);
    ok_button = gm_utils_get_data (dialog, "ok_button");
    cancel_button = gm_utils_get_data (dialog, "cancel_button");

    gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
                        GTK_SIGNAL_FUNC (gm_dialogs_color_selection_canceled),
                        &data);

    gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        &data);

    gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
                        GTK_SIGNAL_FUNC (gm_dialogs_color_selection_canceled),
                        &data);

    gtk_signal_connect (GTK_OBJECT (col_sel_dialog->colorsel), "color_changed",
                        GTK_SIGNAL_FUNC (gm_dialogs_color_selection_color_changed),
                        &data);


    gtk_widget_show (dialog);

    gtk_main ();

    if (!data.canceled) {
        hex_color = gm_dialog_color_selection_get_hex_color(GTK_COLOR_SELECTION (col_sel_dialog->colorsel));
        gtk_widget_destroy (dialog);
        return hex_color;
    }else {
        gtk_widget_destroy (dialog);
        return NULL;
    }
}



/* new database */


static void dialogs_new_database_ok_button_clicked               (GtkButton  *button,
        gint *answer)

{

    *answer = 1;
    gtk_main_quit ();
}

static void dialogs_new_database_cancel_button_clicked               (GtkButton  *button,
        gint *answer)

{
    *answer = 0;
    gtk_main_quit ();
}


static gint dialogs_new_database_dialog_key_pressed     (GtkWidget       *widget,
        GdkEventKey     *event,
        gint * answer)
{


    if  (event->keyval == GDK_Escape) {
        *answer = 0;
        gtk_main_quit ();
        return TRUE;
    }
    return FALSE;

}



gchar *
gm_dialogs_new_database (gchar * old_name)

{
    GtkWidget * new_db_dialog;
    GtkWidget * button;
    GtkWidget * database_name_entry;
    gchar * database_name = NULL;

    gint answer = 0;
    new_db_dialog = create_new_db_dialog ();

    gtk_signal_connect (GTK_OBJECT (new_db_dialog), "key_press_event",
                        GTK_SIGNAL_FUNC (dialogs_new_database_dialog_key_pressed),
                        &answer);

    gtk_widget_show (new_db_dialog);

    button = gm_utils_get_data (GTK_OBJECT(new_db_dialog),"ok_button");

    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (dialogs_new_database_ok_button_clicked),
                        &answer);

    button = gm_utils_get_data (GTK_OBJECT(new_db_dialog),"cancel_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (dialogs_new_database_cancel_button_clicked),
                        &answer);

    database_name_entry = gm_utils_get_data (GTK_OBJECT (new_db_dialog),"database_name_entry");
    if (old_name)
        gtk_entry_set_text (GTK_ENTRY (database_name_entry), old_name);

    gtk_signal_connect (GTK_OBJECT (database_name_entry), "activate",
                        GTK_SIGNAL_FUNC (dialogs_new_database_ok_button_clicked),
                        &answer);

    gtk_main();
    database_name = gtk_entry_get_text (GTK_ENTRY (database_name_entry));
    if (answer)
    {
        if (database_name && strlen(database_name)>1)
        {
            database_name = g_strdup (database_name);
        }else
            database_name = NULL;
        gtk_widget_destroy (new_db_dialog);
        return database_name;
    }
    else
    {
        gtk_widget_destroy (new_db_dialog);
        return NULL;
    }

}


/* flush_options_dialog */

static gboolean
dialogs_flush_options_deleted           (GtkWidget       *widget,
        GdkEvent        *event,
        gint *canceld)
{
    gtk_main_quit();
    *canceld = 1;
    return TRUE;
}


static void
dialogs_flush_options_canceled (void * ignore, gint *canceld)
{
    *canceld = 1;
    gtk_main_quit();
}

static void
dialogs_flush_options_ok_clicked (void * ignore, gint *canceld)
{
    *canceld = 0;
    gtk_main_quit();
}


static gboolean
gm_dialogs_show_fcr_dialog (gint page)
{

    GtkWidget * dialog;
    gint canceled = 0;
    GtkWidget *ok_button;
    GtkWidget *cancel_button;
    GtkWidget * note_book;


    dialog = global_app->flush_options_dialog;

    if (!dialog) {
        dialog = global_app->flush_options_dialog = create_flush_options_dialog();
        ok_button = gm_utils_get_data (dialog, "ok_button");
        cancel_button = gm_utils_get_data (dialog, "cancel_button");
        gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
                            GTK_SIGNAL_FUNC (dialogs_flush_options_deleted),
                            &canceled);


        gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
                            GTK_SIGNAL_FUNC (dialogs_flush_options_canceled),
                            &canceled);

        gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                            GTK_SIGNAL_FUNC (dialogs_flush_options_ok_clicked),
                            &canceled);
    }
    note_book = gm_utils_get_data (dialog, "note_book");
    switch (page){
    case FLUSH_PAGE :
        gtk_window_set_title (GTK_WINDOW (dialog),_("Flush the mysql Server"));
        break;
    case CHECK_PAGE :
        gtk_window_set_title (GTK_WINDOW (dialog),_("Check table(s)"));
        break;
    case REPAIR_PAGE :
        gtk_window_set_title (GTK_WINDOW (dialog),_("Repair table(s)"));
        break;
    }

    gdm ("page number %d", page);

    gtk_notebook_set_page (GTK_NOTEBOOK (note_book), page);
    gtk_widget_show (dialog);
    gtk_main ();
    gtk_widget_hide (dialog);
    return canceled;
}


gint
gm_dialogs_flush_options_dialog (void)
{

    GtkWidget * dialog;
    gint canceled = 0;
    GtkToggleButton *tb;
    GmFlushFlags flag = 0;

    canceled = gm_dialogs_show_fcr_dialog (FLUSH_PAGE);
    dialog = global_app->flush_options_dialog;
    if (canceled) {
        return FALSE;
    }
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "all"));
    if (gtk_toggle_button_get_active (tb))
        return FLUSH_ALL;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "hosts"));
    if (gtk_toggle_button_get_active (tb))
        flag |= FLUSH_HOSTS;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "logs"));
    if (gtk_toggle_button_get_active (tb))
        flag |= FLUSH_LOGS;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "status"));
    if (gtk_toggle_button_get_active (tb))
        flag |= FLUSH_STATUS;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "tables"));
    if (gtk_toggle_button_get_active (tb))
        flag |= FLUSH_TABLES;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "locked_tables"));
    if (gtk_toggle_button_get_active (tb))
        flag |= FLUSH_LOCKED_TABLES;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "privileges"));
    if (gtk_toggle_button_get_active (tb))
        flag |= FLUSH_PRIVILEGES;
    return flag;
}



gint
gm_dialogs_check_table_options_dialog (void)
{

    GtkWidget * dialog;
    gint canceled = 0;
    GtkToggleButton *tb;


    canceled = gm_dialogs_show_fcr_dialog (CHECK_PAGE);
    dialog = global_app->flush_options_dialog;
    if (canceled) {
        return FALSE;
    }
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "c_normal"));
    if (gtk_toggle_button_get_active (tb))
        return CHECK_NORMAL;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "c_quick"));
    if (gtk_toggle_button_get_active (tb))
        return CHECK_QUICK;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "c_fast"));
    if (gtk_toggle_button_get_active (tb))
        return CHECK_FAST;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "c_changed"));
    if (gtk_toggle_button_get_active (tb))
        return CHECK_CHANGED;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "c_extended"));
    if (gtk_toggle_button_get_active (tb))
        return CHECK_EXTENDED;
    return FALSE;

}


gint
gm_dialogs_repair_table_options_dialog (void)
{

    GtkWidget * dialog;
    gint canceled = 0;
    GtkToggleButton *tb;


    canceled = gm_dialogs_show_fcr_dialog (REPAIR_PAGE);
    dialog = global_app->flush_options_dialog;
    if (canceled) {
        return FALSE;
    }
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "r_normal"));
    if (gtk_toggle_button_get_active (tb))
        return REPAIR_NORMAL;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (dialog, "r_quick"));
    if (gtk_toggle_button_get_active (tb))
        return REPAIR_QUICK;
    return FALSE;

}

