/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "gm.h"
#include "gm-bpanel.h"
#include "gm-dumpin.h"
#include "glade_interface.h"


static void
dumpin_ok_clicked   (GtkButton *button,
                     GmConnection * connection)
{
    GtkWidget * win;
    GtkCList * clist;
    GtkWidget * dumpin_database_entry = NULL;
    gchar * filename;
    gchar * database;
    GList * items, *l, *row_list;
    gboolean database_exist = FALSE;
    gint files_count;


    gint row_num;

    win = gm_utils_get_data (GTK_OBJECT (button),"win");
    dumpin_database_entry = gm_utils_get_data (GTK_OBJECT (win),"dumpin_database_entry");
    clist = GTK_CLIST (gm_utils_get_data (GTK_OBJECT (win), "files_clist"));

    if (!clist->row_list)
        return;

    database = gtk_entry_get_text (GTK_ENTRY (dumpin_database_entry));
    l = items = gm_utils_get_data (GTK_OBJECT (button), "existing_databases");

    while (l)
    {
        if (!strcmp (database, (gchar *) l->data))
        {
            database_exist = TRUE;
            break;
        }
        l = l->next;
    }

    if (!database_exist && database && strlen(database))
    {
        if (!gm_db_create_database(connection, database))
        {
            gm_utils_warning ("Could not create database \n");
            return;
        }
    }
    /* Achtung */
    //    gm_db_change_database (connection, database);

    row_list = clist->row_list;
    files_count = clist->rows;
    row_num = 0;
    while (row_list)
    {
        filename = (gchar *) gtk_clist_get_row_data (clist, row_num);
        if (!gm_db_dump_in (connection, database, filename))
        {
            gchar * message ;
            if (files_count>1) {
                message = g_strdup_printf ("An error occur while dumping file %s \n. Continue Dumping files ? \n",filename);
                if (!gm_utils_yes_no_dialog(message)) {
                    g_free (message);
                    break;
                }
            }else {
                message = g_strdup_printf ("An error occur while dumping file %s \n",filename);
                gm_utils_warning (message);
                g_free (message);
                break;
            }
        }
        row_list = row_list->next;
        row_num++;
    }
    gm_bpanel_reload (NULL, connection);
    gtk_main_quit();
}


static void
dumpin_cancel_clicked  (GtkButton *button,
                        GmConnection * connection)
{
    gtk_main_quit();
}


static void
dumpin_add_file_button_clicked  (GtkButton *button,
                                 GtkWidget * win)
{



    GList * l, * files_list = NULL;
    GList * row_list = NULL;
    GtkCList * clist;
    gchar *row[2];
    gint i, row_num ;
    gboolean list_mode = TRUE;
    GmConnection * connection;


    clist = GTK_CLIST (gm_utils_get_data (GTK_OBJECT (win), "files_clist"));

    connection = gm_utils_get_data (GTK_OBJECT (win), "connection");




    l = files_list = gm_utils_create_filebrowser (list_mode);
    i = g_list_length (clist->row_list);
    while (l)
    {
        row[0] = g_basename (l->data);
        row[1] = g_dirname  (l->data);
        row_list  =  clist->row_list;
        row_num = 0;
        while (row_list)
        {
            /*  find duplicates entry */
            gchar * path ;
            path = (gchar *) gtk_clist_get_row_data (clist, row_num);
            if (!strcmp (path, l->data))
            {
                g_free (l->data);
                l->data = NULL;
                break;
            }
            row_num++;
            row_list = row_list->next;
        }
        if (!l->data)
        {
            l = l->next;
            continue;
        }
        row_num = gtk_clist_append (clist, row);
        gtk_clist_set_row_data_full (clist, row_num, l->data, g_free);
        g_free(row[1]);
        i++;
        l = l->next;
    }
    g_list_g_free (files_list);
}




static void
dumpin_remove_file_button_clicked  (GtkButton *button,
                                    GtkWidget * win)

{

    GtkCList * clist;
    gint row;


    clist = GTK_CLIST (gm_utils_get_data (GTK_OBJECT (win), "files_clist"));


    while (clist->selection)
    {
        row = GPOINTER_TO_INT ((g_list_first (clist->selection))->data);
        gtk_clist_remove (clist, row);
    }
}





static gint quick_cmp_AZ  (gconstpointer a,   gconstpointer b)

{
    return   (GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b));
}

static gint quick_cmp_ZA  (gconstpointer a,   gconstpointer b)

{
    return -1 * (GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b));
}





static void
dumpin_move_up_button_clicked  (GtkButton *button,
                                GtkWidget * win)

{

    GtkCList * clist;
    gint row;
    gint dest_row = 0;
    GList * l;
    GList * new_list = NULL;



    clist = GTK_CLIST (gm_utils_get_data (GTK_OBJECT (win), "files_clist"));

    if (!clist->selection)
        return;

    l = g_list_first(clist->selection);
    while (l)
    {
        new_list = g_list_append (new_list, l->data);
        l = l->next;
    }
    l =  g_list_sort (new_list, quick_cmp_AZ);
    while (l)
    {
        row = GPOINTER_TO_INT (l->data);

        if (row <= 0)
            return;

        dest_row = row - 1;
        gtk_clist_row_move (clist,row, dest_row);
        l = l->next;
    }

    dest_row = GPOINTER_TO_INT(g_list_first (new_list)->data);
    dest_row++;

    if (gtk_clist_row_is_visible (clist,dest_row) != GTK_VISIBILITY_FULL)
        gtk_clist_moveto (clist, dest_row,0, 0.0,0.5);

    g_list_free (new_list);


}

static void
dumpin_move_down_button_clicked (GtkButton *button,
                                 GtkWidget * win)

{


    GtkCList * clist;
    gint row;
    gint dest_row = 0;
    GList * l;
    GList * new_list = NULL;



    clist = GTK_CLIST (gm_utils_get_data (GTK_OBJECT (win), "files_clist"));

    if (!clist->selection)
        return;

    l = g_list_first(clist->selection);
    while (l)
    {
        new_list = g_list_append (new_list, l->data);
        l = l->next;
    }
    l =  g_list_sort (new_list, quick_cmp_ZA);
    while (l)
    {
        row = GPOINTER_TO_INT (l->data);

        if (row >= g_list_length (clist->row_list) -1)
            return;

        dest_row = row + 1;
        gtk_clist_row_move (clist,row, dest_row);
        l = l->next;
    }
    dest_row = GPOINTER_TO_INT(g_list_first (new_list)->data);
    dest_row++;

    if (gtk_clist_row_is_visible (clist,dest_row) != GTK_VISIBILITY_FULL)
        gtk_clist_moveto (clist, dest_row,0, 1.0,0.5);

    g_list_free (new_list);

}

static gint gm_dumpin_dialog_key_pressed     (GtkWidget       *widget,
        GdkEventKey     *event,
        GmConnection * connection)
{
    if  (event->keyval == GDK_Escape) {
        gtk_main_quit ();
        return TRUE;
    }
    return FALSE;
}




void gm_dumpin_dialog_new (GmConnection * connection, gchar * sel_database)

{
    GtkWidget * win;
    GtkWidget * but;
    GtkWidget * cancel_but;
    GtkCList * clist;
    GList *items = NULL;
    GtkWidget *combo;
    DbQuery * q;
    MYSQL_ROW row;








    win = create_dumpin_file_selection_dialog ();


    gtk_signal_connect (GTK_OBJECT (win), "key_press_event",
                        GTK_SIGNAL_FUNC (gm_dumpin_dialog_key_pressed),
                        connection);


    gtk_object_set_data (GTK_OBJECT (win), "connection", connection);
    clist = GTK_CLIST (gm_utils_get_data (GTK_OBJECT (win), "files_clist"));
    gtk_clist_set_column_auto_resize (clist, 0, TRUE);
    gtk_clist_set_column_auto_resize (clist, 1, TRUE);
    gtk_clist_set_column_auto_resize (clist, 2, TRUE);
    combo = gm_utils_get_data (GTK_OBJECT (win),"dumpin_database_combo");





    q = gm_db_new_query ("show databases");
    if (gm_db_request (connection, q))
    {
        mysql_data_seek(q->mysql_result,0);
        while ((row = mysql_fetch_row(q->mysql_result)))
        {
            items = g_list_append (items, row[0]);
        }
        gtk_combo_set_popdown_strings (GTK_COMBO (combo), items);
    }

    if (sel_database)
        gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), sel_database);

    but = gm_utils_get_data (GTK_OBJECT (win),"add_file_button");
    gtk_signal_connect (GTK_OBJECT (but), "clicked",
                        GTK_SIGNAL_FUNC (dumpin_add_file_button_clicked),
                        win);

    but = gm_utils_get_data (GTK_OBJECT (win),"remove_file_button");
    gtk_signal_connect (GTK_OBJECT (but), "clicked",
                        GTK_SIGNAL_FUNC (dumpin_remove_file_button_clicked),
                        win);

    but = gm_utils_get_data (GTK_OBJECT (win),"move_up_button");
    gtk_signal_connect (GTK_OBJECT (but), "clicked",
                        GTK_SIGNAL_FUNC (dumpin_move_up_button_clicked),
                        win);

    but = gm_utils_get_data (GTK_OBJECT (win),"move_down_button");
    gtk_signal_connect (GTK_OBJECT (but), "clicked",
                        GTK_SIGNAL_FUNC (dumpin_move_down_button_clicked),
                        win);


    but = gm_utils_get_data (GTK_OBJECT (win),"start_execute");
    gm_utils_create_button_label (but , "Execute ", GNOME_STOCK_PIXMAP_EXEC, TRUE);

    gtk_object_set_data (GTK_OBJECT (but), "win", win);
    if (items)
        gtk_object_set_data (GTK_OBJECT (but), "existing_databases", items);

    gtk_signal_connect (GTK_OBJECT (but), "clicked",
                        GTK_SIGNAL_FUNC (dumpin_ok_clicked),
                        connection);


    cancel_but = gm_utils_get_data (GTK_OBJECT (win),"cancel_button");
    gtk_object_set_data (GTK_OBJECT (cancel_but), "win", win);
    gtk_signal_connect (GTK_OBJECT (cancel_but), "clicked",
                        GTK_SIGNAL_FUNC (dumpin_cancel_clicked),
                        connection);

    gtk_signal_connect (GTK_OBJECT (win), "destroy",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);

    gtk_widget_show(win);
    gtk_main ();
    gm_db_free_query(q);
    if (GTK_IS_WIDGET (win)){
        gtk_signal_disconnect_by_func(GTK_OBJECT (win), gtk_main_quit, NULL);
        gtk_widget_destroy (win);
    }

}
