/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "gm.h"
#include "gm-dumpout.h"
#include "gm-bpanel.h"
#include "glade_interface.h"
#include "gm-plugin.h"
#include "gm-clist.h"

static void           dumpout_start_executing   (GtkButton *button, GmConnection * connection);
static void           dumpout_export_format_entry_changed (GtkEditable * editable, GmConnection * connection );
static void           dumpout_fill_colums_clist (GmNpageData * npage_data);
static GmPluginData * dumpout_get_exporter_plugin_selected (GmConnection * connection );
static void dumpout_mysql_export (GmConnection * connection) ;
static GList * gm_dumpout_configure_selection(GtkCList * clist, GmRowsSelectionTypes stype);
static GList * dumpout_get_columns_list (void) ;



static void on_dumpout_dialog_destroy (GtkWidget *dumpout_dialog, GmConnection *connection)
{
    global_app->dumpout_dialog = NULL;
}



static void
dumpout_cancel_clicked  (GtkButton *button,
                         GmConnection * connection)
{
    gtk_widget_destroy (global_app->dumpout_dialog);

}


static gint gm_dumpout_dialog_key_pressed     (GtkWidget       *widget,
        GdkEventKey     *event,
        GmConnection * connection)
{
    if  (event->keyval == GDK_Escape) {
        gtk_widget_hide (global_app->dumpout_dialog);
        return TRUE;
    }
    return FALSE;
}


static void
dumpout_fill_format_combo (GmConnection * connection)
{

    GtkCombo  * export_format_combo = NULL;
    GtkWidget * dialog;
    GList * plugins_list;
    GtkWidget *li;

    dialog =global_app->dumpout_dialog;

    export_format_combo = GTK_COMBO (gm_utils_get_data (dialog, "export_format_combo"));

    plugins_list = global_app->plugins_list;

    if (!plugins_list) {
        gdm ("No Plugins are installed ");
        return;
    }
    while (plugins_list)
    {
        GmPluginData * pd;
        pd = (GmPluginData *) plugins_list->data;
        if (pd->type == PLUGIN_EXPORTER) {
            li = gtk_list_item_new_with_label ((gchar *) pd->desc);
            gtk_widget_show (li);
            gtk_container_add (GTK_CONTAINER (export_format_combo->list), li);
        }
        plugins_list=plugins_list->next;
    }

}




static void dumpout_setup_selection (GtkWidget * dumpout_dialog, GtkCList *clist, GmNpageData * npage_data)
{

    GList * selection;
    GtkWidget * rb_all_rows;
    GtkWidget * rb_selected_rows;
    GtkWidget * rb_invert_selection;
    GtkWidget * selection_note_book;
    GtkWidget * cpage;
    gboolean first_row_is_selected = FALSE;
    gint selection_len = 0;
    GtkCListRow  *clist_row;



    selection = clist->selection;
    if (selection)
        selection_len = g_list_length (selection);

    clist_row = ROW_ELEMENT (clist, 0)->data;
    if (clist_row->state == GTK_STATE_SELECTED )
        first_row_is_selected = TRUE;


    rb_all_rows = gm_utils_get_data (dumpout_dialog, "rb_all_rows");
    rb_selected_rows = gm_utils_get_data (dumpout_dialog, "rb_selected_rows");
    rb_invert_selection = gm_utils_get_data (dumpout_dialog, "rb_invert_selection");

    selection_note_book = gm_utils_get_data (dumpout_dialog, "selection_note_book");
    cpage = gtk_notebook_get_nth_page (GTK_NOTEBOOK (selection_note_book), 1);





    if (npage_data->type == GM_NPAGE_BROWSER ) {
        BpanelViewInfo * view_info;
        view_info = gm_utils_get_data (GTK_OBJECT (clist), "view_info");
        gtk_widget_set_sensitive (cpage, FALSE);
        if (view_info->view_type == DATABASES_VIEW) {
            if (selection_len == clist->rows  || !selection_len) { /* all rows are selected or none */
                gtk_widget_set_sensitive (rb_selected_rows, FALSE);
                gtk_widget_set_sensitive (rb_invert_selection, FALSE);
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_all_rows), TRUE);
                return;
            }
            if (selection_len > 0 ) {
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_selected_rows), TRUE);
                return;
            }
        }
        else {
            if (selection) {
                gtk_clist_unselect_row (clist,0,0);
                selection = clist->selection;
                selection_len = g_list_length(selection) ;
            }
            if (selection_len == clist->rows - 1 || !selection_len) { /* all rows are selected or none */
                gtk_widget_set_sensitive (rb_selected_rows, FALSE);
                gtk_widget_set_sensitive (rb_invert_selection, FALSE);
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_all_rows), TRUE);
                return;
            }
            if (selection_len > 0 ) {
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_selected_rows), TRUE);
                return;
            }
        }
    }else {
        gtk_widget_set_sensitive (cpage, TRUE);
        if (selection_len == clist->rows  || !selection_len) { /* all rows are selected or none */
            gtk_widget_set_sensitive (rb_selected_rows, FALSE);
            gtk_widget_set_sensitive (rb_invert_selection, FALSE);
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_all_rows), TRUE);
            return;
        }
        if (selection_len > 0 ) {
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_selected_rows), TRUE);
            return;
        }

    }
}





void gm_dumpout_dialog_new (void *ignore , GmConnection * connection)
{
    GtkWidget * dumpout_dialog=NULL;
    GtkWidget * but;
    GtkWidget * cancel_but;
    GmNpageData * npage_data;
    GtkWidget * data_list;
    BpanelViewInfo * view_info = NULL;
    GtkCList * clist;
    GtkWidget *     mysql_dump_options_frame;
    GList * list ;
    GtkCombo * export_format_combo ;






    g_return_if_fail ( connection != NULL);

    npage_data = gm_utils_get_current_npage_data (connection);

    data_list = npage_data->data_list;

    if (npage_data->type != GM_NPAGE_BROWSER && npage_data->type != GM_NPAGE_USER_QUERY)
    {
        gm_utils_warning (_("Please select the page you want to export !!!"));
        return;
    }
    clist = GTK_CLIST (data_list);
    list = clist->selection;

    if (npage_data->type == GM_NPAGE_BROWSER ) {
        view_info = gm_utils_get_data (GTK_OBJECT (data_list), "view_info");
        if (!clist->rows || (clist->rows == 1 && view_info->view_type!= DATABASES_VIEW))
        {
            gm_utils_warning (_("Datalist is empty !!!"));
            return;
        }
    }else {
        if (!clist->rows )
        {
            gm_utils_warning (_("Datalist is empty !!!"));
            return;
        }
    }
    global_app->dumpout_dialog = dumpout_dialog =  create_dumpout_dialog ();
    gtk_object_set_data (GTK_OBJECT (dumpout_dialog), "connection", connection);

    but = gm_utils_get_data (GTK_OBJECT (dumpout_dialog),"start_execute");
    gm_utils_create_button_label (but , "Execute ", GNOME_STOCK_PIXMAP_EXEC, TRUE);
    gtk_object_set_data (GTK_OBJECT (but), "dumpout_dialog", dumpout_dialog);

    gtk_signal_connect (GTK_OBJECT (but), "clicked",
                        GTK_SIGNAL_FUNC (dumpout_start_executing),
                        connection);


    cancel_but = gm_utils_get_data (GTK_OBJECT (dumpout_dialog),"cancel_button");
    gtk_object_set_data (GTK_OBJECT (cancel_but), "dumpout_dialog", dumpout_dialog);
    gtk_signal_connect (GTK_OBJECT (cancel_but), "clicked",
                        GTK_SIGNAL_FUNC (dumpout_cancel_clicked),
                        connection);


    gtk_signal_connect (GTK_OBJECT (dumpout_dialog), "destroy",
                        GTK_SIGNAL_FUNC (on_dumpout_dialog_destroy),
                        connection);

    gtk_object_set_data (GTK_OBJECT (dumpout_dialog), "data_list", data_list);
    gtk_signal_connect (GTK_OBJECT (dumpout_dialog), "key_press_event",
                        GTK_SIGNAL_FUNC (gm_dumpout_dialog_key_pressed),
                        connection);





    dumpout_fill_colums_clist (npage_data) ;
    dumpout_setup_selection (dumpout_dialog, clist, npage_data);
    export_format_combo = GTK_COMBO (gm_utils_get_data (global_app->dumpout_dialog, "export_format_combo"));
    mysql_dump_options_frame = gm_utils_get_data (global_app->dumpout_dialog, "mysql_dump_options_frame");

    if (npage_data->type != GM_NPAGE_BROWSER ) {
        gtk_list_clear_items (GTK_LIST (export_format_combo->list), 0, -1);
        gtk_widget_hide (mysql_dump_options_frame);
    }else {
        if (view_info && view_info->view_type == ROWS_VIEW)
            gtk_widget_hide (mysql_dump_options_frame);
    }


    /* fill out the formats Combolist */

    dumpout_fill_format_combo (connection);
    gtk_signal_connect (GTK_OBJECT (GTK_ENTRY (export_format_combo->entry)), "changed",
                        GTK_SIGNAL_FUNC (dumpout_export_format_entry_changed),
                        connection);


    /* restore the last used plugin */

    if (GE_CONFIG(global_app)->last_used_exporter) {
        /* FIXME:  we must check if the plugin exist */
        gtk_entry_set_text (GTK_ENTRY (export_format_combo->entry), GE_CONFIG(global_app)->last_used_exporter);
    }
    gtk_widget_show (dumpout_dialog);
}



static void
dumpout_start_executing   (GtkButton *button, GmConnection * connection)
{
    GtkWidget * data_list;
    GtkWidget * dumpout_entry;
    GmNpageData * npage_data;
    gchar * file_name=NULL;
    GtkToggleButton * tb;
    GmPluginData * pd;
    GmRowsSelectionTypes stype ;
    GtkCList * columns_clist;
    GtkCombo * export_format_combo ;
    gchar * last_used_exporter =NULL;



    g_return_if_fail ( connection != NULL);
    npage_data = gm_utils_get_current_npage_data (connection);
    g_return_if_fail (npage_data != NULL);

    export_format_combo = GTK_COMBO (gm_utils_get_data (global_app->dumpout_dialog, "export_format_combo"));
    dumpout_entry = gm_utils_get_data (GTK_OBJECT (global_app->dumpout_dialog),"dumpout_entry");
    file_name = gtk_entry_get_text (GTK_ENTRY (dumpout_entry));

    if (!file_name || strlen (file_name)<=0){
        gm_utils_warning (_("No Target File Selected, please select a file !! "));
        return;
    }


    columns_clist = GTK_CLIST(gm_utils_get_data (global_app->dumpout_dialog, "columns_clist"));
    if (g_list_length (columns_clist->selection) == columns_clist->rows) {
        gm_utils_warning (_("You need at least one column to export your data   !! "));
        return;
    }


    data_list = npage_data->data_list;
    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (global_app->dumpout_dialog),"rb_all_rows"));
    stype = ALL_ROWS;
    if (gtk_toggle_button_get_active (tb)) {
        stype = ALL_ROWS;
    }else {
        tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (global_app->dumpout_dialog),"rb_selected_rows"));
        if (gtk_toggle_button_get_active (tb)) {
            stype = SELECTED_ROWS;
        }else {
            tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (global_app->dumpout_dialog),"rb_invert_selection"));
            if (gtk_toggle_button_get_active (tb)) {
                stype = INVERTED_ROWS_SELECTION;
            }

        }
    }



    pd  = dumpout_get_exporter_plugin_selected (connection);

    gtk_widget_hide(global_app->dumpout_dialog);
    if (pd) {
        GtkCListRow  *clist_row;
        GmExporterPluginData *epd;
        GtkCList * columns_clist;
        gint res;

        /* fill in the needed Informations in the private data structure */

        columns_clist = GTK_CLIST (gm_utils_get_data (global_app->dumpout_dialog, "columns_clist"));
        epd = g_new (GmExporterPluginData, 1);
        epd->output_filename = file_name;
        epd->rows_selection = gm_dumpout_configure_selection (GTK_CLIST(data_list), stype);
        epd->stype = stype;
        epd->data_clist = GTK_CLIST (data_list);
        epd->columns_list = dumpout_get_columns_list ();
        clist_row = ROW_ELEMENT (GTK_CLIST (data_list), 0)->data;
        if (clist_row->cell[0].type != GTK_CELL_TEXT)
            epd->ignore_first_row = TRUE;
        else
            epd->ignore_first_row = FALSE;

        pd->private_data = (gpointer) epd;

        if (!pd->progressbar)
            pd->progressbar = connection->progressbar;

        if (!pd->mini_buffer)
            pd->mini_buffer = connection->mini_buffer;

        /* note the plugin has it's own loop and we will wait until the plugin return a state to continue in the main part */

        res = pd->run_plugin (pd);

        if (epd->stype == INVERTED_ROWS_SELECTION)
            g_list_free (epd->rows_selection);

        g_list_free (epd->columns_list);
        g_free(epd);
        pd->private_data = NULL;
        if (res == PLUGIN_ERROR || res == PLUGIN_DEAD){
            gm_utils_warning (_("Error, Export Plugin returned an error while executing "));
            gtk_widget_show (global_app->dumpout_dialog);
            return;
        }else if (res == PLUGIN_CANCELED){
            gtk_widget_show (global_app->dumpout_dialog);
            return;
        }
    }else {
        /* build in exporter */
        dumpout_mysql_export (connection);
    }

    last_used_exporter = gtk_entry_get_text (GTK_ENTRY (export_format_combo->entry));
    if (last_used_exporter) {
        if (GE_CONFIG(global_app)->last_used_exporter) {
            g_free (GE_CONFIG(global_app)->last_used_exporter);
        }
        GE_CONFIG(global_app)->last_used_exporter = g_strdup (last_used_exporter);
    }
    gtk_widget_destroy (global_app->dumpout_dialog);
}

static void dumpout_fill_colums_clist (GmNpageData * npage_data)
{
    GtkCList * columns_clist, * data_clist;
    gint column = 0;
    gchar *text[1];

    columns_clist = GTK_CLIST (gm_utils_get_data (global_app->dumpout_dialog, "columns_clist"));
    data_clist = GTK_CLIST (npage_data->data_list);

    for (column=0; column<data_clist->columns; column++) {
        text[0] = gm_clist_get_column_title (data_clist, column);
        gtk_clist_append (GTK_CLIST (columns_clist), text);
    }

}

static void dumpout_export_format_entry_changed (GtkEditable * editable, GmConnection * connection )
{
    gchar * text ;
    GList * plugins_list;
    GtkWidget *mysql_dump_options_frame;
    GtkWidget * selection_note_book;
    GtkWidget * cpage;

    mysql_dump_options_frame = gm_utils_get_data (global_app->dumpout_dialog, "mysql_dump_options_frame");

    text = gtk_entry_get_text (GTK_ENTRY (editable));
    g_return_if_fail (text != NULL);

    selection_note_book = gm_utils_get_data (global_app->dumpout_dialog, "selection_note_book");
    cpage = gtk_notebook_get_nth_page (GTK_NOTEBOOK (selection_note_book), 1);
    gtk_widget_set_sensitive (cpage, TRUE);
    gtk_widget_set_sensitive (mysql_dump_options_frame, FALSE);

    plugins_list = global_app->plugins_list;
    while (plugins_list) {
        GmPluginData * pd;
        pd = (GmPluginData *) plugins_list->data;
        if (pd->type == PLUGIN_EXPORTER && pd->desc && !strcmp (text, pd->desc))
            return;
        plugins_list = plugins_list->next;
    }

    /* Buildin exporter */

    if (!strcmp (text, "Mysqldump")) {
        gtk_widget_set_sensitive (mysql_dump_options_frame, TRUE);
        gtk_widget_set_sensitive (cpage, FALSE);
    }
}



static GmPluginData  * dumpout_get_exporter_plugin_selected (GmConnection * connection )
{
    gchar * text ;
    GList * plugins_list;
    GtkCombo * export_format_combo;



    export_format_combo = GTK_COMBO (gm_utils_get_data (global_app->dumpout_dialog, "export_format_combo"));
    text = gtk_entry_get_text (GTK_ENTRY (export_format_combo->entry));

    g_return_val_if_fail (text != NULL, NULL);

    plugins_list = global_app->plugins_list;
    while (plugins_list) {
        GmPluginData * pd;
        pd = (GmPluginData *) plugins_list->data;
        if (pd->type == PLUGIN_EXPORTER && pd->desc && !strcmp (text, pd->desc)){
            return pd;
        }
        plugins_list = plugins_list->next;
    }
    return NULL;
}




static void dumpout_mysql_export (GmConnection * connection)
{

    BpanelViewInfo * view_info;
    GmConnectionAccessData * access_data;
    GString *dump_command;
    GtkCList  * clist;
    GtkWidget * data_list;
    GtkWidget * dumpout_entry;
    GtkToggleButton * structure_only_tb;
    gboolean    structure_only = FALSE;
    GtkToggleButton * add_drop_tb;
    gboolean    add_drop = FALSE;
    GList     * list ;
    GmNpageData * npage_data;

    dumpout_entry = gm_utils_get_data (GTK_OBJECT (global_app->dumpout_dialog),"dumpout_entry");
    structure_only_tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (global_app->dumpout_dialog),"structure_only_tb"));
    structure_only    = gtk_toggle_button_get_active (structure_only_tb);

    add_drop_tb = GTK_TOGGLE_BUTTON (gm_utils_get_data (GTK_OBJECT (global_app->dumpout_dialog),"add_drop_tb"));
    add_drop    = gtk_toggle_button_get_active (add_drop_tb);

    npage_data = gm_utils_get_current_npage_data (connection);
    g_return_if_fail (npage_data != NULL);

    data_list = npage_data->data_list;
    view_info = gm_utils_get_data (GTK_OBJECT (data_list), "view_info");

    clist = GTK_CLIST (data_list);
    list = clist->selection;



    access_data  = connection->access_data;
    dump_command = g_string_new ("");

    g_string_sprintfa (dump_command,"mysqldump --host=%s --port=%d -u%s -p%s ", access_data->host, access_data->port, access_data->user, access_data->password);


    if (structure_only)
        g_string_append (dump_command, " -d ");


    if (add_drop)
        g_string_append (dump_command, " --add-drop-table ");

    if (view_info->view_type != ROWS_VIEW) {
        if (view_info->view_type == DATABASES_VIEW) {
            g_string_append (dump_command, " -B ");
        }else {
            g_string_sprintfa (dump_command, " %s ", view_info->database_in_use);
        }
        while (list) {
            gchar * name;
            gint sel_row = GPOINTER_TO_INT (list->data);
            if (!gtk_clist_get_row_data (clist, sel_row)) {
                gtk_clist_get_text (clist, sel_row, 0, &name);
                g_string_sprintfa (dump_command, " %s ", name);
            }
            list = list->next;
        }
    }else {
        g_string_sprintfa (dump_command, " -t %s %s ", view_info->database_in_use, view_info->table_in_use);
    }

    g_string_sprintfa (dump_command, " > %s ", gtk_entry_get_text (GTK_ENTRY (dumpout_entry)));
    gm_debug_message (GM_DUMPOUT_CHANEL, "%s ", dump_command->str);
    if (dump_command->str)
        system (dump_command->str);


    g_string_free (dump_command, TRUE);
}



static GList * gm_dumpout_configure_selection(GtkCList * clist, GmRowsSelectionTypes stype)
{

    GList *list = NULL;
    switch (stype){
    case ALL_ROWS :
        {
            list =  clist->row_list;
            break;
        }
    case SELECTED_ROWS :
        {
            list = clist->selection;
            break;
        }
    case INVERTED_ROWS_SELECTION :
        {
            GList * l;
            GList * selection;
            gint row = 0;
            gint selected_row = 0;

            selection = clist->selection;
            if (selection) {

                l = clist->row_list;
                while (l){
                    if (selection)
                        selected_row = GPOINTER_TO_INT (selection->data);
                    if (row == selected_row) {
                        selection = selection->next;
                    }else {
                        list = g_list_append (list, GINT_TO_POINTER (row));
                    }
                    row++;
                    l = l ->next;
                }
            }else {
                list = clist->row_list;
            }
            break;
        }
    }
    return list;
}







static GList * dumpout_get_columns_list (void)
{
    GtkCList * clist;
    GList * selection;
    GList * list=NULL;
    GList *l;
    gint selected_col = -1;
    gint col = 0;



    clist = GTK_CLIST(gm_utils_get_data (global_app->dumpout_dialog, "columns_clist"));
    selection = clist->selection;
    l = clist->row_list;
    while (l){
        if (selection)
            selected_col = GPOINTER_TO_INT (selection->data);
        if (col == selected_col) {
            selection = selection->next;
        }else {
            list = g_list_append (list, GINT_TO_POINTER (col));
        }
        col++;
        l = l ->next;
    }
    return list;
}
