#include <gnome.h>
#include "gm.h"
#include "glade_interface.h"
#include "gm-filter.h"
#include "gm-bpanel.h"


enum {
    HOSTS_VIEW_PAGE = 0,
    DATABASES_VIEW_PAGE,
    TABLES_VIEW_PAGE,
    ROWS_VIEW_PAGE
};




static void filter_cancel_button_clicked               (void  *ignore,
        GtkWidget * dialog)
{

    gtk_widget_destroy (dialog);

}


static void filter_ok_button_clicked               (void  *ignore,
        GtkWidget * dialog)
{

    BpanelViewInfo    * view_info;
    DbBrowserNodeInfo * info = NULL;
    GtkWidget * button;
    GtkEntry * entry;
    gboolean dirty, active;
    gboolean reload = FALSE;
    gboolean old_state;
    gchar * new_default_query;
    GmConnection * connection;



    view_info = gm_utils_get_data (dialog, "view_info");
    dirty     = $int_pointer (dialog, "dirty") ;
    old_state = $int_pointer (dialog, "old_state") ;
    connection = gm_utils_get_data (dialog, "connection");

    info = (DbBrowserNodeInfo *) gm_utils_get_data (dialog, "info");

    if (!dirty){
        gtk_widget_destroy (dialog);
        return;
    }



    entry = GTK_ENTRY (gm_utils_get_data (dialog, "db_default_filterq"));
    new_default_query = gtk_entry_get_text (entry);
    g_strstrip (new_default_query);
    if (!new_default_query || !strlen(new_default_query)) {
        gm_utils_warning (_("You must Provide a default query to fetch databases !!"));
        return;
    }
    if (strcmp (GE_CONFIG(global_app)->default_databases_query,new_default_query) || old_state) {
        g_free (GE_CONFIG(global_app)->default_databases_query);
        GE_CONFIG(global_app)->default_databases_query = g_strdup (new_default_query);
        reload = TRUE;
    }


    entry = GTK_ENTRY (gm_utils_get_data (dialog, "table_default_filterq"));
    new_default_query = gtk_entry_get_text (entry);
    g_strstrip (new_default_query);
    if (!new_default_query || !strlen(new_default_query)) {
        gm_utils_warning (_("You must Provide a default query to fetch tables !!"));
        return;
    }

    if (strcmp (GE_CONFIG(global_app)->default_tables_query,new_default_query) || old_state) {
        g_free (GE_CONFIG(global_app)->default_tables_query);
        GE_CONFIG(global_app)->default_tables_query = g_strdup (new_default_query);
        reload = TRUE;
    }

    entry = GTK_ENTRY (gm_utils_get_data (dialog, "row_default_filterq"));
    new_default_query = gtk_entry_get_text (entry);
    g_strstrip (new_default_query);
    if (!new_default_query || !strlen(new_default_query)) {
        gm_utils_warning (_("You must Provide a default query to fetch rows !!"));
        return;
    }
    if (strcmp (GE_CONFIG(global_app)->default_rows_query,new_default_query) || old_state) {
        g_free (GE_CONFIG(global_app)->default_rows_query);
        GE_CONFIG(global_app)->default_rows_query = g_strdup (new_default_query);
        reload = TRUE;
    }

    switch (view_info->view_type)
    {
    case DATABASES_VIEW :
        {
            button = gm_utils_get_data  (dialog, "db_tb_user");
            active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
            if (active) {
                gchar * new_user_query;
                entry = GTK_ENTRY (gm_utils_get_data (dialog, "db_user_entry"));
                new_user_query = gtk_entry_get_text (entry);
                if (!info->filter_query || strcmp(info->filter_query, new_user_query) || !old_state) {
                    g_free (info->filter_query);
                    info->filter_query = g_strdup (new_user_query);
                    reload = TRUE;
                }
                if (!info->filter_enabled)
                    reload = TRUE;
                info->filter_enabled = TRUE;
            }else {
                info->filter_enabled = FALSE;
            }
            break;
        }
    case TABLES_VIEW :
        {
            button = gm_utils_get_data  (dialog, "table_tb_user");
            active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
            if (active) {
                gchar * new_user_query;
                entry = GTK_ENTRY (gm_utils_get_data (dialog, "table_user_entry"));
                new_user_query = gtk_entry_get_text (entry);
                if (!info->filter_query || strcmp(info->filter_query, new_user_query) || !old_state) {
                    g_free (info->filter_query);
                    info->filter_query = g_strdup (new_user_query);
                    reload = TRUE;
                }
                if (!info->filter_enabled)
                    reload = TRUE;
                info->filter_enabled = TRUE;
            }else {
                info->filter_enabled = FALSE;
            }
            break;
        }
    case ROWS_VIEW :
        {
            button = gm_utils_get_data  (dialog, "row_tb_user");
            active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
            if (active) {
                gchar * new_user_query;
                entry = GTK_ENTRY (gm_utils_get_data (dialog, "row_user_entry"));
                new_user_query = gtk_entry_get_text (entry);
                if (!info->filter_query || strcmp(info->filter_query, new_user_query) || !old_state) {
                    g_free (info->filter_query);
                    info->filter_query = g_strdup_printf ("SELECT * FROM %s.%s WHERE %s", view_info->database_in_use, view_info->table_in_use, new_user_query);
                    reload = TRUE;
                }
                if (!info->filter_enabled)
                    reload = TRUE;
                info->filter_enabled = TRUE;
            }else {
                info->filter_enabled = FALSE;
            }
            break;
        }
    }
    gtk_widget_destroy (dialog);
    if (reload)
        gm_bpanel_reload (NULL, connection);
}






static void filter_user_entry_enable_disable (GtkToggleButton *togglebutton,
        GtkWidget * entry)
{
    gtk_widget_set_sensitive (entry, gtk_toggle_button_get_active (togglebutton));
}



static void make_it_dirty            (GtkEditable     *editable,
                                      GtkWidget * dialog)

{
    gtk_object_set_data (GTK_OBJECT (dialog), "dirty", GINT_TO_POINTER(1));
}




static void configure_entries (GtkWidget * dialog,
                               DbBrowserNodeInfo * info,
                               GtkWidget * entry,
                               GtkWidget * button)
{


    if (info->filter_query ) {
        gtk_entry_set_text (GTK_ENTRY (entry), info->filter_query);
        if (info->filter_enabled) {
            gtk_widget_set_sensitive (entry, TRUE);
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
        }
    }

    gtk_signal_connect (GTK_OBJECT (button), "toggled",
                        GTK_SIGNAL_FUNC (filter_user_entry_enable_disable),
                        entry);

    gtk_signal_connect (GTK_OBJECT (button), "toggled",
                        GTK_SIGNAL_FUNC (make_it_dirty),
                        dialog);

    gtk_signal_connect (GTK_OBJECT (entry), "changed",
                        GTK_SIGNAL_FUNC (make_it_dirty),
                        dialog);

    /* the old state is used to make sure if we have to update the browser view or not */

    gtk_object_set_data (GTK_OBJECT (dialog), "old_state", GINT_TO_POINTER(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (button))));
}



void gm_filter_show_dialog (GmConnection * connection, BpanelViewInfo * view_info)
{
    DbBrowserNodeInfo * info = NULL;
    GtkWidget * dialog;
    GtkWidget * filter_main_notebook ;
    GtkWidget * button;
    GtkWidget * entry;
    GtkWidget * default_entry;
    GtkWidget * frame;
    GmCcenterGeneral * ge_config;


    g_return_if_fail (view_info != NULL);

    info = gm_bpanel_get_node_info (connection, view_info);

    if (!info)
        g_assert ("this should never happen info was NULL in gm-filter line 125 \n");

    dialog = create_filter_dialog ();
    gtk_object_set_data (GTK_OBJECT (dialog), "view_info", view_info);
    gtk_object_set_data (GTK_OBJECT (dialog), "dirty", GINT_TO_POINTER(0));
    gtk_object_set_data (GTK_OBJECT (dialog), "info", info);
    gtk_object_set_data (GTK_OBJECT (dialog), "connection", connection);


    filter_main_notebook = gm_utils_get_data (dialog, "filter_main_notebook");


    button = gm_utils_get_data (dialog, "cancel_button");

    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (filter_cancel_button_clicked),
                        dialog);
    button = gm_utils_get_data  (dialog, "ok_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (filter_ok_button_clicked),
                        dialog);

    default_entry = gm_utils_get_data  (dialog, "db_default_filterq");
    ge_config = GE_CONFIG(global_app);



    gtk_entry_set_text (GTK_ENTRY (default_entry), ge_config->default_databases_query) ;

    gtk_signal_connect (GTK_OBJECT (default_entry), "changed",
                        GTK_SIGNAL_FUNC (make_it_dirty),
                        dialog);

    default_entry = gm_utils_get_data  (dialog, "table_default_filterq");
    gtk_entry_set_text (GTK_ENTRY (default_entry), GE_CONFIG(global_app)->default_tables_query );
    gtk_signal_connect (GTK_OBJECT (default_entry), "changed",
                        GTK_SIGNAL_FUNC (make_it_dirty),
                        dialog);

    default_entry = gm_utils_get_data  (dialog, "row_default_filterq");
    gtk_entry_set_text (GTK_ENTRY (default_entry), GE_CONFIG(global_app)->default_rows_query);
    gtk_signal_connect (GTK_OBJECT (default_entry), "changed",
                        GTK_SIGNAL_FUNC (make_it_dirty),
                        dialog);


    if (view_info->view_type == DATABASES_VIEW) {
        frame =  gm_utils_get_data  (dialog, "db_user_frame");
        gtk_widget_set_sensitive (frame, TRUE);
        entry = gm_utils_get_data  (dialog, "db_user_entry");
        button = gm_utils_get_data  (dialog, "db_tb_user");
        configure_entries (dialog,
                           info,
                           entry,
                           button);
    }


    if (view_info->view_type == TABLES_VIEW) {
        frame =  gm_utils_get_data  (dialog, "table_user_frame");
        gtk_widget_set_sensitive (frame, TRUE);
        button = gm_utils_get_data  (dialog, "table_tb_user");
        entry = gm_utils_get_data  (dialog, "table_user_entry");
        configure_entries (dialog,
                           info,
                           entry,
                           button);
    }


    if (view_info->view_type == ROWS_VIEW) {
        gchar *where_body;

        frame =  gm_utils_get_data  (dialog, "row_user_frame");
        gtk_widget_set_sensitive (frame, TRUE);
        button = gm_utils_get_data  (dialog, "row_tb_user");
        entry = gm_utils_get_data  (dialog, "row_user_entry");
        configure_entries (dialog,
                           info,
                           entry,
                           button);
        /* hack around the where body */
        if (info->filter_query) {
            where_body = strstr (info->filter_query, "WHERE");
            if (where_body) {
                where_body += 6;
                gtk_entry_set_text (GTK_ENTRY (entry), where_body);
            }
        }


    }
    gtk_widget_show (dialog);
}









