/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "gm.h"
#include "gm-dbapi.h"
#include "gm-formular.h"
#include "glade_interface.h"
#include "gm-bpanel.h"
#include "gm-date-entry.h"


static GtkWidget* formular_create_entry_text_box (gchar *entry_value);
static void formular_apply_button_clicked (GtkButton *button, GtkWidget *formular_vbox) ;
gboolean formular_on_entry_focus_in_event (GtkWidget *widget, GdkEventFocus *event, gpointer user_data);
gboolean
formular_on_entry_focus_out_event           (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data);


gboolean
formular_on_text_focus_in_event           (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data);


gboolean
formular_on_text_focus_out_event           (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data);

static void formular_edit_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox);


static void formular_close_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox);

static void formular_open_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox);

static void formular_save_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox);

static void formular_copy_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox);

static void formular_paste_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox);
static void formular_show_text_widget (GtkWidget * formular_vbox, GtkWidget * focused_widget) ;






static void formular_update_data_without_useofkey (GtkWidget * formular_vbox, gboolean cloned)
{
    DbQuery * q;
    BpanelViewInfo * view_info;
    GSList * old_data_list;
    GSList * slist, * dlist = NULL;
    gint row = 0;
    gchar * query ;
    gchar * set_query = NULL;
    gchar * where_query = NULL;
    GmConnection * connection;
    GtkWidget * entry;
    gchar* escaped_str=NULL;




    view_info = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "view_info");
    old_data_list = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "old_data_list");
    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    slist = g_slist_nth (view_info->fields_list, 0);
    if (old_data_list)
        dlist = g_slist_nth (old_data_list, 0);
    slist = g_slist_nth (view_info->fields_list, 0);

    while (slist)
    {
        MYSQL_FIELD *field = (MYSQL_FIELD *) slist->data;
        gchar * col_name;
        gchar * entry_name, *s1 =NULL , *s2= NULL;
        gchar * w1=NULL;
        gchar * new_value = NULL, *tmp_value=NULL;

        entry_name = g_strdup_printf ("entry_widget_%d",row);
        entry = gm_utils_get_data  (GTK_OBJECT (formular_vbox), entry_name);
        if (!GTK_IS_EDITABLE (entry)){
            GtkWidget * entry_text_vbox = entry;
            entry = gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "fentry");
            if (!GTK_WIDGET_VISIBLE (entry)){
                entry = gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "text");
            }
        }

        tmp_value = gtk_editable_get_chars (GTK_EDITABLE (entry), 0,-1);


        new_value = NULL;
        if (tmp_value){
            new_value = gm_utils_strescape (tmp_value, NULL);
        }



        col_name = field->name;
        if (new_value && strlen (new_value))
        {
            s1 = g_strdup_printf (" %s='%s' ",col_name , new_value );
            if (set_query)
            {
                s2 = set_query;
                set_query = g_strdup_printf ("%s, %s", set_query, s1);
                g_free (s2);
                g_free (s1);
            }else
            {
                set_query = s1;
            }
        }
        g_free (new_value);
        escaped_str = NULL;
        if (old_data_list && dlist && dlist->data)
            escaped_str = gm_utils_strescape ( (gchar *) dlist->data, NULL);

        if (old_data_list)
        {
            if (where_query)
            {
                if (escaped_str)
                    w1 = g_strdup_printf ("%s AND %s='%s' ",where_query, col_name, escaped_str);
                else
                    w1 = g_strdup_printf ("%s AND %s IS NULL ",where_query, col_name);

                g_free (where_query);
                where_query = w1;
            }else
            {
                if (escaped_str)
                    where_query = g_strdup_printf ("where %s='%s' ",col_name, escaped_str);
                else
                    where_query = g_strdup_printf ("where %s IS NULL ",col_name);
            }
        }
        if (tmp_value && dlist && dlist->data && strcmp ( (gchar *) dlist->data, tmp_value))
        {
            g_free (dlist->data);
            dlist->data = g_strdup (tmp_value);
        }



        g_free (tmp_value);
        g_free (escaped_str);
        row++;
        slist=slist->next;
        if (dlist)
            dlist = dlist->next;
    }
    if (old_data_list && !cloned)
        query = g_strdup_printf ("UPDATE %s SET %s %s ", view_info->table_in_use, set_query, where_query);
    else
        query = g_strdup_printf ("INSERT INTO  %s set %s ", view_info->table_in_use, set_query);
    g_free (set_query);
    g_free (where_query);

    q = gm_db_new_query (query);
    q->database = view_info->database_in_use;
    if (!gm_db_request (connection, q))
    {
        gchar * error_message;
        error_message = g_strdup_printf ( "Mysql Error :%s\n",  q->error);
        gm_utils_write_log_message (connection, error_message,  ERROR_LOGMODE);
        g_free (error_message);
        gm_db_free_query (q);
        g_free (query);
        return;
    }

    gm_db_free_query (q);
    g_free (query);

}

static void formular_update_data_with_useofkey (GtkWidget * formular_vbox,
        DbTableInfo *table_info,
        gboolean cloned)
{
    DbQuery * q;
    BpanelViewInfo * view_info;
    GSList * old_data_list;
    GSList * slist, * dlist = NULL;
    gint row = 0;
    gchar * query ;
    gchar * set_query = NULL;
    gchar * where_query = NULL;
    GmConnection * connection;
    GtkWidget * entry;
    gchar* escaped_str=NULL;


    view_info = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "view_info");
    old_data_list = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "old_data_list");
    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    slist = g_slist_nth (view_info->fields_list, 0);
    if (old_data_list)
        dlist = g_slist_nth (old_data_list, 0);
    slist = g_slist_nth (view_info->fields_list, 0);

    while (slist)
    {
        MYSQL_FIELD *field = (MYSQL_FIELD *) slist->data;
        gchar * col_name;
        gchar * entry_name, *s1 =NULL , *s2= NULL;
        gchar * w1=NULL;
        gchar * new_value = NULL, *tmp_value=NULL;

        entry_name = g_strdup_printf ("entry_widget_%d",row);
        entry = gm_utils_get_data  (GTK_OBJECT (formular_vbox), entry_name);
        if (!GTK_IS_EDITABLE (entry)){
            GtkWidget * entry_text_vbox = entry;
            entry = gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "fentry");
            if (!GTK_WIDGET_VISIBLE (entry)){
                entry = gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "text");
            }
        }

        tmp_value = gtk_editable_get_chars (GTK_EDITABLE (entry), 0,-1);


        new_value = NULL;
        if (tmp_value){
            new_value = gm_utils_strescape (tmp_value, NULL);
        }



        col_name = field->name;
        if (new_value && strlen (new_value))
        {
            s1 = g_strdup_printf (" %s='%s' ",col_name , new_value );
            if (set_query)
            {
                s2 = set_query;
                set_query = g_strdup_printf ("%s, %s", set_query, s1);
                g_free (s2);
                g_free (s1);
            }else
            {
                set_query = s1;
            }
        }
        g_free (new_value);
        escaped_str = NULL;
        if (old_data_list && dlist && dlist->data)
            escaped_str = gm_utils_strescape ( (gchar *) dlist->data, NULL);

        if (old_data_list)
        {
            GList *keys_info_list = table_info->keys_info_list;
            gboolean set_where_query = FALSE;

            while (keys_info_list){
                DbKeyInfo * key_info = (DbKeyInfo *) keys_info_list->data;
                gdm ("key %s", key_info->name);


                if (key_info->is_unique || key_info->is_primary_key){
                    GList *l = key_info->affected_column_list;
                    while (l) {
                        gchar *name = (gchar * )l->data;
                        if (name && !strcmp (col_name,name)){
                            set_where_query = TRUE;
                            break;
                        }
                        l=l->next;
                    }
                    if (set_where_query)
                        break;
                }
                keys_info_list = keys_info_list->next;
            }

            if (set_where_query) {
                if (where_query)
                {
                    if (escaped_str)
                        w1 = g_strdup_printf ("%s AND %s='%s' ",where_query, col_name, escaped_str);
                    else
                        w1 = g_strdup_printf ("%s AND %s IS NULL ",where_query, col_name);

                    g_free (where_query);
                    where_query = w1;
                }else
                {
                    if (escaped_str)
                        where_query = g_strdup_printf ("WHERE %s='%s' ",col_name, escaped_str);
                    else
                        where_query = g_strdup_printf ("WHERE %s IS NULL ",col_name);
                }
            }
        }
        if (tmp_value && dlist && dlist->data && strcmp ( (gchar *) dlist->data, tmp_value))
        {
            g_free (dlist->data);
            dlist->data = g_strdup (tmp_value);
        }



        g_free (tmp_value);
        g_free (escaped_str);
        row++;
        slist=slist->next;
        if (dlist)
            dlist = dlist->next;
    }
    if (old_data_list && !cloned)
        query = g_strdup_printf ("UPDATE %s SET %s %s ", view_info->table_in_use, set_query, where_query);
    else
        query = g_strdup_printf ("INSERT INTO  %s set %s ", view_info->table_in_use, set_query);
    g_free (set_query);
    g_free (where_query);

    q = gm_db_new_query (query);
    q->database = view_info->database_in_use;
    if (!gm_db_request (connection, q))
    {
        gchar * error_message;
        error_message = g_strdup_printf ( "Mysql Error :%s\n",  q->error);
        gm_utils_write_log_message (connection, error_message,  ERROR_LOGMODE);
        g_free (error_message);
        gm_db_free_query (q);
        g_free (query);
        return;
    }

    gm_db_free_query (q);
    g_free (query);

}



static void formular_apply_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox)
{
    GmConnection *connection;
    BpanelViewInfo *view_info;
    DbTableInfo *table_info;
    gboolean has_unique_keys = FALSE;

    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    view_info  = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "view_info");

    table_info = gm_db_has_unique_keys(connection, view_info->database_in_use, view_info->table_in_use, &has_unique_keys);
    if (has_unique_keys)
        formular_update_data_with_useofkey (formular_vbox, table_info, FALSE);
    else {
        formular_update_data_without_useofkey (formular_vbox, FALSE);
    }
    gm_db_free_table_info (table_info);
    gm_bpanel_reload (NULL, connection);
}


void formular_ok_button_clicked (GtkButton * button,
                                 GtkWidget * formular_vbox)
{

    GmConnection * connection;

    BpanelViewInfo *view_info;
    DbTableInfo *table_info;

    gboolean has_unique_keys = FALSE;

    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    view_info  = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "view_info");

    table_info = gm_db_has_unique_keys(connection, view_info->database_in_use, view_info->table_in_use, &has_unique_keys);
    if (has_unique_keys)
        formular_update_data_with_useofkey (formular_vbox, table_info, FALSE);
    else {
        formular_update_data_without_useofkey (formular_vbox, FALSE);
    }

    gm_db_free_table_info (table_info);
    gtk_widget_destroy (formular_vbox);
    gm_bpanel_select_db_browser_page (connection);
    gm_bpanel_reload (NULL, connection);
}


void formular_clone_button_clicked (GtkButton * button,
                                    GtkWidget * formular_vbox)
{

    GmConnection * connection;
    BpanelViewInfo *view_info;
    DbTableInfo *table_info;

    gboolean has_unique_keys = FALSE;

    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    view_info  = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "view_info");

    table_info = gm_db_has_unique_keys(connection, view_info->database_in_use, view_info->table_in_use, &has_unique_keys);
    if (has_unique_keys)
        formular_update_data_with_useofkey (formular_vbox, table_info, TRUE);
    else {
        formular_update_data_without_useofkey (formular_vbox, TRUE);
    }
    gm_db_free_table_info (table_info);
    gtk_widget_destroy (formular_vbox);
    gm_bpanel_select_db_browser_page (connection);
    gm_bpanel_reload (NULL, connection);
}



static void formular_cancel_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox)
{
    GmConnection * connection;
    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    gtk_widget_destroy (formular_vbox);
    gm_bpanel_select_db_browser_page (connection);
}





static GtkWidget * formular_create_gm_entry (DbFieldInfo * field, gchar *entry_value)
{

    GtkWidget * entry_widget = NULL;
    gint type;


    type = field->type;
    switch (type) {
    case GM_MYSQL_DATE:
    case GM_MYSQL_DATETIME:
    case GM_MYSQL_TIMESTAMP:
    case GM_MYSQL_TIME:
    case GM_MYSQL_YEAR:
        {
            GtkWidget * dentry_widget;
            dentry_widget = gm_date_entry_new ( (time_t) 0, TRUE, TRUE, type);
            if (entry_value)
                gtk_entry_set_text (GTK_ENTRY (GM_DATE_ENTRY (dentry_widget)->date_entry), entry_value);
            entry_widget = GM_DATE_ENTRY (dentry_widget)->date_entry;
            gtk_widget_show (dentry_widget);
            break;
        }
    case GM_MYSQL_ENUM:
        {
            GtkWidget * enum_widget;
            enum_widget = gtk_combo_new ();
            gtk_widget_show (enum_widget);
            if (field->enum_list)
                gtk_combo_set_popdown_strings (GTK_COMBO (enum_widget), field->enum_list);
            entry_widget = GTK_COMBO (enum_widget)->entry;
            gtk_entry_set_editable (GTK_ENTRY (entry_widget), FALSE);
            if (entry_value)
                gtk_entry_set_text (GTK_ENTRY (entry_widget), entry_value);
            break;
        }
    default :
        {
            entry_widget = formular_create_entry_text_box (entry_value);
            break;
        }

    }
    return entry_widget;
}





static GtkWidget*
create_formular_vbox (GmConnection * connection)
{

    GtkWidget *formular_vbox;
    GtkWidget *formular_frame;
    GtkWidget *scrolledwindow4;
    GtkWidget *formular_viewport;
    GtkWidget *cancel_button;
    GtkWidget *ok_button;
    GtkWidget *clone_button;
    GtkWidget *apply_button;
    GtkWidget *formular_toolbar;
    GtkWidget * status_frame;
    GtkWidget * status_bar;
    GtkWidget *tmp_toolbar_icon;
    GtkWidget *close_button;
    GtkWidget *edit_button;
    GtkWidget *save_button;
    GtkWidget *open_button;
    GtkWidget *copy_button;
    GtkWidget *paste_button;

    formular_vbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (formular_vbox);

    gtk_object_set_data (GTK_OBJECT (formular_vbox), "connection", connection);



    /* Cancel  Apply Ok  toolbar  */

    formular_toolbar = gtk_toolbar_new (GTK_ORIENTATION_VERTICAL, GTK_TOOLBAR_ICONS);
    gtk_widget_ref (formular_toolbar);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "formular_toolbar", formular_toolbar,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (formular_toolbar);
    gtk_box_pack_start (GTK_BOX (formular_vbox), formular_toolbar, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (formular_toolbar), 2);
    gtk_toolbar_set_space_style (GTK_TOOLBAR (formular_toolbar), GTK_TOOLBAR_SPACE_LINE);
    gtk_toolbar_set_button_relief (GTK_TOOLBAR (formular_toolbar), GTK_RELIEF_NONE);


    tmp_toolbar_icon = gm_stock_pixmap_widget (formular_vbox, "cancel");
    cancel_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                    GTK_TOOLBAR_CHILD_BUTTON,
                    NULL,
                    "cancel",
                    "Close this Page", NULL,
                    tmp_toolbar_icon, NULL, NULL);
    gtk_widget_ref (cancel_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "cancel_button", cancel_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (cancel_button);



    tmp_toolbar_icon = gm_stock_pixmap_widget (formular_vbox, "apply");
    apply_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                   GTK_TOOLBAR_CHILD_BUTTON,
                   NULL,
                   "Apply",
                   "Apply the changes without closing the page", NULL,
                   tmp_toolbar_icon, NULL, NULL);
    gtk_widget_ref (apply_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "apply_button", apply_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (apply_button);



    tmp_toolbar_icon = gm_stock_pixmap_widget (formular_vbox, "ok");
    ok_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                                            GTK_TOOLBAR_CHILD_BUTTON,
                                            NULL,
                                            "Ok",
                                            "Apply the changes and close the page ", NULL,
                                            tmp_toolbar_icon, NULL, NULL);

    gtk_widget_ref (ok_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "ok_button", ok_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (ok_button);



    tmp_toolbar_icon = gm_stock_pixmap_widget (formular_vbox, "clone");
    clone_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                   GTK_TOOLBAR_CHILD_BUTTON,
                   NULL,
                   "Clone",
                   "Save as a new datarecord (clone)", NULL,
                   tmp_toolbar_icon, NULL, NULL);

    gtk_widget_ref (clone_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "clone_button", clone_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (clone_button);



    /* edit buttons */

    gtk_toolbar_append_space (GTK_TOOLBAR (formular_toolbar));

    tmp_toolbar_icon = gnome_stock_pixmap_widget (formular_vbox, GNOME_STOCK_PIXMAP_CLOSE);
    close_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                   GTK_TOOLBAR_CHILD_BUTTON,
                   NULL,
                   "cancel",
                   "back to the entry view", NULL,
                   tmp_toolbar_icon, NULL, NULL);
    gtk_widget_ref (close_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "close_button", close_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (close_button);

    gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
                        GTK_SIGNAL_FUNC (formular_close_button_clicked),
                        formular_vbox);


    tmp_toolbar_icon = gnome_stock_pixmap_widget (formular_vbox, GNOME_STOCK_PIXMAP_PREFERENCES);
    edit_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                  GTK_TOOLBAR_CHILD_BUTTON,
                  NULL,
                  "Edit",
                  NULL, NULL,
                  tmp_toolbar_icon, NULL, NULL);
    gtk_widget_ref (edit_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "edit_button", edit_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (edit_button);
    gtk_signal_connect (GTK_OBJECT (edit_button), "clicked",
                        GTK_SIGNAL_FUNC (formular_edit_button_clicked),
                        formular_vbox);



    gtk_toolbar_append_space (GTK_TOOLBAR (formular_toolbar));

    tmp_toolbar_icon = gnome_stock_pixmap_widget (formular_vbox, GNOME_STOCK_PIXMAP_SAVE_AS);
    save_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                  GTK_TOOLBAR_CHILD_BUTTON,
                  NULL,
                  "Apply",
                  "save as ", NULL,
                  tmp_toolbar_icon, NULL, NULL);
    gtk_widget_ref (save_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "save_button", save_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (save_button);
    gtk_signal_connect (GTK_OBJECT (save_button), "clicked",
                        GTK_SIGNAL_FUNC (formular_save_button_clicked),
                        formular_vbox);



    tmp_toolbar_icon = gnome_stock_pixmap_widget (formular_vbox, GNOME_STOCK_PIXMAP_OPEN);
    open_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                  GTK_TOOLBAR_CHILD_BUTTON,
                  NULL,
                  "Ok",
                  "open", NULL,
                  tmp_toolbar_icon, NULL, NULL);
    gtk_widget_ref (open_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "open_button", open_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (open_button);
    gtk_signal_connect (GTK_OBJECT (open_button), "clicked",
                        GTK_SIGNAL_FUNC (formular_open_button_clicked),
                        formular_vbox);


    gtk_toolbar_append_space (GTK_TOOLBAR (formular_toolbar));

    tmp_toolbar_icon = gnome_stock_pixmap_widget (formular_vbox, GNOME_STOCK_PIXMAP_COPY);
    copy_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                  GTK_TOOLBAR_CHILD_BUTTON,
                  NULL,
                  "copy",
                  "copy", NULL,
                  tmp_toolbar_icon, NULL, NULL);
    gtk_widget_ref (copy_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "copy_button", copy_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (copy_button);
    gtk_signal_connect (GTK_OBJECT (copy_button), "clicked",
                        GTK_SIGNAL_FUNC (formular_copy_button_clicked),
                        formular_vbox);


    tmp_toolbar_icon = gnome_stock_pixmap_widget (formular_vbox, GNOME_STOCK_PIXMAP_PASTE);
    paste_button = gtk_toolbar_append_element (GTK_TOOLBAR (formular_toolbar),
                   GTK_TOOLBAR_CHILD_BUTTON,
                   NULL,
                   "paste",
                   "paste", NULL,
                   tmp_toolbar_icon, NULL, NULL);
    gtk_widget_ref (paste_button);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "paste_button", paste_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_signal_connect (GTK_OBJECT (paste_button), "clicked",
                        GTK_SIGNAL_FUNC (formular_paste_button_clicked),
                        formular_vbox);





    formular_frame = gtk_frame_new (NULL);
    gtk_widget_ref (formular_frame);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "formular_frame", formular_frame,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (formular_frame);
    gtk_box_pack_start (GTK_BOX (formular_vbox), formular_frame, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (formular_frame), 2);
    scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_ref (scrolledwindow4);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "scrolledwindow4", scrolledwindow4,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (scrolledwindow4);
    gtk_container_add (GTK_CONTAINER (formular_frame), scrolledwindow4);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    formular_viewport = gtk_viewport_new (NULL, NULL);
    gtk_widget_ref (formular_viewport);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "formular_viewport", formular_viewport,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (formular_viewport);
    gtk_container_add (GTK_CONTAINER (scrolledwindow4), formular_viewport);






    /* Status bar */

    status_frame = gtk_frame_new (NULL);
    gtk_widget_ref (status_frame);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "status_frame", status_frame,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (status_frame);
    gtk_box_pack_start (GTK_BOX (formular_vbox), status_frame, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (status_frame), 1);

    status_bar = gtk_label_new ("");
    gtk_widget_ref (status_bar);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "status_bar", status_bar,
                              (GtkDestroyNotify) gtk_widget_unref);
    //gtk_widget_show (status_bar);
    gtk_container_add (GTK_CONTAINER (status_frame), status_bar);
    gtk_label_set_justify (GTK_LABEL (status_bar), GTK_JUSTIFY_LEFT);
    gtk_misc_set_alignment (GTK_MISC (status_bar), 0, 0.5);
    gtk_misc_set_padding (GTK_MISC (status_bar), 2, 0);


    //gtk_widget_hide (paste_button);

    /* disable the toolbar buttons */

    gtk_widget_set_sensitive (close_button, FALSE);
    gtk_widget_set_sensitive (edit_button, FALSE);
    gtk_widget_set_sensitive (save_button, FALSE);
    gtk_widget_set_sensitive (open_button, FALSE);



    return formular_vbox;
}





GtkWidget * gm_formular_new (GmConnection * connection, BpanelViewInfo * view_info , GSList * old_data_list)
{

    DbTableInfo * table_info;
    DbBrowserNodeInfo * info = NULL;
    GtkWidget * formular_vbox;
    GtkWidget * formular_viewport;

    GtkWidget * entry_widget;
    GtkWidget * button;
    GtkWidget * formular_table;
    GtkWidget * rahmen;
    GtkWidget * fieldlabel;
    GtkWidget * status_bar;
    GSList    * dlist=NULL;
    GList     * flist = NULL;
    gchar     * message ;
    gint  row=0;
    gint spalten = 2;
    gint zeilen ;







    zeilen = g_slist_length (view_info->fields_list);

    formular_vbox = create_formular_vbox (connection);
    info = gm_bpanel_get_node_info (connection, view_info);
    gtk_object_set_data (GTK_OBJECT (formular_vbox), "info", info);

    if (old_data_list)
        gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "old_data_list", old_data_list,
                                  (GtkDestroyNotify) g_slist_g_free);

    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "view_info", view_info,
                              (GtkDestroyNotify) gm_bpanel_free_view_info);

    gtk_object_set_data (GTK_OBJECT (formular_vbox), "connection", connection);

    formular_viewport = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "formular_viewport");

    formular_table = gtk_table_new (zeilen, spalten, FALSE);
    gtk_widget_ref (formular_table);
    gtk_object_set_data_full (GTK_OBJECT (formular_vbox), "formular_table", formular_table,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (formular_table);
    gtk_container_add (GTK_CONTAINER (formular_viewport), formular_table);
    gtk_container_set_border_width (GTK_CONTAINER (formular_table), 6);
    gtk_table_set_row_spacings (GTK_TABLE (formular_table), 1);
    gtk_table_set_col_spacings (GTK_TABLE (formular_table), 1);

    /* ueberschrift */

    rahmen = gtk_frame_new (NULL);
    gtk_widget_show (rahmen);
    /*    gtk_table_attach (GTK_TABLE (formular_table), rahmen, 0, 2, 0, 1,
          (GtkAttachOptions) (GTK_FILL),
          (GtkAttachOptions) (GTK_FILL), 0, 0);*/
    gtk_container_set_border_width (GTK_CONTAINER (rahmen), 1);
    fieldlabel = gtk_label_new (" Please make your changes then hit the update button to  update your database");
    gtk_widget_show (fieldlabel);
    gtk_container_add (GTK_CONTAINER (rahmen), fieldlabel);





    row = 0;
    if (old_data_list)
        dlist = g_slist_nth (old_data_list, 0);
    table_info = view_info->table_info;

    flist = table_info->fields_info_list;

    while (flist)
    {

        DbFieldInfo *field = (DbFieldInfo *) flist->data;
        gchar * entry_name;
        gchar * entry_value=NULL ;



        if (dlist)
            entry_value = (gchar *) dlist->data;

        fieldlabel = gtk_label_new (field->name);
        gtk_misc_set_alignment (GTK_MISC (fieldlabel), 0.0, 0.5);
        /*          gtk_misc_set_padding (GTK_MISC (fieldlabel), 1, 1); */
        gtk_widget_show (fieldlabel);

        gtk_table_attach (GTK_TABLE (formular_table), fieldlabel, 0, 1, row+1, row+2,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        entry_widget = formular_create_gm_entry (field, entry_value);
        if (!GTK_IS_EDITABLE (entry_widget)){
            GtkWidget * entry;
            GtkWidget * top_fieldlabel;

            gtk_object_set_data (GTK_OBJECT (entry_widget), "left_fieldlabel", fieldlabel);

            entry = gm_utils_get_data  (GTK_OBJECT (entry_widget), "fentry");
            top_fieldlabel = gm_utils_get_data  (GTK_OBJECT (entry_widget), "top_fieldlabel");
            gtk_label_set_text (GTK_LABEL (top_fieldlabel), field->name);

            gtk_signal_connect (GTK_OBJECT (entry), "focus_in_event",
                                GTK_SIGNAL_FUNC (formular_on_entry_focus_in_event),
                                formular_vbox);

            gtk_signal_connect (GTK_OBJECT (entry), "focus_out_event",
                                GTK_SIGNAL_FUNC (formular_on_entry_focus_out_event),
                                formular_vbox);
        }







        if (!entry_widget->parent){
            GtkWidget * entry;
            gtk_table_attach (GTK_TABLE (formular_table), entry_widget, 1, 2, row+1, row+2,
                              (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                              (GtkAttachOptions) (GTK_FILL), 2, 2);
            entry = gm_utils_get_data  (GTK_OBJECT (entry_widget), "fentry");
            if (info && info->text_fields) {
                GList * l = info->text_fields;

                while (l) {
                    if (l->data) {
                        if (!strcmp( (gchar *) (l->data), GTK_LABEL (fieldlabel)->label)) {
                            formular_show_text_widget (formular_vbox, entry);
                            break;
                        }
                    }
                    l = l->next;
                }

                //
            }

        }
        else{

            gtk_table_attach (GTK_TABLE (formular_table), entry_widget->parent, 1, 2, row+1, row+2,
                              (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                              (GtkAttachOptions) (GTK_FILL), 2, 2);
        }


        entry_name = g_strdup_printf ("entry_widget_%d",row);
        gtk_widget_ref (entry_widget);
        gtk_object_set_data_full (GTK_OBJECT (formular_vbox), entry_name, entry_widget,
                                  (GtkDestroyNotify) gtk_widget_unref);
        g_free (entry_name);

        gtk_widget_show (entry_widget);

        flist = flist->next;
        if (dlist)
            dlist = dlist->next;
        row++;
    }

    button = gm_utils_get_data  (GTK_OBJECT (formular_vbox),"ok_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (formular_ok_button_clicked),
                        formular_vbox);


    button = gm_utils_get_data  (GTK_OBJECT (formular_vbox),"clone_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (formular_clone_button_clicked),
                        formular_vbox);


    button = gm_utils_get_data  (GTK_OBJECT (formular_vbox),"apply_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (formular_apply_button_clicked),
                        formular_vbox);


    button = gm_utils_get_data  (GTK_OBJECT (formular_vbox),"cancel_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (formular_cancel_button_clicked),
                        formular_vbox);

    /* and finaly fill out some information into the status bar */

    status_bar = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "status_bar");
    message = g_strdup_printf ("Working on localhost:/%s/%s", view_info->database_in_use, view_info->table_in_use);
    gtk_label_set_text (GTK_LABEL (status_bar), message);
    g_free (message);
    gtk_widget_show (formular_vbox);
    return formular_vbox;
}




static void
formular_on_entry_changed (GtkWidget *entry, gpointer user_data)
{
    gm_utils_set_widget_background (entry, "#000000", "#FFFF00");
    gtk_signal_disconnect_by_func (GTK_OBJECT (entry), formular_on_entry_changed, NULL);
}


static GtkWidget* formular_create_entry_text_box (gchar *entry_value)
{
    GtkWidget *entry_text_vbox;
    GtkWidget *fentry;
    GtkWidget *top_fieldlabel;

    entry_text_vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (entry_text_vbox);

    fentry = gm_utils_create_entry_widget ();
    gtk_widget_ref (fentry);
    gtk_object_set_data_full (GTK_OBJECT (entry_text_vbox), "fentry", fentry,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (fentry);

    if (entry_value)
        gtk_entry_set_text (GTK_ENTRY (fentry), entry_value);
    gtk_box_pack_start (GTK_BOX (entry_text_vbox), fentry, FALSE, FALSE, 0);
    top_fieldlabel = gtk_label_new ("");
    gtk_box_pack_start (GTK_BOX (entry_text_vbox),  top_fieldlabel, TRUE, TRUE, 0);
    gtk_object_set_data (GTK_OBJECT (entry_text_vbox), "top_fieldlabel", top_fieldlabel);

    gtk_signal_connect (GTK_OBJECT (fentry), "changed",
                        GTK_SIGNAL_FUNC (formular_on_entry_changed),
                        NULL);
    return entry_text_vbox;
}








gboolean
formular_on_entry_focus_in_event           (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data)
{

    GtkWidget * box = user_data;
    GtkWidget * button;

    button = gm_utils_get_data (box, "edit_button");
    gtk_widget_set_sensitive (button, TRUE);

    button = gm_utils_get_data (box, "copy_button");
    gtk_widget_set_sensitive (button, TRUE);

    button = gm_utils_get_data (box, "paste_button");
    gtk_widget_set_sensitive (button, TRUE);





    return FALSE;
}

gboolean
formular_on_entry_focus_out_event           (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data)
{

    GtkWidget * box = user_data;
    GtkWidget * button;


    button = gm_utils_get_data (box, "edit_button");
    gtk_widget_set_sensitive (button, FALSE);

    button = gm_utils_get_data (box, "copy_button");
    gtk_widget_set_sensitive (button, FALSE);

    button = gm_utils_get_data (box, "paste_button");
    gtk_widget_set_sensitive (button, FALSE);



    return FALSE;
}


gboolean
formular_on_text_focus_in_event           (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data)
{


    GtkWidget * box = user_data;
    GtkWidget * button;

    button = gm_utils_get_data (box, "close_button");
    gtk_widget_set_sensitive (button, TRUE);

    button = gm_utils_get_data (box, "edit_button");
    gtk_widget_set_sensitive (button, FALSE);

    button = gm_utils_get_data (box, "open_button");
    gtk_widget_set_sensitive (button, TRUE);

    button = gm_utils_get_data (box, "save_button");
    gtk_widget_set_sensitive (button, TRUE);

    button = gm_utils_get_data (box, "copy_button");
    gtk_widget_set_sensitive (button, TRUE);

    button = gm_utils_get_data (box, "paste_button");
    gtk_widget_set_sensitive (button, TRUE);


    return FALSE;
}

gboolean
formular_on_text_focus_out_event           (GtkWidget       *widget,
        GdkEventFocus   *event,
        gpointer         user_data)
{

    GtkWidget * box = user_data;
    GtkWidget * button;


    button = gm_utils_get_data (box, "close_button");
    gtk_widget_set_sensitive (button, FALSE);

    button = gm_utils_get_data (box, "open_button");
    gtk_widget_set_sensitive (button, FALSE);

    button = gm_utils_get_data (box, "save_button");
    gtk_widget_set_sensitive (button, FALSE);

    button = gm_utils_get_data (box, "copy_button");
    gtk_widget_set_sensitive (button, FALSE);

    button = gm_utils_get_data (box, "paste_button");
    gtk_widget_set_sensitive (button, FALSE);

    return FALSE;
}





static GtkWidget * formular_widget_focused (GtkWidget * widget)
{
    GList *children;
    GtkContainer * container;
    GtkWidget * child;
    GtkWidget * focused_widget;



    if (!GTK_IS_CONTAINER (widget))
        return NULL;
    container = GTK_CONTAINER (widget);

    children = gtk_container_children (container);
    while (children)
    {
        child = children->data;
        children = children->next;
        focused_widget = formular_widget_focused (child);
        if (focused_widget)
            return focused_widget;
        if (GTK_WIDGET_VISIBLE (child) && GTK_WIDGET_HAS_FOCUS (child))
            return child;
    }
    return NULL;
}



static void formular_open_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox)
{
    GtkWidget * focused_widget;
    GmConnection * connection;
    GtkTable	  *table;

    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    table = GTK_TABLE (gm_utils_get_data  (GTK_OBJECT (formular_vbox), "formular_table"));
    focused_widget = formular_widget_focused (formular_vbox);

    if (focused_widget)
        gm_utils_text_open_activated (NULL, GTK_EDITABLE (focused_widget));
}



static void formular_save_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox)
{
    GtkWidget * focused_widget;
    GmConnection * connection;
    GtkTable	  *table;

    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    table = GTK_TABLE (gm_utils_get_data  (GTK_OBJECT (formular_vbox), "formular_table"));
    focused_widget = formular_widget_focused (formular_vbox);

    if (focused_widget)
        gm_utils_text_save_under_activated (NULL, GTK_EDITABLE (focused_widget));
}


static void formular_copy_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox)
{
    GtkWidget * focused_widget;
    GmConnection * connection;
    GtkTable	  *table;

    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    table = GTK_TABLE (gm_utils_get_data  (GTK_OBJECT (formular_vbox), "formular_table"));
    focused_widget = formular_widget_focused (formular_vbox);

    if (focused_widget)
        gm_utils_text_copy_activated (NULL, GTK_EDITABLE (focused_widget));
}

static void formular_paste_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox)
{
    GtkWidget * focused_widget;
    GmConnection * connection;
    GtkTable	  *table;

    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    table = GTK_TABLE (gm_utils_get_data  (GTK_OBJECT (formular_vbox), "formular_table"));
    focused_widget = formular_widget_focused (formular_vbox);

    if (focused_widget)
        gm_utils_text_paste_activated (NULL, GTK_EDITABLE (focused_widget));
}





static void formular_show_text_widget (GtkWidget * formular_vbox, GtkWidget * focused_widget)
{


    GmConnection * connection;
    GtkTable	  *table;

    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    table = GTK_TABLE (gm_utils_get_data  (GTK_OBJECT (formular_vbox), "formular_table"));

    if (focused_widget){
        GdkColor color;
        GtkWidget * entry_text_vbox = focused_widget->parent;
        GtkWidget * text_sw;
        GtkWidget * text;
        GtkWidget * entry;
        gchar * entry_value;
        GList * l;


        entry = gm_utils_get_data  (entry_text_vbox, "fentry");
        text_sw = gtk_object_get_data  (GTK_OBJECT (entry_text_vbox), "text_sw");
        text = gtk_object_get_data  (GTK_OBJECT (entry_text_vbox), "text");
        if (!text_sw) {
            text_sw = gtk_scrolled_window_new (NULL, NULL);
            gtk_widget_ref (text_sw);
            gtk_object_set_data_full (GTK_OBJECT (entry_text_vbox), "text_sw", text_sw,
                                      (GtkDestroyNotify) gtk_widget_unref);
            gtk_widget_show (text_sw);
            gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (text_sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
            gtk_box_pack_start (GTK_BOX (entry_text_vbox), text_sw, TRUE, TRUE, 0);

            text = gm_utils_create_text_widget ();
            gtk_signal_connect (GTK_OBJECT (text), "focus_in_event",
                                GTK_SIGNAL_FUNC (formular_on_text_focus_in_event),
                                formular_vbox);
            gtk_signal_connect (GTK_OBJECT (text), "focus_out_event",
                                GTK_SIGNAL_FUNC (formular_on_text_focus_out_event),
                                formular_vbox);

            gtk_widget_ref (text);
            gtk_object_set_data_full (GTK_OBJECT (entry_text_vbox), "text", text,
                                      (GtkDestroyNotify) gtk_widget_unref);
            gtk_widget_show (text);
            gtk_text_set_editable (GTK_TEXT (text), TRUE);
            gtk_widget_set_usize (text, -1, 250);
            gtk_container_add (GTK_CONTAINER (text_sw), text);
            gtk_widget_show (text_sw);

            /*            button = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "close_button");
                        gtk_widget_set_sensitive (button, TRUE);    

                        button = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "edit_button");
                        gtk_widget_set_sensitive (button, FALSE);
                        button = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "open_button");
                        gtk_widget_set_sensitive (button, TRUE);
                        button = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "save_button");
                        gtk_widget_set_sensitive (button, TRUE);            */
        }


        gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
        entry_value = gtk_entry_get_text (GTK_ENTRY (entry));
        gdk_color_parse ("black",&color);
        gtk_text_insert ( GTK_TEXT (text), NULL,
                          &color,
                          NULL,
                          entry_value,
                          strlen (entry_value) );
        gtk_widget_hide (entry);
        gtk_widget_hide (gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "left_fieldlabel"));
        gtk_widget_show (gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "top_fieldlabel"));
        for (l=g_list_first (table->children);l;l=l->next) {
            GtkTableChild *table_child = l->data;

            if (table_child->widget == entry_text_vbox) {
                table_child->left_attach = 0;
                gtk_widget_queue_resize (table_child->widget);
            }
        }
        gtk_widget_show (text_sw);
        gtk_widget_grab_focus (text);

    }
}

static void formular_edit_button_clicked (GtkButton * ignore,
        GtkWidget * formular_vbox)
{

    DbBrowserNodeInfo * info = NULL;
    GtkWidget * focused_widget;
    GmConnection * connection;
    GtkTable	  *table;
    GtkLabel  * label = NULL;
    gboolean already_in_the_list = FALSE;
    GList * l ;


    connection = gm_utils_get_data  (GTK_OBJECT (formular_vbox), "connection");
    table = GTK_TABLE (gm_utils_get_data  (GTK_OBJECT (formular_vbox), "formular_table"));
    focused_widget = formular_widget_focused (formular_vbox);
    formular_show_text_widget (formular_vbox, focused_widget);
    if (focused_widget->parent)
        label = GTK_LABEL (gm_utils_get_data  (focused_widget->parent, "left_fieldlabel"));
    info = gm_utils_get_data (formular_vbox, "info");
    if (info && label) {
        for (l=g_list_first (info->text_fields);l;l=l->next) {
            if (l->data) {
                if (!strcmp( (gchar *) (l->data), label->label)) {
                    already_in_the_list = TRUE;
                    break;
                }

            }
        }

        if (!already_in_the_list)
            info->text_fields = g_list_append (info->text_fields, g_strdup (label->label));
    }


}


static void formular_close_button_clicked (GtkButton * button,
        GtkWidget * formular_vbox)
{
    GtkWidget * focused_widget;
    GtkTable	  *table;
    GtkWidget * entry_text_vbox;
    GtkWidget * text_sw;
    GtkWidget * text;
    GtkWidget * entry;
    gchar * entry_value;
    GList * l;
    GtkLabel * label;
    DbBrowserNodeInfo * info = NULL;


    focused_widget = formular_widget_focused (formular_vbox);


    if (focused_widget && GTK_IS_TEXT (focused_widget)){
        entry_text_vbox = (focused_widget->parent)->parent;  // scrolled win --> vbox
        label = GTK_LABEL (gm_utils_get_data  (entry_text_vbox, "left_fieldlabel"));
        info = gm_utils_get_data (formular_vbox, "info");
        if (info && info->text_fields) {
            for (l=g_list_first (info->text_fields);l;l=l->next) {
                if (l->data) {
                    if (!strcmp( (gchar *) (l->data), label->label)) {
                        g_free (l->data);
                        info->text_fields = g_list_remove (info->text_fields, l->data);
                        break;
                    }
                }
            }
        }


        table = GTK_TABLE (gm_utils_get_data  (GTK_OBJECT (formular_vbox), "formular_table"));
        entry = gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "fentry");
        text_sw = gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "text_sw");
        text = gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "text");

        entry_value = gtk_editable_get_chars (GTK_EDITABLE (text), 0,-1);
        gtk_entry_set_text (GTK_ENTRY (entry), entry_value);
        g_free (entry_value);
        gtk_widget_hide (text_sw);
        gtk_widget_hide (gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "top_fieldlabel"));
        gtk_widget_show (gm_utils_get_data  (GTK_OBJECT (entry_text_vbox), "left_fieldlabel"));
        for (l=g_list_first (table->children);l;l=l->next) {
            GtkTableChild *table_child = l->data;

            if (table_child->widget == entry_text_vbox) {
                table_child->left_attach = 1;
                gtk_widget_queue_resize (table_child->widget);
            }
        }
        gtk_widget_show (entry);
        gtk_widget_grab_focus (entry);
    }

}






