/*
 *   Based on Code from 
 *   Authors: Ramiro Estrugo <ramiro@eazel.com>
 *            Modified to fit the needs of gmyclient by :
 *            Mohamed Elassadi <mohamed@users.sourceforge.net>
 */

#include "gm.h"
#include "gm-horizontal-splitter.h"

#include "gm-gtk-macros.h"

#include <stdlib.h>

struct GmHorizontalSplitterDetails {
    gboolean down;
    double down_position;
    guint32 down_time;
};

/* Bar width currently hardcoded to 7 */
#define BAR_WIDTH 6
#define CLOSED_THRESHOLD 14
#define NOMINAL_SIZE 148
#define SPLITTER_CLICK_SLOP 1
#define SPLITTER_CLICK_TIMEOUT	400

enum {
    COLLAPSED,
    LAST_SIGNAL
};
static guint splitter_signals[LAST_SIGNAL] = { 0 };

/* GmHorizontalSplitterClass methods */
static void     gm_horizontal_splitter_initialize_class (GmHorizontalSplitterClass *horizontal_splitter_class);
static void     gm_horizontal_splitter_initialize       (GmHorizontalSplitter      *horizontal_splitter);
static gboolean gm_horizontal_splitter_button_press     (GtkWidget                       *widget,
        GdkEventButton                  *event);
static gboolean gm_horizontal_splitter_button_release   (GtkWidget                       *widget,
        GdkEventButton                  *event);
static gboolean gm_horizontal_splitter_motion           (GtkWidget      		       *widget,
        GdkEventMotion 		       *event);

/* GtkObjectClass methods */
static void     gm_horizontal_splitter_destroy          (GtkObject                       *object);

/* GtkWidgetClass methods */
static void     gm_horizontal_splitter_draw             (GtkWidget                       *widget,
        GdkRectangle                    *area);

GM_DEFINE_CLASS_BOILERPLATE (GmHorizontalSplitter,
                             gm_horizontal_splitter,
                             E_TYPE_HPANED)

/* GtkObjectClass methods */
static void
gm_horizontal_splitter_initialize_class (GmHorizontalSplitterClass *horizontal_splitter_class)
{
    GtkObjectClass *object_class;
    GtkWidgetClass *widget_class;

    object_class = GTK_OBJECT_CLASS (horizontal_splitter_class);
    widget_class = GTK_WIDGET_CLASS (horizontal_splitter_class);

    /* GtkObjectClass */
    object_class->destroy = gm_horizontal_splitter_destroy;

    /* GtkWidgetClass */
    widget_class->draw = gm_horizontal_splitter_draw;
    widget_class->button_press_event = gm_horizontal_splitter_button_press;
    widget_class->button_release_event = gm_horizontal_splitter_button_release;
    widget_class->motion_notify_event = gm_horizontal_splitter_motion;

    splitter_signals[COLLAPSED] =
        gtk_signal_new ("collapsed",
                        GTK_RUN_FIRST,
                        object_class->type,
                        GTK_SIGNAL_OFFSET (GmHorizontalSplitterClass, collapsed),
                        gtk_marshal_NONE__INT,
                        GTK_TYPE_NONE, 1,
                        GTK_TYPE_INT);



    gtk_object_class_add_signals (object_class, splitter_signals, LAST_SIGNAL);


}

static void
gm_horizontal_splitter_initialize (GmHorizontalSplitter *horizontal_splitter)
{
    horizontal_splitter->details = g_new0 (GmHorizontalSplitterDetails, 1);
    e_paned_set_handle_size (E_PANED (horizontal_splitter), BAR_WIDTH);
}

/* GtkObjectClass methods */
static void
gm_horizontal_splitter_destroy (GtkObject *object)
{
    GmHorizontalSplitter *horizontal_splitter;

    horizontal_splitter = GM_HORIZONTAL_SPLITTER (object);

    g_free (horizontal_splitter->details);

    /* Chain */
    GM_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
}

static void
draw_resize_bar (GtkWidget		*widget,
                 GdkWindow		*window,
                 const GdkRectangle	*area)
{
    GtkStyle	*style;
    EPaned		*paned;
    GdkGC		*gcs[BAR_WIDTH];
    guint		i;




    g_assert (widget != NULL);
    g_assert (window != NULL);
    g_assert (area != NULL);
    g_assert (area->width == BAR_WIDTH);

    style = widget->style;
    paned = E_PANED(widget);

    g_assert (style != NULL);




    gcs[0] = style->dark_gc[GTK_STATE_NORMAL];
    gcs[1] = style->mid_gc[GTK_STATE_ACTIVE];
    gcs[2] = style->light_gc[GTK_STATE_ACTIVE];
    gcs[3] = style->bg_gc[GTK_STATE_NORMAL];
    gcs[4] = style->mid_gc[GTK_STATE_ACTIVE];
    gcs[5] = style->dark_gc[GTK_STATE_NORMAL];



    for (i = 0; i < BAR_WIDTH; i++)
    {
        gdk_draw_line (window,
                       gcs[i],
                       area->x + i ,
                       area->y,
                       area->x + i ,
                       area->y + area->height - 1);
    }

    i = 0;
    gdk_draw_line (window,
                   gcs[i],
                   area->x + i ,
                   area->y,
                   area->x + i ,
                   area->y + area->height - 1);
    i = 5;
    gdk_draw_line (window,
                   gcs[i],
                   area->x + i ,
                   area->y,
                   area->x + i ,
                   area->y + area->height - 1);

    /*  	gc = gdk_gc_new(gpixmap->pixmap);
    	gdk_draw_pixmap(window, gc, gpixmap->pixmap, 0, 0, 0, 0,
            20, 40); 
      	gdk_gc_destroy(gc);*/
}

/* Number of ridges in the thumb currently hardcoded to 8 */
#define NUM_RIDGES 8

/* These control the layout of the ridges */
#define RIDGE_HEIGHT 2
#define RIDGE_EDGE_OFFSET 2
#define BETWEEN_RIDGE_OFFSET 1


static void
draw_resize_bar_thumb (GtkWidget		*widget,
                       GdkWindow		*window,
                       const GdkRectangle	*area)
{
    EPaned		*paned;
    GtkStyle	*style;
    GdkGC		*light_gc;
    GdkGC		*dark_gc;
    guint		total_thumb_height;

    g_assert (widget != NULL);
    g_assert (window != NULL);
    g_assert (area != NULL);
    g_assert (area->width == BAR_WIDTH);

    style = widget->style;
    g_assert (style != NULL);

    paned = E_PANED (widget);

    light_gc = style->light_gc[GTK_STATE_ACTIVE];
    dark_gc = style->fg_gc[GTK_STATE_SELECTED];

    total_thumb_height = (NUM_RIDGES * RIDGE_HEIGHT) + ((NUM_RIDGES - 1) * BETWEEN_RIDGE_OFFSET);

    /* Make sure the thumb aint bigger than the handle */
    if (total_thumb_height > paned->handle_height)
    {
        total_thumb_height = paned->handle_height;
    }

    /* Draw the thumb only if we have enough space for at least one ridge */
    if (total_thumb_height > RIDGE_HEIGHT)
    {
        gint	y = paned->handle_ypos + (paned->handle_height - total_thumb_height) / 2;
        guint	i;

        for (i = 0; i < NUM_RIDGES; i++)
        {
            gint x1 = area->x + RIDGE_EDGE_OFFSET;
            gint x2 = area->x + BAR_WIDTH - RIDGE_EDGE_OFFSET;

            gdk_draw_line (window,
                           dark_gc,
                           x1,
                           y,
                           x2,
                           y-2);

            y += BETWEEN_RIDGE_OFFSET;

            gdk_draw_line (window,
                           light_gc,
                           x1,
                           y,
                           x2,
                           y-2);

            y += RIDGE_HEIGHT;
        }
    }
}

/* GtkWidgetClass methods */
static void
gm_horizontal_splitter_draw (GtkWidget    *widget,
                             GdkRectangle *area)
{
    EPaned *paned;
    GdkRectangle handle_area, child_area;
    guint16 border_width;

    g_return_if_fail (widget != NULL);
    g_return_if_fail (E_IS_PANED (widget));

    /*          GTK_WIDGET_CLASS (parent_class)->draw (widget, area);         */
    /*          return; */

    if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget)) {
        paned = E_PANED (widget);
        border_width = GTK_CONTAINER (paned)->border_width;

        gdk_window_clear_area (widget->window,
                               area->x, area->y, area->width,
                               area->height);

        if (e_paned_handle_shown(paned)) {
            handle_area.x = paned->handle_xpos;
            handle_area.y = paned->handle_ypos;
            handle_area.width = paned->handle_size;
            handle_area.height = paned->handle_height;

            if (gdk_rectangle_intersect (&handle_area, area, &child_area)) {
                child_area.x -= paned->handle_xpos;
                child_area.y -= paned->handle_ypos;

                /* Simply things by always drawing the full width of the bar. */
                child_area.x = 0;
                child_area.width = BAR_WIDTH;

                draw_resize_bar (widget, paned->handle, &child_area);
                draw_resize_bar_thumb (widget, paned->handle, &child_area);
            }
        }

        /* Redraw the children
         */
        if (paned->child1 && gtk_widget_intersect (paned->child1, area, &child_area)) {
            gtk_widget_draw (paned->child1, &child_area);
        }

        if (paned->child2 && gtk_widget_intersect (paned->child2, area, &child_area)) {
            gtk_widget_draw (paned->child2, &child_area);
        }
    }
}

void
gm_horizontal_splitter_expand (GmHorizontalSplitter *splitter)
{
    int position;

    g_return_if_fail (GM_IS_HORIZONTAL_SPLITTER (splitter));

    position = e_paned_get_position (E_PANED (splitter));

    if (position >= CLOSED_THRESHOLD && position <= (E_PANED (splitter)->max_position - CLOSED_THRESHOLD)) {
        return;
    }

    position = E_PANED(splitter)->saved_position;


    if (position < CLOSED_THRESHOLD || position > (E_PANED (splitter)->max_position - CLOSED_THRESHOLD) ) {
        position = NOMINAL_SIZE;
    }

    e_paned_set_position (E_PANED (splitter), position);
    gtk_widget_show (GTK_BIN(E_PANED(splitter)->child1)->child);
    gtk_signal_emit (GTK_OBJECT (splitter), splitter_signals[COLLAPSED], SPLITTER_OPENED);
}

static void        gm_horizontal_splitter_set_saved_position (GmHorizontalSplitter * splitter, gint position)
{
    gm_spliter_set_saved_position (E_PANED (splitter), position);
    GE_CONFIG(global_app)->horizontal_paned_position = position;
}


void
gm_horizontal_splitter_collapse (GmHorizontalSplitter *splitter, gint direction)
{
    int position;

    g_return_if_fail (GM_IS_HORIZONTAL_SPLITTER (splitter));

    position = e_paned_get_position (E_PANED (splitter));

    if (position < CLOSED_THRESHOLD) {
        return;
    }
    gm_horizontal_splitter_set_saved_position (splitter, position);

    switch (direction) {
    case GO_LEFT:
        {
            e_paned_set_position (E_PANED (splitter), 0);
            gtk_widget_hide (GTK_BIN(E_PANED(splitter)->child1)->child);
            break;
        }
    case GO_RIGHT :
        {
            e_paned_set_position (E_PANED (splitter), E_PANED (splitter)->max_position);
        }
    }
    gtk_signal_emit (GTK_OBJECT (splitter), splitter_signals[COLLAPSED], SPLITTER_CLOSED);
}

/* routine to toggle the open/closed state of the splitter */

void
gm_horizontal_splitter_toggle_position (GmHorizontalSplitter *splitter, gint direction)
{
    gint position;

    g_return_if_fail (GM_IS_HORIZONTAL_SPLITTER (splitter));

    position = e_paned_get_position (E_PANED (splitter));

    if (position >= CLOSED_THRESHOLD && position <= (E_PANED(splitter)->max_position - CLOSED_THRESHOLD)) {
        gm_horizontal_splitter_collapse (splitter,  direction);
    } else {
        gm_horizontal_splitter_expand (splitter);
    }


}

/* GmHorizontalSplitter public methods */
GtkWidget *
gm_horizontal_splitter_new (void)
{
    return gtk_widget_new (gm_horizontal_splitter_get_type (), NULL);
}

/* handle mouse downs by remembering the position and the time */
static gboolean
gm_horizontal_splitter_button_press (GtkWidget *widget, GdkEventButton *event)
{
    GmHorizontalSplitter *splitter;

    splitter = GM_HORIZONTAL_SPLITTER (widget);

    if (event->window != E_PANED (widget)->handle) {
        splitter->details->down = FALSE;
    } else {
        splitter->details->down = TRUE;
        splitter->details->down_position = event->x;
        splitter->details->down_time = event->time;
    }

    return GM_CALL_PARENT_WITH_RETURN_VALUE
           (GTK_WIDGET_CLASS, button_press_event, (widget, event));
}


static void
splitter_xor_line (EPaned *paned)
{
    GtkWidget *widget;
    GdkGCValues values;
    guint16 xpos, half_width;
    gint8 dash_list[2];

    widget = GTK_WIDGET(paned);

    if (!paned->xor_gc) {
        values.function = GDK_INVERT;
        values.subwindow_mode = GDK_INCLUDE_INFERIORS;
        paned->xor_gc = gdk_gc_new_with_values (widget->window, &values,
                                                GDK_GC_FUNCTION | GDK_GC_SUBWINDOW);
    }

    gdk_gc_set_line_attributes (paned->xor_gc, 1, GDK_LINE_ON_OFF_DASH,
                                GDK_CAP_NOT_LAST, GDK_JOIN_BEVEL);

    /* Make line appear as every other pixel dash */
    dash_list[0] = 1;
    dash_list[1] = 1;
    gdk_gc_set_dashes (paned->xor_gc, 1, dash_list, 2);

    xpos = paned->child1_size + GTK_CONTAINER (paned)->border_width + paned->handle_size / 2;
    half_width = paned->handle_size / 2;

    gdk_draw_line (widget->window, paned->xor_gc, xpos - half_width, 0, xpos - half_width,
                   widget->allocation.height - 1);

    gdk_draw_line (widget->window, paned->xor_gc, xpos + half_width, 0, xpos + half_width,
                   widget->allocation.height - 1);

}


/* handle mouse ups by seeing if it was a tap and toggling the open state accordingly */

static gboolean
gm_horizontal_splitter_button_release (GtkWidget *widget, GdkEventButton *event)
{
    GmHorizontalSplitter *splitter;
    int delta, delta_time;
    EPaned *paned;
    gboolean priority;

    splitter = GM_HORIZONTAL_SPLITTER (widget);
    paned = E_PANED (widget);
    priority = e_paned_get_priority (paned);
    if (event->window == E_PANED (widget)->handle
            && splitter->details->down) {
        delta = abs (event->x - splitter->details->down_position);
        delta_time = abs (splitter->details->down_time - event->time);
        if (delta < SPLITTER_CLICK_SLOP && delta_time < SPLITTER_CLICK_TIMEOUT)  {
            if (!priority){
                gm_horizontal_splitter_toggle_position (splitter, GO_RIGHT);
            }else {
                gm_horizontal_splitter_toggle_position (splitter, GO_LEFT);
            }
        }else {
            gint position;

            position = e_paned_get_position (paned);
            gm_horizontal_splitter_set_saved_position (splitter, position);
            gtk_widget_show (GTK_BIN(E_PANED(splitter)->child1)->child);

        }



    }

    splitter->details->down = FALSE;
    if (paned->in_drag && (event->button == 1)) {
        splitter_xor_line (paned);
        paned->in_drag = FALSE;
        paned->position_set = TRUE;
        gdk_pointer_ungrab (event->time);
        gtk_widget_queue_resize (GTK_WIDGET (paned));
        return TRUE;
    }

    return FALSE;
}

static gboolean
gm_horizontal_splitter_motion (GtkWidget *widget, GdkEventMotion *event)
{
    EPaned *paned;
    gint x;

    g_return_val_if_fail (widget != NULL, FALSE);
    g_return_val_if_fail (E_IS_PANED (widget), FALSE);

    paned = E_PANED (widget);

    if (event->is_hint || event->window != widget->window) {
        gtk_widget_get_pointer(widget, &x, NULL);
    } else {
        x = event->x;
    }

    if (paned->in_drag) {
        gint size = x - GTK_CONTAINER (paned)->border_width - paned->handle_size / 2;
        splitter_xor_line (paned);
        paned->child1_size = CLAMP (e_paned_quantized_size (paned, size), paned->min_position, paned->max_position);
        splitter_xor_line (paned);
    }

    return TRUE;
}





