/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include "gm.h"
#include "gm-ccci.h"
#include "gm-entry.h"
#include "gm-bpanel.h"
#include "gm-qbuffer.h"
#include "glade_interface.h"
#include "gm-interface.h"
#include "gm-control-center.h"
#include "gm-dbapi.h"
#include "gm-dumpin.h"
#include "gm-dumpout.h"
#include "gm-formular.h"
#include "gm-table-properties.h"
#include "gm-ccci.h"
#include "gm-vertical-splitter.h"
#include "gm-dbtree.h"
#include "gm-horizontal-splitter.h"
#include "gm-print.h"
#include "gm-dialogs.h"
#include "gm-mysql-admin.h"







static void interface_change_panes (GtkWidget *connection_window);
static void interface_horizontal_splitter_collapsed     (GmVerticalSplitter *horizontal_splitter, gint state, GtkWidget *connection_window);
static void interface_vertical_splitter_collapsed       (GmVerticalSplitter *vertical_splitter, gint state, GtkWidget *connection_window);
static void interface_find_dialog_on_destroy            (GtkWidget *find_dialog, GmConnection *connection);
static void interface_find_dialog_find_button_clicked   (GtkButton *button, GmConnection *connection);
static void interface_find_dialog_cancel_button_clicked (GtkButton *button, GmConnection *connection);
static void interface_close_tree_panel                  (void *ignore, GmConnection *connection);
static void interface_clear_selected_window             (GtkButton *button, GmConnection *connection);
static void interface_clip_window_on_off_toggled        (GtkToggleButton *togglebutton, GmConnection *connection);
static void interface_sql_log_panel_on_off_toggled          (GtkToggleButton *togglebutton, GmConnection *connection);
static void interface_tree_panel_view_hide_toggled      (GtkToggleButton *togglebutton, GmConnection *connection);
static void interface_help_button_clicked               (void  *ignore, GmConnection *connection);
static void interface_update_button_clicked               (void  *ignore, GmConnection *connection);
static void interface_exit_button_clicked               (void  *ignore, GmConnection *connection);
static void interface_about_button_clicked              (void  *ignore, GmConnection *connection);
static void interface_dumpin_button_clicked             (void  *ignore, GmConnection *connection);
static void interface_new_connection_button_clicked     (GtkButton *button, GmConnection *connection);
static void interface_new_query_page_button_clicked     (GtkButton *button, GmConnection *connection);
static void interface_remove_page_button_clicked        (GtkButton *button, GmConnection *connection);
static void interface_create_query_buffer               (GmConnection *connection);
static void interface_options_button_clicked            (GtkButton *button, GmConnection *connection);
static void interface_reload                            (void *ignore, GmConnection *connection);
static void interface_configure_menu_signals            (GmConnection *connection );
static void interface_show_find_dialog                  (void *ignore, GmConnection *connection);
static void interface_close_tree_panel                  (void *ignore, GmConnection *connection) ;

static gint interface_find_dialog_key_pressed               (GtkWidget *widget, GdkEventKey  *event, GmConnection *connection);
static gboolean interface_window_deleted                    (GtkWidget *widget, GdkEvent *event, GmConnection *connection);
static gboolean interface_connection_window_key_press_event (GtkWidget       *widget, GdkEventKey     *event, GmConnection *connection);
static gboolean interface_execute_current_npage             (GmConnection *connection);
static void interface_sql_log_book_switch_page              (GtkWidget *sql_log_book, GtkNotebookPage *page, gint page_num, GmConnection *connection);
static void interface_main_note_book_switch_page              (GtkWidget *sql_log_book, GtkNotebookPage *page, gint page_num, GmConnection *connection);
static void interface_configure_sql_log_signals             (GmConnection *connection) ;



static void
interface_reload (void *ignore, GmConnection *connection)
{
    GtkWidget *dbtree;
    GtkWidget *main_note_book;
    GtkWidget *current_npage;
    GmNpageData *npage_data;


    dbtree = connection->dbtree;
    if (GTK_WIDGET_HAS_FOCUS (dbtree)){        
        gm_dbtree_rebuild_tree (NULL, connection);
        return;
    }
    
    

    main_note_book  = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
    current_npage   = gtk_notebook_get_nth_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_note_book)));
    npage_data      = (GmNpageData *) gm_utils_get_data_ww  (GTK_OBJECT (current_npage), "npage_data");

    if (!npage_data)
        return ;

    
    switch (npage_data->type)
    {
        case GM_NPAGE_BROWSER:
        {
            gm_bpanel_reload (NULL, connection);
            break;
        }
        case GM_NPAGE_VARIABLES :            
        case GM_NPAGE_USER_QUERY:
        {
            if (npage_data->query) {
                gm_interface_execute_query_and_show_result (connection, npage_data->query, FALSE);                
            }
            break;            
        }            
    case GM_NPAGE_ROWEDIT:
    case GM_NPAGE_TABLE_PROP:
        default:            
        return;        
    }

};


static gboolean
interface_execute_current_npage (GmConnection *connection)
{

    GtkWidget *main_note_book;
    GtkWidget *current_npage;
    GmNpageData *npage_data;


    main_note_book  = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
    current_npage   = gtk_notebook_get_nth_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_note_book)));
    npage_data      = (GmNpageData *) gm_utils_get_data_ww  (GTK_OBJECT (current_npage), "npage_data");

    if (!npage_data)
        return FALSE;
    switch (npage_data->type)
    {
    case GM_NPAGE_BROWSER:
    case GM_NPAGE_USER_QUERY:
        return FALSE;
    case GM_NPAGE_ROWEDIT:
        formular_ok_button_clicked (NULL,
                                    current_npage);
        return TRUE;
    case GM_NPAGE_TABLE_PROP:

        {
            GtkWidget *fp_win = gm_utils_get_data  (GTK_OBJECT (current_npage), "fields_properties_window");
            gm_table_properties_execute (fp_win) ;
            return TRUE;
        }
    }
    return FALSE;
}

static void
interface_options_button_clicked (GtkButton *button,
                                  GmConnection *connection)
{
    gm_cc_show_option_dialog (global_app->cc, OPTIONS_GENERAL_PAGE);
}



static void
interface_show_print_options (GtkButton *button,
                              GmConnection *connection)
{
    gm_cc_show_option_dialog (global_app->cc, OPTIONS_PRINT_PAGE);
}



static void
interface_remove_page_button_clicked (GtkButton       *button,
                                      GmConnection *connection)
{

    GtkWidget *main_note_book;

    main_note_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
    if (g_list_length (GTK_NOTEBOOK (main_note_book)->children)>1)
    {
        gint page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (main_note_book));
        if (page_num != 0){
            gtk_notebook_remove_page (GTK_NOTEBOOK (main_note_book), page_num);
        }
    }
}





static void
interface_new_connection_button_clicked (GtkButton       *button,
        GmConnection *connection)
{
    gm_app_new_connection ();
}


static void
interface_new_query_page_button_clicked (GtkButton       *button,
        GmConnection *connection)
{


    gm_interface_new_query_page (connection, TRUE);

}


static void
interface_dumpin_button_clicked (void *ignore,
                                 GmConnection *connection)
{
    BpanelViewInfo *view_info;
    GmNpageData *npage_data;
    GtkWidget *data_list;
    GtkWidget *npage;
    gchar *database = NULL;
    g_return_if_fail ( connection != NULL);
    npage = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "dbbrowser_page_vbox");
    npage_data = (GmNpageData *) gm_utils_get_data  (GTK_OBJECT (npage), "npage_data");
    data_list = npage_data->data_list;
    if (!data_list)
        return;
    view_info = gm_utils_get_data  (GTK_OBJECT (data_list), "view_info");

    switch (view_info->view_type)
    {

    case DATABASES_VIEW :
        {
            gint sel_row;
            GList *list = GTK_CLIST (data_list)->selection;

            if (!list)
                break;
            if (g_list_length (list) == 1 && GPOINTER_TO_INT (list->data) == 0)
                break;
            sel_row = GPOINTER_TO_INT (list->data);
            gtk_clist_get_text              (GTK_CLIST (data_list), sel_row, 0, &database);
            break;
        }

    case TABLES_VIEW :
    default:

        {
            database = view_info->database_in_use;
            break;
        }
    }



    gm_dumpin_dialog_new (connection, database);


}


static void
interface_about_button_clicked (void  *ignore,
                                GmConnection *connection)
{
    GtkWidget *about_window;


    about_window = create_about_window ();
    gtk_widget_show (about_window);
}





static void
interface_exit_button_clicked (void  *ignore,
                               GmConnection *connection)
{
    gm_application_exit_immidiatly (NULL, connection) ;
}



static void
interface_help_button_clicked (void  *ignore,
                               GmConnection *connection)
{
    gchar *path;

    path = g_strdup_printf ("ghelp://%s/doc/index.html", PACKAGE_DATA_DIR);
    gnome_help_goto              (NULL, path);
    g_free (path);
}

static void
interface_update_button_clicked (void  *ignore,
                               GmConnection *connection)
{
    gm_utils_check_update();    
}




static void
interface_tree_panel_view_hide_toggled (GtkToggleButton *togglebutton,
                                        GmConnection *connection)
{
    gboolean active;

    GmHorizontalSplitter *horizontal_splitter;
    GmCcenterGeneral *ge_config;

    
    ge_config = GE_CONFIG (global_app);
    active    = gtk_toggle_button_get_active (togglebutton);
    horizontal_splitter = GM_HORIZONTAL_SPLITTER (gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "horizontal_splitter"));

    if (active)
    {
        if (gm_splitter_is_collapsed (E_PANED (horizontal_splitter)))
            gm_horizontal_splitter_toggle_position (GM_HORIZONTAL_SPLITTER (horizontal_splitter), GO_DOWN);
    }
    else
    {
        if (!gm_splitter_is_collapsed (E_PANED (horizontal_splitter)))
            gm_horizontal_splitter_toggle_position (GM_HORIZONTAL_SPLITTER (horizontal_splitter), GO_UP);
    }
    ge_config->view_state= (active)? ge_config->view_state | GM_DBTREE_VISIBLE:ge_config->view_state &  ~ (GM_DBTREE_VISIBLE);
}


static void
interface_sql_log_panel_on_off_toggled (GtkToggleButton *togglebutton,
                                    GmConnection *connection)
{
    gboolean active;
    GmVerticalSplitter *vertical_splitter;
    GmCcenterGeneral *ge_config;

    ge_config = GE_CONFIG (global_app);
    active    = gtk_toggle_button_get_active (togglebutton);
    vertical_splitter = GM_VERTICAL_SPLITTER (gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "vertical_splitter"));
    if (active)
    {
        if (gm_vertical_splitter_is_collapsed (vertical_splitter)) {
            gm_vertical_splitter_toggle_position (GM_VERTICAL_SPLITTER (vertical_splitter), GO_UP);
        }
    }
    else
    {
        gboolean is_collapsed =gm_vertical_splitter_is_collapsed (vertical_splitter);
        
        if (!is_collapsed){
            gm_vertical_splitter_toggle_position (GM_VERTICAL_SPLITTER (vertical_splitter), GO_DOWN);
        }
        
    }
    ge_config->view_state= (active)? ge_config->view_state | GM_SQL_LOG_PANEL_VISIBLE:ge_config->view_state &  ~ (GM_SQL_LOG_PANEL_VISIBLE);

}




static void
interface_clip_window_on_off_toggled (GtkToggleButton *togglebutton,
                                      GmConnection *connection)
{
    gboolean active;
    GtkWidget *clip_window;
    active    = gtk_toggle_button_get_active (togglebutton);
    clip_window = global_app->clip_window;

    if (active)
    {
        if (!clip_window)
            clip_window = gm_ccci_create_clip_window (connection);
        else
            gm_ccci_update_clip_window (connection);
        gtk_widget_show (clip_window);
    }
    else
    {
        if (clip_window)
            gtk_widget_hide (clip_window);
    }
}


static void
interface_clear_selected_window (GtkButton *button,
                                 GmConnection *connection)
{

    GtkWidget *log_book;
    gint  current_npage;
    GtkWidget *window;

    log_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "sql_log_book");
    current_npage   = gtk_notebook_get_current_page (GTK_NOTEBOOK (log_book));

    switch (current_npage) {
        case SQL_NOTEBOOK_PAGE:
            gtk_editable_delete_text (GTK_EDITABLE (connection->query_buffer), 0, -1);            
            break;            
    case     HISTORY_NOTEBOOK_PAGE:
        {
            GtkCList * clist = GTK_CLIST (connection->qhistory_mlist);
            if (!clist->selection)
                gtk_clist_clear (GTK_CLIST (connection->qhistory_mlist));
            else {
                while (clist->selection){
                    gint row = GPOINTER_TO_INT ((clist->selection)->data);
                    gtk_clist_remove (clist, row);
                }
            }
            
            break;
        }
    case     MESSAGES_NOTEBOOK_PAGE:
        {
            window = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "messages_window");
            gtk_editable_delete_text (GTK_EDITABLE (window), 0, -1);
            break;
        }
    case      ERRORS_NOTEBOOK_PAGE:
        {
            window = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "errors_window");
            gtk_editable_delete_text (GTK_EDITABLE (window), 0, -1);
            break;
        }
    }
}










static void
interface_close_tree_panel (void *ignore,
                            GmConnection *connection)
{
    gm_interface_show_hide_tree_panel ( connection);
}






static void
interface_find_dialog_cancel_button_clicked (GtkButton *button,
        GmConnection *connection)
{
    gtk_widget_hide (global_app->find_dialog);

}

static void
interface_find_dialog_find_button_clicked (GtkButton *button,
        GmConnection *connection)
{

    GtkWidget *main_note_book;
    GtkWidget *current_npage;
    GtkWidget *find_entry;
    GtkToggleButton  *tb;
    gboolean case_sensitive;
    gboolean wrap_around;
    GmNpageData *npage_data;
    GmClist *mlist;
    GtkCList *clist;
    gchar *str;



    main_note_book  = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
    current_npage   = gtk_notebook_get_nth_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_note_book)));
    npage_data = (GmNpageData *) gm_utils_get_data  (GTK_OBJECT (current_npage), "npage_data");
    find_entry = gm_utils_get_data  (GTK_OBJECT (global_app->find_dialog), "find_entry");

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data  (GTK_OBJECT (global_app->find_dialog), "uplow_tb"));
    case_sensitive = gtk_toggle_button_get_active (tb);

    tb = GTK_TOGGLE_BUTTON (gm_utils_get_data  (GTK_OBJECT (global_app->find_dialog), "wrap_around_tb"));
    wrap_around = gtk_toggle_button_get_active (tb);

    if (npage_data) {
        clist = GTK_CLIST (npage_data->data_list);
        if (!clist)
            return;
        str = gtk_entry_get_text (GTK_ENTRY (find_entry));
        if (!case_sensitive)
            g_strdown (str);

        mlist = GM_CLIST (clist);
        if (str && strlen (str) >= 1) {
            if (!gm_clist_search (mlist, str, wrap_around, case_sensitive))
                gm_utils_warning ("The text you entered was not found !");
        }

    }
}




static void
interface_find_dialog_on_destroy (GtkWidget *find_dialog,
                                  GmConnection *connection)
{

    global_app->find_dialog = NULL;
}


static gint
interface_find_dialog_key_pressed (GtkWidget    *widget,
                                   GdkEventKey  *event,
                                   GmConnection *connection)
{

    if  (event->keyval == GDK_Escape) {
        gtk_widget_hide (global_app->find_dialog);
    }
    return FALSE;
}





static void
interface_show_find_dialog (void *ignore,
                            GmConnection *connection)
{
    GtkWidget *find_dialog ;
    GtkWidget *button;
    GtkWidget *find_entry;


    find_dialog = global_app->find_dialog;

    if (!find_dialog) {

        global_app->find_dialog = find_dialog = create_find_dialog ();
        button = gm_utils_get_data  (GTK_OBJECT (find_dialog), "find_button");
        gm_utils_create_button_label (button, "  Find", GNOME_STOCK_PIXMAP_SEARCH, TRUE);


        find_entry = gm_utils_get_data  (GTK_OBJECT (find_dialog), "find_entry");

        gtk_signal_connect (GTK_OBJECT (find_entry), "activate",
                            GTK_SIGNAL_FUNC (interface_find_dialog_find_button_clicked),
                            connection);

        gtk_signal_connect (GTK_OBJECT (find_entry), "key_press_event",
                            GTK_SIGNAL_FUNC (interface_find_dialog_key_pressed),
                            connection);

        button = gm_utils_get_data  (GTK_OBJECT (find_dialog), "find_button");
        gtk_signal_connect (GTK_OBJECT (button), "clicked",
                            GTK_SIGNAL_FUNC (interface_find_dialog_find_button_clicked),
                            connection);

        button = gm_utils_get_data  (GTK_OBJECT (find_dialog), "cancel_button");
        gtk_signal_connect (GTK_OBJECT (button), "clicked",
                            interface_find_dialog_cancel_button_clicked,
                            connection);

        gtk_signal_connect (GTK_OBJECT (find_dialog), "destroy",
                            interface_find_dialog_on_destroy,
                            connection);
    }
    gtk_widget_show (find_dialog);
}


/*====================================================================================================*/
/*  Main Window  */
/*====================================================================================================*/



static gboolean
interface_window_deleted (GtkWidget    *widget,
                          GdkEvent     *event,
                          GmConnection *connection)
{
    gm_application_close_connection (NULL, connection);
    return TRUE;
}











/*====================================================================================================*/
/*  key events  */
/*====================================================================================================*/



static gboolean
interface_connection_window_key_press_event (GtkWidget    *widget,
        GdkEventKey  *event,
        GmConnection *connection)
{

    GtkWidget *data_list;
    GtkWidget *main_note_book;
    GtkWidget *current_npage;
    GmNpageData *npage_data;
    gboolean stop_emit = FALSE;
    static gint key_buffer = 0;

    g_return_val_if_fail (connection!=NULL, FALSE);

    main_note_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
    current_npage  = gtk_notebook_get_nth_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_note_book)));
    npage_data     = (GmNpageData *) gm_utils_get_data  (GTK_OBJECT (current_npage), "npage_data");

    if (event->state & GDK_CONTROL_MASK)
    {
        GtkWidget * note_book;
        
        if (!GTK_WIDGET_HAS_FOCUS (connection->query_buffer)) 
            note_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
        else 
            note_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "sql_log_book");
        if ( (event->keyval == GDK_Left) || (event->keyval == GDK_KP_Left))
        {
            gtk_notebook_prev_page (GTK_NOTEBOOK (note_book));
            gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
            stop_emit =  TRUE;
        }else if ( (event->keyval == GDK_Right) || (event->keyval == GDK_KP_Right))
        {
                gtk_notebook_next_page (GTK_NOTEBOOK (note_book));
                gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
                stop_emit =  TRUE;
        }
    }
    if (event->state & GDK_CONTROL_MASK)
    {
        gint page_num;
        if (event->keyval>=48 && event->keyval<58)
        {
            GtkWidget *main_note_book;
            page_num = event->keyval - 49;
            if (page_num<0)
                page_num = 10;
            main_note_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
            if (main_note_book)
                gtk_notebook_set_page (GTK_NOTEBOOK (main_note_book), page_num);
            stop_emit =  TRUE;
        }
    }

    if (event->state & GDK_MOD1_MASK)
    {
        switch (event->keyval)
        {
        case GDK_Q:
        case GDK_q:
            {
                gm_application_exit_immidiatly (NULL, connection);
                stop_emit =  TRUE;
                break;
            }
        case GDK_C:
        case GDK_c:
            {
                gm_application_close_connection (NULL, connection);
                stop_emit =  TRUE;
                break;
            }

        case GDK_x:
            {
                /* Query Mode */
                if (npage_data)
                {
                    data_list = npage_data->data_list;

                    if (!GTK_WIDGET_HAS_FOCUS (connection->query_buffer))
                        gtk_widget_grab_focus (connection->query_buffer);
                    else
                    {
                        if (data_list && !GTK_WIDGET_HAS_FOCUS (data_list))
                            gtk_widget_grab_focus (data_list);
                    }
                }else
                {
                    if (!GTK_WIDGET_HAS_FOCUS (connection->query_buffer))
                        gtk_widget_grab_focus (connection->query_buffer);
                }
                stop_emit =  TRUE;
                break;
            }
        }
    }

    /* direct access with functionkeys short keys */

    switch (event->keyval)
    {
    case GDK_F1:
        {
            interface_help_button_clicked (NULL, connection);
            stop_emit =  TRUE;
            break;
        }

    case GDK_F8:
        {
            gm_bpanel_delete_selected_row (NULL, connection);
            stop_emit =  TRUE;
            break;
        }

    case GDK_F10 :
        {
            gm_application_exit (NULL, connection);
            stop_emit =  TRUE;
            break;
        }
    }

    /* direct access with control short keys */
    if (event->state & GDK_CONTROL_MASK)
    {
        switch (event->keyval)
        {

        case GDK_x:
            {
                connection->key_mode = APP_COMMAND_MODE;
                stop_emit =  TRUE;
                break;
            }

        case GDK_n:
            {
                interface_new_connection_button_clicked (NULL, connection);
                stop_emit =  TRUE;
                break;
            }

        case GDK_d:
            {
                interface_remove_page_button_clicked (NULL, connection);
                stop_emit =  TRUE;
                break;
            }
        case GDK_l:
            {
                gm_interface_show_hide_sql_log_panel (connection);
                stop_emit =  TRUE;
                break;
            }

        case GDK_t:
            {
                gm_interface_show_hide_tree_panel (connection);
                stop_emit =  TRUE;
                break;
            }

        case GDK_q:
            {
                GtkWidget *sql_log_book;
                sql_log_book = gm_utils_get_data (connection->connection_window, "sql_log_book");
                gtk_notebook_set_page (GTK_NOTEBOOK (sql_log_book), SQL_NOTEBOOK_PAGE);
                gtk_widget_grab_focus (connection->query_buffer);
                stop_emit =  TRUE;
                break;
            }



        case GDK_r:
            {
                interface_reload (NULL, connection);
                stop_emit =  TRUE;
                break;
            }
        case GDK_o:
            {
                gm_cc_show_option_dialog (global_app->cc, OPTIONS_GENERAL_PAGE);
                stop_emit =  TRUE;
                break;
            }
        case GDK_f:
        case GDK_s:
            {
                interface_show_find_dialog (NULL, connection);
                stop_emit =  TRUE;
                break;
            }
        case GDK_h:
            {
                interface_help_button_clicked (NULL, connection);
                stop_emit =  TRUE;
                break;
            }
        case GDK_u:
            {
                gm_bpanel_go_up (NULL, connection);
                stop_emit =  TRUE;
                break;
            }

        default:
        }
    }
    if (connection->key_mode == APP_COMMAND_MODE && !stop_emit && npage_data && npage_data->type == GM_NPAGE_BROWSER) {
        switch (event->keyval)
        {
        case GDK_c:
            {
                gm_ccci_copy (NULL, connection);
                stop_emit =  TRUE;
                break;
            }
        case GDK_v:
            {
                gm_ccci_paste (NULL, connection);
                stop_emit =  TRUE;
                break;
            }
        default:
        }
    }



    if (connection->key_mode == APP_COMMAND_MODE && !stop_emit) {
        switch (event->keyval)
        {
        case GDK_n:
            {
                if (npage_data)
                {
                    data_list = npage_data->data_list;
                    if (data_list)
                        gm_bpanel_new_data_list_row (GM_CLIST (data_list), connection);
                }
                stop_emit = TRUE;
                connection->key_mode = APP_DEFAULT_MODE;
                break;
            }
        case GDK_a:
            {
                stop_emit = interface_execute_current_npage (connection);
                stop_emit =  TRUE;
                break;
            }
        case GDK_e :
            {
                gm_bpanel_edit_selected_row (NULL, connection);
                stop_emit = TRUE;
                connection->key_mode = APP_DEFAULT_MODE;
                break;
            }
        case GDK_r :
            {
                gm_bpanel_delete_selected_row (NULL, connection);
                stop_emit = TRUE;
                connection->key_mode = APP_DEFAULT_MODE;
                break;
            }

        case GDK_d :
            {
                /* waiting for next key */
                key_buffer = GDK_d;
                break;
            }
        case GDK_i :
            {
                if (key_buffer == GDK_d)
                {
                    interface_dumpin_button_clicked (NULL, connection);
                    connection->key_mode = APP_DEFAULT_MODE;
                    key_buffer = 0;
                    stop_emit =  TRUE;
                }
                break;
            }
        case GDK_o :
            {
                if (key_buffer == GDK_d)
                {
                    gm_dumpout_dialog_new (NULL, connection);
                    connection->key_mode = APP_DEFAULT_MODE;
                    key_buffer = 0;
                    stop_emit =  TRUE;
                }
                break;
            }

        default :
            {
                connection->key_mode = APP_DEFAULT_MODE;
                key_buffer = 0;
                break;
            }
        }
    }



    if (stop_emit) {
        gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
    }



    return FALSE;
}


static void
interface_vertical_splitter_collapsed (GmVerticalSplitter *vertical_splitter,
                                       gint state,
                                       GtkWidget *connection_window)
{

    GtkWidget *tb;
    guint handler_id;
    GmCcenterGeneral *ge_config;

    ge_config = GE_CONFIG (global_app);

    tb = gm_utils_get_data  (GTK_OBJECT (connection_window), "sql_log_panel_on_off");
    handler_id = GPOINTER_TO_INT (gm_utils_get_data (tb, "handler_id"));
    gtk_signal_handler_block (GTK_OBJECT (tb), handler_id);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tb), !state);
    gtk_signal_handler_unblock (GTK_OBJECT (tb), handler_id);
    ge_config->view_state= (!state)? ge_config->view_state | GM_SQL_LOG_PANEL_VISIBLE:ge_config->view_state &  ~ (GM_SQL_LOG_PANEL_VISIBLE);
}


static void
interface_horizontal_splitter_collapsed (GmVerticalSplitter *horizontal_splitter,
        gint state,
        GtkWidget *connection_window)
{

    GtkWidget *tb;
    guint handler_id;
    GmCcenterGeneral *ge_config;





    ge_config = GE_CONFIG (global_app);

    tb = gm_utils_get_data  (GTK_OBJECT (connection_window), "tree_panel_view_hide");
    handler_id = GPOINTER_TO_INT (gm_utils_get_data (tb, "handler_id"));
    gtk_signal_handler_block (GTK_OBJECT (tb), handler_id);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tb), !state);
    gtk_signal_handler_unblock (GTK_OBJECT (tb), handler_id);
    ge_config->view_state= (!state)? ge_config->view_state | GM_DBTREE_VISIBLE:ge_config->view_state &  ~ (GM_DBTREE_VISIBLE);
}





static void
interface_change_panes (GtkWidget *connection_window)
{

    GtkWidget *old_vpaned;
    GtkWidget *vertical_splitter;
    GtkWidget *horizontal_splitter;
    GtkWidget *sql_log_frame;
    GtkWidget *main_notebook_hbox;
    GtkWidget *right_vbox;
    GtkWidget *dbtree_frame;
    GtkWidget *main_hbox;
    GmCcenterGeneral *ge_config;




    ge_config = GE_CONFIG (global_app);

    /* vertical */
    sql_log_frame = gm_utils_get_data  (GTK_OBJECT (connection_window), "sql_log_frame");
    old_vpaned = gm_utils_get_data  (GTK_OBJECT (connection_window), "vpaned");
    main_notebook_hbox = gm_utils_get_data  (GTK_OBJECT (connection_window), "main_notebook_hbox");
    right_vbox = gm_utils_get_data  (GTK_OBJECT (connection_window), "right_vbox");

    vertical_splitter = gm_vertical_splitter_new ();
    e_paned_set_priority (E_PANED (vertical_splitter), 0);
    gm_spliter_set_saved_position (E_PANED (vertical_splitter), ge_config->vertical_paned_position);

    gtk_signal_connect (GTK_OBJECT (vertical_splitter), "collapsed",
                        GTK_SIGNAL_FUNC (interface_vertical_splitter_collapsed),
                        connection_window);


    gtk_widget_ref (vertical_splitter);
    gtk_object_set_data_full (GTK_OBJECT (connection_window), "vertical_splitter", vertical_splitter,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vertical_splitter);

    gtk_widget_unparent (sql_log_frame);
    gtk_widget_unparent (main_notebook_hbox);
    gtk_widget_unparent (old_vpaned);
    gtk_widget_hide (old_vpaned);

    e_paned_pack1 (E_PANED (vertical_splitter), main_notebook_hbox, TRUE, TRUE);
    e_paned_pack2 (E_PANED (vertical_splitter), sql_log_frame, TRUE, TRUE);
    gtk_container_add (GTK_CONTAINER (right_vbox), vertical_splitter);


    /* horizontal */
    dbtree_frame = gm_utils_get_data  (GTK_OBJECT (connection_window), "dbtree_frame");
    old_vpaned = gm_utils_get_data  (GTK_OBJECT (connection_window), "hpaned");
    main_hbox = gm_utils_get_data  (GTK_OBJECT (connection_window), "main_hbox");

    horizontal_splitter = gm_horizontal_splitter_new ();

    gtk_signal_connect (GTK_OBJECT (horizontal_splitter), "collapsed",
                        GTK_SIGNAL_FUNC (interface_horizontal_splitter_collapsed),
                        connection_window);

    gtk_widget_ref (horizontal_splitter);
    gtk_object_set_data_full (GTK_OBJECT (connection_window), "horizontal_splitter", horizontal_splitter,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (horizontal_splitter);
    gm_spliter_set_saved_position (E_PANED (horizontal_splitter), ge_config->horizontal_paned_position);

    gtk_widget_unparent (dbtree_frame);
    gtk_widget_unparent (right_vbox);
    gtk_widget_unparent (old_vpaned);
    gtk_widget_hide (old_vpaned);

    e_paned_pack1 (E_PANED (horizontal_splitter), dbtree_frame, FALSE, TRUE);
    e_paned_pack2 (E_PANED (horizontal_splitter), right_vbox, FALSE, TRUE);
    gtk_container_add (GTK_CONTAINER (main_hbox), horizontal_splitter);

}



static void
interface_configure_debug_menu (GmConnection *connection) 
{

    GtkWidget *menu_item;
    GtkWidget *connection_window;
    connection_window = connection->connection_window;
    menu_item = gm_utils_get_data (connection_window, "debug");
#if GM_DEBUG_ON
    gtk_widget_show (menu_item);
    menu_item = gm_utils_get_data (connection_window, "query_chanel");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (gm_debug_toggle_chanel),
                        GINT_TO_POINTER (GM_QUERY_CHANEL));
    gtk_widget_show (menu_item);
    menu_item = gm_utils_get_data (connection_window, "printer_chanel");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (gm_debug_toggle_chanel),
                        GINT_TO_POINTER (GM_PRINTER_CHANEL));
#else
    gtk_widget_hide (menu_item);
#endif
}


static void
interface_configure_menu_signals (GmConnection *connection )
{
    GtkWidget *menu_item;
    GtkWidget *connection_window;


    connection_window = connection->connection_window;


    interface_configure_debug_menu(connection);
    

    menu_item = gm_utils_get_data (connection_window, "new_connection_menu");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (interface_new_connection_button_clicked),
                        connection);


    menu_item = gm_utils_get_data (connection_window, "disconnect_menu");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (gm_application_close_connection),
                        connection);

    menu_item = gm_utils_get_data (connection_window, "new_querypage");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (interface_new_query_page_button_clicked),
                        connection);


    menu_item = gm_utils_get_data (connection_window, "close_current_page");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (interface_remove_page_button_clicked),
                        connection);

    menu_item = gm_utils_get_data (connection_window, "print_options");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (interface_show_print_options),
                        connection);
#if GM_PRINT_SUPPORT
#else
    gtk_widget_set_sensitive (menu_item, FALSE);
#endif    

    menu_item = gm_utils_get_data (connection_window, "print");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (gm_print),
                        connection);
#if GM_PRINT_SUPPORT
#else
    gtk_widget_set_sensitive (menu_item, FALSE);
#endif    

    menu_item = gm_utils_get_data (connection_window, "preview");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (gm_print_preview),
                        connection);
#if GM_PRINT_SUPPORT
#else
    gtk_widget_set_sensitive (menu_item, FALSE);
#endif    

    /* server */
    menu_item = gm_utils_get_data (connection_window, "variables");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) gm_mysql_admin_show_variables ,
                        connection);
    menu_item = gm_utils_get_data (connection_window, "processlist");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) gm_mysql_admin_show_processlist ,
                        connection);
    menu_item = gm_utils_get_data (connection_window, "status");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) gm_mysql_admin_show_status ,
                        connection);
    menu_item = gm_utils_get_data (connection_window, "flush");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) gm_mysql_admin_flush ,
                        connection);


    /*tools */

    menu_item = gm_utils_get_data (connection_window, "options");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc)interface_options_button_clicked,
                        connection);

    menu_item = gm_utils_get_data (connection_window, "export");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) gm_dumpout_dialog_new,
                        connection);


    menu_item = gm_utils_get_data (connection_window, "import");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) interface_dumpin_button_clicked,
                        connection);



    /* help menu */

    menu_item = gm_utils_get_data (connection_window, "about");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) interface_about_button_clicked,
                        connection);

    menu_item = gm_utils_get_data (connection_window, "help");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) interface_help_button_clicked,
                        connection);

    menu_item = gm_utils_get_data (connection_window, "update");
    gm_utils_register_hint (GTK_WIDGET (menu_item), _("Check our updateserver for a newer Version, Note : works only if you have GET installed"), connection) ;
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        (GtkSignalFunc) interface_update_button_clicked,
                        connection);

    /* EXIT */
    menu_item = gm_utils_get_data (connection_window, "menu_exit");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        gm_application_exit_immidiatly, connection);

    /*Edit */

    menu_item = gm_utils_get_data (connection_window, "menu_copy");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        gm_ccci_copy, connection);

    menu_item = gm_utils_get_data (connection_window, "menu_paste");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        gm_ccci_paste, connection);



    menu_item = gm_utils_get_data (connection_window, "menu_add_row");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        gm_bpanel_new_row, connection);

    menu_item = gm_utils_get_data (connection_window, "menu_delete_row");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        gm_bpanel_delete_selected_row, connection);


    menu_item = gm_utils_get_data (connection_window, "menu_edit_row");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        gm_bpanel_edit_selected_row, connection);


    menu_item = gm_utils_get_data (connection_window, "menu_find");
    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                        interface_show_find_dialog, connection);
}





static void
interface_create_query_buffer (GmConnection *connection)
{
    GtkWidget *query_buffer;
    GtkWidget *query_buffer_sw;

    connection->query_buffer = query_buffer = gm_query_buffer_new (connection);
    query_buffer_sw = gm_utils_get_data (connection->connection_window, "query_buffer_sw");
    gtk_widget_set_usize (query_buffer_sw, -1, 100);
    gtk_widget_set_usize (connection->query_buffer, -1, 100);
    gtk_widget_ref (query_buffer);
    gtk_object_set_data_full (GTK_OBJECT (connection->connection_window), "query_buffer", query_buffer,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (query_buffer);
    gtk_container_add (GTK_CONTAINER (query_buffer_sw), query_buffer);
    gtk_text_set_editable (GTK_TEXT (query_buffer), TRUE);
    gm_utils_text_configure_sql_syntax_highlighting (GTK_EDITABLE (query_buffer));
    gm_utils_register_hint (GTK_WIDGET (query_buffer), _("Hold the control key and press right or left to move to next, last notebook page . Control-Return to execute your Query"), connection) ;
}




static void
interface_create_message_window (GmConnection * connection)
{
    GtkWidget *messages_window;
    GtkWidget *messages_scrolled_window;


    messages_window = gm_utils_create_text_widget ();
    messages_scrolled_window = gm_utils_get_data (connection->connection_window, "messages_text_scrolledwindow");
    gtk_object_set_data (GTK_OBJECT (connection->connection_window), "messages_window", messages_window);
    gtk_widget_show (messages_window);
    gtk_container_add (GTK_CONTAINER (messages_scrolled_window), messages_window);
    gtk_text_set_editable(GTK_TEXT(messages_window), FALSE);
}

static void 
interface_qbuffer_rehighlight (void *ignore, GtkWidget *qbuffer) 
{
    gm_utils_text_colorize (GTK_EDITABLE (qbuffer), 0, -1);
}




/*****************************************************************************************************/
/* Public functions */
/*****************************************************************************************************/


GtkWidget *
gm_interface_new_query_page (GmConnection *connection,
                             gboolean with_info_entry)

{

    GtkWidget *main_note_book;
    GtkWidget *empty_notebook_page;
    GtkWidget *label;




    main_note_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");

    /* create a frame a toolbar  and 2 buttons */
    empty_notebook_page = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (empty_notebook_page);
    gtk_object_set_data_full (GTK_OBJECT (empty_notebook_page), "empty_notebook_page", empty_notebook_page,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (empty_notebook_page);
#if 0
    frame = gtk_frame_new (NULL);
    gtk_widget_ref (frame);
    gtk_object_set_data_full (GTK_OBJECT (empty_notebook_page), "frame", frame,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (frame);
    gtk_box_pack_start (GTK_BOX (empty_notebook_page), frame, FALSE, FALSE, 0);

    toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
    gtk_widget_ref (toolbar);
    gtk_object_set_data_full (GTK_OBJECT (empty_notebook_page), "toolbar", toolbar,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (toolbar);
    gtk_container_add (GTK_CONTAINER (frame), toolbar);
    export_button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                    GTK_TOOLBAR_CHILD_BUTTON,
                    NULL,
                    _ ("button1"),
                    NULL, NULL,
                    NULL, NULL, NULL);
    gtk_widget_destroy (GTK_BIN (export_button)->child);
    pixmap = gm_stock_pixmap_widget (export_button, "export");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (export_button), pixmap);
    gtk_widget_ref (export_button);

    
    gtk_object_set_data_full (GTK_OBJECT (empty_notebook_page), "export_button", export_button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (export_button);
    gtk_widget_show (frame);
#endif
    if (with_info_entry)
    {
        label = gtk_label_new (_ ("Type your query below hold the control Key and hit return to execute it (Ctl-d to delete this page) "));
        gtk_widget_ref (label);
        gtk_object_set_data_full (GTK_OBJECT (empty_notebook_page), "label", label,
                                  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (label);
        gtk_box_pack_start (GTK_BOX (empty_notebook_page), label, TRUE, TRUE, 0);
    }

    gtk_notebook_append_page        (GTK_NOTEBOOK (main_note_book),
                                     empty_notebook_page, NULL);
    bpanel_append_notebook_page_with_icon (GTK_NOTEBOOK (main_note_book), empty_notebook_page , " Query-Result", "query_page");
    gtk_notebook_set_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_page_num (GTK_NOTEBOOK (main_note_book), empty_notebook_page));
    return empty_notebook_page;
}

void
gm_interface_show_hide_sql_log_panel (GmConnection *connection)
{
    GtkWidget *tb;
    gboolean active;
    tb = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "sql_log_panel_on_off");
    active =  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tb));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tb), !active);
}


void
gm_interface_show_hide_tree_panel (GmConnection *connection)
{
    GtkWidget *tb;
    gboolean active;
    tb = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "tree_panel_view_hide");
    active =  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tb));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tb), !active);
}



GtkWidget*
gm_interface_create_connection_window (GmConnection *connection)
{
    GtkWidget *sql_log_panel_on_off;
    GtkWidget *about_button;
    GtkWidget *exit_button;
    GtkWidget *button;
    GtkWidget *main_note_book;
    GtkWidget *npage;
    GtkWidget *pixmap;
    GtkWidget *dbtree_sw;
    GtkWidget *dbtree;
    GtkWidget *sql_log_book;
    
    guint handler_id;
    GmCcenterGeneral *ge_config;

    connection->connection_window = create_connection_window ();
    connection->mini_buffer = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "mini_buffer");
    connection->progressbar = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "progressbar");

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "new_connection_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "new_connection");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);


    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_new_connection_button_clicked),
                        connection);
    gtk_widget_show (connection->connection_window);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "new_page_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "newpage");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_new_query_page_button_clicked),
                        connection);
    gtk_widget_show (connection->connection_window);
    ge_config = GE_CONFIG (global_app);
    interface_create_message_window (connection);
    interface_create_query_buffer (connection);


    sql_log_book = gm_utils_get_data (connection->connection_window, "sql_log_book");
    gtk_signal_connect (GTK_OBJECT (sql_log_book), "switch-page", 
                        GTK_SIGNAL_FUNC (interface_sql_log_book_switch_page),
                        connection);
    



    interface_configure_sql_log_signals (connection);

        /* replace the gtk_vpaned with ours */
    interface_change_panes (connection->connection_window);

    gtk_signal_connect (GTK_OBJECT (connection->connection_window), "key_press_event",
                        GTK_SIGNAL_FUNC (interface_connection_window_key_press_event),
                        connection);

    interface_configure_menu_signals (connection);
    gtk_signal_connect (GTK_OBJECT (connection->connection_window), "delete_event", GTK_SIGNAL_FUNC (interface_window_deleted), connection);

        /* main window toolbar */

    sql_log_panel_on_off =   gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "sql_log_panel_on_off");
    gm_utils_create_button_label (sql_log_panel_on_off, NULL, "query_page", FALSE);
    handler_id = gtk_signal_connect (GTK_OBJECT (sql_log_panel_on_off), "toggled",
                                     GTK_SIGNAL_FUNC (interface_sql_log_panel_on_off_toggled),
                                     connection);
    gtk_object_set_data (GTK_OBJECT (sql_log_panel_on_off), "handler_id", GINT_TO_POINTER (handler_id));

    button =   gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "tree_panel_view_hide");
    handler_id = gtk_signal_connect (GTK_OBJECT (button), "toggled",
                                     GTK_SIGNAL_FUNC (interface_tree_panel_view_hide_toggled),
                                     connection);
    gtk_object_set_data (GTK_OBJECT (button), "handler_id", GINT_TO_POINTER (handler_id));




    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "clip_win_on_off");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "clip_board");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);

    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_clip_window_on_off_toggled),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "tree_panel_view_hide");
    gm_utils_create_button_label (button, NULL, "tree", FALSE);


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "help_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_help_button_clicked),
                        connection);

    about_button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "about_button");
    gtk_signal_connect (GTK_OBJECT (about_button), "clicked",
                        GTK_SIGNAL_FUNC (interface_about_button_clicked),
                        connection);

    exit_button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "exit_button");
    gtk_signal_connect (GTK_OBJECT (exit_button), "clicked",
                        GTK_SIGNAL_FUNC (interface_exit_button_clicked),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "close_connection_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "close_connection");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_application_close_connection),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "remove_page_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "deletepage");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_remove_page_button_clicked),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "options_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "options");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_options_button_clicked),
                        connection);


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "reload_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_reload),
                        connection);


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "find_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_show_find_dialog),
                        connection);

        /* print */

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "print_preview_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "print-preview");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_print_preview),
                        connection);
#if GM_PRINT_SUPPORT
#else
    gtk_widget_set_sensitive (button, FALSE);
#endif   


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "print_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_print),
                        connection);

#if GM_PRINT_SUPPORT
#else
    gtk_widget_set_sensitive (button, FALSE);
#endif    
        /* browser panel toolbar */

    main_note_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
    gtk_signal_connect (GTK_OBJECT (main_note_book), "switch-page", 
                        GTK_SIGNAL_FUNC (interface_main_note_book_switch_page),
                        connection);


    npage = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "dbbrowser_hbox");

    bpanel_append_notebook_page_with_icon (GTK_NOTEBOOK (main_note_book), npage , "DbBrowser", "db-browser");


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "edit_row_button");
    gm_utils_create_vbutton_label (button, "Edit", "row-edit", FALSE);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_bpanel_edit_selected_row),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "new_row_button");
    gm_utils_create_vbutton_label (button, "New", "new_row", FALSE);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_bpanel_new_row),
                        connection);


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "delete_row_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "delete_row");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_bpanel_delete_selected_row),
                        connection);


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "save_result_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_bpanel_save_under),
                        connection);



    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "export_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "export");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_dumpout_dialog_new),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "dumpin_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "import");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_dumpin_button_clicked),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "up_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_bpanel_go_up),
                        connection);


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "filter_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "cute");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_bpanel_show_filter_dialog),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "copy_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_ccci_copy),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "paste_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_ccci_paste),
                        connection);
        /* status toolbar  */

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "filter_sbutton");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "cute");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);

        /* dbtree_scrolled_window */

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "close_tree_view_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "kreuz");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_close_tree_panel),
                        connection);

    dbtree_sw = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "dbtree_sw");
    connection->dbtree = dbtree = gm_dbtree_new (connection);
    gtk_widget_ref (dbtree);
    gtk_object_set_data_full (GTK_OBJECT (connection->connection_window), "dbtree", dbtree,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (dbtree);
    gtk_container_add (GTK_CONTAINER (dbtree_sw), dbtree);

    gm_dbtree_build_tree (connection);


    gtk_widget_show (connection->connection_window);
    gtk_widget_realize (connection->connection_window);

    /* restore the state of the dbtree */

    button =   gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "tree_panel_view_hide");
    if (! (ge_config->view_state & GM_DBTREE_VISIBLE)){
        GtkWidget *splitter;
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
        splitter = gm_utils_get_data (connection->connection_window, "horizontal_splitter");
        e_paned_set_position (E_PANED (splitter), 0);
    }else {
        GtkWidget *splitter;
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
        splitter = gm_utils_get_data (connection->connection_window, "horizontal_splitter");
        e_paned_set_position (E_PANED (splitter), ge_config->horizontal_paned_position);
    }

    if (! (ge_config->view_state & GM_SQL_LOG_PANEL_VISIBLE)){
        GtkWidget *splitter;
        button =   gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "sql_log_panel_on_off");
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);

        splitter = gm_utils_get_data (connection->connection_window, "vertical_splitter");
        e_paned_set_position (E_PANED (splitter), 10000);
    }else
    {
        GtkWidget *splitter;
        splitter = gm_utils_get_data (connection->connection_window, "vertical_splitter");
        e_paned_set_position (E_PANED (splitter), ge_config->vertical_paned_position);
    }
    return connection->connection_window;
}



void
gm_interface_create_new_database (void *ignore, GmConnection *connection)
{

    gchar * database_name;
    BpanelViewInfo * view_info;

    database_name = gm_dialogs_new_database (NULL);
    if (!database_name)
        return;

    if (gm_db_create_database (connection, database_name)){

        view_info = gm_utils_get_data (GTK_OBJECT (connection->data_list), "view_info");
        if (view_info && view_info->view_type == DATABASES_VIEW)
            gm_bpanel_reload (NULL, connection);

        gm_dbtree_rebuild_tree (NULL, connection);
    }
    g_free (database_name);
}

void
gm_interface_delete_databases (GmConnection *connection, GList *db_list)
{
    GList * list;
    gboolean reload = FALSE;
    BpanelViewInfo * view_info;

    if (!db_list)
        return;
    list = db_list;


    if (gm_utils_yes_no_dialog (_("Do you realy want to delete the selected databases ?")))
    {
        while (list)
        {
            gchar * database = list->data;
            if (database) {
                if (gm_db_delete_database (connection, database))
                    reload = TRUE;
            }

            list = list->next;
        }
        if (reload) {
            view_info = gm_utils_get_data (GTK_OBJECT (connection->data_list), "view_info");
            if (view_info && view_info->view_type == DATABASES_VIEW)
                gm_bpanel_reload (NULL, connection);

            gm_dbtree_rebuild_tree (NULL, connection);
        }
    }
}



void
gm_interface_delete_tables (GmConnection *connection, gchar * database, GList *table_list)
{
    GList * list;
    gboolean reload = FALSE;
    BpanelViewInfo * view_info;

    if (!table_list)
        return;
    list = table_list;


    if (gm_utils_yes_no_dialog (_("Do you realy want to drop the selected Table (s) ?")))
    {


        while (list)
        {
            gchar * table  = list->data;
            if (table) {
                if (gm_db_delete_table (connection, database, table))
                    reload = TRUE;
            }
            list = list->next;
        }
        if (reload) {
            view_info = gm_utils_get_data (GTK_OBJECT (connection->data_list), "view_info");
            if (view_info && view_info->view_type == TABLES_VIEW)
                gm_bpanel_reload (NULL, connection);

            gm_dbtree_rebuild_tree (NULL, connection);
        }
    }
}

void
gm_interface_show_tables (GmConnection * connection, gchar * database)
{

    gm_bpanel_show_tables (connection, database);
}




void
gm_interface_edit_table (GmConnection * connection, gchar *database, gchar *table)
{


    DbTableInfo * table_info;
    gchar * t_label;
    GtkWidget *npage;
    GtkWidget * main_note_book;
    GtkWidget * fields_properties_window;
    GmNpageData * npage_data;

    g_return_if_fail (connection != NULL);


    g_return_if_fail (table != NULL || database != NULL );


    table_info = gm_db_get_table_info (connection, table, database);

    main_note_book = gm_utils_get_data (GTK_OBJECT(connection->connection_window),"main_note_book");
    fields_properties_window =  gm_table_properties_new (connection, table_info);
    npage = gm_utils_get_data (GTK_OBJECT(fields_properties_window),"properties_box");
    gtk_object_set_data (GTK_OBJECT (npage), "fields_properties_window", fields_properties_window);
    npage_data = gm_app_new_npage_data ();
    npage_data->type = GM_NPAGE_TABLE_PROP;
    gtk_object_set_data_full (GTK_OBJECT (npage), "npage_data", npage_data,
                              (GtkDestroyNotify) gm_app_free_npage_data);

    gtk_widget_unparent (npage);

    gtk_object_set_data (GTK_OBJECT (npage), "main_note_book", main_note_book);
    gtk_widget_show (npage);
    gtk_notebook_append_page        (GTK_NOTEBOOK (main_note_book),
                                     npage, NULL);
    t_label = g_strdup_printf ("/%s/%s", database, table);

    bpanel_append_notebook_page_with_icon (GTK_NOTEBOOK (main_note_book), npage, t_label, "table-prop" );
    gtk_notebook_set_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_page_num (GTK_NOTEBOOK (main_note_book), npage));
    g_free(t_label);
}




void gm_interface_new_table_properties_window (GmConnection * connection, gchar * database)
{
    DbTableInfo * table_info;
    GmNpageData * npage_data;
    gchar * t_label;
    GtkWidget *npage;
    GtkWidget * main_note_book;
    GtkWidget * fields_properties_window;

    g_return_if_fail (connection != NULL);



    table_info = db_new_table_info ();
    table_info->database = g_strdup (database);


    main_note_book = gm_utils_get_data (GTK_OBJECT (connection->connection_window), "main_note_book");

    fields_properties_window =  gm_table_properties_new (connection, table_info);
    npage = gm_utils_get_data (GTK_OBJECT (fields_properties_window), "properties_box");
    gtk_object_set_data (GTK_OBJECT (npage), "fields_properties_window", fields_properties_window);
    npage_data = gm_app_new_npage_data ();
    npage_data->type = GM_NPAGE_TABLE_PROP;
    gtk_object_set_data_full (GTK_OBJECT (npage), "npage_data", npage_data,
                              (GtkDestroyNotify) gm_app_free_npage_data);
    gtk_widget_unparent (npage);
    gtk_object_set_data (GTK_OBJECT (npage), "main_note_book", main_note_book);
    gtk_widget_show (npage);
    gtk_notebook_append_page        (GTK_NOTEBOOK (main_note_book),
                                     npage, NULL);
    t_label = g_strdup_printf ("/%s/New Table", database);
    bpanel_append_notebook_page_with_icon (GTK_NOTEBOOK (main_note_book), npage, t_label, "new_table" );
    g_free (t_label);
    gtk_notebook_set_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_page_num (GTK_NOTEBOOK (main_note_book), npage));
}



void
gm_interface_show_query_result (GmConnection *connection, DbQuery * q, gint type)
{

    GmNpageData * npage_data;
    GtkWidget * main_note_book;
    GtkWidget * current_npage;
    GtkWidget * data_list_sw;
    GtkWidget * data_list;
    GtkWidget * label;
    GtkWidget * frame;
    GtkWidget * pixmap;
    GtkWidget * hbox;
    GtkWidget * close_npage_button;
    GSList * f;
    MYSQL_RES *result;
    MYSQL_ROW row;
    gint i = 0;




    main_note_book = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
    current_npage   = gtk_notebook_get_nth_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_note_book)));
    npage_data = (GmNpageData *) gm_utils_get_data  (GTK_OBJECT (current_npage), "npage_data");

    if (!npage_data || npage_data->type !=type){
        npage_data = gm_app_new_npage_data ();
        npage_data->type = type;
        data_list_sw = gtk_scrolled_window_new (NULL, NULL);
        gtk_widget_show (data_list_sw);
        gtk_container_set_border_width (GTK_CONTAINER (data_list_sw), 2);
        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (data_list_sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
        label = gm_utils_get_data_ww  (GTK_OBJECT (current_npage), "label");
        if (label){
            gtk_widget_destroy (label);
            gtk_object_remove_data (GTK_OBJECT (current_npage), "label");
        }else {
            current_npage = gm_interface_new_query_page (connection, FALSE);
        }
        frame = gtk_frame_new (NULL);
        gtk_widget_show (frame);
        gtk_box_pack_start (GTK_BOX (current_npage), frame, FALSE, FALSE, 0);
        gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
        hbox = gtk_hbox_new (FALSE, 0);
        gtk_widget_show (hbox);
        gtk_container_add (GTK_CONTAINER (frame), hbox);
        label = gtk_label_new ("");
        gtk_widget_show (label);
        gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
        gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
        gtk_misc_set_padding (GTK_MISC (label), 2, 2);
        close_npage_button = gtk_button_new_with_label (_("X"));
        gtk_widget_show (close_npage_button);
        gtk_box_pack_start (GTK_BOX (hbox), close_npage_button, FALSE, FALSE, 0);
        gtk_button_set_relief (GTK_BUTTON (close_npage_button), GTK_RELIEF_NONE);
        gtk_widget_destroy (GTK_BIN (close_npage_button)->child);
        pixmap = gm_stock_pixmap_widget (close_npage_button, "kreuz");
        gtk_widget_show (pixmap);
        gtk_container_add (GTK_CONTAINER (close_npage_button), pixmap);
        gtk_signal_connect (GTK_OBJECT (close_npage_button), "clicked",
                            GTK_SIGNAL_FUNC (interface_remove_page_button_clicked),
                            connection);
        gtk_box_pack_start (GTK_BOX (current_npage),  data_list_sw, TRUE, TRUE, 0);
        npage_data->data_list_sw = data_list_sw;
        gtk_object_set_data_full (GTK_OBJECT (current_npage), "npage_data", npage_data,
                                  (GtkDestroyNotify) gm_app_free_npage_data);
        npage_data->label = label;
        npage_data->query = g_strdup (q->query);
    }else {
        gtk_widget_destroy (npage_data->data_list);
        data_list_sw = npage_data->data_list_sw;
        label = npage_data->label;
        g_free(npage_data->query);
        npage_data->query = g_strdup (q->query);
    }

     /*   
      gchar * temp;
      temp = g_strdup_printf ("QUERY: %s", q->query);
      gtk_label_set_text (GTK_LABEL (label), temp);
      g_free (temp);*/


    result = q->mysql_result;
    npage_data->data_list = data_list = gm_clist_new (q->fields_count);
    gtk_widget_show (data_list);
    gtk_container_add (GTK_CONTAINER (data_list_sw), data_list);
    gtk_clist_set_selection_mode (GTK_CLIST (data_list), GTK_SELECTION_EXTENDED);
    gtk_clist_freeze (GTK_CLIST (data_list));
    gtk_clist_column_titles_show (GTK_CLIST (data_list));
    f = g_slist_first (q->fields_list);
    while (f)
    {
        GtkWidget * label;
        MYSQL_FIELD * field;

        field = (MYSQL_FIELD *) f->data;
        gtk_clist_set_column_auto_resize (GTK_CLIST (data_list), i, TRUE);
        label = gtk_label_new (field->name);
        gtk_widget_show (label);
        gm_clist_set_column_widget (GM_CLIST (data_list), i, label);
        gm_clist_set_column_sortdata_type (GM_CLIST (data_list), i, field->type);
        i++;
        f = f->next;
    }
    mysql_data_seek (result, 0);

    while ( (row = mysql_fetch_row (result)))
        gtk_clist_append (GTK_CLIST (data_list), row);
    gtk_clist_thaw (GTK_CLIST (data_list));
}




static void
interface_main_note_book_switch_page (GtkWidget       *sql_log_book,
                                    GtkNotebookPage *page,
                                    gint             page_num,
                                    GmConnection    *connection)
{


    GtkWidget *main_note_book;
    GtkWidget *current_npage;
    GmNpageData *npage_data;


    main_note_book  = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "main_note_book");
    current_npage   = gtk_notebook_get_nth_page (GTK_NOTEBOOK (main_note_book), page_num);
    npage_data      = (GmNpageData *) gm_utils_get_data_ww  (GTK_OBJECT (current_npage), "npage_data");

    if (!npage_data)
        return ;
    if (npage_data->data_list)
        gtk_widget_grab_focus (npage_data->data_list);
}



static void
interface_sql_log_book_switch_page (GtkWidget       *sql_log_book,
                                    GtkNotebookPage *page,
                                    gint             page_num,
                                    GmConnection    *connection)
{
    GtkWidget *open_file_button;
    GtkWidget *save_file_button;
    GtkWidget *clear_window_button;
    GtkWidget *execute_button;
    GtkWidget *rehighlight_button;    

    open_file_button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "open_file_button");
    save_file_button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "save_file_button");
    clear_window_button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "clear_window_button");
    execute_button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "execute_button");
    rehighlight_button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "rehighlight_button");


    switch (page_num) {
        case SQL_NOTEBOOK_PAGE:
        {
            gtk_widget_set_sensitive (open_file_button, TRUE);
            gtk_widget_set_sensitive (rehighlight_button, TRUE);
            gtk_widget_set_sensitive (execute_button, TRUE);

            break;
        }
        case HISTORY_NOTEBOOK_PAGE:
        {
            gtk_widget_set_sensitive (open_file_button, TRUE);
            gtk_widget_set_sensitive (rehighlight_button, FALSE);
            gtk_widget_set_sensitive (execute_button, FALSE);
            break;
        }
        case MESSAGES_NOTEBOOK_PAGE:
        case ERRORS_NOTEBOOK_PAGE:
        {
            gtk_widget_set_sensitive (open_file_button, FALSE);
            gtk_widget_set_sensitive (rehighlight_button, FALSE);
            gtk_widget_set_sensitive (execute_button, FALSE);
            break;
        }   
    }
    

}




static void 
interface_sql_log_open_file_button_clicked (void *ignore, GmConnection *connection) 
{
    GtkWidget *sql_log_book;
    gint page_num;    
    GtkWidget *text_widget;
    
    sql_log_book = gm_utils_get_data (connection->connection_window, "sql_log_book");
    
    page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (sql_log_book));
    
    switch (page_num) {
        case SQL_NOTEBOOK_PAGE:
        {
            text_widget = connection->query_buffer;
            gm_utils_text_open_activated (NULL, GTK_EDITABLE (connection->query_buffer));            
            break;
        }
        case HISTORY_NOTEBOOK_PAGE:
        {
            gm_utils_load_file_to_clist (NULL, -1, GTK_CLIST (connection->qhistory_mlist)) ;
            break;
        }
        case MESSAGES_NOTEBOOK_PAGE:
        case ERRORS_NOTEBOOK_PAGE:
        {

            break;
        }   
    }
    

        
}

static void 
interface_sql_log_save_file_button_clicked (void *ignore, GmConnection *connection) 
{

    GtkWidget *sql_log_book;
    gint page_num;    
    GtkWidget *text_widget;
    
    sql_log_book = gm_utils_get_data (connection->connection_window, "sql_log_book");    
    page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (sql_log_book));
    
    switch (page_num) {
        case SQL_NOTEBOOK_PAGE:
        {
            gm_utils_text_save_under_activated (NULL, GTK_EDITABLE(connection->query_buffer));
            break;
        }
        case HISTORY_NOTEBOOK_PAGE:
        {
            gm_utils_save_clist_to_file (NULL, -1, GTK_CLIST (connection->qhistory_mlist)) ;
            break;
        }
        case MESSAGES_NOTEBOOK_PAGE:
        {            
            text_widget = gm_utils_get_data  (connection->connection_window, "messages_window");
            gm_utils_text_save_under_activated (NULL, GTK_EDITABLE(text_widget));
            break;            
        }        
        case ERRORS_NOTEBOOK_PAGE:
        {
            text_widget = gm_utils_get_data  (connection->connection_window,"errors_window");
            gm_utils_text_save_under_activated (NULL, GTK_EDITABLE(text_widget));
            break;
        }   
    }

        
}




static void
interface_configure_sql_log_signals (GmConnection *connection) 
{
    GtkWidget * button;
    

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "open_file_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_sql_log_open_file_button_clicked),
                        connection);


    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "save_file_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_sql_log_save_file_button_clicked),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "clear_window_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_clear_selected_window),
                        connection);

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "execute_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gm_query_buffer_execute_query),
                        GM_QUERY_BUFFER (connection->query_buffer));

    button = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "rehighlight_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (interface_qbuffer_rehighlight),
                        connection->query_buffer);
}



gboolean 
gm_interface_execute_query_and_show_result (GmConnection * connection, gchar * query, gboolean set_cursor_position)
{
    DbQuery * q;
    gint logmode;
    gchar * message, *info;
    gboolean  ret_val;
    glong time;

    

    g_return_val_if_fail (query != NULL, FALSE);

    time = clock();
    q = gm_db_new_query (query);
    if (gm_db_request(connection, q))
    {
        logmode = SIMPLE_LOGMODE;
        message = q->query;
        ret_val = TRUE;
    }else
    {
        logmode = ERROR_LOGMODE;
        message = q->error;
        ret_val = FALSE;
    }

    time = clock()- time;
    time = time /1000;

    gm_utils_write_log_message (connection, message ,logmode);
    if (logmode != ERROR_LOGMODE)
    {
        logmode = SIMPLE_LOGMODE;
        info = g_strdup_printf ("%ld row in set time (%ld ms)", q->affected_rows, time);
        gm_utils_write_log_message (connection, info, logmode);
        g_free (info);
    }else
    {
        if (set_cursor_position)
            gm_query_buffer_set_cursor_at_error_position(connection, q); 
    }

    if (q->mysql_result)
    {
        gm_interface_show_query_result (connection, q, GM_NPAGE_USER_QUERY);
    }
    gm_db_free_query (q);
    return ret_val ;
}

