/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "gm.h"
#include "gm-mysql-admin.h"
#include "gm-interface.h"



static gboolean
mysql_admin_execute_query (GmConnection * connection, DbQuery * q)
{
    g_return_val_if_fail (q != NULL, FALSE);

    if (!gm_db_request(connection, q)){
        gm_utils_write_log_message (connection, q->error ,ERROR_LOGMODE);
        return FALSE;
    }
    else
        if (q->mysql_result)
            gm_interface_show_query_result (connection, q, GM_NPAGE_VARIABLES);
    return TRUE;
}


void
gm_mysql_admin_show_variables (void *ignore, GmConnection *connection)
{

    DbQuery * q;
    q = gm_db_new_query ("SHOW VARIABLES");
    mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
}


void
gm_mysql_admin_show_processlist (void *ignore, GmConnection *connection)
{

    DbQuery * q;
    q = gm_db_new_query ("SHOW PROCESSLIST");
    mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
}


void
gm_mysql_admin_show_status (void *ignore, GmConnection *connection)
{
    DbQuery * q;
    q = gm_db_new_query ("SHOW STATUS");
    mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
}

static gboolean
mysql_admin_flush_hosts (GmConnection *connection)
{
    DbQuery * q;
    gboolean ret;

    q = gm_db_new_query ("FLUSH HOSTS");
    ret = mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    return ret;
}


static gboolean
mysql_admin_flush_logs (GmConnection *connection)
{
    DbQuery * q;
    gboolean ret;

    q = gm_db_new_query ("FLUSH LOGS");
    ret = mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    return ret;
}

static gboolean
mysql_admin_flush_status (GmConnection *connection)
{
    DbQuery * q;
    gboolean ret;

    q = gm_db_new_query ("FLUSH STATUS");
    ret = mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    return ret;
}

static gboolean
mysql_admin_flush_tables (GmConnection *connection)
{
    DbQuery * q;
    gboolean ret;

    q = gm_db_new_query ("FLUSH TABLES");
    ret = mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    return ret;
}

static gboolean
mysql_admin_flush_locked_tables (GmConnection *connection)
{
    DbQuery * q;
    gboolean ret;

    q = gm_db_new_query ("FLUSH TABLES WITH READ LOCK");
    ret = mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    return ret;
}

static gboolean
mysql_admin_flush_privileges (GmConnection *connection)
{
    DbQuery * q;
    gboolean ret;

    q = gm_db_new_query ("FLUSH PRIVILEGES");
    ret = mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    return ret;
}



void
gm_mysql_admin_flush (void *ignore, GmConnection *connection)
{

    GmFlushFlags flag;

    flag = gm_dialogs_flush_options_dialog ();
    if (!flag)
        return;

    if (flag & FLUSH_HOSTS || flag == FLUSH_ALL)
        if (!mysql_admin_flush_hosts (connection))
            return;

    if (flag & FLUSH_LOGS || flag == FLUSH_ALL)
        if (!mysql_admin_flush_logs (connection))
            return;

    if (flag & FLUSH_STATUS || flag == FLUSH_ALL)
        if (!mysql_admin_flush_status (connection))
            return;

    if (flag & FLUSH_TABLES || flag == FLUSH_ALL)
        if (!mysql_admin_flush_tables (connection))
            return;

    if (flag & FLUSH_LOCKED_TABLES || flag == FLUSH_ALL)
        if (!mysql_admin_flush_locked_tables (connection))
            return;

    if (flag & FLUSH_PRIVILEGES || flag == FLUSH_ALL)
        if (!mysql_admin_flush_privileges (connection))
            return;
}


void
gm_mysql_admin_check_tables (GmConnection *connection, gchar *database, GList * tables_list)
{

    GmCheckTableEnums check_option;
    DbQuery * q;
    gchar * type = NULL;
    GString * s;
    GList * l;

    g_return_if_fail (tables_list != NULL || database != NULL);

    check_option = gm_dialogs_check_table_options_dialog ();
    if (!check_option)
        return;
    switch (check_option){
    case CHECK_NORMAL:
        break;
    case CHECK_QUICK:
        type = g_strdup ("QUICK");
        break;
    case CHECK_FAST:
        type = g_strdup ("FAST");
        break;
    case CHECK_CHANGED:
        type = g_strdup ("CHANGED");
        break;
    case CHECK_EXTENDED:
        type = g_strdup ("EXTENDED");
        break;
    }
    l = tables_list;
    s= g_string_new ("CHECK TABLE ");
    while (l) {
        g_string_append (s, (gchar *)l->data);
        if (l->next)
            g_string_append (s, ", ");
        l = l->next;
    }
    if (type)
        g_string_sprintfa (s, " TYPE=%s", type);

    q = gm_db_new_query (s->str);
    q->database = database;
    mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    g_free (type);
    g_string_free (s, TRUE);
}


void
gm_mysql_admin_repair_tables (GmConnection *connection, gchar *database, GList * tables_list)
{

    GmRepairTableEnums repair_option;
    DbQuery * q;
    gchar * type = NULL;
    GString * s;
    GList * l;

    g_return_if_fail (tables_list != NULL || database != NULL);

    repair_option = gm_dialogs_repair_table_options_dialog ();
    if (!repair_option)
        return;
    switch (repair_option){
    case REPAIR_NORMAL:
        break;
    case REPAIR_QUICK:
        type = g_strdup ("QUICK");
        break;
    }
    l = tables_list;
    s= g_string_new ("REPAIR TABLE ");
    while (l) {
        g_string_append (s, (gchar *)l->data);
        if (l->next)
            g_string_append (s, ", ");
        l = l->next;
    }
    if (type)
        g_string_sprintfa (s, " TYPE=%s", type);

    q = gm_db_new_query (s->str);
    q->database = database;
    mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    g_free (type);
    g_string_free (s, TRUE);
}

void
gm_mysql_admin_optimize_tables (GmConnection *connection, gchar *database, GList * tables_list)
{


    DbQuery * q;
    GString * s;
    GList * l;

    g_return_if_fail (tables_list != NULL || database != NULL);


    l = tables_list;
    s= g_string_new ("OPTIMIZE TABLE ");
    while (l) {
        g_string_append (s, (gchar *)l->data);
        if (l->next)
            g_string_append (s, ", ");
        l = l->next;
    }
    q = gm_db_new_query (s->str);
    q->database = database;
    mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    g_string_free (s, TRUE);
}


void
gm_mysql_admin_analyze_tables (GmConnection *connection, gchar *database, GList * tables_list)
{


    DbQuery * q;
    GString * s;
    GList * l;


    g_return_if_fail (tables_list != NULL || database != NULL);


    l = tables_list;
    s= g_string_new ("ANALYZE TABLE ");
    while (l) {
        g_string_append (s, (gchar *)l->data);
        if (l->next)
            g_string_append (s, ", ");
        l = l->next;
    }
    q = gm_db_new_query (s->str);
    q->database = database;
    mysql_admin_execute_query (connection, q);
    gm_db_free_query (q);
    g_string_free (s, TRUE);
}



void
gm_mysql_admin_truncate_tables (GmConnection *connection, gchar *database, GList * tables_list)
{


    DbQuery * q;
    gchar *query;
    GList * l;

    g_return_if_fail (tables_list != NULL || database != NULL);
    if (!gm_utils_yes_no_dialog (_("All the records in the selected table(s) will be deleted \nDo you want to continue ?")))
        return;


    l = tables_list;

    while (l) {
        query = g_strdup_printf ("DELETE FROM %s", (gchar *)l->data);
        q = gm_db_new_query (query);
        q->database = database;
        mysql_admin_execute_query (connection, q);
        gm_db_free_query (q);
        g_free (query);
        l = l->next;
    }
}
