/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include "gm.h"
#include <gmodule.h>
#include <dirent.h>
#include "gm-plugin.h"


#ifndef DATA_PLUGINS_TO_REMOVE
#define DATA_PLUGINS_TO_REMOVE "plugins_to_remove"
#endif








void			gm_plugins_init (void);

static GmPluginData *	plugin_load (const gchar *file);
#if 0
static void		gm_plugin_unload (GmPluginData *pd);
#endif
static void		plugin_load_dir (char *dir);
static void		plugin_load_all (void);





void
gm_plugins_init (void)
{

    if (!g_module_supported ())
    {
        gdm(_("Dynamic Module loading is not supported Plugins will not be available"));
        return;
    }


    plugin_load_all ();
}

#if 0
void
gm_plugins_menu_add (void)
{
    GmPluginData  *pd;
    gint         n;
    gchar       *path;
    GnomeUIInfo *menu;
    gint *items_to_remove;
    gint menu_pos;





    path = g_strdup_printf ("%s/", _("_Plugins"));

    items_to_remove = gtk_object_get_data (GTK_OBJECT(app), DATA_PLUGINS_TO_REMOVE);

    if(items_to_remove != NULL)
    {

        gnome_app_remove_menus (app, path, *items_to_remove);

        gtk_object_remove_data(GTK_OBJECT(app), DATA_PLUGINS_TO_REMOVE);
    }

    n = g_list_length (plugins_list) + 1 ;

    menu = g_malloc0 ( (n + 3)* sizeof (GnomeUIInfo));

    /* Add the plugin manager item */
    menu[0].type = GNOME_APP_UI_ITEM;
    menu[0].label = _("_Manager ...");
    menu[0].hint  = _("Manage available plugin modules");
    menu[0].moreinfo = gm_plugin_manager_create;
    menu[0].pixmap_type = GNOME_APP_PIXMAP_STOCK;
    menu[0].pixmap_info = GNOME_STOCK_MENU_BOOK_OPEN;

    /* Add a Separator*/
    menu[1].type = GNOME_APP_UI_SEPARATOR;

    menu_pos = 2;

    for (n = 0; n < g_list_length (plugins_list); n++)
    {
        pd = g_list_nth_data (plugins_list, n);

        if (!pd->installed)
            continue;

        menu [menu_pos].type = GNOME_APP_UI_ITEM;
        menu [menu_pos].label = pd->name;
        menu [menu_pos].hint = pd->desc;
        menu [menu_pos].moreinfo = pd->private_data;
        menu [menu_pos].pixmap_type = GNOME_APP_PIXMAP_NONE;
        menu_pos++;
    }

    menu [menu_pos].type = GNOME_APP_UI_ENDOFINFO;

    gnome_app_insert_menus (app, path, menu);
    gnome_app_install_menu_hints (app, menu);

    g_free (path);
    g_free (menu);

    items_to_remove = g_new0 (gint, 1);
    *items_to_remove = menu_pos;
    gtk_object_set_data_full (GTK_OBJECT(app), DATA_PLUGINS_TO_REMOVE, items_to_remove, g_free);
}
#endif

void
gm_plugin_save_settings (void)
{
    gint n;
    gchar * config_string;
    GmPluginData *nth_plugin;

    for (n = 0; n < g_list_length (global_app->plugins_list); n++)
    {
        nth_plugin = g_list_nth_data (global_app->plugins_list, n);

        config_string = g_strdup_printf ("/gmyclient/installed_plugins/%s=",
                                         g_basename(nth_plugin->file));
        gnome_config_set_bool (config_string, nth_plugin->installed);
        g_free (config_string);
    }
}







static GmPluginData *
plugin_append (GmPluginData *pd)
{

    GList * plugins_list;

    plugins_list = global_app->plugins_list;
    while (plugins_list)
    {
        GmPluginData *pdata;
        pdata = (GmPluginData *) plugins_list->data;
        if (pdata->name)
        {
            if (!strcmp(pdata->name, pd->name))
            {
                return NULL;
            }
        }
        plugins_list = plugins_list->next;
    }
    global_app->plugins_list = g_list_append (global_app->plugins_list, pd);
    return pd;
}


static GmPluginData *
plugin_load (const gchar *file)
{
    GmPluginData *pd;
    guint res;

    gchar * config_string;




    g_return_val_if_fail (file != NULL, NULL);

    gdm("loading plugin %s", file);

    if (!(pd = g_new0 (GmPluginData, 1)))
    {
        gdw ("plugin allocation error");
        return NULL;
    }


    pd->file = g_strdup (file);
    pd->handle = g_module_open (file, 0);

    if (!pd->handle)
    {
        gdw (_("Error, unable to open module file, %s"),
             g_module_error ());

        g_free (pd);
        return NULL;
    }

    if (!g_module_symbol (pd->handle, "init_plugin",
                          (gpointer*)&pd->init_plugin))
    {
        gdw (_("Error, plugin does not contain init_plugin function."));
        goto error;
    }

    if (!g_module_symbol (pd->handle, "run_plugin",
                          (gpointer*)&pd->run_plugin))
    {
        gdw (_("Error, plugin does not contain run_plugin function."));
        goto error;
    }


    pd->installed_by_default = FALSE;
    pd->desc = NULL;
    pd->long_desc = NULL;

    res = pd->init_plugin (pd);
    if (res != PLUGIN_OK)
    {
        gdw (_("Error, init_plugin returned an error"));
        goto error;
    }
    if (pd->name == NULL)
    {
        gdw ("Error, the plugin did not specified a name" );
        goto error;
    }
    if (pd->desc == NULL)
    {
        gdw ("Error, the plugin did not specified a description" );
        goto error;
    }
    if (pd->long_desc == NULL)
    {
        gdw ("Error, the plugin did not specified a long description" );
        goto error;
    }


    config_string = g_strdup_printf ("/gmyclient/installed_plugins/%s=%s",
                                     g_basename(file),
                                     pd->installed_by_default?"TRUE":"FALSE");
    pd->installed = gnome_config_get_bool(config_string);
    g_free (config_string);




    return plugin_append (pd);

error:
    g_module_close (pd->handle);
    g_free (pd->file);
    g_free (pd);
    return NULL;
}



#if 0
/* NOt used, but we migth need it latter */
static void
gm_plugin_unload (GmPluginData *pd)
{
    int w, n;
    char *path;
    GnomeApp *app;

    g_return_if_fail (pd != NULL);

    if (pd->can_unload && !pd->can_unload (pd))
    {
        g_warning (_("Error, plugin is still in use"));
        return;
    }

    if (pd->destroy_plugin)
        pd->destroy_plugin (pd);

    n = g_slist_index (plugins_list, pd);
    plugins_list = g_slist_remove (plugins_list, pd);

    path = g_strdup_printf ("%s/", _("_Plugins"));

    for (w = 0; w < g_list_length (mdi->windows); w++)
    {
        app = g_list_nth_data (mdi->windows, w);
        gnome_app_remove_menu_range (app, path, n, 1);
    }

    g_module_close (pd->handle);
    g_free (pd->file);
    g_free (pd);
}
#endif

static void
plugin_load_dir (char *dir)
{
    DIR *d;
    struct dirent *e;



    if ((d = opendir (dir)) == NULL)
        return;

    while ((e = readdir (d)) != NULL)
    {
        if (strncmp (e->d_name + strlen (e->d_name) - 3, ".so", 3) == 0)
        {
            char *plugin = g_strconcat (dir, e->d_name, NULL);
            plugin_load (plugin);
            g_free (plugin);
        }
    }
    closedir (d);
}

static void
plugin_load_all (void)
{
    gchar *pdir;
    gchar const * const home = g_get_home_dir ();


    gdm ("DEBUG_PLUGINS");


    /* load user plugins */
    if (home != NULL)
    {
        pdir = gnome_util_prepend_user_home (".gmyclient/plugins");
        plugin_load_dir (pdir);
        g_free (pdir);
    }

    /* load system plgins */
    plugin_load_dir (PLUGINS_DIR "/");
}















