/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __GM_PLUGIN_H__
#define __GM_PLUGIN_H__

#include <gmodule.h>


enum
{
    PLUGIN_OK,
    PLUGIN_CANCELED,
    PLUGIN_ERROR,
    PLUGIN_DEAD
};

typedef enum
{
    PLUGIN_UNKNOWN=0,
    PLUGIN_EXPORTER
}GmPluginTypes;


typedef enum
{
    ALL_ROWS,
    SELECTED_ROWS,
    INVERTED_ROWS_SELECTION
}GmRowsSelectionTypes;





typedef struct _GmPluginData GmPluginData;
typedef struct _GmExporterPluginData GmExporterPluginData;

struct _GmPluginData
{
    gchar	     *file;
    GmPluginTypes    type;
    GModule	     *handle;

    gint	(*init_plugin) 		(GmPluginData *);
    gint	(*run_plugin) 		(GmPluginData *);
    gint	(*can_unload)		(GmPluginData *);
    void	(*destroy_plugin)	(GmPluginData *);

    gchar	*name;
    gchar	*desc;
    gchar	*long_desc;
    gchar	*author;
    gchar       *preset_file;


    /* filled in by caller */
    void	*private_data;

    /* Is this plugin installed ? */
    gint installed;
    gint installed_by_default;

    /* mainview progressbar and status mini_bufferto be used if needed */
    GtkWidget * progressbar;
    GtkWidget * mini_buffer;

};


struct _GmExporterPluginData
{
    gboolean ignore_first_row;
    gchar    *output_filename;
    GList    *rows_selection;
    GmRowsSelectionTypes stype;
    GList    *columns_list;
    GtkCList *data_clist;
};






extern GList	*plugins_list;
extern GList	*plugins_installed_list;

/* Plugin MUST have this function */
/*extern gint init_plugin (GmPluginData *pd); */

gint init_plugin (GmPluginData *pd);
gint run_plugin (GmPluginData *pd);

void			gm_plugins_init (void);


void			gm_plugin_save_settings (void);

gchar *			gm_plugin_program_location_get (gchar *program_name, gchar *plugin_name, gint dont_guess);
gchar *			gm_plugin_program_location_change (gchar * program_name, gchar * plugin_name);

#endif /* __GM_PLUGIN_H__ */


