/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <math.h>
#include "gm.h"


#if GM_PRINT_SUPPORT

#include "gm-clist.h"
#include "gm-print.h"
#include "print-util.h"








static void           print_create_pages (PrintJobInfo *pji);
static void           print_document (PrintJobInfo *pji);
static void           real_print (gboolean preview, GmConnection *connection);
static gboolean       print_run_dialog (PrintJobInfo *pji);
static PrintJobInfo * print_job_info_new (GmConnection *connection);
static void           print_job_info_destroy (PrintJobInfo *pji);
static GtkWidget    * print_get_data_list (GmConnection * connection, gint *ignore_first_row) ;
static void           print_calculate_pages (PrintJobInfo *pji);

static gint           print_determine_lines (PrintJobInfo *pji);
static void           print_start_job (PrintJobInfo *pji);
static void           print_end_job (PrintJobInfo *pji);
static void           print_header (PrintJobInfo *pji, unsigned int page);
static void           print_table_header (PrintJobInfo *pji, gint page_num);
static void           print_set_orientation (PrintJobInfo *pji);
static void           print_start_page (PrintJobInfo *pji, gint current_page);
static void           print_end_page (PrintJobInfo *pji);
static void           print_preview_real (PrintJobInfo *pji);
static gchar          *print_compose_print_line (PrintJobInfo * pji, gint row);
static void           print_row (PrintJobInfo * pji, PrintRowData *prd);
static GList          *print_get_virtuell_columns (PrintJobInfo * pji);





static void
print_start_page (PrintJobInfo *pji, gint current_page)
{

    gchar *pagenumbertext;

            /* We need to call gnome_print_beginpage so that it adds
            "%% Page x" comment needed for viewing postcript files (i.e. in gv)*/
    
    pagenumbertext = g_strdup_printf ("%d", current_page);
    gnome_print_beginpage (pji->pc, pagenumbertext);
    print_set_orientation(pji);
    g_free (pagenumbertext);
}

static void
print_set_orientation (PrintJobInfo *pji)
{
    double affine [6];

    if (pji->orientation == PRINT_ORIENT_PORTRAIT)
        return;

    art_affine_rotate (affine, 90.0);
    gnome_print_concat (pji->pc, affine);
    gdm("calling gnome_print_concat 1 ");
    

    art_affine_translate (affine, 0, -pji->page_height);
    gnome_print_concat (pji->pc, affine);
    gdm("calling gnome_print_concat 2 ");
    

}






static void
print_header (PrintJobInfo *pji, unsigned int page)
{

    guchar* text2 = g_strdup_printf (_("Page: %i/%i"), page, pji->pages);
    float x,y,len;



    gnome_print_setfont (pji->pc, pji->font_header);
    gdm("calling  gnome_print_setfont 3 ");
    


    y = pji->page_height - pji->margin_top ;
    len = gnome_font_get_width_string (pji->font_header, text2);
    x = pji->page_width - len - 36;
    gnome_print_moveto (pji->pc, x, y);
    gdm("calling  t_moveto 4 ");
    
    gm_print_show (pji->pc, text2,0);
    g_free (text2);
}



static void
print_start_job (PrintJobInfo *pji)
{
    gm_print_progress_start (pji);
}


static void
print_end_page (PrintJobInfo *pji)
{


    gnome_print_showpage (pji->pc);
    gdm("calling  t_showpage 5 ");
}

static void
print_end_job (PrintJobInfo *pji)
{
    gnome_print_context_close (pji->pc);
    gm_print_progress_end (pji);
    gm_debug_message (GM_PRINTER_CHANEL, "Printing done");
}
static gint
print_determine_lines (PrintJobInfo *pji)
{

    g_return_val_if_fail (pji->data_list != NULL, 0);

    if (GTK_IS_CLIST(pji->data_list)) {
        GtkCList * clist = GTK_CLIST (pji->data_list);
        return clist->rows;
    }
    return 0;
}


static float print_calculate_columns_width (PrintJobInfo *pji)
{
    float max_width = 0  ;
    gint i = 0 ;
    GtkCList * clist = GTK_CLIST (pji->data_list);
    gint row = 0;
    GtkCListRow *clist_row;
    gchar * text;
    float max = 0;

    g_return_val_if_fail ( clist != NULL || !clist->columns, 0);

    pji->columns_width        =  g_new (float, clist->columns);
    pji->max_chars_per_column =  g_new (gint, clist->columns);
    pji->split_columns        =  g_new (gint, clist->columns);

    for (i=0; i<clist->columns; i++) {
        pji->columns_width[i] = 2 * pji->font_char_width;
        pji->max_chars_per_column[i] = 0;
    }



    for (row=0; row<clist->rows; row++)
    {
        gint column;

        clist_row = ROW_ELEMENT (clist, row)->data;
        for (column=0; column<clist->columns; column++) {

            if (clist_row->cell[column].type != GTK_CELL_TEXT)
                continue;
            text = GTK_CELL_TEXT (clist_row->cell[column])->text;
            if (text ) {
                gint len = strlen (text);
                if (len) {
                    float w = pji->font_char_width * len ;

                    pji->columns_width[column] = (w>pji->columns_width[column])?w:pji->columns_width[column];
                    if (len > pji->max_chars_per_column[column])
                        pji->max_chars_per_column[column] = len;
                }
            }
        }
    }

    /* checkt  the header labels in the CLIST */


    for (i=0; i<clist->columns; i++) {
        gchar *title;
        title = gm_clist_get_column_title (clist, i);
        if (title) {
            if (strlen(title) > pji->max_chars_per_column[i])
                pji->max_chars_per_column[i]= strlen(title);
        }
    }
    max_width = 0;
    for (i=0; i<clist->columns; i++) {
        max += (2 +pji->max_chars_per_column[i]) * pji->font_char_width ;
        gm_debug_message (GM_PRINTER_CHANEL, "Max chars column %d  %d ", i,pji->max_chars_per_column[i]);
        max_width += pji->columns_width[i] ;
        /* add the margin sizes right and left one char */
    }


    return max;
}







static void
print_calculate_pages (PrintJobInfo *pji)
{


}


static
GtkWidget * print_get_data_list (GmConnection * connection, gint *ignore_first_row)
{
    GtkWidget * main_note_book;
    GtkWidget * current_npage;
    GmNpageData * npage_data;




    main_note_book = gm_utils_get_data  (GTK_OBJECT(connection->connection_window),"main_note_book");
    current_npage   = gtk_notebook_get_nth_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_note_book)));
    npage_data = (GmNpageData *) gm_utils_get_data  (GTK_OBJECT(current_npage),"npage_data");


    if (npage_data->data_list){
        if (gtk_clist_get_row_data (GTK_CLIST (npage_data->data_list), 0))
            *ignore_first_row = 1;

        if (GTK_CLIST(npage_data->data_list)->rows - *ignore_first_row > 0)
            return npage_data->data_list;
        else
            return NULL;
    }



    return NULL;
}






static void
print_job_info_destroy (PrintJobInfo *pji)
{
    gnome_print_master_close (pji->master);
    gdm("calling  t_master_close 6 ");
    


    pji->buffer = NULL;

    gtk_object_unref (GTK_OBJECT (pji->font_body));
    gtk_object_unref (GTK_OBJECT (pji->font_header));
    gtk_object_unref (GTK_OBJECT (pji->font_table_header));
    g_free (pji);
}

static PrintJobInfo *
print_job_info_new (GmConnection *connection)
{
    PrintJobInfo *pji;
    pji = g_new0 (PrintJobInfo, 1);

    /* CList Widget */
    pji->ignore_first_row  = 0;
    pji->data_list = print_get_data_list (connection, &pji->ignore_first_row);
    if (!pji->data_list){
        gm_utils_message (_("No data in your result to print "));
        g_free(pji);
        return NULL;
    }


    pji->preview     = FALSE;

    /* GnomePrint Master and Context */


    pji->master  = gnome_print_master_new ();
    gdm("calling  t_master_new 7 ");
    
    pji->pc      = gnome_print_master_get_context (pji->master);
    gdm("calling  t_master_get_context 8 ");
    
    pji->printer = NULL;

    g_return_val_if_fail (GNOME_IS_PRINT_MASTER (pji->master), NULL);
    g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pji->pc), NULL);



    /* Paper and Orientation */
    pji->paper       = gnome_paper_with_name (PR_CONFIG(global_app)->paper_size);
    gnome_print_master_set_paper (pji->master, pji->paper);
    gdm("calling  t_master_set_paper 9 ");
    
    g_return_val_if_fail (pji->paper != NULL, NULL);


    pji->orientation = PR_CONFIG(global_app)->print_orientation;

    if (pji->orientation == PRINT_ORIENT_LANDSCAPE)
    {
        pji->page_width  = gnome_paper_psheight (pji->paper);
        pji->page_height = gnome_paper_pswidth (pji->paper);
    }
    else
    {
        pji->page_width  = gnome_paper_pswidth (pji->paper);
        pji->page_height = gnome_paper_psheight (pji->paper);
    }

    /* Margins and printable area */
    pji->margin_numbers = .25 * 72;
    pji->margin_top     = .75 * 72; /* Printer margins, not page margins */
    pji->margin_bottom  = 72;       /* We should "pull" this from gnome-print when */
    pji->margin_left    = .75 * 72; /* gnome-print implements them */
    pji->margin_right   = .75 * 72;
    pji->header_height  = .25 * 72;


    pji->printable_width  = pji->page_width -
                            pji->margin_left -
                            pji->margin_right ;
    pji->printable_height = pji->page_height -
                            pji->margin_top -
                            pji->margin_bottom;

    /* Fonts */
    pji->font_char_width  = 0.0808 * 72; /* Get this from the font itself !! */
    pji->font_char_height = .14 * 72;






    pji->font_body    = gnome_font_new (GM_PRINT_FONT_BODY, GM_PRINT_FONT_SIZE_BODY);
    pji->font_header  = gnome_font_new (GM_PRINT_FONT_HEADER, GM_PRINT_FONT_SIZE_HEADER);
    pji->font_char_width = gnome_font_get_width_string (pji->font_body, "D");

    pji->font_table_header = gnome_font_new_closest (GM_PRINT_FONT_BODY,GNOME_FONT_BOLD,FALSE,GM_PRINT_FONT_SIZE_BODY);


    g_return_val_if_fail (GNOME_IS_FONT (pji->font_body),    NULL);
    g_return_val_if_fail (GNOME_IS_FONT (pji->font_header),  NULL);
    g_return_val_if_fail (GNOME_IS_FONT (pji->font_table_header),  NULL);


    /* Chars, lines and pages */
    pji->wrapping         = 0;
    pji->print_line_numbers = 0;
    pji->print_grid     = PR_CONFIG(global_app)->print_grid;
    pji->print_header     = 1;



    pji->chars_per_line   = (gint)(pji->printable_width /
                                   pji->font_char_width);
    pji->lines_per_page   = ((pji->printable_height - pji->header_height ) /
                             (pji->font_char_height -  1 +  ROWS_MARGIN));
    pji->lines_per_page--;
    pji->tab_size         = 0 ;
    pji->range            = GNOME_PRINT_RANGE_ALL;

    print_calculate_pages (pji);
    print_create_pages    (pji);

    pji->vcol_list = print_get_virtuell_columns (pji);
    pji->canceled = FALSE;

    return pji;
}



static gboolean
print_run_dialog (PrintJobInfo *pji)
{
    GnomeDialog *dialog;





    dialog = (GnomeDialog *) gnome_print_dialog_new (
                 (const char *) _("Print Your data"),
                 GNOME_PRINT_DIALOG_RANGE);
    gdm(" gnome_print_dialog_new 10");
    
    gnome_print_dialog_construct_range_page ( (GnomePrintDialog * ) dialog,
            GNOME_PRINT_RANGE_ALL |
            GNOME_PRINT_RANGE_RANGE |
            FALSE,
            1, pji->pages, "A",
            _("Pages"));

    switch (gnome_dialog_run (GNOME_DIALOG (dialog))) {
    case GNOME_PRINT_PRINT:
        break;
    case GNOME_PRINT_PREVIEW:
        pji->preview = TRUE;
        break;
    case -1:
        return TRUE;
    default:
        gnome_dialog_close (GNOME_DIALOG (dialog));
        return TRUE;
    }

    pji->printer = gnome_print_dialog_get_printer (GNOME_PRINT_DIALOG (dialog));

    /* If preview, do not set the printer so that the print button in the preview
     * window will pop another print dialog */
    if (pji->printer && !pji->preview)
        gnome_print_master_set_printer (pji->master, pji->printer);

    pji->range = gnome_print_dialog_get_range_page (
                     GNOME_PRINT_DIALOG (dialog),
                     &pji->page_first,
                     &pji->page_last);

    if (pji->range == GNOME_PRINT_RANGE_SELECTION){
        //    gm_print_range_is_selection (pji, start_pos, end_pos);
    }


    gnome_dialog_close (GNOME_DIALOG (dialog));

    return FALSE;
}












static void
real_print (gboolean preview, GmConnection *connection)
{
    PrintJobInfo *pji;
    gboolean cancel = FALSE;


    if (!gm_print_verify_fonts ())
        return;

    pji = print_job_info_new (connection);
    if (!pji)
        return;


    pji->preview = preview;


    if (!pji->preview){
        cancel = print_run_dialog (pji);
    }



    /* The canceled button on the dialog was clicked */

    if (cancel) {
        print_job_info_destroy (pji);
        return;
    }
    print_document (pji);

    /* The printing was canceled while in progress */
    if (pji->canceled) {
        print_job_info_destroy (pji);
        return;
    }

    if (pji->preview)
        print_preview_real (pji);
    else
        gnome_print_master_print (pji->master);

    print_job_info_destroy (pji);

}







/**
 * print_preview_real:
 * @pji: 
 * 
 * Create and show the print preview window
 **/
static void
print_preview_real (PrintJobInfo *pji)
{
    GnomePrintMasterPreview *gpmp;
    gchar *title;

    title = g_strdup_printf (_("Gnome Mysql client : Print Preview\n"));
    if (pji->orientation == PRINT_ORIENT_LANDSCAPE)
        gpmp = gnome_print_master_preview_new_with_orientation (pji->master, title, TRUE);
    else
        gpmp = gnome_print_master_preview_new_with_orientation (pji->master, title, FALSE);
    g_free (title);

    gtk_widget_show (GTK_WIDGET (gpmp));
}





static void
print_create_pages (PrintJobInfo *pji)
{

    GtkCList * clist = GTK_CLIST (pji->data_list);
    gint page = 0;
    gint row = 0;
    gint current_row = 0;
    PrintPageData * ppd = NULL;
    float total_width;

    pji->total_lines = print_determine_lines (pji);
    pji->total_width = total_width  = print_calculate_columns_width (pji);
    pji->hpages = ceil(total_width / pji->printable_width);

    for (row=pji->ignore_first_row; row<clist->rows; row++){

        gint p = 0;
        PrintRowData * prd ;
        gint offset = 0;
        gchar * line;


        line = print_compose_print_line (pji, row);


        for (p=0; p<pji->hpages; p++) {
            GString * s;
            GList * l = NULL;
            s = g_string_new ("");
            prd = g_new (PrintRowData, 1);
            g_string_append_len (s, line, offset, pji->chars_per_line);
            offset +=pji->chars_per_line;
            prd->line = s->str;
            g_string_free (s, FALSE);
            ppd = NULL;
            l   = g_list_nth (pji->pages_list, page + p);
            if (l){
                ppd = (PrintPageData *) (l)->data;
            }
            else
            {
                ppd = g_new (PrintPageData, 1);
                ppd->page_number = p + page + 1; /* real page number */
                ppd->rows = NULL;
                pji->pages_list = g_list_append (pji->pages_list, ppd);
            }
            ppd->rows = g_list_append (ppd->rows, prd);
        }
        current_row++;
        if (current_row == pji->lines_per_page){
            page += pji->hpages;
            current_row = 0;
        }
        //    pji->pages = ceil ((float) (pji->total_lines) / (float) pji->lines_per_page) * pji->hpages;
        pji->pages = g_list_length (pji->pages_list);
        pji->page_first = 1;
        pji->page_last = pji->pages;
    }
}





static void
print_row (PrintJobInfo * pji, PrintRowData *prd)
{
    gfloat y;
    gfloat x;


    /* Calculate the y position */

    y = pji->page_height -  pji->margin_top - pji->header_height -
        ( (pji->font_char_height + ROWS_MARGIN )* ( (pji->current_row % pji->lines_per_page)+1 ));
    x = pji->margin_left;
    if (prd->line) {
        gnome_print_moveto (pji->pc, x, y );
        gnome_print_show (pji->pc, prd->line);
        /*                  draw_cell_grid (pji, x, y, pcd->len); */        
        g_free (prd->line);
    }

}


static
gchar * print_compose_print_line (PrintJobInfo * pji, gint row)
{

    GtkCList * clist = GTK_CLIST (pji->data_list);
    GtkCListRow *clist_row = NULL;
    GString * string = NULL;
    gint text_size;
    gint column;
    gchar * line;
    gint i =0;

    string = g_string_new ("");
    /* if the given row is negativ we will get the buttonstext */
    if (row>=0)
        clist_row = ROW_ELEMENT (clist, row)->data;

    for (column=0; column<clist->columns; column++) {
        gchar * text = NULL;
        gint  spaces = 0;

        if (clist_row) {
            if (clist_row->cell[column].type == GTK_CELL_TEXT)
                text = GTK_CELL_TEXT (clist_row->cell[column])->text;
            else
                text = NULL;
        }else {
            text = gm_clist_get_column_title (clist, column);
        }

        text_size = 0;
        if (text)
            text_size = strlen (text);
        spaces = pji->max_chars_per_column[column] - text_size;


        spaces++; /* right margin is one character */
        g_string_append (string, " "); /* first leading space */

        if (text){
            g_string_append (string, text);
        }

        for (i=0; i<spaces; i++)
            g_string_append (string, " ");
    }
    line = string->str;
    g_string_free (string, FALSE);

    return line;
}


static void
print_table_header (PrintJobInfo *pji, gint page_num)
{


    float x,y;
    gchar * line;
    gint offset = 0;
    GString * s;
    gint rest;
    gint hpage_num;


    rest  = page_num / pji->hpages;

    if (page_num >= pji->hpages)
        hpage_num = page_num - (rest * pji->hpages);
    else
        hpage_num = page_num;
        gm_debug_message (GM_PRINTER_CHANEL,"Printing Table header page_num %d  hpages %d", page_num, pji->hpages);

    line = print_compose_print_line (pji, -1);
    offset = hpage_num * pji->chars_per_line;
    s = g_string_new ("");
    g_string_append_len (s, line, offset, pji->chars_per_line);
    gnome_print_setfont (pji->pc, pji->font_table_header);
    /* Print the file name */
    y = pji->page_height - pji->margin_top - pji->header_height;

    x = pji->margin_left;

    gnome_print_moveto (pji->pc, x, y);
    gm_print_show (pji->pc, s->str, 0);
    gnome_print_setfont (pji->pc, pji->font_body);
    g_string_free (s, TRUE);
    g_free (line);

}






static GList * print_get_virtuell_columns (PrintJobInfo * pji)
{
    GtkCList * clist = GTK_CLIST (pji->data_list);
    GList * vcol_list=NULL;
    gint column;
    gint max_chars = 0;
    gint col_size = 0;
    gint p = 0;


    for (column=0; column<clist->columns; column++) {
        PrintVirtuellColumns *vcol ;
        if (!col_size)
            col_size = pji->max_chars_per_column[column];

        vcol = g_new (PrintVirtuellColumns, 1);
        vcol->origin = column;
        vcol->left_open  = FALSE;
        vcol->right_open = FALSE;
        vcol->page = p;
        if (max_chars + col_size + 2 > pji->chars_per_line) {
            if (col_size != pji->max_chars_per_column[column])
                vcol->left_open = TRUE;
            vcol->right_open = TRUE;
            vcol->size = pji->chars_per_line - max_chars - 1;
            col_size -= (pji->chars_per_line - max_chars) - 1;
            max_chars = 0;
            column--;
            p++;
        }else {
            vcol->size = col_size;

            if (col_size != pji->max_chars_per_column[column]){
                vcol->left_open = TRUE;
                max_chars += col_size + 1 ;
            }else
                max_chars += col_size + 2 ;

            col_size = 0;
        }
        vcol_list = g_list_append (vcol_list, vcol);
    }
    return vcol_list;
}



static void
print_draw_grid (PrintJobInfo *pji, gint page_num)
{

    GList * vcol_list;
    gint hpage_num;
    gint rest;
    gfloat x,y,x1,y1;
    gint line;
    PrintPageData * ppd ;
    gint lines;
    GList * l;



    gnome_print_gsave (pji->pc);
    gnome_print_setlinewidth (pji->pc, 0.5);

    rest  = page_num / pji->hpages;

    if (page_num >= pji->hpages)
        hpage_num = page_num - (rest * pji->hpages);
    else
        hpage_num = page_num;

    y = pji->page_height - pji->margin_top - pji->header_height ;
    y += pji->font_char_height + 2; /* to get the line over the heder text */

    x = pji->margin_left;



    l = g_list_nth (pji->pages_list,page_num );
    g_return_if_fail (l != NULL);

    ppd = (PrintPageData *) (l)->data;
    lines = g_list_length (ppd->rows) - 1;
    y1 = pji->page_height -  pji->margin_top - pji->header_height -
         ( (pji->font_char_height + ROWS_MARGIN )* ( (lines % pji->lines_per_page)+1 ));
    y1 -=2;

    for (vcol_list = pji->vcol_list; vcol_list; vcol_list = vcol_list->next) {

        PrintVirtuellColumns *vcol = vcol_list->data;
        gint margin = 0;

        if (vcol->page != hpage_num)
            continue;
        gnome_print_moveto (pji->pc, x, y);

        x1 = x;
        if (!vcol->left_open)
            margin++;

        if (!vcol->right_open)
            margin++;
        x += ( (margin + vcol->size) * pji->font_char_width);

        if (!vcol->left_open)
            gnome_print_lineto (pji->pc, x1 , y1);
        /* print the last right vertical line */
        if (!vcol_list->next){
            gnome_print_moveto (pji->pc, x, y);
            gnome_print_lineto (pji->pc, x, y1);
        }
    }


    y = pji->page_height - pji->margin_top - pji->header_height;

    y += pji->font_char_height + 2; /* to get the line over the heder text */


    x1 = x ;

    x = pji->margin_left  ;
    for (line = 0; line<= lines + 1; line++){
        gnome_print_moveto (pji->pc, x, y);
        gnome_print_lineto (pji->pc, x1, y);
        y -= (pji->font_char_height + ROWS_MARGIN);
    }
    gnome_print_moveto (pji->pc, x, y);
    gnome_print_lineto (pji->pc, x1, y);
    gnome_print_stroke (pji->pc);
    gnome_print_grestore (pji->pc);
}

static void
print_document (PrintJobInfo *pji)
{
    int current_page, current_row, current_hpage = 1;
    gboolean first_time = TRUE;
    gboolean font_set = FALSE;

    current_row = 0;
    print_start_job (pji);
    

    for (current_page = 1; current_page <= pji->pages; current_page++)
    {

        pji->current_row = 0;

        if (current_page <  pji->hpages) {
            first_time = TRUE;
            font_set = FALSE;
        }
        if (pji->range != GNOME_PRINT_RANGE_ALL)
            pji->print_this_page = (current_page >= pji->page_first &&
                                    current_page <= pji->page_last) ? TRUE:FALSE;
        else
            pji->print_this_page = TRUE;        

        if (pji->print_this_page)
        {

            GList * rows;
            GList * p;
            PrintPageData * ppd ;
            PrintRowData *prd;

            print_start_page (pji, current_page);

            /* Print the header of the page */
                print_header (pji, current_page);

            gnome_print_setfont (pji->pc, pji->font_body);

            if (pji->print_grid)
                print_draw_grid (pji, current_page -1);


            p = g_list_nth (pji->pages_list, current_page-1);
            if (p) {
                ppd = (PrintPageData *) p->data;
                if (pji->print_header)
                    print_table_header (pji, current_page-1);
                rows = ppd->rows;
                while (rows) {
                    prd = (PrintRowData * ) rows->data;
                    print_row (pji, prd);
                    rows = rows->next;
                    pji->current_row++;
                }
            }
        }
        if (pji->print_this_page)
            print_end_page (pji);

        gm_print_progress_tick (pji, current_page);
        if (pji->canceled)
            break;
        if (pji->hpages > 1 && current_hpage < pji->hpages) {
            current_hpage++;
        }else {
            current_hpage = 1;
        }
    }

    print_end_job (pji);
}

#endif # gnome-print-support


void gm_print (void * ignore, GmConnection * connection)
{
#if GM_PRINT_SUPPORT
    real_print (FALSE, connection);
#endif
}


void gm_print_preview (void * ignore, GmConnection * connection)
{
#if GM_PRINT_SUPPORT
    real_print (TRUE, connection);
#endif
}
