/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GM_PRINT_H__
#define _GM_PRINT_H__
typedef enum {
    PRINT_ORIENT_LANDSCAPE,
    PRINT_ORIENT_PORTRAIT
} GmPrintOrientation;

#define GM_PRINT_FONT_BODY   "Courier"
#define GM_PRINT_FONT_HEADER "Helvetica"
#define GM_PRINT_FONT_SIZE_BODY   10
#define GM_PRINT_FONT_SIZE_TABLE_HEADER 10
#define GM_PRINT_FONT_SIZE_HEADER 8
#define COL_MARGIN (.14 * 72)
#define ROWS_MARGIN (.07 * 72)
#include "gm.h"


#if GM_PRINT_SUPPORT

#include <libgnomeprint/gnome-printer.h>
#include <libgnomeprint/gnome-printer-dialog.h>
#include <libgnomeprint/gnome-print-master.h>
#include <libgnomeprint/gnome-print-master-preview.h>
#include <libgnomeprint/gnome-print-preview.h>



typedef struct _PrintPageData PrintPageData;
typedef struct _PrintRowData PrintRowData;
typedef struct _PrintCellData PrintCellData;
typedef struct _HPagesInfo HPagesInfo;


struct _PrintRowData
{
    gchar * line;
};




typedef struct
{
    gint origin;
    gboolean left_open;
    gboolean right_open;
    gint size;
    gint page;
}
PrintVirtuellColumns;

struct _PrintPageData
{
    gint page_number;
    gint total_lines;
    GList * rows;
};



typedef struct _PrintJobInfo {
    /* gnome print stuff */
    GnomePrintMaster *master;
    GnomePrintContext *pc;
    GnomePrinter *printer;
    const GnomePaper *paper;

    /* Data stuff */

    GtkWidget * data_list;
    gint columns;
    gint ignore_first_row;
    gboolean print_header;
    gboolean print_grid;
    gint print_line_numbers;
    guchar *buffer;
    guint buffer_size;
    float *columns_width;
    gint *split_columns;
    gint  *max_chars_per_column; /* contains the maximal size in chars that contains the column */
    float total_width;
    gint hpages;
    GList *pages_list;
    HPagesInfo *hpages_info;




    /* Font stuff */
    float   font_char_width;
    float   font_char_height;
    GnomeFont *font_body;
    GnomeFont *font_header;
    GnomeFont *font_table_header;


    /* Page stuff */
    guint   pages;
    float   page_width, page_height;
    float   margin_top, margin_bottom, margin_left, margin_right, margin_numbers;
    float   printable_width, printable_height;
    float   header_height;
    guint   total_lines, total_lines_real;
    guint   lines_per_page;
    guint   chars_per_line;
    guchar* temp;
    GmPrintOrientation orientation;

    /* Range stuff */
    gint range;
    gint page_first;
    gint page_last;
    gboolean print_this_page;
    gboolean preview;

    /* buffer stuff */
    gint file_offset;
    gint current_row;

    /* Text Wrapping */
    gboolean wrapping;
    gint tab_size;

    /* Progress */
    GtkWidget   *progress_bar;
    GnomeDialog *progress_dialog;
    gboolean canceled;
    GList * vcol_list;
} PrintJobInfo;




#endif # gnome-print-support
void gm_print (void * ignore, GmConnection * connection);
void gm_print_preview (void * ignore, GmConnection * connection);




#endif
