/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gm.h"
#include "gm-qbuffer.h"
#include "gm-clist.h"
#include "gm-interface.h"

#include <sys/types.h>
#include <regex.h>

#define DEFAULT_HISTORY_FILE_NAME ".gmyclient-history"
#define MAX_LINES 1000
#define REMOVE_MESSAGE_TIMEOUT 700
#define UNENDLICHKEIT 10000000
#define GM_CONFIG_DIR_NAME ".gmyclient"

typedef struct _GmQuery GmQueryPositon;

struct _GmQuery
{
    gint start_pos;
    gint length;
};







static void    gm_query_buffer_class_init         (GmQueryBufferClass *klass);
static void    gm_query_buffer_init               (GmQueryBuffer      *query_buffer);

static gint    gm_query_buffer_key_press          (GtkWidget    *widget, GdkEventKey    *event);

static void    gm_query_buffer_size_allocate      (GtkWidget     *widget, GtkAllocation  *allocation);
static void    gm_query_buffer_size_request       (GtkWidget     *widget, GtkRequisition *requisition);



static void    query_buffer_load_history          (GmConnection * connection);


static void    gm_query_buffer_destroy            (GtkObject *object);
static void    query_buffer_dynamic_expansion     (GmQueryBuffer   *query_buffer);
static gchar * query_buffer_get_completion_prefix (GtkEditable *editable, gint * start_pos, gint * end_pos);
static void    query_buffer_cycle_expand          (GmQueryBuffer *query_buffer);
static void    gm_query_buffer_configure_clist (GmConnection *connection);





static GtkWidgetClass *parent_class = NULL;

guint
gm_query_buffer_get_type (void)
{
    static guint query_buffer_type = 0;

    if (!query_buffer_type)
    {
        static const GtkTypeInfo query_buffer_info =
            {
                "GmQueryBuffer",
                sizeof (GmQueryBuffer),
                sizeof (GmQueryBufferClass),
                (GtkClassInitFunc) gm_query_buffer_class_init,
                (GtkObjectInitFunc) gm_query_buffer_init,
                /* reserved_1 */ NULL,
                /* reserved_2 */ NULL,
                (GtkClassInitFunc) NULL,
            };

        query_buffer_type = gtk_type_unique (gtk_text_get_type (), &query_buffer_info);
    }
    return query_buffer_type;
}



static void
gm_query_buffer_class_init (GmQueryBufferClass *class)
{
    GtkWidgetClass *widget_class;
    GtkObjectClass *object_class;

    widget_class = (GtkWidgetClass*) class;
    parent_class = gtk_type_class (gtk_text_get_type ());
    object_class = (GtkObjectClass *) class;


    widget_class->key_press_event = gm_query_buffer_key_press;
    widget_class->size_request    = gm_query_buffer_size_request;
    widget_class->size_allocate   = gm_query_buffer_size_allocate;

    object_class->destroy = gm_query_buffer_destroy;
}


static void
gm_query_buffer_init (GmQueryBuffer *query_buffer)
{

    query_buffer->compl_cycling   = FALSE;
    query_buffer->remember_message = NULL;
    query_buffer->remember_cursor_pos = 0;
    query_buffer->remove_messages_timer_tag = 0;
}



static void
query_buffer_on_insert_text              (GtkEditable     *editable,
        gchar           *new_text,
        gint             new_text_length,
        gint            *position,
        gpointer         user_data)
{

}




static void
query_buffer_on_delete_text                   (GtkEditable     *editable,
        gint             start_pos,
        gint             end_pos,
        gpointer         user_data)
{




}





GtkWidget*
gm_query_buffer_new (GmConnection *connection)
{

    GmQueryBuffer *query_buffer;
    gint block_id;

    query_buffer = gtk_type_new (gm_query_buffer_get_type ());

    gm_query_buffer_configure_clist(connection);
    {
        gint i=0;
        for (i=0; i<GTK_CLIST (connection->qhistory_mlist)->rows ; i++) {
            gchar *text;            
            gtk_clist_get_text (GTK_CLIST (connection->qhistory_mlist), i, 0, &text);
            gdm ("After loading --- %s", text);
            
        }            
    }
    if (connection->qhistory_mlist)
        query_buffer->history_position = GTK_CLIST(connection->qhistory_mlist)->rows - 1;
    else
        query_buffer->history_position = 0;

    
    query_buffer->completion = global_app->keys_funcs_completion_list;

    gtk_object_set_data (GTK_OBJECT (query_buffer), "connection", connection);

    gm_utils_attach_menu_to_text_widget (GTK_WIDGET (query_buffer));

    block_id = gtk_signal_connect (GTK_OBJECT (query_buffer), "insert_text",
                                   GTK_SIGNAL_FUNC (query_buffer_on_insert_text),
                                   NULL);


    gtk_signal_connect (GTK_OBJECT (query_buffer), "delete_text",
                        GTK_SIGNAL_FUNC (query_buffer_on_delete_text),
                        NULL);

    return GTK_WIDGET (query_buffer);
}





static gint gm_query_buffer_key_press (GtkWidget   *widget,
                                       GdkEventKey *event)
{
    GmQueryBuffer * query_buffer ;
    GtkEditable *editable;


    query_buffer =  GM_QUERY_BUFFER(widget);
    editable = GTK_EDITABLE (widget);


    if (event->state & GDK_CONTROL_MASK)
    {
        switch (event->keyval)
        {
        case GDK_v :
            {		/* paste the ringdata */
                gtk_selection_convert(widget, GDK_SELECTION_PRIMARY,
                                      GDK_TARGET_STRING, 0);
                return TRUE;
                break;
            }
        case GDK_Return :
            {		/* Execute  */
                gm_query_buffer_execute_query (NULL, query_buffer);
                return TRUE;
                break;
            }
        }
    }



    if (query_buffer->compl_cycling) { /* select the current selected completion item */
        if (event->keyval == GDK_Return || event->keyval == GDK_Up || event->keyval == GDK_Down) {
            gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
            gtk_editable_select_region (GTK_EDITABLE (widget), 0,0);
            query_buffer->compl_cycling = FALSE;
            return TRUE;
        }
    }

    if ((event->keyval == GDK_Tab) )
    {
        query_buffer_dynamic_expansion (GM_QUERY_BUFFER (widget));
        return TRUE;		/* return TRUE to stop event-propag */
    }
    GM_QUERY_BUFFER (widget)->compl_cycling = FALSE;


    if ((* GTK_WIDGET_CLASS (parent_class)->key_press_event) (widget, event))
    {
        GM_QUERY_BUFFER (widget)->compl_cycling = FALSE;
        return TRUE;
    }


    return FALSE;
}





static void
gm_query_buffer_size_allocate (GtkWidget     *widget,
                               GtkAllocation *allocation)
{

    if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
        (* GTK_WIDGET_CLASS (parent_class)->size_allocate) (widget, allocation);
}


static void
gm_query_buffer_size_request (GtkWidget      *widget,
                              GtkRequisition *requisition)
{

    if (GTK_WIDGET_CLASS (parent_class)->size_request)
        (* GTK_WIDGET_CLASS (parent_class)->size_request) (widget, requisition);

}













void gm_query_buffer_save_history (GmQueryBuffer * query_buffer)
{

    gchar * home_dir = NULL;
    gchar * filename = NULL;

    GmConnectionAccessData *access_data;
    GmConnection * connection;
    GtkCList *clist;
    






    g_return_if_fail(query_buffer != NULL);
    connection = gm_utils_get_data (query_buffer, "connection");
    access_data = connection->access_data;

    g_assert (access_data);

    home_dir = g_get_home_dir();

    filename =g_strdup_printf("%s/%s/%s_%s",home_dir,GM_CONFIG_DIR_NAME, access_data->host, access_data->user);

    clist = GTK_CLIST(connection->qhistory_mlist);
    if (!clist || !clist->rows){        
        gchar *command;        
        command =g_strdup_printf("rm -f %s", filename);
        system (command);
        g_free(command); 
        return;
    }
    gm_utils_save_clist_to_file (filename, MAX_LINES, clist);
}


static void
gm_query_buffer_destroy (GtkObject * object)
{

    GmQueryBuffer * query_buffer;





    query_buffer = GM_QUERY_BUFFER (object);
    gm_query_buffer_save_history (query_buffer);

    if (GTK_OBJECT_CLASS (parent_class)->destroy)
        (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}





/* -------------------------------------------------------------------------------- Completion stuff ----------------------------------------*/






/* this function return a new allocated prefix must be freed by the caller */

static gchar * query_buffer_get_completion_prefix (GtkEditable *editable, gint * start_pos, gint * end_pos)
{

    gint cursor_pos = 0;
    GString *s_pre = NULL;
    gchar *prefix = NULL;
    gint i=0;

    gchar *ch;



    g_return_val_if_fail (editable != NULL, NULL);



    cursor_pos = editable->current_pos;
    if (!cursor_pos) return NULL;
    i = cursor_pos ;
    *end_pos = i;

    while (i >=1) {
        ch = gtk_editable_get_chars(editable, i-1, i);
        if (*ch !=' ' && *ch !='\n' && *ch !='\r' && *ch !='\t' && *ch!='(') {
            if (s_pre)
                g_string_prepend_c(s_pre, *ch);
            else {
                s_pre = g_string_new(ch);
            }
        }else
            break;
        i--;
    }
    *start_pos = i;
    if (s_pre) {
        prefix = s_pre->str;
        g_string_free (s_pre, FALSE);
    }
    return prefix;
}






static void  query_buffer_dynamic_expansion (GmQueryBuffer * query_buffer)
{

    g_return_if_fail (query_buffer!=NULL);



    if (GTK_TEXT(query_buffer)->text_end - GTK_TEXT(query_buffer)->gap_size <=0)
        return;

    query_buffer_cycle_expand (query_buffer);
    return;
}


typedef struct 
{
    gchar * prefix;
    gint type;
}CompletionHit;


static void query_buffer_cycle_expand (GmQueryBuffer *query_buffer)
{
    GtkEditable * editable;
    static gchar * new_prefix = NULL;
    static gchar * prefix=NULL;
    static GList *compl_items = NULL;
    static GList *item = NULL;
    static gint   cursor_pos=0;
    static gint last_insert_length    = 0;
    gboolean first_time = FALSE;
    gchar * s, *inserted_string = NULL;
    gint i = 0;
    gint new_cursor_pos;
    gint start_pos=0, end_pos=0;


    editable  = GTK_EDITABLE(query_buffer);
    if (!query_buffer->compl_cycling)
    {
        cursor_pos = editable->current_pos;
        if (prefix)
            g_free(prefix);        
        if (!(prefix = query_buffer_get_completion_prefix (editable, &start_pos, &end_pos)))
        {
            return;
        }
        g_strup (prefix);
        query_buffer->compl_cycling = TRUE;

        item = compl_items = g_completion_complete (query_buffer->completion, prefix, &new_prefix);
        if (item) {
            gint temp_pos = start_pos;
            gtk_editable_delete_text (editable, start_pos, end_pos);
            gtk_editable_insert_text (editable, prefix, strlen(prefix), &temp_pos);
        }

        first_time = TRUE;
    }
    if (!compl_items)
        return;

    if (!first_time &&  g_list_length (compl_items)==1)
        return;
    if (!item)
        item = compl_items;
    if (strlen(prefix) == strlen (item->data))
        return;

    s = item->data;
    for (i=0;i<strlen(prefix);i++)
        s++;
    if (!first_time)
    {
            /* remove the last inserted completion and restore the cursor position */
        gtk_editable_delete_text (editable, cursor_pos, last_insert_length + cursor_pos);
        gtk_editable_set_position (editable, cursor_pos);        
    }
    new_cursor_pos = cursor_pos;

    inserted_string = g_strdup_printf ("%s",s);

    gtk_editable_insert_text (editable, inserted_string, strlen(inserted_string), &new_cursor_pos);
    g_free (inserted_string);
    last_insert_length = new_cursor_pos - cursor_pos;
    gtk_editable_set_position (editable, new_cursor_pos);
    gtk_editable_select_region (editable, cursor_pos + (strlen (new_prefix)- strlen(prefix)),new_cursor_pos);
    gdm ("after selection current_pos = %d", new_cursor_pos);    
    item = item->next;
}




















#if 0
static gboolean mb_prepare_query (GmConnection * connection, DbQuery * q)
{

    gchar * firstcommand = NULL;
    gchar * param[100];
    gchar * query;
    gchar **commands;
    gint i=0, p=0;


    query  = g_strdup (q->query);

        /* FIXME: this function is bugy */

        /* split the lines */


    commands = g_strsplit (query, " ", 0);
    param[p] = NULL;
    while (commands[i])
    {
        param[p]= NULL;
        g_strstrip(commands[i]);
        if (strlen(commands[i])>=1)
        {
            if (!i)
                firstcommand = commands[i];
            else
            {
                param[p] =  commands[i];
                p++;
            }
        }
        i++;
    }
        /* see if we have some intern command attached with */

    if (firstcommand)
    {
        if ( !strcmp (firstcommand, "use") && param[0])
        {
            gchar *s;
            s = param[0];
            s += strlen(s)-1;

            if (*s == ';')
                *s = '\0';


            gm_db_change_database (connection, param[0]);
        }else if (!strcmp(firstcommand,"lt"))
        {
            g_free(q->query);
            q->query = g_strdup ("show tables");
        }else if (!strcmp(firstcommand,"ld"))
        {
            g_free(q->query);
            q->query = g_strdup ("show databases");
        }
    }
    g_free (query);
    g_strfreev (commands);
    return TRUE;
}
#endif



void 
gm_query_buffer_set_cursor_at_error_position (GmConnection * connection, DbQuery * q)
{
    gchar * cmp= "SQL syntax near '";
    gchar * position;
    gchar *query;
    
    g_return_if_fail (q != NULL);
    
    query = q->query;
    
    position = strstr (q->error, cmp);
    if (position) {
        gint size;
        GString *error_string;
            
        position += 17; /* strlen(cmp) */
        size = strlen (position);
        size--;            
        while (size) {
            if (position[size]=='\'')
                break;
            size--;                
        }
        error_string = g_string_new("");
        if (size>0){
            gchar *position_in_line;                
            g_string_append_len (error_string, position, 0,size);
            position_in_line = strstr (query, error_string->str);
            if (position_in_line){
                gint p_start;
                gint p_end;                    
                p_start = position_in_line - query;
                p_end = p_start + size;                    
                gm_utils_text_colorize_error (GTK_EDITABLE(connection->query_buffer), p_start, p_end);
                gtk_editable_set_position (GTK_EDITABLE(connection->query_buffer), p_start);
            }
        }
        g_string_free (error_string, TRUE);            
    }
}


/* return a list of query_matches stucture */

static GList * query_buffer_lines_scanner (gchar * buffer)
{
    gchar * s = NULL;
    gchar * index;
    gboolean quote_open = FALSE;
    gboolean double_quote_open = FALSE;
    gboolean escaped = FALSE;
    GList   * result_list = NULL;
    GmQueryPositon * sql_query_line;
    gint start_pos = 0;
    gint length = 0;

    s = index = buffer ;
    while (*s)
    {
        if (!escaped && *s == '\\')
        {
                /* ignore the next charachter */
            s++;
            escaped = TRUE;
        }else if (!escaped && *s == '\'')
        {
            quote_open = !quote_open;
        } else if (!quote_open && !escaped && *s == '"')
        {
            double_quote_open = !double_quote_open;
        } else if (!escaped && !quote_open  && !double_quote_open)
        {
            if (*s == ';')
            {
                length = (s - index);
                s++;
                index = s ;
                sql_query_line = g_new0 (GmQueryPositon, 1);
                sql_query_line->start_pos = start_pos;
                sql_query_line->length = length;
                start_pos += length + 1;
                result_list = g_list_append (result_list, sql_query_line);
                if (!*s)
                    break;
            }
        }
        escaped = FALSE;
        s++;
    }
    return result_list ;
}




void gm_query_buffer_execute_query (void * ignore, GmQueryBuffer *query_buffer)
{

    GList *list;
    gchar * buffer;
    gboolean abgebrochen = FALSE;
    GmConnection * connection;



    connection = gm_utils_get_data (query_buffer, "connection");
    buffer = gtk_editable_get_chars (GTK_EDITABLE (query_buffer), 0 , -1);


    list = query_buffer_lines_scanner (buffer);
    g_free (buffer);
    if (!list) {
        gdk_beep();
        gm_utils_show_status_information (connection, "You have to put a semikolone at the end of the mysql-query ");
        return;
    }

    while (list) {
        GmQueryPositon * q_line = list->data;
        gchar *sql_query,  *free_it;
        sql_query = free_it = gtk_editable_get_chars (GTK_EDITABLE (query_buffer), q_line->start_pos , q_line->start_pos + q_line->length);
        if (!gm_interface_execute_query_and_show_result (connection, sql_query, TRUE)){
            abgebrochen = TRUE;
            if (list->next){
                if (!gm_utils_yes_no_dialog ("Would you like to continue ")){
                    break;
                }
            }
        }else{
            gm_query_buffer_add_to_history (query_buffer, sql_query);
        }
        g_free(free_it);
        list = list->next;
    }
    if (!abgebrochen)
        gtk_editable_delete_text (GTK_EDITABLE (query_buffer), 0 , -1);
    return;
}



void 
gm_query_buffer_add_to_history (GmQueryBuffer *query_buffer, gchar * sql_query)
{
    gchar *line;
    gchar *first_row_text = NULL;
    
    GtkWidget * qhistory_mlist;
    gchar * history_row[1];
    GmConnection * connection;

    g_return_if_fail(query_buffer != NULL);


    if (!sql_query)
        return;

    line = g_strdup (sql_query);


    g_strstrip (line);
    if (!strlen (line)){
        g_free(line);
        return ;
    }
    connection = gm_utils_get_data (query_buffer, "connection");
    qhistory_mlist = connection->qhistory_mlist;

    gtk_clist_get_text (GTK_CLIST (qhistory_mlist), 0,0, &first_row_text);
    if (!first_row_text || (first_row_text && strcmp (first_row_text, line))) 
    {
        
        history_row[0] = line;
        gtk_clist_prepend (GTK_CLIST (qhistory_mlist), history_row);
        gtk_clist_unselect_all(GTK_CLIST (qhistory_mlist));
        gtk_clist_select_row (GTK_CLIST (qhistory_mlist), 0,0);
        query_buffer->history_position = GTK_CLIST (qhistory_mlist)->rows - 1;    
    }
    
}








static void gm_query_buffer_import_line ( GmConnection * connection, gint row)
{
    gchar * query;
    GtkWidget * query_buffer;
    gint new_cursor_pos;


    query_buffer = gm_utils_get_data (connection->connection_window, "query_buffer");


    gtk_clist_get_text              (GTK_CLIST (connection->qhistory_mlist), row, 0,&query);

    new_cursor_pos = GTK_EDITABLE (query_buffer)->current_pos;

    if (query)
        gtk_editable_insert_text (GTK_EDITABLE (query_buffer), query , strlen(query), &new_cursor_pos);

    gtk_editable_insert_text (GTK_EDITABLE (query_buffer), ";\n" , 2, &new_cursor_pos);

}


static void history_mlist_row_activated (GmClist * mlist, gint row, GmConnection * connection)
{

    GList * l = GTK_CLIST (mlist)->selection;
    GtkWidget * sql_log_book;
    
    while (l) {
        gm_query_buffer_import_line (connection, GPOINTER_TO_INT (l->data));
        l = l->next;
    }

    sql_log_book = gm_utils_get_data (connection->connection_window, "sql_log_book");
    gtk_notebook_set_page (GTK_NOTEBOOK (sql_log_book), SQL_NOTEBOOK_PAGE);
gtk_widget_grab_focus (connection->query_buffer);
}


static void  
query_buffer_load_history (GmConnection * connection)
{
    gchar * home_dir;
    gchar * file_name;
    GmConnectionAccessData *access_data;

    access_data = connection->access_data;

    g_assert (access_data);


    home_dir = g_get_home_dir();
    file_name = g_strdup_printf("%s/%s/%s_%s",home_dir,GM_CONFIG_DIR_NAME, access_data->host, access_data->user);
    gm_utils_load_file_to_clist (file_name, MAX_LINES,GTK_CLIST (connection->qhistory_mlist));
    g_free (file_name);
}




static void
gm_query_buffer_configure_clist (GmConnection *connection)
{
    GtkWidget * qhistory_mlist;
    GtkWidget * mlist_sw;
    GList *list = NULL;
    



    mlist_sw = gm_utils_get_data  (connection->connection_window, "qhistory_scrolled_window");

    connection->qhistory_mlist = qhistory_mlist = gm_clist_new (1);
    gtk_widget_show (qhistory_mlist);
    gtk_container_add (GTK_CONTAINER (mlist_sw), qhistory_mlist);
    gtk_clist_set_column_auto_resize (GTK_CLIST (qhistory_mlist), 0, TRUE);
    gtk_clist_set_selection_mode (GTK_CLIST (qhistory_mlist), GTK_SELECTION_EXTENDED);

    gtk_signal_connect (GTK_OBJECT (qhistory_mlist), "row_activated",
                        GTK_SIGNAL_FUNC (history_mlist_row_activated),
                        connection);
    query_buffer_load_history(connection);


    list = g_list_append (list,(gchar *)  _("Doppel click on a row, or press return to copy the query to the sql-query window, mutiple selection are possible"));
    list = g_list_append (list,(gchar *)  _("for completion use the tabkey, Return to select, repress tab key to cycle in the completion list "));
    list = g_list_append (list,(gchar *)  _("Ctrl + Return to fire your query"));

    gm_utils_register_hints (GTK_WIDGET (qhistory_mlist), list, connection) ;

}



