/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include "glade_interface.h"
#include "gm-app.h"
#include "gm-clist.h"
#include "gm-dbapi.h"
#include "gm-table-properties.h"
#include "gm-entry.h"
#include "gm-utils.h"
#include "gm-bpanel.h"
#include "gm-debug.h"
#include "gm-dbtree.h"

extern gchar * mysql_attributes[];

static GtkWidget * table_properties_create_notebook_box       (DbFieldInfo * field_info, gint fcounter);
static void        table_properties_remove_key_button_clicked                  (GtkButton * button, GtkWidget * fields_properties_window);
static void        table_properties_old_apply_changes_clicked (GtkButton * button, GtkWidget * fp_win);
static void        table_properties_new_apply_changes_clicked  (GtkButton * button, GtkWidget * fp_win);
static gchar     * table_properties_get_field_create_definition                (GtkWidget * field_page);
static GtkWidget * table_properties_fetch_describe_result                      (GmConnection *connection, gchar * database, gchar * table);
static void        table_properties_fill_general_data_list (DbQuery * q, GtkWidget *data_list);




/* callbacks */



void gm_table_properties_execute (GtkWidget * fp_win)
{
    gboolean is_new_table;

    is_new_table = GPOINTER_TO_INT (gm_utils_get_data_ww  (GTK_OBJECT (fp_win), "new_table"));

    if (is_new_table){
        table_properties_new_apply_changes_clicked (NULL, fp_win);
    }else {
        table_properties_old_apply_changes_clicked (NULL, fp_win);
    }
}


static void
on_columns_clist_select_row                   (GmClist        *mlist,
        gint             row,
        gint             column,
        GdkEvent        *event,
        GtkWidget * fields_properties_window)
{
    GtkWidget * fields_notebook;


    fields_notebook = gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "fields_notebook");
    gtk_notebook_set_page (GTK_NOTEBOOK (fields_notebook), row);

}







static gchar * key_name_in_one_string (GList *l)
{
    gchar * col_name;
    GString * string = NULL;

    l = g_list_first(l);
    while (l)
    {
        col_name = (gchar *) l->data;
        if (!string)
            string = g_string_new (col_name);
        else
            g_string_sprintfa (string, ", %s", col_name);

        l = l->next;
    }
    if (string){
        gchar *ret = g_strdup (string->str);
        g_string_free(string, TRUE);
        return ret;
    }else {
        return NULL;
    }
}


static gboolean check_table_properties_entries (GtkWidget * fields_properties_window)
{


    GtkWidget * fields_notebook;
    GtkWidget * notebook_page;
    GtkWidget * entry_widget;
    GtkEntry  * entry;
    GtkWidget * fields_spin;
    gchar *tmp_text;
    gint fields_count;
    gint i = 0;

    g_return_val_if_fail (fields_properties_window != NULL, FALSE);

    fields_notebook = gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "fields_notebook");

    entry_widget = gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "table_name_entry");
    entry = GTK_ENTRY (entry_widget);
    tmp_text = gtk_entry_get_text (entry);
    if (!tmp_text || strlen(tmp_text)<1)
    {
        gm_utils_warning ("Table name is empty ");
        return FALSE;
    }
    fields_spin =  gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fields_spin");
    fields_count = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fields_spin));
    for (i=0; i<fields_count; i++)
    {
        notebook_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), i);
        entry_widget = gm_utils_get_data  (GTK_OBJECT (notebook_page), "field_name_entry");
        entry = GTK_ENTRY (entry_widget);
        tmp_text = gtk_entry_get_text (entry);
        if (!tmp_text || strlen(tmp_text)<1)
        {
            gchar * warning;
            warning = g_strdup_printf ("Field number %d has no name ", i+1);
            gm_utils_warning (warning);
            g_free (warning);
            return FALSE;
        }
    }
    return TRUE;
}





static void rename_field_in_columns_clist (GtkCList *clist, gchar *old_name, gchar *new_name)
{
    GList * l;
    gint  row = 0;

    g_return_if_fail (clist != NULL);
    g_return_if_fail (old_name != NULL);
    g_return_if_fail (new_name != NULL);
    l = clist->row_list;

    while (l)
    {
        gchar *cdata = NULL;
        gtk_clist_get_text (clist, row, 0, &cdata);
        if (cdata && !strcmp (cdata, old_name))
        {
            gtk_clist_set_text (clist, row, 0 , new_name);
            return ;
        }
        l = l->next;
        row++;
    }
}


static void rename_field_in_keys_clist (GtkCList *clist, gchar *old_name, gchar *new_name)
{
    GList * l;
    gint  row = 0;

    g_return_if_fail (clist != NULL);
    g_return_if_fail (old_name != NULL);
    g_return_if_fail (new_name != NULL);


    l = clist->row_list;

    while (l)
    {
        GList * list, * lpointer = NULL;
        DbKeyInfo * key_info ;
        key_info = (DbKeyInfo *) gtk_clist_get_row_data (clist, row);
        list = lpointer = g_list_first (key_info->affected_column_list);

        while (lpointer)
        {
            gchar * this_col_name = (gchar *) lpointer->data;
            if (!strcmp (old_name, this_col_name))
            {
                gchar * key_names = NULL;
                g_free (this_col_name);
                lpointer->data = g_strdup (new_name);
                key_names = key_name_in_one_string (list);
                gtk_clist_set_text (clist, row, 1, key_names);
                g_free (key_names);
                break;
            }
            lpointer = lpointer->next;
        }
        l = l->next;
        row++;
    }
}




static gchar * get_create_keys_definition (GtkCList *keys_clist, gboolean add)
{

    GList *l=NULL;
    gint row =0;
    GString * create_keys_definition = NULL;
    g_return_val_if_fail (keys_clist != NULL, NULL);

    l = keys_clist->row_list;
    while (l)
    {
        DbKeyInfo * key_info ;
        gchar * columns;
        gchar * tmp_query = NULL;

        key_info = (DbKeyInfo *) gtk_clist_get_row_data (keys_clist, row);

        if (!key_info || key_info->key_exists){
            l = l->next;
            row++;
            continue;
        }


        gtk_clist_get_text (keys_clist, row, 1, &columns);




        if (key_info->is_primary_key){
            tmp_query = g_strdup_printf("PRIMARY KEY (%s) ", columns);
        }
        else if (key_info->is_unique)
        {
            if (key_info->name)
                tmp_query= g_strdup_printf("UNIQUE %s (%s) ", key_info->name, columns);
            else
                tmp_query = g_strdup_printf("UNIQUE (%s) ", columns);
        }
        else
        {
            if (key_info->name)
                tmp_query = g_strdup_printf("INDEX %s (%s) ", key_info->name, columns);
            else
                tmp_query = g_strdup_printf("INDEX (%s) ", columns);
        }
        if (!create_keys_definition){
            if (add){
                create_keys_definition = g_string_new ("add ");
                g_string_sprintfa (create_keys_definition, "%s", tmp_query);
            }else {
                create_keys_definition = g_string_new (tmp_query);
            }
        }else {
            if (add)
                g_string_sprintfa (create_keys_definition,", Add %s", tmp_query);
            else
                g_string_sprintfa (create_keys_definition,", %s", tmp_query);
        }
        g_free (tmp_query);
        l = l->next;
        row++;
    }

    if (create_keys_definition){
        gchar * str;
        str = g_strdup (create_keys_definition->str);
        g_string_free(create_keys_definition, TRUE);
        return str;
    }

    return NULL;

}




static void field_name_entry_changed            (GtkEditable     *editable,
        GtkWidget * notebook_box)
{

    GtkWidget * fields_notebook;
    GtkWidget * label;
    gchar * new_field_name;
    gchar  * old_field_name;
    GtkWidget *fields_properties_window;
    GtkCList * columns_clist;
    GtkCList * keys_clist;



    fields_properties_window = gm_utils_get_data  (GTK_OBJECT (notebook_box),"fields_properties_window");



    keys_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"keys_clist"));
    fields_notebook = gm_utils_get_data  (GTK_OBJECT (notebook_box), "fields_notebook");



    label =  gtk_notebook_get_tab_label      (GTK_NOTEBOOK(fields_notebook),
             notebook_box);
    old_field_name = GTK_LABEL(label)->label;
    new_field_name = gtk_entry_get_text (GTK_ENTRY(editable));

    columns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"columns_clist"));
    rename_field_in_columns_clist (columns_clist, old_field_name, new_field_name);
    columns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fcolumns_clist"));
    rename_field_in_columns_clist (columns_clist, old_field_name, new_field_name);

    rename_field_in_keys_clist (keys_clist, old_field_name, new_field_name);
    gtk_label_set_text (GTK_LABEL (label), new_field_name);
    gtk_object_set_data (GTK_OBJECT (notebook_box), "dirty", GINT_TO_POINTER (TRUE));
}






static void remove_field_from_columns_clist (GtkCList *clist, gchar *column_name)
{
    GList * l;
    gint  row = 0;

    l = clist->row_list;

    while (l)
    {
        gchar *cdata = NULL;
        gtk_clist_get_text (clist, row, 0, &cdata);
        if (cdata && !strcmp (cdata, column_name))
        {
            gtk_clist_remove (clist, row);
            return ;
        }
        l = l->next;
        row++;
    }
}







static void remove_field_from_keys_clist (GtkCList *clist, gchar *column_name, GtkWidget * fields_properties_window)
{

    GList * l;
    gint  row = 0;
    gboolean remove_row = FALSE;

    l = clist->row_list;

    while (l)
    {
        GList * list, *lpointer = NULL;
        DbKeyInfo * key_info ;
        key_info = (DbKeyInfo *) gtk_clist_get_row_data (clist, row);
        lpointer = g_list_first (key_info->affected_column_list);
        while (lpointer)
        {
            gchar * this_col_name = (gchar *) lpointer->data;
            if (!strcmp (column_name, this_col_name))
            {
                g_free (this_col_name);
                list = key_info->affected_column_list = g_list_remove (key_info->affected_column_list, lpointer->data);
                if (list)
                {
                    gchar * key_names;
                    key_names = key_name_in_one_string (list);
                    gtk_clist_set_text (clist, row, 1, key_names);
                    g_free (key_names);
                }
                else
                {
                    remove_row = TRUE;
                    gtk_clist_select_row (clist, row, 0);
                }
                break;

            }
            lpointer = lpointer->next;
        }
        l = l->next;
        row++;
    }

    if (remove_row)
        table_properties_remove_key_button_clicked              (NULL,fields_properties_window);
}


static void  fields_spin_changed            (GtkAdjustment     *adjustment,
        GtkWidget * fields_properties_window)
{

    gint old_fields_count ;
    gint fields_count ;
    GtkWidget * fields_spin;
    GtkWidget * fields_notebook;
    GtkCList  * columns_clist;
    GtkCList  * fcolumns_clist;
    GtkCList  * keys_clist;

    gint i = 0 ;



    old_fields_count =   GPOINTER_TO_INT (gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "fields_count"));
    fields_spin =  gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fields_spin");
    fields_count = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fields_spin));
    fields_notebook  = gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fields_notebook");
    columns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"columns_clist"));
    fcolumns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fcolumns_clist"));
    keys_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"keys_clist"));

    if (fields_count < old_fields_count)
    {
        for (i = old_fields_count; i>fields_count; i--)
        {
            GtkWidget * notebook_label;
            notebook_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (fields_notebook),
                             gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), i-1));
            remove_field_from_columns_clist (columns_clist,GTK_LABEL(notebook_label)->label);
            remove_field_from_columns_clist (fcolumns_clist,GTK_LABEL(notebook_label)->label);
            remove_field_from_keys_clist (keys_clist, GTK_LABEL(notebook_label)->label, fields_properties_window);
            gtk_notebook_remove_page  (GTK_NOTEBOOK (fields_notebook), i-1);
        }
    }else
    {
        for (i = old_fields_count; i<fields_count; i++)
        {
            gint new_row;
            GtkWidget * notebook_label;
            gchar     * notebook_label_name;
            gchar     * tmp_name;
            GtkWidget * notebook_box;
            gchar     * column_row_name[1];

            notebook_box = table_properties_create_notebook_box (NULL, i);
            gtk_container_add (GTK_CONTAINER (fields_notebook), notebook_box);
            tmp_name= g_strdup_printf("New field %d",i+1);
            column_row_name[0] = tmp_name;
            gtk_clist_append (columns_clist, column_row_name);
            new_row = gtk_clist_append (fcolumns_clist, column_row_name);
            gtk_clist_select_row (fcolumns_clist, new_row, 0);

            notebook_label = gtk_label_new (tmp_name);
            g_free (tmp_name);
            gtk_widget_ref (notebook_label);
            notebook_label_name = g_strdup_printf ("notebook_label%d", i);
            gtk_object_set_data_full (GTK_OBJECT (fields_properties_window), notebook_label_name, notebook_label,
                                      (GtkDestroyNotify) gtk_widget_unref);
            g_free (notebook_label_name);
            gtk_widget_show (notebook_label);
            gtk_notebook_set_tab_label (GTK_NOTEBOOK (fields_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), i), notebook_label);
            gtk_widget_ref (fields_notebook);
            gtk_object_set_data_full (GTK_OBJECT (notebook_box), "fields_notebook", fields_notebook,
                                      (GtkDestroyNotify) gtk_widget_unref);

            gtk_object_set_data (GTK_OBJECT (notebook_box), "fields_properties_window", fields_properties_window);
            gtk_object_set_data (GTK_OBJECT (notebook_box), "index", GINT_TO_POINTER (i));
            gtk_notebook_set_page (GTK_NOTEBOOK (fields_notebook), i);
            gtk_widget_grab_focus (GTK_WIDGET(gm_utils_get_data  (GTK_OBJECT (gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), i)), "field_name_entry")));
        }
    }
    gtk_object_set_data (GTK_OBJECT (fields_properties_window), "fields_count",GINT_TO_POINTER (fields_count));

}



static void table_properties_window_cancel_clicked (GtkButton * button ,
        GtkWidget * fp_win)
{
    GmConnection * connection;

    connection = gm_utils_get_data  (GTK_OBJECT (fp_win), "connection");
    gm_bpanel_select_db_browser_page(connection);
    gtk_widget_destroy (fp_win);
}

static void table_properties_remove_key_button_clicked              (GtkButton * button ,
        GtkWidget * fields_properties_window)
{
    GtkWidget * keys_clist_widget;
    GtkCList * keys_clist;
    gchar * drop_key_query ;
    gint row = 0;
    GList * keys_info_list;

    keys_clist_widget = gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"keys_clist");
    keys_clist = GTK_CLIST (keys_clist_widget);
    drop_key_query = $pointer (keys_clist_widget, "drop_key_query");
    keys_info_list = $pointer (keys_clist_widget, "keys_info_list");
    while (keys_clist->selection)
    {
        gchar * key_name;
        GList * l;
        row = GPOINTER_TO_INT (keys_clist->selection->data);
        gtk_clist_get_text (keys_clist, row, 0, &key_name);
        l = g_list_first (keys_info_list);
        while (l)
        {
            DbKeyInfo * key_info = (DbKeyInfo *) l->data;
            gchar * tmp_query = NULL;
            if (key_info->key_exists) {
                if (!strcmp(key_info->name, key_name))
                {
                    key_info->key_exists = FALSE;
                    if (key_info->is_primary_key)
                    {
                        tmp_query = g_strdup ("DROP PRIMARY KEY");
                    }else
                    {
                        tmp_query = g_strdup_printf("DROP INDEX %s", key_name);
                    }
                    if (drop_key_query)
                    {
                        gchar *free_it;
                        free_it = g_strdup_printf ("%s, %s",drop_key_query, tmp_query);
                        g_free(drop_key_query);
                        drop_key_query = free_it;
                    }else
                    {
                        drop_key_query = g_strdup(tmp_query);
                    }
                    g_free (tmp_query);
                    gtk_object_set_data (GTK_OBJECT (keys_clist_widget), "drop_key_query", drop_key_query);
                    break;
                }
            }

            l = l->next;
        }
        gtk_clist_remove (keys_clist, row);
    }
    row = (row>0)? row-1 :0;
    keys_clist->focus_row = row;
    gtk_clist_select_row (keys_clist, row ,0 );
}

static void new_key_button_clicked                 (GtkButton * button ,
        GtkWidget * fields_properties_window)
{
    GtkWidget * keys_clist_widget;
    GtkWidget * columns_clist_widget;
    GtkCList * keys_clist;
    GtkCList * columns_clist;
    GtkWidget * key_type_entry_widget;
    GtkEntry * key_type_entry;
    GtkEntry * key_name_entry;
    gchar * key_name = NULL;
    gchar * key_type_string;
    gint key_type = 0;
    GList * l;
    gint row=0;
    gchar * row_vector[3];
    GList * this_key_list = NULL;
    DbKeyInfo * key_info = NULL;



    keys_clist_widget     = $pointer(fields_properties_window, "keys_clist");
    columns_clist_widget  = $pointer (fields_properties_window, "columns_clist");
    key_type_entry_widget = $pointer (fields_properties_window, "key_type_entry");
    key_name_entry        = GTK_ENTRY ($pointer (fields_properties_window, "key_name_entry"));



    key_name        = gtk_entry_get_text (key_name_entry);
    keys_clist      = GTK_CLIST (keys_clist_widget);
    columns_clist   = GTK_CLIST (columns_clist_widget);
    key_type_entry  = GTK_ENTRY (key_type_entry_widget);
    key_type_string = gtk_entry_get_text (key_type_entry);

    if (!columns_clist->selection)
    {
        gm_utils_warning ("Please select one or more columns to associate the new key with\n");
        return;
    }

    if (!strcmp (key_type_string, "PRIMARY KEY"))
    {
        key_type = 0;
    }else if (!strcmp (key_type_string, "UNIQUE"))
    {
        key_type = 1;
    }else if (!strcmp (key_type_string, "INDEX"))
    {
        key_type = 2;
    }

    if (key_type)
    {
        row_vector[0]= key_name;
    }
    else
    {
        row_vector[0]= "PRIMARY";
    }


    if (gm_clist_find_row_from_column (keys_clist, row_vector[0], 0) != -1)
    {
        if (!key_type)
        {
            gm_utils_warning ("Multiple primary key defined !!!\n");
        }else
        {
            gchar * message;
            message = g_strdup_printf ("Duplicate key name '%s' !!!\n", row_vector[0]);
            gm_utils_warning (message);
            g_free (message);
        }
        return;
    }

    key_info = db_new_key_info ();
    key_info->key_exists = FALSE;
    l = columns_clist->selection ;
    key_info->name = g_strdup (row_vector[0]);

    if (!key_type)
        key_info->is_primary_key =TRUE;

    if (!key_type || key_type == 1)
        key_info->is_unique =  TRUE;

    if (!key_type || key_type == 1)
        row_vector[2] = "Yes";
    else
        row_vector[2] = "No";

    while (l)
    {
        gchar * col_name;
        row = GPOINTER_TO_INT (l->data);
        gtk_clist_get_text (columns_clist, row, 0, &col_name);
        l = l->next;
        this_key_list = g_list_append (this_key_list, g_strdup (col_name));
    }
    row_vector[1] = key_name_in_one_string (this_key_list);
    key_info->affected_column_list = this_key_list;

    gtk_clist_set_row_data_full (keys_clist, gtk_clist_append (keys_clist, row_vector), key_info, (GtkDestroyNotify) db_free_key_info);
    g_free (row_vector[1]);

}






static void remove_field_button_clicked            (GtkButton * button ,
        GtkWidget * fields_properties_window)
{



    GtkWidget *fields_notebook;
    GtkWidget *notebook_page;
    GtkWidget * fields_spin;
    gint selected_page;
    gint fields_count;

    fields_notebook = gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "fields_notebook");
    selected_page = gtk_notebook_get_current_page(GTK_NOTEBOOK (fields_notebook));
    notebook_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), selected_page);
    fields_count =   GPOINTER_TO_INT (gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "fields_count"));
    if (fields_count > 1)
    {
        gtk_widget_hide(notebook_page);
        gtk_notebook_reorder_child      (GTK_NOTEBOOK (fields_notebook),
                                         notebook_page,fields_count-1);
        fields_spin =  gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fields_spin");
        gtk_spin_button_set_value       (GTK_SPIN_BUTTON (fields_spin),
                                         (gfloat) fields_count-1);
    }



}

static void new_field_button_clicked               (GtkButton * button ,
        GtkWidget * fields_properties_window)
{

    gint old_fields_count ;
    GtkWidget * fields_spin;

    old_fields_count =   GPOINTER_TO_INT (gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "fields_count"));
    fields_spin =  gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fields_spin");
    gtk_spin_button_set_value       (GTK_SPIN_BUTTON (fields_spin),
                                     (gfloat) old_fields_count+1);

}







static void fill_table_properties_window (GtkWidget * fields_properties_window, DbTableInfo * table_info)
{
    GtkWidget * widget;
    GtkWidget * button;
    GtkWidget * notebook_box;
    GtkWidget * fields_notebook;
    GtkCList * columns_clist;
    GtkCList * fcolumns_clist;
    GList * list;
    GtkCList * keys_clist;
    gint i = 0;


    widget = gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"table_name_entry");
    if (table_info && table_info->name)
        gtk_entry_set_text (GTK_ENTRY (widget), table_info->name);
    else
        gtk_entry_set_text (GTK_ENTRY (widget), "NewTable");

    button = gm_utils_get_data  (GTK_OBJECT(fields_properties_window),"ok_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (table_properties_old_apply_changes_clicked),
                        fields_properties_window);

    fields_notebook = gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "fields_notebook");
    keys_clist    = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"keys_clist"));
    columns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"columns_clist"));
    fcolumns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fcolumns_clist"));

    list = table_info->fields_info_list;
    while (list)
    {
        DbFieldInfo  * field_info;
        GtkWidget * notebook_label;
        gchar     * notebook_label_name;
        gchar     * column_row_name[1];


        field_info = (DbFieldInfo *) list->data;
        notebook_box = table_properties_create_notebook_box(field_info, i);
        gtk_container_add (GTK_CONTAINER (fields_notebook), notebook_box);
        notebook_label = gtk_label_new (field_info->name);

        gtk_widget_ref (notebook_label);
        notebook_label_name = g_strdup_printf ("notebook_label%d", i);
        gtk_object_set_data_full (GTK_OBJECT (fields_properties_window), notebook_label_name, notebook_label,
                                  (GtkDestroyNotify) gtk_widget_unref);
        g_free (notebook_label_name);
        gtk_widget_show (notebook_label);
        gtk_notebook_set_tab_label (GTK_NOTEBOOK (fields_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), i), notebook_label);
        gtk_widget_ref (fields_notebook);
        gtk_object_set_data_full (GTK_OBJECT (notebook_box), "fields_notebook", fields_notebook,
                                  (GtkDestroyNotify) gtk_widget_unref);
        gtk_object_set_data (GTK_OBJECT (notebook_box), "fields_properties_window", fields_properties_window);
        gtk_object_set_data (GTK_OBJECT (notebook_box), "index", GINT_TO_POINTER (i));
        gtk_object_set_data (GTK_OBJECT (notebook_box), "field_info", field_info);
        column_row_name[0]=field_info->name;
        gtk_clist_append (columns_clist, column_row_name);
        gtk_clist_append (fcolumns_clist, column_row_name);
        list = list->next;
        i++;
    }
    gtk_object_set_data (GTK_OBJECT (fields_properties_window), "fields_count",GINT_TO_POINTER (i));



    if (table_info->keys_info_list)
    {
        GList * l;
        gchar * key_names;

        l =  g_list_first (table_info->keys_info_list);
        key_names = key_name_in_one_string (list);
        while (l)
        {
            gchar * key_row[3];
            DbKeyInfo * key_info = (DbKeyInfo *) l->data;

            if (key_info)
            {
                key_row[0] = key_info->name;
                if (key_info->is_unique)
                    key_row[2] = "Yes";
                else
                    key_row[2] = "No";
                key_names = key_name_in_one_string (key_info->affected_column_list);
                key_row[1] = key_names;
                gtk_clist_set_row_data (keys_clist, gtk_clist_append (keys_clist, key_row), key_info);
                g_free (key_names);
            }
            l = l->next;
        }
        gtk_object_set_data (GTK_OBJECT (keys_clist), "keys_info_list", table_info->keys_info_list);
    }


    {
        /* general infos feched with describe */
        GtkWidget * data_list ;
        GtkWidget * data_list_sw ;
        GmConnection * connection;

        connection = gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "connection");
        data_list = table_properties_fetch_describe_result (connection, table_info->database, table_info->name);
        data_list_sw = gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "data_list_sw");
        gtk_container_add (GTK_CONTAINER (data_list_sw), data_list);

    }






}

static void init_table_properties_window (GtkWidget * fields_properties_window)
{

    GtkWidget * button;
    GtkWidget * notebook_label;
    gchar * notebook_label_name;
    GtkWidget * notebook_box;
    GtkWidget * fields_notebook;
    GtkCList * columns_clist;
    GtkCList * fcolumns_clist;
    gchar * column_row_name[1];

    gint i =0;

    fields_notebook = gm_utils_get_data  (GTK_OBJECT (fields_properties_window), "fields_notebook");
    button = gm_utils_get_data  (GTK_OBJECT(fields_properties_window),"ok_button");
    columns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"columns_clist"));
    fcolumns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fcolumns_clist"));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (table_properties_new_apply_changes_clicked),
                        fields_properties_window);

    notebook_box = table_properties_create_notebook_box(NULL, i);
    gtk_object_set_data (GTK_OBJECT (notebook_box), "fields_properties_window", fields_properties_window);
    gtk_widget_show (notebook_box);

    gtk_container_add (GTK_CONTAINER (fields_notebook), notebook_box);
    column_row_name[0] ="NewField";
    notebook_label = gtk_label_new (column_row_name[0]);
    gtk_clist_append (columns_clist, column_row_name);
    gtk_clist_append (fcolumns_clist, column_row_name);

    gtk_widget_ref (notebook_label);
    notebook_label_name = g_strdup_printf ("notebook_label%d", i);
    gtk_object_set_data_full (GTK_OBJECT (fields_properties_window), notebook_label_name, notebook_label,
                              (GtkDestroyNotify) gtk_widget_unref);
    g_free (notebook_label_name);
    gtk_widget_show (notebook_label);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (fields_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), i), notebook_label);
    gtk_widget_ref (fields_notebook);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "fields_notebook", fields_notebook,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_object_set_data (GTK_OBJECT (notebook_box), "index", GINT_TO_POINTER (i));
    gtk_object_set_data (GTK_OBJECT (fields_properties_window), "fields_count",GINT_TO_POINTER (i+1));
}






GtkWidget * gm_table_properties_new (GmConnection *connection, DbTableInfo * table_info)
{

    GtkWidget * fields_properties_window=NULL;
    GtkWidget * fields_properties_halter_box;
    GtkWidget * fields_notebook;
    GtkWidget * table_properties_notebook = NULL;
    GtkWidget * key_name_entry;
    GtkWidget * pixmap;
    GtkWidget * button;
    GtkCList * keys_clist;
    GtkCList * columns_clist;
    GtkCList * fcolumns_clist;
    GtkWidget * widget;
    GtkAdjustment * adj;

    fields_properties_window = create_fields_properties_window();

    button = gm_utils_get_data  (GTK_OBJECT(fields_properties_window),"ok_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "ok");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);

    /* remove the first notebookpage if we have a new table */

    table_properties_notebook = $pointer (fields_properties_window, "table_properties_notebook");
    if (!table_info->fields_info_list)
        gtk_widget_destroy (gtk_notebook_get_nth_page (GTK_NOTEBOOK (table_properties_notebook), 0));


    fields_properties_halter_box = gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fields_properties_halter_box");

    if (table_info){
        gtk_object_set_data_full (GTK_OBJECT (fields_properties_window), "table_info", table_info,
                                  (GtkDestroyNotify) gm_db_free_table_info);
    }



    gtk_object_set_data (GTK_OBJECT (fields_properties_window), "connection", connection);



    columns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"columns_clist"));
    fcolumns_clist  = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fcolumns_clist"));

    gtk_clist_set_column_auto_resize (columns_clist, 0, TRUE);
    gtk_clist_set_column_auto_resize (fcolumns_clist, 0, TRUE);

    keys_clist    = GTK_CLIST (gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"keys_clist"));
    gtk_clist_set_column_auto_resize (keys_clist   , 0, TRUE);
    gtk_clist_set_column_auto_resize (keys_clist   , 1, TRUE);
    gtk_clist_set_column_auto_resize (keys_clist   , 2, TRUE);


    gtk_signal_connect (GTK_OBJECT (fcolumns_clist), "select_row",
                        GTK_SIGNAL_FUNC (on_columns_clist_select_row),
                        fields_properties_window);



    /* note book */
    fields_notebook = gtk_notebook_new ();
    gtk_widget_ref (fields_notebook);
    gtk_object_set_data_full (GTK_OBJECT (fields_properties_window), "fields_notebook", fields_notebook,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (fields_notebook);
    gtk_box_pack_start (GTK_BOX (fields_properties_halter_box), fields_notebook, TRUE, TRUE, 0);
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (fields_notebook), GTK_POS_RIGHT);
    gtk_notebook_set_scrollable (GTK_NOTEBOOK (fields_notebook), TRUE);
    gtk_notebook_set_show_tabs (GTK_NOTEBOOK (fields_notebook), FALSE);

    /* signals connection */



    /* fields toolbar */

    button = gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"new_field_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "add_column");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC (new_field_button_clicked),
                        fields_properties_window);

    button = gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"remove_field_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "delete_column");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC (remove_field_button_clicked),
                        fields_properties_window);
    /* key toolbar */

    button = gm_utils_get_data  (GTK_OBJECT(fields_properties_window),"new_key_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "add_key");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (new_key_button_clicked),
                        fields_properties_window);
    key_name_entry        = $pointer (fields_properties_window, "key_name_entry");
    gtk_signal_connect (GTK_OBJECT (key_name_entry), "activate",
                        GTK_SIGNAL_FUNC (new_key_button_clicked),
                        fields_properties_window);

    button = gm_utils_get_data  (GTK_OBJECT(fields_properties_window),"remove_key_button");
    gtk_widget_destroy (GTK_BIN (button)->child);
    pixmap = gm_stock_pixmap_widget (button, "del_key");
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (table_properties_remove_key_button_clicked),
                        fields_properties_window);

    button = gm_utils_get_data  (GTK_OBJECT(fields_properties_window),"cancel_button");
    gm_utils_create_button_label (button,"Cancel","cancel", FALSE);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (table_properties_window_cancel_clicked),
                        fields_properties_window);

    if (table_info->fields_info_list)
    {
        fill_table_properties_window (fields_properties_window, table_info);
    }
    else
    {
        gtk_object_set_data (GTK_OBJECT (fields_properties_window), "new_table",GINT_TO_POINTER (TRUE));
        init_table_properties_window (fields_properties_window);
    }


    widget = gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"comment_entry");
    if (table_info && table_info->comment)
        gtk_entry_set_text (GTK_ENTRY (widget), table_info->comment);


    widget = gm_utils_get_data  (GTK_OBJECT (fields_properties_window),"fields_spin");

    if (table_info && table_info->fields_count)
        gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),(gfloat) table_info->fields_count);
    else
        gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),(gfloat) 1);

    adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (widget));

    gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                        GTK_SIGNAL_FUNC (fields_spin_changed),
                        fields_properties_window);


    return fields_properties_window;
}




static GtkWidget * table_properties_create_notebook_box (DbFieldInfo * field_info, gint fcounter)
{

    GtkWidget *notebook_box;
    GtkWidget *field_name_entry;
    GtkWidget *type_combo;
    GtkWidget *type_entry;
    GtkWidget *length_entry;
    GtkWidget *attribute_combo;
    GtkWidget *attribute_entry;
    GtkWidget *null_combo;
    GtkWidget *null_entry;
    GtkWidget *default_entry;
    GtkWidget *extra_combo;
    GtkWidget *extra_entry;

    GList *type_combo_items = NULL;
    GList *attribute_combo_items = NULL;
    GList *null_combo_items = NULL;
    GList *extra_combo_items = NULL;






    notebook_box = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (notebook_box);
    gtk_container_set_border_width (GTK_CONTAINER (notebook_box), 1);

    field_name_entry = gtk_entry_new ();
    gtk_widget_ref (field_name_entry);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "field_name_entry", field_name_entry,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (field_name_entry);
    if (field_info && field_info->name)
        gtk_entry_set_text (GTK_ENTRY(field_name_entry),field_info->name);

    gtk_signal_connect (GTK_OBJECT (field_name_entry), "changed",
                        GTK_SIGNAL_FUNC (field_name_entry_changed),
                        notebook_box);

    gtk_box_pack_start (GTK_BOX (notebook_box), field_name_entry, FALSE, FALSE, 0);
    gtk_widget_set_usize (field_name_entry, -2, 24);
    type_combo = gtk_combo_new ();

    gtk_widget_ref (type_combo);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "type_combo", type_combo,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (type_combo);
    gtk_box_pack_start (GTK_BOX (notebook_box), type_combo, FALSE, FALSE, 0);
    gtk_widget_set_usize (type_combo, -2, 24);
    type_combo_items = g_list_append (type_combo_items, "Tinyint");
    type_combo_items = g_list_append (type_combo_items, "Smallint");
    type_combo_items = g_list_append (type_combo_items, "Mediumint");
    type_combo_items = g_list_append (type_combo_items, "Int");
    type_combo_items = g_list_append (type_combo_items, "Bigint");
    type_combo_items = g_list_append (type_combo_items, "Float");
    type_combo_items = g_list_append (type_combo_items, "Double");
    type_combo_items = g_list_append (type_combo_items, "Decimal");
    type_combo_items = g_list_append (type_combo_items, "Date");
    type_combo_items = g_list_append (type_combo_items, "Datetime");
    type_combo_items = g_list_append (type_combo_items, "Timestamp");
    type_combo_items = g_list_append (type_combo_items, "Time");
    type_combo_items = g_list_append (type_combo_items, "Year");
    type_combo_items = g_list_append (type_combo_items, "Char");
    type_combo_items = g_list_append (type_combo_items, "Varchar");
    type_combo_items = g_list_append (type_combo_items, "Tinyblob");
    type_combo_items = g_list_append (type_combo_items, "Tinytext");
    type_combo_items = g_list_append (type_combo_items, "Text");
    type_combo_items = g_list_append (type_combo_items, "Blob");
    type_combo_items = g_list_append (type_combo_items, "Mediumblob");
    type_combo_items = g_list_append (type_combo_items, "Mediumtext");
    type_combo_items = g_list_append (type_combo_items, "Longblob");
    type_combo_items = g_list_append (type_combo_items, "Longtext");
    type_combo_items = g_list_append (type_combo_items, "Enum");
    type_combo_items = g_list_append (type_combo_items, "SET");
    gtk_combo_set_popdown_strings (GTK_COMBO (type_combo), type_combo_items);
    g_list_free (type_combo_items);

    type_entry = GTK_COMBO (type_combo)->entry;
    gtk_widget_ref (type_entry);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "type_entry", type_entry,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (type_entry);
    if (field_info)
    {
        gtk_entry_set_text (GTK_ENTRY(type_entry), gm_db_get_mysql_type_string (field_info->type));
    }


    gtk_entry_set_editable (GTK_ENTRY (type_entry), FALSE);
    length_entry = gtk_entry_new ();
    gtk_widget_ref (length_entry);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "length_entry", length_entry,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (length_entry);
    if (field_info)
    {
        gchar * tmp_string=NULL;
        GList * list=NULL;
        if (!field_info->enum_list && field_info->length)
        {
            tmp_string = g_strdup (field_info->length);
        }else
        {
            list = field_info->enum_list;
            while (list)
            {
                gchar * s = (gchar *) list->data;
                if (tmp_string)
                {
                    gchar * back = tmp_string;
                    tmp_string = g_strdup_printf ("%s ,'%s' ",tmp_string, s);
                    g_free (back);
                }else
                {
                    tmp_string = g_strdup_printf ("'%s' ", s);
                }
                list = list->next;
            }
        }
        if (tmp_string)
            gtk_entry_set_text (GTK_ENTRY(length_entry), tmp_string);
        g_free (tmp_string);
    }

    gtk_box_pack_start (GTK_BOX (notebook_box), length_entry, FALSE, FALSE, 0);
    gtk_widget_set_usize (length_entry, -2, 24);

    attribute_combo = gtk_combo_new ();
    gtk_widget_ref (attribute_combo);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "attribute_combo", attribute_combo,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (attribute_combo);
    gtk_box_pack_start (GTK_BOX (notebook_box), attribute_combo, FALSE, FALSE, 0);
    gtk_widget_set_usize (attribute_combo, -2, 24);
    attribute_combo_items = g_list_append (attribute_combo_items, "");
    attribute_combo_items = g_list_append (attribute_combo_items, "Binary");
    attribute_combo_items = g_list_append (attribute_combo_items, "Unsigned");
    attribute_combo_items = g_list_append (attribute_combo_items, "Unsigned zerofill");
    gtk_combo_set_popdown_strings (GTK_COMBO (attribute_combo), attribute_combo_items);
    g_list_free (attribute_combo_items);

    attribute_entry = GTK_COMBO (attribute_combo)->entry;
    gtk_widget_ref (attribute_entry);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "attribute_entry", attribute_entry,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (attribute_entry);
    gtk_entry_set_editable (GTK_ENTRY (attribute_entry), FALSE);
    if (field_info && field_info->attributes != GM_MYSQL_UNKNOWN_TYPE)
    {
        gtk_entry_set_text (GTK_ENTRY (attribute_entry), mysql_attributes[field_info->attributes]);
    }


    null_combo = gtk_combo_new ();
    gtk_widget_ref (null_combo);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "null_combo", null_combo,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (null_combo);
    gtk_box_pack_start (GTK_BOX (notebook_box), null_combo, FALSE, FALSE, 0);
    gtk_widget_set_usize (null_combo, -2, 24);
    null_combo_items = g_list_append (null_combo_items, "No");
    null_combo_items = g_list_append (null_combo_items, "Yes");
    gtk_combo_set_popdown_strings (GTK_COMBO (null_combo), null_combo_items);
    g_list_free (null_combo_items);

    null_entry = GTK_COMBO (null_combo)->entry;
    gtk_widget_ref (null_entry);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "null_entry", null_entry,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (null_entry);
    gtk_entry_set_editable (GTK_ENTRY (null_entry), FALSE);
    if (field_info && !field_info->not_null)
        gtk_entry_set_text (GTK_ENTRY(null_entry), "Yes");

    default_entry = gtk_entry_new ();
    gtk_widget_ref (default_entry);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "default_entry", default_entry,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (default_entry);
    if (field_info && field_info->default_value)
    {
        gtk_entry_set_text (GTK_ENTRY(default_entry), field_info->default_value);
    }
    gtk_box_pack_start (GTK_BOX (notebook_box), default_entry, FALSE, FALSE, 0);
    gtk_widget_set_usize (default_entry, -2, 24);

    extra_combo = gtk_combo_new ();
    gtk_widget_ref (extra_combo);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "extra_combo", extra_combo,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (extra_combo);
    gtk_box_pack_start (GTK_BOX (notebook_box), extra_combo, FALSE, FALSE, 0);
    gtk_widget_set_usize (extra_combo, -2, 24);
    extra_combo_items = g_list_append (extra_combo_items, "");
    extra_combo_items = g_list_append (extra_combo_items, "Auto_increment");
    gtk_combo_set_popdown_strings (GTK_COMBO (extra_combo), extra_combo_items);
    g_list_free (extra_combo_items);

    extra_entry = GTK_COMBO (extra_combo)->entry;
    gtk_widget_ref (extra_entry);
    gtk_object_set_data_full (GTK_OBJECT (notebook_box), "extra_entry", extra_entry,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (extra_entry);
    gtk_entry_set_editable (GTK_ENTRY (extra_entry), FALSE);
    if (field_info && field_info->extra)
        gtk_entry_set_text (GTK_ENTRY(extra_entry), "Auto_increment");
    else
        gtk_entry_set_text (GTK_ENTRY (extra_entry), "");







    return notebook_box;
}


static void table_properties_new_apply_changes_clicked (GtkButton * button ,
        GtkWidget * fp_win)
{
    GtkWidget * fields_notebook;
    GtkWidget * notebook_page;
    GtkWidget * entry_widget;
    GtkEntry  * entry;
    GtkWidget * fields_spin;
    GtkCList * keys_clist;
    gint fields_count, i=0;
    gchar * comment = NULL;
    DbQuery * q;
    GmConnection * connection;
    DbTableInfo * table_info;
    gchar *tmp_text;
    GString * create_table_query;






    g_return_if_fail (fp_win != NULL);

    if (!check_table_properties_entries (fp_win))
        return;


    if (!check_table_properties_entries (fp_win))
        return;

    connection = gm_utils_get_data  (GTK_OBJECT (fp_win), "connection");
    fields_notebook = gm_utils_get_data  (GTK_OBJECT (fp_win), "fields_notebook");
    table_info = gm_utils_get_data  (GTK_OBJECT (fp_win), "table_info");

    entry_widget = gm_utils_get_data  (GTK_OBJECT (fp_win), "comment_entry");
    entry = GTK_ENTRY (entry_widget);
    comment = gtk_entry_get_text (entry);

    entry_widget = gm_utils_get_data  (GTK_OBJECT (fp_win), "table_name_entry");
    entry = GTK_ENTRY (entry_widget);
    tmp_text = gtk_entry_get_text (entry);

    create_table_query = g_string_new ("CREATE TABLE ");
    g_string_sprintfa(create_table_query, "%s ",tmp_text);

    fields_spin =  gm_utils_get_data  (GTK_OBJECT (fp_win),"fields_spin");
    fields_count = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fields_spin));

    keys_clist    = GTK_CLIST ($pointer (fp_win, "keys_clist"));

    for (i=0; i<fields_count; i++)
    {
        gchar *field_query;
        gchar *field_name;
        gchar *field_type;
        gchar *field_length;
        gchar *field_attribute;
        gchar *field_default;
        gchar *field_type_length;
        gchar *field_null;
        gchar *field_extra;


        notebook_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), i);
        entry_widget = gm_utils_get_data  (GTK_OBJECT (notebook_page), "field_name_entry");
        entry = GTK_ENTRY (entry_widget);
        field_name = gtk_entry_get_text (entry);

        entry_widget = gm_utils_get_data  (GTK_OBJECT (notebook_page), "type_entry");
        entry = GTK_ENTRY (entry_widget);
        field_type = gtk_entry_get_text (entry);

        entry_widget = gm_utils_get_data  (GTK_OBJECT (notebook_page), "length_entry");
        entry = GTK_ENTRY (entry_widget);
        field_length = gtk_entry_get_text (entry);

        entry_widget = gm_utils_get_data  (GTK_OBJECT (notebook_page), "attribute_entry");
        entry = GTK_ENTRY (entry_widget);
        field_attribute = gtk_entry_get_text (entry);

        entry_widget = gm_utils_get_data  (GTK_OBJECT (notebook_page), "default_entry");
        entry = GTK_ENTRY (entry_widget);
        tmp_text = gtk_entry_get_text (entry);

        if (tmp_text && strlen(tmp_text)>=1)
            field_default = g_strdup_printf ("default '%s' ",tmp_text);
        else
            field_default = g_strdup ("");

        entry_widget = gm_utils_get_data  (GTK_OBJECT (notebook_page), "extra_entry");
        entry = GTK_ENTRY (entry_widget);
        field_extra = gtk_entry_get_text (entry);

        entry_widget = gm_utils_get_data  (GTK_OBJECT (notebook_page), "null_entry");
        entry = GTK_ENTRY (entry_widget);
        field_null = (!strcmp(gtk_entry_get_text (entry),"Yes"))?"NULL":"NOT NULL";

        if (field_length && strlen(field_length)>=1)
            field_type_length = g_strdup_printf("%s(%s)", field_type, field_length);
        else
            field_type_length = g_strdup_printf("%s", field_type);




        field_query = g_strdup_printf("%s %s %s %s %s %s", field_name, field_type_length, field_attribute, field_null, field_default, field_extra);
        g_free (field_type_length);


        if (i)
            g_string_sprintfa (create_table_query, ", %s", field_query);
        else
            g_string_sprintfa (create_table_query, " ( %s", field_query);
        g_free (field_query);
        g_free (field_default);
    }


    {
        gchar * create_keys_definition = NULL;
        create_keys_definition  = get_create_keys_definition (keys_clist, FALSE);
        if (create_keys_definition){
            g_string_sprintfa (create_table_query, ", %s", create_keys_definition);
            g_free (create_keys_definition);
        }
    }

    g_string_append (create_table_query, " )");
    if (comment && strlen (comment)>1)
        g_string_sprintfa (create_table_query, " comment='%s' ", comment);

    q = gm_db_new_query (create_table_query->str);
    gm_debug_message (GM_TABLE_PROPERTIES_CHANEL,"Table Definition : %s\n",create_table_query->str);
    q->database = table_info->database;

    if (!gm_db_request(connection, q) )
    {
        gchar * error_message;
        error_message = g_strdup_printf ("An error occur while executing this query %s \n\n. Mysql said :%sa",
                                         create_table_query->str, q->error);
        gm_utils_write_log_message (connection, error_message,  ERROR_LOGMODE);
    }else
    {
        gm_bpanel_reload (NULL, connection);
        gm_dbtree_rebuild_tree (NULL, connection);
        gm_bpanel_select_db_browser_page (connection);
        gtk_widget_destroy (fp_win);
    }
    gm_db_free_query (q);
    g_string_free (create_table_query, TRUE);

}



static void table_properties_old_apply_changes_clicked (GtkButton * button ,
        GtkWidget * fp_win)
{


    GtkWidget * fields_spin;
    GtkWidget * fields_notebook;
    GtkEntry  * entry;
    gchar * entry_text; /* never free it */
    GString * alter_query_string = NULL;
    gchar * drop_columns_query = NULL;
    gchar * columns_alter_change_query = NULL;
    gchar * drop_key_query = NULL;
    gchar * add_keys_query = NULL;
    gchar * free_it;
    gint   fields_count;
    DbTableInfo * table_info;
    GmConnection *connection;

    DbQuery * q;



    if (!check_table_properties_entries (fp_win))
        return;

    table_info = gm_utils_get_data  (GTK_OBJECT (fp_win), "table_info");

    alter_query_string = g_string_new("");
    g_string_sprintfa (alter_query_string,"ALTER TABLE %s", table_info->name);

    connection = gm_utils_get_data  (GTK_OBJECT (fp_win), "connection");
    fields_notebook = gm_utils_get_data  (GTK_OBJECT (fp_win), "fields_notebook");

    /* table name */
    entry           =  GTK_ENTRY ($pointer(fp_win, "table_name_entry"));
    entry_text      =  gtk_entry_get_text (entry);


    if (strcmp(entry_text, table_info->name))
        g_string_sprintfa (alter_query_string,", rename %s",entry_text);

    /* table comment */

    entry           =  GTK_ENTRY ($pointer(fp_win, "comment_entry"));
    entry_text      =  gtk_entry_get_text (entry);

    if (strcmp(entry_text, table_info->comment))
        g_string_sprintfa (alter_query_string, ", comment='%s' ",entry_text);


    fields_spin      = $pointer (fp_win,"fields_spin");
    fields_count     = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fields_spin));

    {
        DbFieldInfo * field;
        GtkWidget * field_page ;
        gint i =0;


        for (i=0; i < fields_count; i++)
        {
            gchar * field_create_definition = NULL;

            field_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (fields_notebook), i);
            field = (DbFieldInfo * ) $pointer (field_page,"field_info");
            if (field){
                if (columns_alter_change_query){
                    free_it = columns_alter_change_query;
                    columns_alter_change_query = g_strdup_printf ("%s, change %s ",columns_alter_change_query, field->name);
                    g_free (free_it);
                }
                else {
                    columns_alter_change_query = g_strdup_printf ("change %s ",field->name);
                }
                field->drop_it = FALSE;
            }else {
                if (columns_alter_change_query){
                    free_it = columns_alter_change_query;
                    columns_alter_change_query = g_strdup_printf ("%s, add ",columns_alter_change_query);
                    g_free (free_it);
                }
                else {
                    columns_alter_change_query = g_strdup ("add ");
                }
            }
            field_create_definition = table_properties_get_field_create_definition (field_page);
            free_it = columns_alter_change_query;
            columns_alter_change_query = g_strdup_printf ("%s %s", columns_alter_change_query, field_create_definition);
            g_free (free_it);
            g_free (field_create_definition);
        }
    }

    if (columns_alter_change_query){
        g_string_sprintfa (alter_query_string, ", %s", columns_alter_change_query);
        g_free (columns_alter_change_query);
    }


    {
        GList * l;
        l = table_info->fields_info_list;
        while (l){
            DbFieldInfo  * field = l->data;
            if (field->drop_it) {
                if (drop_columns_query){
                    free_it = drop_columns_query;
                    drop_columns_query = g_strdup_printf ("%s, drop %s ", drop_columns_query, field->name);
                    g_free (free_it);
                }
                else {
                    drop_columns_query = g_strdup_printf ("drop %s ",field->name);
                }
            }
            l = l->next;
        }
    }
    {
        /* reset the list */
        GList * l;
        l = table_info->fields_info_list;
        while (l){
            DbFieldInfo  * field = l->data;
            field->drop_it = TRUE;
            l = l->next;
        }
    }

    if (drop_columns_query)
    {
        g_string_sprintfa (alter_query_string, ", %s", drop_columns_query);
        g_free (drop_columns_query);
    }
    {
        /* drop keys query */
        GtkWidget * keys_clist;

        keys_clist     = $pointer (fp_win, "keys_clist");
        drop_key_query = $pointer (keys_clist, "drop_key_query");

        if (drop_key_query)
        {
            g_string_sprintfa (alter_query_string, ", %s", drop_key_query);
        }

        /*
          add keys query
        */
        add_keys_query = get_create_keys_definition (GTK_CLIST(keys_clist), TRUE);
        if (add_keys_query){
            g_string_sprintfa (alter_query_string, ", %s", add_keys_query);
            g_free (add_keys_query);
        }
    }
    q = gm_db_new_query (alter_query_string->str);
    gm_debug_message (GM_TABLE_PROPERTIES_CHANEL,"Alter Table Definition : %s\n",alter_query_string->str);
    q->database = table_info->database;
    if (!gm_db_request(connection, q) )
    {
        gchar * error_message;
        error_message = g_strdup_printf ("An error occur while executing this query %s \n\n.Mysql said :%s\n",
                                         alter_query_string->str, q->error);
        gm_utils_write_log_message (connection, error_message,  ERROR_LOGMODE);
    }else
    {
        gm_bpanel_reload (NULL, connection);
        gm_bpanel_select_db_browser_page(connection);
        g_free (drop_key_query);
        gtk_widget_destroy (fp_win);
    }
    gm_db_free_query (q);
    g_string_free (alter_query_string, TRUE);

}




static gchar * table_properties_get_field_create_definition (GtkWidget * field_page)
{

    GtkWidget * entry_widget;
    GtkEntry  * entry;

    gchar *field_query;
    gchar *field_name;
    gchar *field_type;
    gchar *field_length;
    gchar *field_attribute;
    gchar *field_default;
    gchar *field_type_length;
    gchar *field_null;
    gchar *field_extra;
    gchar * tmp_text;



    entry_widget = gm_utils_get_data  (GTK_OBJECT (field_page), "field_name_entry");
    entry = GTK_ENTRY (entry_widget);
    field_name = gtk_entry_get_text (entry);

    entry_widget = gm_utils_get_data  (GTK_OBJECT (field_page), "type_entry");
    entry = GTK_ENTRY (entry_widget);
    field_type = gtk_entry_get_text (entry);

    entry_widget = gm_utils_get_data  (GTK_OBJECT (field_page), "length_entry");
    entry = GTK_ENTRY (entry_widget);
    field_length = gtk_entry_get_text (entry);

    entry_widget = gm_utils_get_data  (GTK_OBJECT (field_page), "attribute_entry");
    entry = GTK_ENTRY (entry_widget);
    field_attribute = gtk_entry_get_text (entry);

    entry_widget = gm_utils_get_data  (GTK_OBJECT (field_page), "default_entry");
    entry = GTK_ENTRY (entry_widget);
    tmp_text = gtk_entry_get_text (entry);
    if (tmp_text && strlen(tmp_text)>=1)
        field_default = g_strdup_printf ("default '%s' ",tmp_text);
    else
        field_default = g_strdup ("");

    entry_widget = gm_utils_get_data  (GTK_OBJECT (field_page), "extra_entry");
    entry = GTK_ENTRY (entry_widget);
    field_extra = gtk_entry_get_text (entry);

    entry_widget = gm_utils_get_data  (GTK_OBJECT (field_page), "null_entry");
    entry = GTK_ENTRY (entry_widget);
    field_null = (!strcmp(gtk_entry_get_text (entry),"Yes"))?"NULL":"NOT NULL";

    if (field_length && strlen(field_length)>=1)
        field_type_length = g_strdup_printf("%s(%s)", field_type, field_length);
    else
        field_type_length = g_strdup_printf("%s", field_type);




    field_query = g_strdup_printf("%s %s %s %s %s %s", field_name, field_type_length, field_attribute, field_null, field_default, field_extra);
    g_free (field_type_length);
    g_free (field_default);

    return field_query;
}


static GtkWidget * table_properties_fetch_describe_result (GmConnection *connection, gchar * database, gchar * table){

    DbQuery * q;
    gchar * query ;
    GtkWidget * data_list = NULL;

    g_return_val_if_fail (connection != NULL, NULL);


    query = g_strdup_printf ("desc %s ", table);
    q = gm_db_new_query (query);
    q->database = database;

    if (gm_db_request(connection, q))
    {
        data_list = gm_clist_new (q->fields_count);
        gtk_clist_set_selection_mode (GTK_CLIST (data_list), GTK_SELECTION_EXTENDED);
        gtk_widget_show (data_list);
        table_properties_fill_general_data_list (q, data_list);
        gm_clist_sort_by_column (GTK_CLIST(data_list), 0, 0);
    }else  {
        gm_utils_write_log_message (connection, q->error, ERROR_LOGMODE);
    }

    gm_db_free_query (q);
    g_free (query);
    return data_list;

}





static void table_properties_fill_general_data_list (DbQuery * q, GtkWidget *data_list){


    MYSQL_RES *result;
    MYSQL_ROW row;
    GSList *f = NULL;
    GtkWidget * label;
    GtkCList * clist;
    gint i =0;
    gboolean alternate;
    GdkColor color;


    g_return_if_fail (q != NULL);

    result = q->mysql_result;
    clist = GTK_CLIST (data_list);
    gtk_clist_freeze (clist);
    gtk_clist_column_titles_show (clist);
    f = g_slist_nth(q->fields_list, 0);
    while (f){
        MYSQL_FIELD * field;
        gchar * fname;
        field = (MYSQL_FIELD *) f->data;
        gtk_clist_set_column_auto_resize (clist, i, TRUE);
        fname = g_strdup_printf (" %s ", field->name);
        label = gtk_label_new (fname);
        g_free (fname);
        gtk_widget_show (label);
        gm_clist_set_column_widget (GM_CLIST (data_list), i, label);
        gm_clist_set_column_sortdata_type (GM_CLIST (data_list), i, field->type);
        i++;
        f = f->next;
    }
    gdk_color_parse                 ("#ededed", &color);
    mysql_data_seek(result,0);
    alternate = FALSE;
    i=0;
    while ((row = mysql_fetch_row(result))) {
        gtk_clist_append (clist, row);
        if (alternate ){
            gtk_clist_set_background (clist, i, &color);
        }
        alternate = !alternate;
        i++;
    }
    gtk_clist_thaw (clist);
}
