/*
 *   Copyright (C) 2001  mohamed Elassadi <mohamed@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include "gm.h"
#include "gm-control-center.h"
#include "glade_interface.h"
#include "gm-debug.h"
#include "gm-interface.h"


#define BUFFER_SIZE 8192
#define REMOVE_MESSAGE_TIMEOUT 8000
#define GMYCLIENT_UPDATE_URL "http://gmyclient.sourceforge.net/update.php"

enum {
    REGEX_KEYWORDS =0,
    REGEX_FUNCTIONS
};




static void utils_real_open_file (GtkEditable * editable, gchar * filename);
static GtkWidget * utils_create_text_entry_popup_menu (GtkEditable * editable) ;
static gint utils_text_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer data);
static gboolean utils_text_key_press  (GtkWidget *widget, GdkEventKey *event,gpointer user_data);
static void utils_text_cut_activated  (GtkMenuItem     *menuitem, GtkEditable * editable);
static void utils_text_configure_highlight_colors (GmSyntaxHighlighting *sh);




static void utils_message (gchar * message, gchar *type)
{
    GtkWidget *messagebox;
    GtkWidget *dialog_vbox;
    GtkWidget *button;
    GtkWidget *dialog_action_area;

    g_return_if_fail(message != NULL);


    /* We create it with an OK button, and then remove the button, to work
       around a bug in gnome-libs. */

    messagebox = gnome_message_box_new (message,
                                        type,
                                        GNOME_STOCK_BUTTON_OK, NULL);
    gtk_container_remove (GTK_CONTAINER (GNOME_DIALOG (messagebox)->action_area), GNOME_DIALOG (messagebox)->buttons->data);
    GNOME_DIALOG (messagebox)->buttons = NULL;
    gtk_object_set_data (GTK_OBJECT (messagebox), "messagebox", messagebox);
    gtk_container_set_border_width (GTK_CONTAINER (messagebox), 2);
    gtk_window_set_title (GTK_WINDOW (messagebox), _("Warning"));
    gtk_window_set_position (GTK_WINDOW (messagebox), GTK_WIN_POS_CENTER);
    gtk_window_set_modal (GTK_WINDOW (messagebox), TRUE);
    gtk_window_set_policy (GTK_WINDOW (messagebox), FALSE, FALSE, FALSE);

    dialog_vbox = GNOME_DIALOG (messagebox)->vbox;
    gtk_object_set_data (GTK_OBJECT (messagebox), "dialog_vbox", dialog_vbox);
    gtk_widget_show (dialog_vbox);

    gnome_dialog_append_button (GNOME_DIALOG (messagebox), GNOME_STOCK_BUTTON_OK);
    button = GTK_WIDGET (g_list_last (GNOME_DIALOG (messagebox)->buttons)->data);
    gtk_widget_ref (button);
    gtk_object_set_data_full (GTK_OBJECT (messagebox), "button", button,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (button);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

    dialog_action_area = GNOME_DIALOG (messagebox)->action_area;
    gtk_widget_ref (dialog_action_area);
    gtk_object_set_data_full (GTK_OBJECT (messagebox), "dialog_action_area", dialog_action_area,
                              (GtkDestroyNotify) gtk_widget_unref);

    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);
    gtk_signal_connect (GTK_OBJECT (messagebox), "destroy",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);
    gtk_widget_show(messagebox);
    gtk_main();
}




void gm_utils_message (gchar * message){
    utils_message (message, GNOME_MESSAGE_BOX_INFO);
}

void gm_utils_warning (gchar * message){
    utils_message (message, GNOME_MESSAGE_BOX_WARNING);
}

void gm_utils_error (gchar * message){
    utils_message (message, GNOME_MESSAGE_BOX_ERROR);
}



GSList *g_slist_first(GSList * list){

    if (list != NULL)
        return g_slist_nth(list, 0);
    return NULL;
}

void g_slist_g_free(GSList * ml){

    GSList * l=NULL;
    l = g_slist_nth(ml, 0);
    while (l){
        g_free(l->data);
        l = l->next;
    }
    g_slist_free(ml);
}


void g_list_g_free(GList * list){

    GList * l =NULL;

    if (!list);
    return;
    l = g_list_first(list);
    while (l){
        g_free(l->data);
        l = l->next;
    }
    g_list_free(list);
}



void gm_utils_write_log_message (GmConnection * connection, gchar *msg, gint logmode){

    //    static GdkFont  * myFont = NULL;
    GdkColor color;
    GtkWidget * output_window= NULL;
    GtkWidget * messages_window;
    GtkWidget * errors_window;
    GtkWidget * log_book;

    gchar *message =NULL, *pmessage = NULL;
    time_t now ;
    struct tm *ptr;
    gchar datum[80];
    GmCcenter * cc;
    GmCcenterGeneral * ge_config;
    FILE * logfilehandler;


    g_return_if_fail (connection != NULL || global_app->cc != NULL);

    if (!msg)
        return;


    cc = GM_CCENTER (global_app->cc);
    ge_config = cc->ge_config;

    logfilehandler = ge_config->logfilehandler;

    time(&now);
    ptr = localtime(&now);
    strftime(datum, 80, "[ %a %d-%b-%Y %X ] ", ptr);
    messages_window = gm_utils_get_data  (connection->connection_window, "messages_window");
    errors_window = gm_utils_get_data  (connection->connection_window,"errors_window");

    pmessage = msg;
    pmessage += strlen(msg)-1;
    if (*pmessage != '\n'){
        pmessage = g_strdup_printf("%s\n",msg);
    }else {
        pmessage = g_strdup (msg);
    }

    if (ge_config->date_time_on && (logmode != DATA_LOGMODE))
        message = g_strdup_printf("%s%s",datum,pmessage);
    else
        message = g_strdup(pmessage);

    switch (logmode) {
    case ERROR_LOGMODE :
        gdk_color_parse("red",&color);
        output_window = errors_window;
        break;
    case DATA_LOGMODE :
    case MESSAGE_LOGMODE :
        gdk_color_parse("black",&color);
        output_window = messages_window;
        break;
    case DATETIME_LOGMODE:
    case SIMPLE_LOGMODE :
    default:
        gdk_color_parse("blue",&color);
        output_window = messages_window;
        break;
    }
    gtk_text_insert( GTK_TEXT(output_window), NULL,
                     &color,
                     NULL,
                     message,
                     strlen(message) );

    g_free(message);

    if (global_app->logfilehandler){
        message = g_strdup_printf("%s%s",datum,pmessage);
        fprintf(logfilehandler, "%s\n",(gchar *) message);
    }
    g_free(pmessage);
    log_book = gm_utils_get_data (connection->connection_window, "sql_log_book");
    if (logmode == ERROR_LOGMODE) {
        gboolean active;
        GtkWidget * log_panel_on_off;

        log_panel_on_off = gm_utils_get_data  (GTK_OBJECT (connection->connection_window), "sql_log_panel_on_off");
        active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(log_panel_on_off));
        gtk_notebook_set_page (GTK_NOTEBOOK (log_book), ERRORS_NOTEBOOK_PAGE);
        if (!active) {
            gm_interface_show_hide_sql_log_panel (connection);
            gm_utils_warning (_("Warning an error occur see Log-window fore more details !!"));
        }
    }else if (logmode == MESSAGE_LOGMODE)
        gtk_notebook_set_page (GTK_NOTEBOOK (log_book), MESSAGES_NOTEBOOK_PAGE);


}

static gint utils_yes_no_dialog_key_pressed     (GtkWidget       *widget,
        GdkEventKey     *event,
        gint *answer){

    if  (event->keyval == GDK_Escape) {
        *answer = 0;
        gtk_main_quit ();
        return TRUE;
    }
    return FALSE;

}


static void utils_yes_button_clicked               (GtkButton  *button,
        gint *answer){

    *answer = 1;
    gtk_main_quit ();
}


static void utils_no_button_clicked               (GtkButton  *button,
        gint *answer){
    *answer = 0;
    gtk_main_quit ();
}


gboolean gm_utils_yes_no_dialog(gchar * message){
    GtkWidget * yes_no_dialog;
    GtkWidget * button;
    GtkWidget * questions_label;
    gint answer = 0;
    yes_no_dialog = create_yes_no_dialog ();
    gtk_widget_show (yes_no_dialog);

    gtk_signal_connect (GTK_OBJECT (yes_no_dialog), "key_press_event",
                        GTK_SIGNAL_FUNC (utils_yes_no_dialog_key_pressed),
                        &answer);


    button = gm_utils_get_data  (GTK_OBJECT(yes_no_dialog),"yes_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (utils_yes_button_clicked),
                        &answer);

    button = gm_utils_get_data(GTK_OBJECT(yes_no_dialog),"no_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (utils_no_button_clicked),
                        &answer);

    gtk_signal_connect (GTK_OBJECT (yes_no_dialog), "destroy",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);


    questions_label = gm_utils_get_data  (GTK_OBJECT(yes_no_dialog),"questions_label");

    gtk_label_set_text ( GTK_LABEL(questions_label), message);
    gtk_main();
    if (GTK_IS_WIDGET(yes_no_dialog)){
        gtk_signal_disconnect_by_func(GTK_OBJECT (yes_no_dialog), gtk_main_quit, NULL);
        gtk_widget_destroy (yes_no_dialog);
    }

    return answer;
}





gchar *
gm_utils_strescape (const gchar *source,
                    const gchar *exceptions)
{
    const guchar *p;
    gchar *dest;
    gchar *q;
    guchar excmap[256];

    g_return_val_if_fail (source != NULL, NULL);

    p = (guchar *) source;
    /* Each source byte needs maximally four destination chars (\777) */
    q = dest = g_malloc (strlen (source) * 4 + 1);

    memset (excmap, 0, 256);
    if (exceptions)
    {
        guchar *e = (guchar *) exceptions;

        while (*e)
        {
            excmap[*e] = 1;
            e++;
        }
    }

    while (*p)
    {
        if (excmap[*p])
            *q++ = *p;
        else
        {
            switch (*p)
            {
            case '\b':
                *q++ = '\\';
                *q++ = 'b';
                break;
            case '\f':
                *q++ = '\\';
                *q++ = 'f';
                break;
            case '\n':
                *q++ = '\\';
                *q++ = 'n';
                break;
            case '\r':
                *q++ = '\\';
                *q++ = 'r';
                break;
            case '\t':
                *q++ = '\\';
                *q++ = 't';
                break;
            case '\\':
                *q++ = '\\';
                *q++ = '\\';
                break;
            case '"':
                *q++ = '\\';
                *q++ = '"';
                break;
            case '\'':
                *q++ = '\\';
                *q++ = '\'';
                break;
            default:
                *q++ = *p;
                break;
            }
        }
        p++;
    }
    *q = 0;
    return dest;
}

/* return a new allocated char pointer should be freed by the caller */




gchar * g_str_replace (gchar * src, gchar *sub_string, gchar *with_string)
{

    gchar * p = NULL;
    gchar * left_string;
    gchar * right_string;
    gchar * result_string = NULL;
    gint sub_string_len;
    gint new_size;


    g_return_val_if_fail (src != NULL, NULL);
    g_return_val_if_fail (sub_string != NULL, NULL);
    g_return_val_if_fail (with_string != NULL, NULL);



    p = (gchar *) strstr (src, sub_string);

    if (!p)
        return src;


    new_size = (p - src) +1;

    left_string = (gchar *) g_malloc (new_size);
    strncpy (left_string, src, new_size -1);
    left_string [new_size-1] = '\0';

    sub_string_len = strlen (sub_string);
    p += sub_string_len;
    right_string = g_strdup (p);

    result_string = g_strdup_printf("%s%s%s",left_string, with_string, right_string);

    g_free (right_string);
    g_free (left_string);

    return result_string;
}




void gm_utils_set_widget_cursor (GtkWidget *widget,gint cursor){
    CursorStruct * c ;
    static GList *cursors_cache = NULL;
    GList *l;
    GdkCursor * gdcursor=NULL;


    g_return_if_fail (widget != NULL);

    if (!widget->window){
        gtk_widget_realize (widget);
        g_return_if_fail (widget->window != NULL);
    }



    l = g_list_first (cursors_cache);

    while (l){
        c = l->data;
        if (c->cursor_num == cursor){
            gdcursor = c->gdcursor;
            break;
        }
        l = l->next;
    }
    if (gdcursor) {
        gdk_window_set_cursor (widget->window, gdcursor);
    }else {
        gdcursor = gdk_cursor_new(cursor);
        gdk_window_set_cursor (widget->window, gdcursor);
        c = g_new0 (CursorStruct,1);
        c->cursor_num = cursor;
        c->gdcursor = gdcursor;
        cursors_cache = g_list_append (cursors_cache, c);
    }
}

GtkWidget * gm_stock_pixmap_widget (GtkWidget *widget, gchar *icon)
{
    GtkWidget * pixmap;
    GString * icon_file_name;

    icon_file_name = g_string_new (PACKAGE_DATA_DIR);




    g_string_append (icon_file_name, "/pixmaps/");
    g_string_append (icon_file_name, icon);
    g_string_append (icon_file_name, ".png");

    if (g_file_exists(icon_file_name->str))
        pixmap = gnome_stock_pixmap_widget (widget, icon_file_name->str);
    else
        pixmap = gnome_stock_pixmap_widget (widget, icon);



    if (!pixmap)
        gm_debug_warning (GM_UTILS_CHANEL, _("Could not find Pixmap File :%s"), icon_file_name->str);

    g_string_free (icon_file_name, TRUE);
    return pixmap;
}



static void filebrowser_changed(GtkWidget * w, GtkFileSelection * filesel)
{

}



static gboolean utils_filebrowser_is_dir(GtkFileSelection * filesel)
{
    gchar *text;
    struct stat buf;

    text = g_strdup(gtk_file_selection_get_filename(filesel));
    g_strstrip(text);

    if (stat(text, &buf) == 0)
    {
        if (S_ISDIR(buf.st_mode))
        {		/* Selected directory */
            gtk_file_selection_set_filename(filesel, text);
            g_free(text);
            return TRUE;
        }
    }
    g_free(text);
    return FALSE;
}




static void filebrowser_add_files(GtkFileSelection * filesel)
{
    GList *sel_list = NULL, *node=NULL;
    gchar *text, *text2, *ptr;
    gchar * filesel_path;
    GList * files_list;

    /*
     * There got to be some clean way to do this too
     */

    gtk_label_get(GTK_LABEL(GTK_BIN(filesel->history_pulldown)->child), &ptr);

    /* This will give an extra slash if the current dir is the root. */

    filesel_path = g_strconcat(ptr, "/", NULL);
    files_list = gm_utils_get_data_ww  (GTK_OBJECT (filesel), "files_list");
    node = GTK_CLIST(filesel->file_list)->selection;

    while (node)
    {
        sel_list = g_list_prepend(sel_list, node->data);
        node = g_list_next(node);
    }
    node = sel_list;
    if (node)
    {
        while (node)
        {
            gtk_clist_get_text(GTK_CLIST(filesel->file_list), GPOINTER_TO_INT(node->data), 0, &text);
            text2 = g_strconcat(filesel_path, text, NULL);
            files_list = g_list_append (files_list, text2);
            node = g_list_next(node);
        }
    }
    else
    {
        /*
         * No files selected, but the user may have
         * typed a filename.
         */

        text = gtk_file_selection_get_filename(filesel);
        if (text[strlen(text) - 1] != '/' ){
            if (g_file_exists (text))
                files_list = g_list_append (files_list, g_strdup (text));
        }
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), "");
    }
    g_list_free(sel_list);
    gtk_object_set_data (GTK_OBJECT (filesel), "files_list", files_list);
}

static void filebrowser_add_all_files(GtkWidget * w, gpointer data)
{
    GtkFileSelection *filesel = GTK_FILE_SELECTION(data);

    gtk_clist_freeze(GTK_CLIST(filesel->file_list));
    gtk_clist_select_all(GTK_CLIST(filesel->file_list));
    filebrowser_add_files(filesel);
    /*
     * We want gtk_clist_undo_selection() but it seems to be buggy
     * in GTK+ 1.2.6
     */
    /*  gtk_clist_undo_selection(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list)); */
    gtk_clist_unselect_all(GTK_CLIST(filesel->file_list));
    gtk_clist_thaw(GTK_CLIST(filesel->file_list));
    gtk_entry_set_text(GTK_ENTRY(filesel->selection_entry), "");
    gtk_main_quit();
}


static void filebrowser_add_selected_files(GtkWidget * w, gpointer data)
{
    GtkFileSelection *filesel = GTK_FILE_SELECTION(data);

    if (GTK_CLIST(filesel->file_list)->selection){
        filebrowser_add_files(filesel);
        gtk_clist_unselect_all(GTK_CLIST(filesel->file_list));

        /*HACK*/
        gtk_entry_set_text(GTK_ENTRY(filesel->selection_entry), "");
        gtk_main_quit();
    }
}



static void filebrowser_ok(GtkWidget * w, GtkWidget * filesel)
{



    if (utils_filebrowser_is_dir(GTK_FILE_SELECTION(filesel)))
    {
        return;

    }

    gtk_widget_hide(filesel);
    filebrowser_add_files(GTK_FILE_SELECTION(filesel));
    gtk_main_quit();
}




GList * gm_utils_create_filebrowser (gboolean list_mode)
{
    GtkWidget *filebrowser, *bbox, *add_selected, *add_all, *label;
    GList * files_list=NULL;
    gchar * ptr;
    gchar * filesel_path;
    GmCcenterGeneral * ge_config;

    ge_config = GM_CCENTER(global_app->cc)->ge_config;

    filebrowser = gtk_file_selection_new(_("Load file(s)"));

    gtk_window_set_modal (GTK_WINDOW (filebrowser), TRUE);
    if (list_mode)
        gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(filebrowser)->file_list),
                                     GTK_SELECTION_EXTENDED);
    else
        gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(filebrowser)->file_list),
                                     GTK_SELECTION_SINGLE);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->selection_entry),
                       "changed", GTK_SIGNAL_FUNC(filebrowser_changed), filebrowser);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->ok_button),
                       "clicked", GTK_SIGNAL_FUNC(filebrowser_ok), filebrowser);
    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->cancel_button),
                              "clicked", gtk_main_quit,
                              GTK_OBJECT(filebrowser));

    if (ge_config && ge_config->dump_file_browser_path)
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(filebrowser),
                                        ge_config->dump_file_browser_path);
    bbox = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 0);
    gtk_box_pack_end(GTK_BOX(GTK_FILE_SELECTION(filebrowser)->action_area),
                     bbox, TRUE, TRUE, 0);
    add_selected  = gtk_button_new_with_label(_("Add selected files"));
    gtk_box_pack_start(GTK_BOX(bbox), add_selected, FALSE, FALSE, 0);

    gtk_signal_connect(GTK_OBJECT(add_selected), "clicked",
                       GTK_SIGNAL_FUNC(filebrowser_add_selected_files), filebrowser);
    if (list_mode) {
        add_all = gtk_button_new_with_label(_("Add all files in the file list"));
        gtk_box_pack_start(GTK_BOX(bbox), add_all, FALSE, FALSE, 0);
        gtk_signal_connect(GTK_OBJECT(add_all), "clicked",
                           GTK_SIGNAL_FUNC(filebrowser_add_all_files), filebrowser);
    }

    gtk_widget_show_all(bbox);

    /*
     * This is ugly.
     * Change the Cancel buttons caption to Close.
     */

    label = gtk_label_new (_("Close"));
    gtk_misc_set_alignment (GTK_MISC(label), 0.5, 0.5);
    gtk_container_remove (GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button),
                          gtk_container_children(GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button))->data);
    gtk_container_add (GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button), label);
    gtk_widget_show(label);
    gtk_widget_show(filebrowser);

    gtk_main ();
    gtk_label_get(GTK_LABEL(GTK_BIN(GTK_FILE_SELECTION(filebrowser)->history_pulldown)->child), &ptr);
    /* This will give an extra slash if the current dir is the root. */
    filesel_path = g_strconcat(ptr, "/", NULL);
    g_free (ge_config->dump_file_browser_path) ;
    ge_config->dump_file_browser_path = g_strdup (filesel_path);

    g_free (filesel_path);
    files_list = gm_utils_get_data_ww  (GTK_OBJECT (filebrowser), "files_list");

    gtk_widget_destroy (filebrowser);
    return files_list;

}






/* file utils functions this will be removed after moving to glib2.0 */

gint is_directory (gchar *path)
{
    struct stat st;
    if (stat(path, &st) == -1) return 0;
    return S_ISDIR(st.st_mode);
}


gchar * gm_utils_get_basename (gchar * path)
{
    gchar * base ;
    base =  g_basename (path);

    if (base)
        return g_strdup (base);

    return NULL;
}


void g_string_append_len (GString *str, gchar * src, gint start_pos, gint len )
{
    gchar * s;
    gint i = 0;

    g_return_if_fail (str != NULL);
    if (!src || !len )
        return ;

    s = src;
    if (start_pos) {
        for (i=0; i<start_pos; i++)
            s++;
    }
    i=0;
    while (*s){
        g_string_append_c (str, *s);
        i++;
        s++;
        if (i == len)
            break;
    }
}




void gm_utils_create_button_label (GtkWidget * button ,
                                   gchar * label_str,
                                   gchar *pixmap_filename ,
                                   gboolean show_label)

{
    GtkWidget * hbox;
    GtkWidget * label;
    GtkWidget * pixmap;


    g_return_if_fail (button != NULL);
    if (GTK_BIN (button)->child)
        gtk_widget_destroy (GTK_BIN (button)->child);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox);


    if (pixmap_filename) {
        pixmap = gm_stock_pixmap_widget (button, pixmap_filename);
        if (pixmap){
            gtk_box_pack_start (GTK_BOX (hbox), pixmap, FALSE, FALSE, 0);
            gtk_widget_show (pixmap);
        }else
            show_label = TRUE;
    }


    if (label_str && show_label ) {
        label = gtk_label_new (label_str);
        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
        gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
        gtk_misc_set_padding (GTK_MISC (label), 2, 0);
        gtk_widget_show (label);
    }

    gtk_container_add (GTK_CONTAINER (button), hbox);
}


void gm_utils_create_vbutton_label (GtkWidget * button ,
                                    gchar * label_str,
                                    gchar *pixmap_filename ,
                                    gboolean show_label)

{
    GtkWidget * vbox;
    GtkWidget * label;
    GtkWidget * pixmap;


    g_return_if_fail (button != NULL);
    if (GTK_BIN (button)->child)
        gtk_widget_destroy (GTK_BIN (button)->child);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox);


    if (pixmap_filename) {
        pixmap = gm_stock_pixmap_widget (button, pixmap_filename);
        if (pixmap){
            gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
            gtk_widget_show (pixmap);
        }else
            show_label = TRUE;
    }


    if (label_str && show_label ) {
        label = gtk_label_new (label_str);
        gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
        gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
        gtk_misc_set_padding (GTK_MISC (label), 2, 0);
        gtk_widget_show (label);
    }

    gtk_container_add (GTK_CONTAINER (button), vbox);
}


gboolean gm_utils_istrstr (gchar * src, gchar * str )
{
    gchar *src_cp;
    gboolean ret = FALSE;

    g_return_val_if_fail (src != NULL, FALSE);
    g_return_val_if_fail (str != NULL, FALSE);

    src_cp = g_strdup (src);
    g_strdown(src_cp);



    if (strstr (src_cp, str)){
        ret = TRUE;
    }


    g_free (src_cp);

    return ret;
}


GtkWidget*
create_about_window (void)
{


    gchar * logo;
    const gchar *authors[] = {
        "Elassadi Mohamed <mohamed@users.sourceforge.net>",
        NULL
    };
    GtkWidget *about_window;
    logo = g_strdup_printf ("%s/pixmaps/gmyclient.jpg",PACKAGE_DATA_DIR);

    about_window = gnome_about_new ("Gmyclient", VERSION,
                                    _("2001 Elassadi Mohamed"),
                                    authors,
                                    _("http://www.gmyclient.sourceforge.net\nPlease report any bugs or malfunctions\n**Happy Mysqling**\n"),
                                    logo);
    g_free (logo);
    gtk_object_set_data (GTK_OBJECT (about_window), "about_window", about_window);
    gtk_window_set_modal (GTK_WINDOW (about_window), TRUE);

    return about_window;
}

void gtk_refresh(void){
    while ( gtk_events_pending() ) {
        gtk_main_iteration();
    }
}

/* Status bar utils functions */







static gint utils_hide_message (gpointer data)
{

    GmConnection * connection;


    connection = (GmConnection * ) data;

    gtk_editable_delete_text (GTK_EDITABLE (connection->mini_buffer), 0, -1);
    gtk_timeout_remove (connection->remove_messages_timer_tag);
    connection->remove_messages_timer_tag   = 0;
    return FALSE;
}




void gm_utils_show_status_information (GmConnection * connection, gchar * message)

{
    gtk_entry_set_text(GTK_ENTRY (connection->mini_buffer), message);
    if (connection->remove_messages_timer_tag)
        gtk_timeout_remove (connection->remove_messages_timer_tag);
    connection->remove_messages_timer_tag = gtk_timeout_add (REMOVE_MESSAGE_TIMEOUT, utils_hide_message, connection);
}















static gint utils_ignore_dialog_key_pressed     (GtkWidget       *widget,
        GdkEventKey     *event,
        gint *answer){

    if  (event->keyval == GDK_Escape) {
        *answer = 0;
        gtk_main_quit ();
        return TRUE;
    }
    return FALSE;

}


static void utils_ignore_button_clicked               (GtkButton  *button,
        gint *answer){

    *answer = 1;
    gtk_main_quit ();
}

static void utils_ignore_all_button_clicked               (GtkButton  *button,
        gint *answer){

    *answer = 2;
    gtk_main_quit ();
}




gint gm_utils_ignore_dialog(gchar * message){
    GtkWidget * ignore_dialog;
    GtkWidget * button;
    GtkWidget * question_label;
    gint answer = 0;
    ignore_dialog = create_ignore_dialog ();
    gtk_widget_show (ignore_dialog);

    gtk_signal_connect (GTK_OBJECT (ignore_dialog), "key_press_event",
                        GTK_SIGNAL_FUNC (utils_ignore_dialog_key_pressed),
                        &answer);


    button = gm_utils_get_data  (GTK_OBJECT(ignore_dialog),"ignore_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (utils_ignore_button_clicked),
                        &answer);

    button = gm_utils_get_data  (GTK_OBJECT(ignore_dialog),"ignore_all_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (utils_ignore_all_button_clicked),
                        &answer);

    button = gm_utils_get_data  (GTK_OBJECT(ignore_dialog),"cancel_button");
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (utils_no_button_clicked),
                        &answer);

    gtk_signal_connect (GTK_OBJECT (ignore_dialog), "destroy",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);

    question_label = gm_utils_get_data  (GTK_OBJECT(ignore_dialog),"question_label");
    gtk_label_set_text ( GTK_LABEL(question_label), message);

    gtk_main();
    if (GTK_IS_WIDGET(ignore_dialog)){
        gtk_signal_disconnect_by_func(GTK_OBJECT (ignore_dialog), gtk_main_quit, NULL);
        gtk_widget_destroy (ignore_dialog);
    }

    return answer;
}




gpointer gm_utils_get_data (gpointer obj,gchar *key)
{
    gpointer result;
    if (!obj){
        gm_debug_warning (GM_UTILS_CHANEL, _("Object is a Null Pointer for the given key %s"), key);
        return NULL;
    }
    if (!key){
        gm_debug_warning (GM_UTILS_CHANEL, _("No key provided to utils_get_data"));
        return NULL;
    }



    result = gtk_object_get_data  (GTK_OBJECT(obj), key);
    if (!result) {
        gm_debug_warning (GM_UTILS_CHANEL, _("Could not find Pointer for the given key >%s<"), key);
    }

    return result     ;
}

gpointer gm_utils_get_data_ww (gpointer obj,gchar *key)
{

    if (!obj){
        gm_debug_warning (GM_UTILS_CHANEL, _("Object is a Null Pointer for the given key %s"), key);
        return NULL;
    }
    if (!key){
        gm_debug_warning (GM_UTILS_CHANEL, _("No key provided to utils_get_data"));
        return NULL;
    }

    return gtk_object_get_data  (GTK_OBJECT(obj), key);
}


GtkWidget*
gm_utils_lookup_widget                          (GtkWidget       *widget,
                                                 const gchar     *widget_name)
{
    GtkWidget *parent, *found_widget;

    for (;;)
    {
        if (GTK_IS_MENU (widget))
            parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
        else
            parent = widget->parent;
        if (parent == NULL)
            break;
        widget = parent;
    }

    found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                     widget_name);
    if (!found_widget)
        gdm ("Widget not found: %s", widget_name);

    return found_widget;
}


/* gm-text-widget-code */











static void utils_real_open_file (GtkEditable * editable, gchar * filename)
{

    GtkWidget *text;
    FILE *fp;
    gchar buffer[BUFFER_SIZE];
    gint bytes_read;

    text = GTK_WIDGET(editable);
    gtk_text_freeze (GTK_TEXT (text));
    gtk_editable_delete_text (editable, 0, -1);

    fp = fopen (filename, "r");


    if (fp == NULL)
    {
        gtk_text_thaw (GTK_TEXT (text));
        gm_utils_warning (_("Couldn't open file."));
        gdm ("Couldn't open file. %s ", filename);
        return;
    }

    for (;;)
    {
        bytes_read = fread (buffer, sizeof (gchar), BUFFER_SIZE, fp);

        if (bytes_read > 0)
            gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, buffer,
                             bytes_read);

        if (bytes_read != BUFFER_SIZE && (feof (fp) || ferror (fp)))
            break;
    }

        /* If there is an error while loading, we reset everything to a good state.
         */
    if (ferror (fp))
    {
        fclose (fp);
        gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
        gtk_text_thaw (GTK_TEXT (text));
        gm_utils_warning (_("Error while loading File !!"));
        return;
    }

    fclose (fp);

    gtk_text_thaw (GTK_TEXT (text));

}



GList *
gm_utils_load_file_as_list (gchar *filename)
{

    gchar *line   = NULL;
    gchar *buffer = NULL;
    FILE  *file   = NULL;
    GList * list = NULL;
    gint   i =0;
    if ((file = fopen(filename,"r"))!=NULL)
    {
        buffer = (gchar *) g_malloc(1024);
        while (fgets(buffer,1023, file) )
        {
            if (!buffer) break;
            if (*buffer  !='\n' && *buffer != '#')
            {
                g_strdelimit(buffer, "\n",'\0');
                line = g_strdup(buffer);
                list = g_list_append(list,line);
                i++;
            }
        }
        fclose(file);
    }
    g_free(buffer);
    return list;
}


/* the clist widget must have just one column */




void
gm_utils_text_open_activated                         (GtkMenuItem     *menuitem,
                                                      GtkEditable * editable)
{
    GList * f;
    gchar * file_name=NULL;
    GmSyntaxHighlighting *sh;

    f = gm_utils_create_filebrowser (FALSE);
    if (f) {
        file_name = f->data;
        utils_real_open_file (editable, file_name);
        g_free(file_name);
        g_list_free(f);
        sh = gm_utils_get_data (editable, "syntax_highlighting");
        if (sh){
            gm_utils_text_colorize (editable, 0, -1);
            return;
        }
    }

}


static void
utils_real_save_file_from_editable (GtkEditable * editable, gchar *filename)
{
    GtkWidget *text;
    gchar *data;
    FILE *fp;
    gint bytes_written, len;

    g_return_if_fail (filename != NULL);

    text = GTK_WIDGET(editable);
    data = gtk_editable_get_chars (editable, 0, -1);

    fp = fopen (filename, "w");
    if (fp == NULL)
    {
        g_free (data);
        return;
    }

    len = strlen (data);
    bytes_written = fwrite (data, sizeof (gchar), len, fp);
    fclose (fp);
    g_free (data);
    if (len != bytes_written)
    {
        gm_utils_warning (_("Error while saving File !!"));
        return;
    }
}

FILE * gm_utils_open_file (gchar * filename)
{
    FILE *fp;

    fp = fopen (filename, "w");
    if (fp == NULL)
    {
        gm_utils_warning (_("Could not open File for writing!!"));
        return NULL;
    }
    return fp;
}


gint gm_utils_write_to_file (FILE *f, gchar *data )
{

    gint bytes_written, len;

    if (!data) {
        gdm ("No data to be written in the file ");
        return FALSE ;
    }

    len = strlen (data);
    bytes_written = fwrite (data, sizeof (gchar), len, f);
    if (len != bytes_written)
    {
        gm_utils_warning (_("Error while saving File !!"));
        return FALSE ;
    }
    return TRUE;
}




void
gm_utils_text_save_under_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable)
{

    gchar * file_name=NULL;

    file_name = gm_utils_show_save_filesel();
    if (file_name) {
        utils_real_save_file_from_editable (editable, file_name);
        g_free(file_name);
    }

}

static gchar * utils_collect_text_from_clist (GtkCList * clist, gint max_lines)
{

    gint i;

    GList * l;
    GString * string;
    gint row = 0;
    gchar * data = NULL;

    l = clist->row_list;

    string = g_string_new("");
    while (l && (row < max_lines || max_lines == -1)){
        for (i = 0; i < clist->columns; i++){
            gchar *cdata = NULL;
            gtk_clist_get_text (clist, row, i, &cdata);
            if (i) {    
                if (cdata)
                    g_string_sprintfa (string,"\t%s", cdata);
                else
                    g_string_append (string, "\t");
            }else {
                if (cdata)
                    g_string_sprintfa (string,"%s", cdata);
            }
        }
        g_string_append (string, "\n");
        l = l->next;
        row++;
    }
    data = string->str;
    g_string_free (string, FALSE);
    return data;
}



void
gm_utils_load_file_to_clist (gchar *file_name, gint max_lines, GtkCList *clist) 
{


    GList * f = NULL;
    GList *l = NULL;    
    GList *dlist = NULL;    
    gint count = 0;
    gboolean  free_it = FALSE;
    gboolean warn_user = FALSE;
    

    g_return_if_fail (clist != NULL);

    gtk_clist_freeze (clist);    
    if (!file_name) {
        warn_user = TRUE;        
        f = gm_utils_create_filebrowser (FALSE);
        if (f) {
            file_name = f->data;
        }
        free_it = TRUE;        
    }
    if (file_name){
        dlist = l = gm_utils_load_file_as_list (file_name);
        if (!l) {
            gchar *error;
            error =g_strdup_printf (_("Could not load file %s"), file_name);
            if (warn_user)
                gm_utils_warning(error);
            else 
                gdm (error);            
            g_free (error);            
        }        
        else 
            gtk_clist_clear (clist);
        if (free_it)
            g_free(file_name);
        g_list_free(f);
    }
    
    
    while (l && (count < max_lines || max_lines == -1)){
        gchar *row[1];
        row[0] = (gchar *) l->data;
        gtk_clist_append (clist, row);
        l = l->next;
        count++;
    }
    if (dlist)
        g_list_g_free (dlist);
    gtk_clist_thaw (clist);    
}


void
gm_utils_save_clist_to_file (gchar *file_name, gint max_lines, GtkCList *clist)
{
    gchar *data;
    FILE *fp;
    gint bytes_written, len;
    GList *f = NULL;    


    g_return_if_fail (clist != NULL);

    if (!file_name) {
        f = gm_utils_create_filebrowser (FALSE);
        if (f) {
            file_name = f->data;
        }        
    }
    data = utils_collect_text_from_clist (clist, max_lines);
    fp = fopen (file_name, "w");
    if (fp == NULL)
    {
        gm_utils_warning (_("Error while saving Clistdata to File  !!"));
        g_free (data);
        g_list_free(f);
        return;
    }
    g_list_free(f);
    len = strlen (data);
    bytes_written = fwrite (data, sizeof (gchar), len, fp);
    fclose (fp);
    g_free (data);
    if (len != bytes_written)
    {
        gm_utils_warning (_("Error while saving File !!"));
        return;
    }
}


void
gm_utils_clist_save_under_activated                         (GtkMenuItem     *menuitem,
        GtkCList * clist)
{

    gchar * file_name=NULL;

    g_return_if_fail (clist != NULL);


    file_name = gm_utils_show_save_filesel();
    if (file_name) {
        gm_utils_save_clist_to_file (file_name, -1,clist);
        g_free(file_name);
    }

}











GtkWidget *gm_utils_create_entry_widget (void)
{
    GtkWidget * entry;
    GtkWidget * entry_popup_menu = NULL;

    entry = gtk_entry_new ();

    entry_popup_menu = utils_create_text_entry_popup_menu (GTK_EDITABLE(entry));

    gtk_signal_connect (GTK_OBJECT (entry), "button_press_event",
                        (GtkSignalFunc) utils_text_button_pressed,
                        entry_popup_menu);

    gtk_object_set_data (GTK_OBJECT (entry), "popup_menu", entry_popup_menu);
    gnome_popup_menu_attach (entry_popup_menu, entry, NULL);


    gtk_signal_connect (GTK_OBJECT (entry), "key_press_event",
                        GTK_SIGNAL_FUNC (utils_text_key_press),
                        NULL);

    return entry;
}


static void utils_save_filesel_ok(GtkWidget * w, gint * answer)
{

    *answer = 1;
    gtk_main_quit();


}


static gint utils_save_filesel_cancel (GtkWidget * w, gint * answer)
{

    *answer = 0;
    gtk_main_quit();
    return TRUE;

}






gchar *gm_utils_show_save_filesel(void)
{
    static GtkWidget *save_filesel=NULL;
    gchar *filename=NULL, *text, *tmp;
    gint answer = 0;

    GtkObject *object;

    if (!save_filesel)
        save_filesel = gtk_file_selection_new(_("Save Result under "));


    object = GTK_OBJECT(GTK_FILE_SELECTION(save_filesel)->ok_button);
    gtk_signal_connect(object, "clicked",
                       GTK_SIGNAL_FUNC(utils_save_filesel_ok),
                       &answer);
    object = GTK_OBJECT(GTK_FILE_SELECTION(save_filesel)->cancel_button);
    gtk_signal_connect(object, "clicked",
                       GTK_SIGNAL_FUNC(utils_save_filesel_cancel),
                       &answer);

    gtk_signal_connect(GTK_OBJECT(save_filesel), "delete_event",
                       GTK_SIGNAL_FUNC(utils_save_filesel_cancel),
                       &answer);
    gtk_widget_show(save_filesel);
    gtk_main();


    gtk_widget_hide(save_filesel);
    if (GTK_IS_WIDGET(save_filesel)) {
        if (answer) {
            filename = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(save_filesel)));
            text = g_strdup(filename);
            if ((tmp = strrchr(text, '/')) != NULL)
                *(tmp + 1) = '\0';
            g_free(text);
        }
    }
    return filename;
}

void
gm_utils_set_widget_background (GtkWidget * widget, gchar * fgcolor_name, gchar * bgcolor_name)
{
    GtkRcStyle *rc_style;
    GdkColor fgcolor;
    GdkColor bgcolor;


    g_return_if_fail (bgcolor_name != NULL);
    g_return_if_fail (widget != NULL);


    if (!gdk_color_parse(bgcolor_name,&bgcolor))
        return;


    if (fgcolor_name && !gdk_color_parse(fgcolor_name,&fgcolor))
        return;


    rc_style=gtk_rc_style_new();
    if (fgcolor_name) {
        rc_style->fg[GTK_STATE_NORMAL]     = fgcolor;
        rc_style->text[GTK_STATE_SELECTED] = fgcolor;
    }

    rc_style->bg[GTK_STATE_NORMAL]=bgcolor;
    rc_style->base[GTK_STATE_NORMAL]=bgcolor;
    rc_style->color_flags[GTK_STATE_NORMAL]|=GTK_RC_FG;
    rc_style->color_flags[GTK_STATE_NORMAL]|=GTK_RC_BG;
    rc_style->color_flags[GTK_STATE_NORMAL]|=GTK_RC_BASE;
    rc_style->color_flags[GTK_STATE_SELECTED]|=GTK_RC_TEXT;
    gtk_widget_modify_style(widget, rc_style);
    gtk_rc_style_unref(rc_style);

}






GmNpageData * gm_utils_get_current_npage_data (GmConnection * connection )
{

    GtkWidget * main_note_book;
    GtkWidget * current_npage;

    g_return_val_if_fail (connection != NULL, NULL);

    main_note_book = gm_utils_get_data  (GTK_OBJECT(connection->connection_window),"main_note_book");
    current_npage   = gtk_notebook_get_nth_page (GTK_NOTEBOOK (main_note_book), gtk_notebook_get_current_page (GTK_NOTEBOOK (main_note_book)));

    return (GmNpageData *) gm_utils_get_data  (GTK_OBJECT(current_npage),"npage_data");
}




/**
 * gm_utils_option_menu_get_selected_index:
 * @optionmenu: a gtkoptionmenu
 *
 * Tries to find out (in an ugly way) the selected
 * item in @optionsmenu
 *
 * Return value: the selected index or -1 on error (or no selection?)
 **/

int
gm_utils_option_menu_get_selected_index (GtkOptionMenu *optionmenu)
{
    GtkMenu *menu;
    GtkMenuItem *selected;
    GList *iterator;
    int index = -1;
    int i = 0;

    g_return_val_if_fail (optionmenu != NULL, -1);

    menu = (GtkMenu *) gtk_option_menu_get_menu (optionmenu);
    iterator = GTK_MENU_SHELL (menu)->children;
    selected = (GtkMenuItem *) gtk_menu_get_active (menu);

    while (iterator) {

        if (iterator->data == selected) {

            index = i;
            break;
        }

        iterator = iterator->next;
        i++;
    }

    return index;
}



/*****************************************************************************************************/
/* Progress Bar
 *
 */




void
gm_utils_progress_tick (GtkWidget * progressbar, gfloat percentage)
{


    while (gtk_events_pending ())
        gtk_main_iteration ();
    if (percentage >= 0.0 && percentage < 1.0 )
        gtk_progress_set_percentage (GTK_PROGRESS (progressbar),
                                     percentage );
}


void
gm_utils_choose_color_button_clicked (GtkWidget *button, GtkEntry *entry)
{
    gchar * hex_color;
    gchar * color;

    color = gtk_entry_get_text (entry);

    if (color && strlen (color)==7)
        hex_color = gm_dialogs_show_color_selection_dialog (color);
    else
        hex_color = gm_dialogs_show_color_selection_dialog ("#FFFFFF");

    if (hex_color)
    {
        gtk_entry_set_text (entry, hex_color);
        g_free (hex_color);
    }
}

void
gm_utils_color_entry_changed (GtkEditable     *editable,
                              GtkWidget       *button)
{

    gchar * color;
    GtkEditable * highlighting_preview;
    GdkColor *gdk_color;


    color = gtk_entry_get_text (GTK_ENTRY (editable));
    if (color && strlen (color)==7) {
        gm_utils_set_widget_background (button, NULL, color);
        highlighting_preview = GTK_EDITABLE (gm_utils_get_data_ww (button, "highlighting_preview"));
        gdk_color = gm_utils_get_data_ww (editable, "gdk_color");
        if (gdk_color)
            gdk_color_parse (color, gdk_color);

        if (gm_utils_get_data_ww (editable, "bgcolor")){ /* bg color of the preview text widget */
            gm_utils_set_widget_background (GTK_WIDGET (highlighting_preview), NULL, color);
        }else if (highlighting_preview)
            gm_utils_text_colorize (highlighting_preview, 0, -1);
    }
}



/*****************************************************************************************************/
/*   text widget routines ( highlighting and common methods ) */




static gboolean
utils_text_key_press                    (GtkWidget       *widget,
        GdkEventKey     *event,
        gpointer         user_data)
{
    if (event->state & GDK_CONTROL_MASK)
    {
        switch (event->keyval)
        {
        case GDK_v :
            {
/*paste the ringdata */
                gtk_selection_convert(widget, GDK_SELECTION_PRIMARY,GDK_TARGET_STRING, 0);
                break;
            }

        case GDK_y :
            {		/* paste the clipboard data */
                gm_utils_text_paste_activated (NULL, GTK_EDITABLE(widget));
                return TRUE;
                break;
            }

        case GDK_c :
            {		/* copy */
                gm_utils_text_copy_activated (NULL, GTK_EDITABLE(widget));
                return TRUE;
                break;
            }
        }
    }
    return FALSE;
}







static GtkWidget *
utils_create_text_entry_popup_menu (GtkEditable * editable)
{

    GtkWidget * text_popup_menu = NULL;
    GtkWidget * pmenu;

    text_popup_menu = create_text_entry_popup_menu ();

    pmenu = gm_utils_get_data (GTK_OBJECT (text_popup_menu), "text_entry_copy");
    gtk_signal_connect (GTK_OBJECT (pmenu), "activate",
                        GTK_SIGNAL_FUNC (gm_utils_text_copy_activated),
                        editable);
    pmenu = gm_utils_get_data (GTK_OBJECT (text_popup_menu), "text_entry_paste");
    gtk_signal_connect (GTK_OBJECT (pmenu), "activate",
                        GTK_SIGNAL_FUNC (gm_utils_text_paste_activated),
                        editable);

    pmenu = gm_utils_get_data (GTK_OBJECT (text_popup_menu), "text_entry_open");
    gtk_signal_connect (GTK_OBJECT (pmenu), "activate",
                        GTK_SIGNAL_FUNC (gm_utils_text_open_activated),
                        editable);

    pmenu = gm_utils_get_data (GTK_OBJECT (text_popup_menu), "text_entry_save_under");
    gtk_signal_connect (GTK_OBJECT (pmenu), "activate",
                        GTK_SIGNAL_FUNC (gm_utils_text_save_under_activated),
                        editable);


    pmenu = gm_utils_get_data (GTK_OBJECT (text_popup_menu), "text_entry_clear");
    gtk_signal_connect (GTK_OBJECT (pmenu), "activate",
                        GTK_SIGNAL_FUNC (gm_utils_text_delete_activated),
                        editable);

    pmenu = gm_utils_get_data (GTK_OBJECT (text_popup_menu), "text_entry_cut");
    gtk_signal_connect (GTK_OBJECT (pmenu), "activate",
                        GTK_SIGNAL_FUNC (utils_text_cut_activated),
                        editable);

    pmenu = gm_utils_get_data (GTK_OBJECT (text_popup_menu), "extern_editor");
    gtk_signal_connect (GTK_OBJECT (pmenu), "activate",
                        GTK_SIGNAL_FUNC (gm_utils_extern_text_editor_activated),
                        editable);

    return text_popup_menu;
}


void gm_utils_attach_menu_to_text_widget (GtkWidget * text)
{

    GtkWidget * text_popup_menu = NULL;

    g_return_if_fail (text != NULL);

    text_popup_menu = utils_create_text_entry_popup_menu (GTK_EDITABLE(text));

    gtk_signal_connect (GTK_OBJECT (text), "button_press_event",
                        (GtkSignalFunc) utils_text_button_pressed,
                        text_popup_menu);

    gtk_object_set_data (GTK_OBJECT (text), "popup_menu", text_popup_menu);
    gnome_popup_menu_attach (text_popup_menu, text, NULL);


    gtk_signal_connect (GTK_OBJECT (text), "key_press_event",
                        GTK_SIGNAL_FUNC (utils_text_key_press),
                        NULL);
}


void  gm_utils_configure_text_widget (GtkWidget *text)
{

    GtkWidget * text_popup_menu = NULL;


    g_return_if_fail (text != NULL);




    text_popup_menu = utils_create_text_entry_popup_menu (GTK_EDITABLE(text));

    gtk_signal_connect (GTK_OBJECT (text), "button_press_event",
                        (GtkSignalFunc) utils_text_button_pressed,
                        text_popup_menu);

    gtk_object_set_data (GTK_OBJECT (text), "popup_menu", text_popup_menu);
    gnome_popup_menu_attach (text_popup_menu, text, NULL);


    gtk_signal_connect (GTK_OBJECT (text), "key_press_event",
                        GTK_SIGNAL_FUNC (utils_text_key_press),
                        NULL);
}

GtkWidget *gm_utils_create_text_widget (void)
{
    GtkWidget * text;
    GtkWidget * text_popup_menu = NULL;


    text = gtk_text_new (NULL, NULL);



    text_popup_menu = utils_create_text_entry_popup_menu (GTK_EDITABLE(text));

    gtk_signal_connect (GTK_OBJECT (text), "button_press_event",
                        (GtkSignalFunc) utils_text_button_pressed,
                        text_popup_menu);

    gtk_object_set_data (GTK_OBJECT (text), "popup_menu", text_popup_menu);
    gnome_popup_menu_attach (text_popup_menu, text, NULL);


    gtk_signal_connect (GTK_OBJECT (text), "key_press_event",
                        GTK_SIGNAL_FUNC (utils_text_key_press),
                        NULL);

    return text;
}


static gint
utils_text_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
    GtkWidget *popup;
    GtkEditable *editable;
    GtkWidget *pmenu;
    gboolean state=TRUE;

    if (event->button != 3)
        return FALSE;

    popup = data;
    editable = GTK_EDITABLE (widget);

    if (editable->selection_end_pos - editable->selection_start_pos  <= 0 )
        state = FALSE;

    pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_copy");
    gtk_widget_set_sensitive (pmenu, state);
    pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_cut");
    gtk_widget_set_sensitive (pmenu, state);
    pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_clear");
    gtk_widget_set_sensitive (pmenu, state);
    if (GTK_IS_ENTRY(widget)) {
        pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_open");
        gtk_widget_set_sensitive (pmenu, FALSE);
        pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_save_under");
        gtk_widget_set_sensitive (pmenu, FALSE);
    }else {
        pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_open");
        gtk_widget_set_sensitive (pmenu, TRUE);
        pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_save_under");
        gtk_widget_set_sensitive (pmenu, TRUE);
    }

    if(!editable->editable){
        pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_open");
        gtk_widget_set_sensitive (pmenu, FALSE);
        pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_cut");
        gtk_widget_set_sensitive (pmenu, FALSE);
        pmenu =     gm_utils_get_data (GTK_OBJECT (popup), "text_entry_clear");
        gtk_widget_set_sensitive (pmenu, FALSE);
        pmenu = gm_utils_get_data (GTK_OBJECT (popup), "text_entry_paste");
        gtk_widget_set_sensitive (pmenu, FALSE);
        pmenu = gm_utils_get_data (GTK_OBJECT (popup), "extern_editor");
        gtk_widget_set_sensitive (pmenu, FALSE);
    }
    pmenu = gm_utils_get_data (GTK_OBJECT (popup), "text_entry_paste");
    if (!global_app->char_clipboard)
        gtk_widget_set_sensitive (pmenu, FALSE);

    return TRUE;
}






void
gm_utils_text_copy_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable)
{


    if (editable->selection_end_pos - editable->selection_start_pos  == 0 )
        return;

    if (global_app->char_clipboard)
        g_free (global_app->char_clipboard);

    global_app->char_clipboard = gtk_editable_get_chars (editable,
                                 (editable->selection_start_pos>editable->selection_end_pos)?editable->selection_end_pos:editable->selection_start_pos,
                                 (editable->selection_start_pos>editable->selection_end_pos)?editable->selection_start_pos:editable->selection_end_pos);
}

void
gm_utils_text_paste_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable)
{
    gint position;

    if (global_app->char_clipboard) {
        position = editable->current_pos;
        gtk_editable_insert_text   (editable,global_app->char_clipboard,
                                    strlen (global_app->char_clipboard),
                                    &position);
    }

}

void
gm_utils_text_delete_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable)
{


    gtk_editable_delete_text (editable,
                              (editable->selection_start_pos>editable->selection_end_pos)?editable->selection_end_pos:editable->selection_start_pos,
                              (editable->selection_start_pos>editable->selection_end_pos)?editable->selection_start_pos:editable->selection_end_pos);

}


void
gm_utils_extern_text_editor_activated            (GtkMenuItem     *menuitem,
        GtkEditable * editable)
{
    gchar * text_editor;
    gchar * text;
    FILE * fh;
    gchar * command;
    gchar * file_name = "/tmp/gmyclient_tempfile";
    time_t now;


    now = time (0);
    file_name = g_strdup_printf ("/tmp/gmyclient_tempfile%ld",now);


    text_editor = (GE_CONFIG (global_app))->text_editor;
    if (!text_editor) {
        gm_utils_warning (_("You didn't select a text editor please select an editor"));
        gm_cc_show_option_dialog (global_app->cc, OPTIONS_GENERAL_PAGE);
    }
    gdm("warten");

    text_editor = (GE_CONFIG (global_app))->text_editor;
    if (!text_editor)
        return;

    fh = gm_utils_open_file (file_name);
    if (fh) {
        gint ret;
        text  =  gtk_editable_get_chars (editable, 0, -1);
        gm_utils_write_to_file (fh, text);
        fclose (fh);
        g_free (text);
        command = g_strdup_printf ("%s %s", text_editor, file_name);
        ret = system (command);
        if (ret != -1)
            utils_real_open_file (editable, file_name);
        else
            gm_utils_warning (_("An error occur while running extern text editor"));
        /* remove temp file */
        g_free (command);
        command = g_strdup_printf ("rm -f %s", file_name);
        system (command);
        g_free (command);
    }
    g_free (file_name);



}



static void
utils_text_cut_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable)
{
    gm_utils_text_copy_activated (NULL, editable);
    gm_utils_text_delete_activated (NULL, editable);
}




/* higlighting part */








static
GmSyntaxHighlighting * gm_syntax_highlighting_new (GtkEditable * editable)
{
    GmSyntaxHighlighting * sh;

    sh = g_new (GmSyntaxHighlighting, 1);
    sh->colors_table = g_hash_table_new (g_str_hash, g_str_equal);
    sh->keywords_regex_pattern = NULL;
    sh->functions_regex_pattern = NULL;
    sh->keywords = NULL;
    sh->functions = NULL;
    sh->gdk_font = NULL;
    sh->editable = editable;
    sh->dirty = FALSE;    
    return sh;
}


static void free_syntax_colors (gpointer key, gpointer value, gpointer user_data)
{

    g_free ((gchar *) value);
}







static void
gm_syntax_highlighting_free  (GmSyntaxHighlighting *sh)
{
    g_hash_table_foreach (sh->colors_table, free_syntax_colors, NULL);
    g_hash_table_destroy (sh->colors_table);
    g_list_g_free (sh->keywords);
    g_list_g_free (sh->functions);
    g_free (sh->keywords_regex_pattern);
    g_free (sh->functions_regex_pattern);
}



static void
utils_text_colorize_functions (GmSyntaxHighlighting * sh,
                               GtkEditable * editable,
                               gchar           *buffer,
                               gint start_pos)
{


    /* keywords */


    regmatch_t *match;
    GList * l, *match_list;



    l = match_list = gm_utils_match_regex (sh->functions_regex_pattern, buffer, FALSE);
    if (match_list) {
        while (l) {
            match = l->data;
            match->rm_eo--; /* exclude the left bracet */
            
            gm_utils_set_text_color (editable, NULL,
                                     match->rm_so + start_pos,
                                     match->rm_eo + start_pos,
                                     g_hash_table_lookup(sh->colors_table, "functions"), sh->gdk_font);
            g_free (match);
            l = l->next;
        }
        g_list_free (match_list);
    }

}



static void
utils_text_colorize_keywords (GmSyntaxHighlighting * sh,
                              GtkEditable * editable,
                              gchar           *buffer,
                              gint start_pos)
{


    /* keywords */
    regmatch_t *match;
    GList * l, *match_list;

    l = match_list = gm_utils_match_regex (sh->keywords_regex_pattern, buffer, FALSE);
    if (match_list) {
        while (l) {
            match = l->data;
            gm_utils_set_text_color (editable, NULL,
                                     match->rm_so + start_pos,
                                     match->rm_eo + start_pos,
                                     g_hash_table_lookup(sh->colors_table, "keywords"), sh->gdk_font);
            g_free (match);
            l = l->next;
        }
        g_list_free (match_list);
    }
}


static void
utils_text_colorize_default (GmSyntaxHighlighting * sh,
                             GtkEditable * editable,gchar * buffer,
                             gint start_pos, gint end_pos)
{
    gm_utils_set_text_color (editable, buffer,
                             start_pos,
                             end_pos,
                             g_hash_table_lookup(sh->colors_table, "default"), sh->gdk_font);
}

static void
utils_text_colorize_strings (GmSyntaxHighlighting * sh,
                             GtkEditable * editable, 
                             gchar *buffer)
{

    regmatch_t *match;
    GList * l, *match_list;

        /* delimiters */
    l = match_list = gm_utils_match_regex ("'[^']*'|\"[^\"]*\"", buffer, FALSE);
    if (match_list) {
        while (l) {
            match = l->data;
            gm_utils_set_text_color (editable, NULL,
                                     match->rm_so,
                                     match->rm_eo,
                                     g_hash_table_lookup(sh->colors_table, "strings"), sh->gdk_font);
            g_free (match);
            l = l->next;
        }
        g_list_free (match_list);
    }

}


static void
utils_text_colorize_comments (GmSyntaxHighlighting * sh,
                              GtkEditable * editable, 
                              gchar * buffer)
{

    regmatch_t *match;
    GList * l, *match_list;



        /* delimiters */
    l = match_list = gm_utils_match_regex ("/\\*[^/]*\\*/", buffer, FALSE);
    if (match_list) {
        while (l) {
            match = l->data;
            gm_utils_set_text_color (editable, NULL,
                                     match->rm_so,
                                     match->rm_eo,
                                     g_hash_table_lookup(sh->colors_table, "comments"), sh->gdk_font);
            g_free (match);
            l = l->next;
        }
        g_list_free (match_list);
    }

}


void static
utils_text_compute_start_end_positions (GmSyntaxHighlighting * sh,
                                        GtkEditable * editable,
                                        gint            new_text_length,
                                        gint            insert_position,
                                        gint * start_pos, gint * end_pos)
{
    GdkWChar ch;
    gint i;

    i = insert_position - new_text_length - 1;
    while (i>=0){
        ch = GTK_TEXT_INDEX (GTK_TEXT (editable), i);
        if (ch == '\t' || ch =='\n' || ch=='\r' || ch ==' ' || ch =='(' || ch ==')' ){
            break;
        }
        i--;
    }

    if (i<0)
        i = 0;
    *start_pos = i;
    i = insert_position ;
    while (i < GTK_TEXT(editable)->text_end - GTK_TEXT(editable)->gap_size){
        ch = GTK_TEXT_INDEX (GTK_TEXT (editable), i);
        if (ch == '\t' || ch =='\n' || ch=='\r' || ch ==' ' || ch =='(' || ch ==')' ){
            break;
        }
        i++;
    }
    *end_pos = i;
}



void
gm_utils_text_colorize (GtkEditable *editable,
                        gint start_pos,
                        gint end_pos)
{
    GmSyntaxHighlighting *sh;    
    gchar *buffer;

    sh = gm_utils_get_data (editable, "syntax_highlighting");
    g_return_if_fail (sh != NULL);
    buffer = gtk_editable_get_chars (editable, start_pos, end_pos);
    utils_text_colorize_default (sh, editable, buffer, start_pos, end_pos);
    utils_text_colorize_keywords (sh, editable, buffer, start_pos);
    utils_text_colorize_functions (sh, editable, buffer, start_pos);
    g_free (buffer);
    buffer = gtk_editable_get_chars (editable, 0, -1);
    utils_text_colorize_strings (sh ,editable, buffer);
    utils_text_colorize_comments (sh ,editable, buffer);
    g_free (buffer);

}




void
gm_utils_text_colorize_error (GtkEditable *editable,
                              gint start_pos,
                              gint end_pos)
{
    GmSyntaxHighlighting *sh;
    GdkColor *color;
    sh = gm_utils_get_data (editable, "syntax_highlighting");
    g_return_if_fail (sh != NULL);
    color = g_hash_table_lookup(sh->colors_table, "errors");
    
    gm_utils_set_text_color (editable, NULL,start_pos, end_pos,
                             color, sh->gdk_font);
    sh->dirty = TRUE;    
}




static void
utils_text_on_insert_text    (GtkEditable     *editable,
                              gchar           *new_text,
                              gint             new_text_length,
                              gint            *position,
                              GmSyntaxHighlighting * sh)
{
    gint current_pos;
    gint start_pos = 0;
    gint end_pos = 0;

    
    g_return_if_fail (sh != NULL);


    current_pos = gtk_editable_get_position (editable);
    if (sh->dirty) {
        start_pos = 0;
        end_pos = -1;
    }else    
        utils_text_compute_start_end_positions (sh, editable, new_text_length, *position ,&start_pos, &end_pos);
    gm_utils_text_colorize (editable, start_pos, end_pos);

    sh->dirty = FALSE;
    /* restore the cursor position */
    gtk_text_set_point (GTK_TEXT(editable), current_pos);    
    gtk_editable_set_position (editable, current_pos);
}


static void
utils_text_on_delete_text    (GtkEditable     *editable,
                              gint             start_pos,
                              gint             end_pos,
                              GmSyntaxHighlighting * sh)
{
    gint current_pos;
    gint s_pos = 0;
    gint e_pos = 0;

    g_return_if_fail (sh != NULL);

    current_pos = gtk_editable_get_position (editable);

    if (sh->dirty) {
        gdm ("colorizing all");        
        s_pos = 0;
        e_pos = -1;
    }else
        utils_text_compute_start_end_positions (sh, editable, 1, start_pos,&s_pos, &e_pos);
    if ((s_pos < e_pos) || (e_pos == -1 && s_pos == 0)) 
        gm_utils_text_colorize (editable, s_pos, e_pos);

    sh->dirty = FALSE;    
    /* restore the cursor position */
    if (current_pos > gtk_text_get_length(GTK_TEXT (editable)))
        current_pos = 0;    
    gtk_text_set_point (GTK_TEXT (editable), current_pos);    
    gtk_editable_set_position (editable, current_pos);
}


static
gchar * utils_compose_regex (GList *list, gint type)
{
    GList * l;
    gchar *regex;
    GString * string;

    if (!list){
        gdm ("trying to compose a regularexpression from an empty list ");
        return NULL;
    }


    string = g_string_new ("");

    l = list;
    while (l) {
        gchar * str = l->data;

        if (str && strlen (str)){
            switch (type){
                case REGEX_KEYWORDS:
                    g_string_sprintfa (string, "\\b%s\\b", str);
                    break;                    
                case REGEX_FUNCTIONS:
                    g_string_sprintfa (string, "\\b%s\\(", str);
                    break;
            }            
            if (l->next)
                g_string_append (string, "|");
        }

        l = l->next;
    }
    regex = string->str;
    g_string_free (string, FALSE);
    return regex;
}



void
gm_utils_text_highlight_colors_changed (void *ignore, GtkEditable * editable)
{
    GmSyntaxHighlighting *sh;



    sh = gm_utils_get_data (editable, "syntax_highlighting");
    g_return_if_fail (sh != NULL);
    gm_utils_set_widget_background (GTK_WIDGET (editable), NULL, SQL_EDITOR_CONFIG(global_app)->bgcolor);
    utils_text_configure_highlight_colors (sh);
    gm_utils_text_colorize (editable, 0, -1);
}



static void
utils_text_configure_highlight_colors (GmSyntaxHighlighting *sh)
{

    GdkColor *color;


    color = g_hash_table_lookup(sh->colors_table, "comments");
    if (!color) {
        color = g_new(GdkColor ,1);
        g_hash_table_insert (sh->colors_table, "comments",  color);
    }
    gdk_color_parse (SQL_EDITOR_CONFIG(global_app)->comments_color, color);

    color = g_hash_table_lookup(sh->colors_table, "functions");
    if (!color){
        color = g_new(GdkColor ,1);
        g_hash_table_insert (sh->colors_table, "functions", color);
    }
    gdk_color_parse (SQL_EDITOR_CONFIG(global_app)->functions_color, color);

    color = g_hash_table_lookup(sh->colors_table, "errors");
    if (!color){
        color = g_new(GdkColor ,1);
        g_hash_table_insert (sh->colors_table, "errors", color);
    }
    gdk_color_parse (SQL_EDITOR_CONFIG(global_app)->errors_color, color);


    color = g_hash_table_lookup(sh->colors_table, "keywords");
    if (!color){
        color = g_new(GdkColor ,1);
        g_hash_table_insert (sh->colors_table, "keywords", color);
    }
    gdk_color_parse (SQL_EDITOR_CONFIG(global_app)->keywords_color, color);


    color = g_hash_table_lookup(sh->colors_table, "strings");
    if (!color){
        color = g_new(GdkColor ,1);
        g_hash_table_insert (sh->colors_table, "strings", color);
    }
    gdk_color_parse (SQL_EDITOR_CONFIG(global_app)->strings_color, color);

    color = g_hash_table_lookup(sh->colors_table, "default");
    if (!color){
        color = g_new(GdkColor ,1);
        g_hash_table_insert (sh->colors_table, "default", color);
    }
    gdk_color_parse (SQL_EDITOR_CONFIG(global_app)->default_color, color);


    if (sh->gdk_font)
        gdk_font_unref (sh->gdk_font);

    sh->gdk_font = NULL;

    if (SQL_EDITOR_CONFIG(global_app)->font_name)
        sh->gdk_font = gdk_font_load (SQL_EDITOR_CONFIG(global_app)->font_name);


}


void
gm_utils_text_configure_sql_syntax_highlighting (GtkEditable * editable)
{

    GmSyntaxHighlighting * sh;
    gint handler_id;
    gchar * filename;

    sh = gm_syntax_highlighting_new (editable);


    filename = g_strdup_printf ("%s/misc/gmyclient-keywords",PACKAGE_DATA_DIR);
    sh->keywords = gm_utils_load_file_as_list (filename);
    g_free (filename);
    sh->keywords_regex_pattern = utils_compose_regex (sh->keywords, REGEX_KEYWORDS);


    filename = g_strdup_printf ("%s/misc/gmyclient-functions",PACKAGE_DATA_DIR);
    sh->functions = gm_utils_load_file_as_list (filename);
    g_free (filename);

    sh->functions_regex_pattern = utils_compose_regex (sh->functions, REGEX_FUNCTIONS);




    gtk_object_set_data_full (GTK_OBJECT(editable), "syntax_highlighting",
                              sh, (GtkDestroyNotify) gm_syntax_highlighting_free );



    handler_id = gtk_signal_connect_after (GTK_OBJECT (editable), "insert_text",
                                           GTK_SIGNAL_FUNC (utils_text_on_insert_text),
                                           sh);
    gtk_object_set_data (GTK_OBJECT (editable), "insert_handler_id", GINT_TO_POINTER(handler_id));

    handler_id = gtk_signal_connect_after (GTK_OBJECT (editable), "delete_text",
                                           GTK_SIGNAL_FUNC (utils_text_on_delete_text),
                                           sh);



    gtk_object_set_data (GTK_OBJECT (editable), "delete_handler_id", GINT_TO_POINTER(handler_id));

    gtk_signal_connect_while_alive  (GTK_OBJECT (global_app), "sql_mode_colors_changed",
                                     GTK_SIGNAL_FUNC (gm_utils_text_highlight_colors_changed),
                                     editable, GTK_OBJECT(editable));

    if (SQL_EDITOR_CONFIG(global_app)->bgcolor)
        gm_utils_set_widget_background (GTK_WIDGET (editable), NULL, SQL_EDITOR_CONFIG(global_app)->bgcolor);

    utils_text_configure_highlight_colors (sh);

}


/**
 * insert the new_text at the start_pos with the given color 
 * set the new_text to null to change the color of existing text 
 * starting at position start_pos
 *
 */

void
gm_utils_set_text_color (GtkEditable *editable,gchar * new_text,
                         gint start_pos,
                         gint end_pos ,
                         GdkColor *color, GdkFont * font)
{
    gint insert_handler_id;
    gint delete_handler_id;
    gchar * text;

    g_return_if_fail (color != NULL);
    g_return_if_fail (editable != NULL);


    gtk_text_freeze (GTK_TEXT (editable));
    insert_handler_id = GPOINTER_TO_INT (gm_utils_get_data (editable, "insert_handler_id"));
    delete_handler_id = GPOINTER_TO_INT (gm_utils_get_data (editable, "delete_handler_id"));
    gtk_signal_handler_block (GTK_OBJECT (editable), insert_handler_id);
    gtk_signal_handler_block (GTK_OBJECT (editable), delete_handler_id);

    if (!new_text){
        text = gtk_editable_get_chars (editable, start_pos, end_pos);
    }else {
        text = new_text;
    }
    gtk_editable_delete_text (editable,start_pos, end_pos);


    if (text ) {
        gtk_text_set_point       (GTK_TEXT(editable), start_pos);
        gtk_text_insert( GTK_TEXT(editable), font,
                         color,
                         NULL,
                         text,
                         strlen(text));
    }
    gtk_signal_handler_unblock (GTK_OBJECT (editable), insert_handler_id);
    gtk_signal_handler_unblock (GTK_OBJECT (editable), delete_handler_id);

    gtk_text_thaw (GTK_TEXT (editable));
    if (!new_text)
        g_free (text);
}





GList *
gm_utils_match_regex (gchar *pattern, gchar * src, gboolean case_sensitive)
{
    int flags= 0;
    regmatch_t match;
    regmatch_t *returned_match;
    gint ret;
    gint count = 1;
    GList * list = NULL;
    regex_t *compiled_regex;
    gint offset = 0;



    if (!case_sensitive)
        flags |= REG_ICASE;

    flags |= REG_EXTENDED;

    compiled_regex =  gm_regex_cache_compile (global_app->regex_cache, pattern,flags);

    if (!compiled_regex)
    {
        gdm("Could not compile the regular expression given %s", pattern);
        return NULL;
    }
    count = compiled_regex->re_nsub;         /* number of parenthesized subexpressions */
    count = 1;                       /* for now we don't allow it */

    while (src && *src !='\0' ){
        ret = regexec (compiled_regex, src, count, &match, flags);
        if  (ret == 0)
        {
            if (match.rm_so == match.rm_eo)
                break;
            returned_match = g_new0 (regmatch_t, 1);
            returned_match->rm_so = match.rm_so + offset;
            returned_match->rm_eo = match.rm_eo + offset;
            list = g_list_append (list, returned_match);
            src += match.rm_eo ;
            offset += match.rm_eo ;
        }else if (ret == REG_NOMATCH)
        {
            break;

        }else
        {
            g_error ("Unexpected error code from regexec: %d.", ret);
            break;
        }
    }
    return list;
}



static gchar *
utils_compose_key (gchar *k, gchar * file_name) 
{
    
    gchar * confkey;
    g_return_val_if_fail (k != NULL, NULL);
    g_return_val_if_fail (file_name != NULL, NULL);
  
    confkey = g_strdup_printf ("=%s=%s", file_name, k);
    return confkey;
}


static void
close_update_info_window (void *ignore, GtkWidget *window) 
{
    gtk_widget_destroy (window);
}



void 
gm_utils_check_update () 
{
    
    gchar *command=NULL ;
    gchar *key = NULL;
    gchar *last_stable_version = NULL;
    gchar *last_beta_version = NULL;
    gchar *devel_version = NULL;
    gchar *news = NULL;
    gchar *info_file = NULL;
    gint   ret;    
    GString * message;
    GtkWidget *text;
    GtkWidget *update_info_window;
    GtkWidget *button;
    
    

    info_file = g_strdup_printf ("%s/%s/UPDATE_INFO",g_get_home_dir(),GM_CONFIG_DIR_NAME);
    command =g_strdup_printf("GET %s?version=%s > %s", GMYCLIENT_UPDATE_URL, VERSION, info_file);
    update_info_window = create_update_info_window();    
    text = gm_utils_get_data (update_info_window, "text");
    button = gm_utils_get_data (update_info_window, "close");    
    gtk_signal_connect(GTK_OBJECT (button), "clicked",
                       GTK_SIGNAL_FUNC(close_update_info_window),
                       update_info_window);
    gtk_widget_show (update_info_window);    
    gtk_refresh();
    
    ret = system (command);
        
    if (!g_file_exists(info_file)) {
        gm_utils_warning (_("an errror occur while trying to get update informations"));
        gtk_widget_destroy (update_info_window);
        g_free(command);
        g_free (info_file);    
        return;
    }

    key  = utils_compose_key ("/Gmyclient/site_ok", info_file);
    if (!gnome_config_get_int (key)){
        gm_utils_warning (_("an errror occur while trying to get update informations"));
    }else {
        key  = utils_compose_key ("/VERSION/LastStableVersion", info_file);
        last_stable_version = gnome_config_get_string (key);
        key  = utils_compose_key ("/VERSION/LastBetaVersion", info_file);
        last_beta_version = gnome_config_get_string (key);
        key  = utils_compose_key ("/VERSION/DevelVersion", info_file);
        devel_version = gnome_config_get_string (key);
        key  = utils_compose_key ("/NEWS/News", info_file);
        news = gnome_config_get_string (key);
        g_free (key);

        message = g_string_new ("");
        g_string_append (message,"-------------------------------------\n");    
        g_string_sprintfa (message,"Your version        : %s\n", VERSION);
        if (last_stable_version)
            g_string_sprintfa (message,"Last stable version : %s\n", last_stable_version);
        if (last_beta_version)
            g_string_sprintfa (message,"Last Beta version : %s\n", last_beta_version);
        if (devel_version)
            g_string_sprintfa (message,"Development version : %s\n", devel_version);
        g_string_append (message, "-------------------------------------\n\n");
        if (news)
            g_string_append (message,news);
        gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, message->str,
                         message->len);
        g_string_free (message, TRUE);
    }
    g_free(command);
    command =g_strdup_printf("rm -f %s", info_file);
    system (command);
    g_free(command);    
    g_free (info_file);    
    g_free (last_stable_version);
    g_free (last_beta_version);
    g_free (devel_version);
    g_free (news);    
}



static gboolean
utils_show_hint (GtkWidget *widget, GmConnection *connection) 
{

    GList *hint_list = NULL;


    hint_list = gm_utils_get_data (widget, "hint_list");

    if (hint_list) {        
        gm_utils_show_status_information (connection, (gchar *)hint_list->data);
        gtk_object_remove_data (GTK_OBJECT (widget), "hint_list");
    }
    if (hint_list && hint_list->next) {
        gtk_object_set_data (GTK_OBJECT(widget), "hint_list", hint_list->next);
        return FALSE;        
    }
    g_list_free (hint_list);    
    return TRUE;
}


static gboolean
utils_show_hint_after_focus (GtkWidget *widget, GdkEventFocus *event, GmConnection * connection)
{

    if (utils_show_hint (widget, connection))
        gtk_signal_disconnect_by_func(GTK_OBJECT (widget), GTK_SIGNAL_FUNC (utils_show_hint_after_focus), connection);
    return FALSE;
}


static void
utils_show_hint_after_selection (GtkWidget *widget, GmConnection * connection)
{
    if (utils_show_hint (widget, connection))
        gtk_signal_disconnect_by_func(GTK_OBJECT (widget), GTK_SIGNAL_FUNC (utils_show_hint_after_selection), connection);
}

void
gm_utils_register_hints (GtkWidget *widget, GList *hint_list, GmConnection *connection) 
{
    GList * l;
    l = hint_list;
    
    if (!GE_CONFIG (global_app)->show_hints)
        return;
    
    g_return_if_fail (hint_list != NULL);
    
    if (GTK_IS_ITEM(widget))
        gtk_signal_connect (GTK_OBJECT (widget), "select",
                            GTK_SIGNAL_FUNC (utils_show_hint_after_selection),
                            connection);
    else
        gtk_signal_connect (GTK_OBJECT (widget), "focus_in_event",
                            GTK_SIGNAL_FUNC (utils_show_hint_after_focus),
                            connection);
    gtk_object_set_data (GTK_OBJECT(widget), "hint_list", hint_list);
}

void
gm_utils_register_hint (GtkWidget *widget, gchar * hint, GmConnection *connection) 
{

    GList * l= NULL;

    g_return_if_fail (hint != NULL);
    
    l = g_list_append (l, hint);    
    gm_utils_register_hints (widget, l, connection);
}
