#ifndef _GM_UTILS__H_
#define _GM_UTILS__H_


#include "gm-app.h"
#include "gm-control-center.h"


#define GM_CONFIG_DIR_NAME ".gmyclient"
#define DEFAULT_LOG_FONT "-Adobe-Helvetica-Medium-R-Normal--12-120-75-75-P-67-ISO8859-1"
#define $pointer(obj,key) gtk_object_get_data (GTK_OBJECT(obj), key)
#define $int_pointer(obj,key) GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT(obj), key))
#define DEFAULT_ROWS_LIMIT 50000

enum {
    SIMPLE_LOGMODE =0,
    ERROR_LOGMODE ,
    MESSAGE_LOGMODE ,
    DATETIME_LOGMODE,
    DATA_LOGMODE
};

typedef struct _CursorStruct CursorStruct;
typedef struct _GmSyntaxHighlighting GmSyntaxHighlighting;

struct _GmSyntaxHighlighting
{
    GHashTable * colors_table;
    GList *keywords;
    GList *functions;
    gchar *keywords_regex_pattern;
    gchar *functions_regex_pattern;
    GdkFont * gdk_font;
    GtkEditable * editable;
    gboolean dirty ;
    
};

struct _CursorStruct {
    gint cursor_num;
    GdkCursor *gdcursor;
};

enum {
    IGNORE_DIALOG_CANCELD=0,
    IGNORE_DIALOG_IGNORE,
    IGNORE_DIALOG_IGNORE_ALL,
};




void    gm_utils_message    (gchar * message);
void    gm_utils_warning    (gchar * message);
void    gm_utils_error      (gchar * message);
void    gm_utils_question   (gchar * message);
void    gm_utils_generic    (gchar * message);
GtkWidget *   gm_stock_pixmap_widget (GtkWidget *widget, gchar *icon);
void gm_utils_create_button_label (GtkWidget * button , gchar * label_str, gchar *pixmap_filename , gboolean show_label);
void gm_utils_create_vbutton_label (GtkWidget * button , gchar * label_str, gchar *pixmap_filename , gboolean show_label);


gpointer gm_utils_get_data    (gpointer obj,gchar *key);
gpointer gm_utils_get_data_ww (gpointer obj,gchar *key);

GtkWidget*
gm_utils_lookup_widget                          (GtkWidget       *widget,
        const gchar     *widget_name);




void gm_utils_write_log_message (GmConnection * connection, gchar *msg, gint logmode);

gboolean gm_utils_yes_no_dialog(gchar * message);
gint     gm_utils_ignore_dialog(gchar * message);

gchar  * gm_utils_strescape (const gchar *source, const gchar *exceptions);
gchar  * g_str_replace (gchar * src, gchar *sub_string, gchar *with_string);
gboolean gm_utils_istrstr (gchar * src, gchar * str );
GtkWidget * create_about_window (void);
void gtk_refresh(void);

gint is_directory (gchar *path);


/* support for GSLIST */

GSList * g_slist_first  (GSList * list);
void     g_slist_g_free (GSList *   ml);
void     g_list_g_free  (GList * list);


/* cursors */
void gm_utils_set_widget_cursor (GtkWidget *widget,gint cursor);


/* filebrowser */
GList * gm_utils_create_filebrowser (gboolean list_mode);


gchar * gm_utils_get_basename (gchar * path);

/* glib2 */
void g_string_append_len (GString *str, gchar * src, gint start_pos, gint len );




/* support  for staus bar */

void gm_utils_show_status_information (GmConnection * connection, gchar * message);


/* gm-text-widget-widget */
void       gm_utils_attach_menu_to_text_widget (GtkWidget * text);
GtkWidget *gm_utils_create_text_widget (void);
void       gm_utils_configure_text_widget (GtkWidget *text);

GtkWidget *gm_utils_create_entry_widget (void);

void
gm_utils_text_open_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable);

void
gm_utils_text_save_under_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable);

void
gm_utils_text_copy_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable);
void
gm_utils_text_paste_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable);

void
gm_utils_text_delete_activated                         (GtkMenuItem     *menuitem,
        GtkEditable * editable);




gchar *gm_utils_show_save_filesel(void);

void gm_utils_clist_save_under_activated                         (GtkMenuItem     *menuitem,GtkCList * clist);
void gm_utils_load_file_to_clist (gchar *file_name, gint max_lines, GtkCList *clist) ;
void gm_utils_save_clist_to_file (gchar *file_name, gint max_lines, GtkCList *clist);





void   gm_utils_set_widget_background(GtkWidget * widget, gchar * fgcolor_name, gchar * bgcolor_name);


/* misc */
GList * gm_utils_load_file_as_list (gchar *filename);



/* return a pointer to the stored information about the focused gtkclist in the main_notebook */
GmNpageData * gm_utils_get_current_npage_data (GmConnection * connection ) ;

/* search for the selected index in the given menu */
int gm_utils_option_menu_get_selected_index (GtkOptionMenu *optionmenu);
FILE * gm_utils_open_file (gchar * filename) ;
gint gm_utils_write_to_file (FILE *f, gchar *data);

void gm_utils_progress_tick (GtkWidget * progressbar, gfloat percentage);
void gm_utils_choose_color_button_clicked (GtkWidget *button, GtkEntry *entry);
void gm_utils_color_entry_changed (GtkEditable *editable, GtkWidget *button);
void gm_utils_extern_text_editor_activated (GtkMenuItem     *menuitem, GtkEditable * editable);
void gm_utils_text_configure_sql_syntax_highlighting (GtkEditable * editable) ;
void gm_utils_set_text_color (GtkEditable *editable,gchar * new_text,
                              gint start_pos,
                              gint end_pos,
                              GdkColor *color, GdkFont * font);




/* regex */

/* return a list of regmatch_t struct */
GList * gm_utils_match_regex (gchar *pattern, gchar * src, gboolean case_sensitive);
void gm_utils_text_colorize (GtkEditable *editable, gint start_pos, gint end_pos);
void gm_utils_text_colorize_error (GtkEditable *editable, gint start_pos, gint end_pos);
void gm_utils_text_highlight_colors_changed (void *ignore, GtkEditable * editable) ;



void gm_utils_register_hints (GtkWidget *widget, GList *hint_list, GmConnection *connection) ;
void gm_utils_register_hint (GtkWidget *widget, gchar *hint, GmConnection *connection) ;

void gm_utils_check_update () ;



#endif
