/*
 *   Based on Code from 
 *   Authors: Ramiro Estrugo <ramiro@eazel.com>
 *            Modified to fit the needs of gmyclient by :
 *            Mohamed Elassadi <mohamed@users.sourceforge.net>
 */

#include "gm.h"
#include "gm-vertical-splitter.h"

#include "gm-gtk-macros.h"

#include <stdlib.h>

struct GmVerticalSplitterDetails {
    gboolean down;
    double down_position;
    guint32 down_time;

};



/* Bar width currently hardcoded to 7 */
#define BAR_HEIGHT 6
#define CLOSED_THRESHOLD 4
#define NOMINAL_SIZE 148
#define SPLITTER_CLICK_SLOP 1
#define SPLITTER_CLICK_TIMEOUT	400



enum {
    COLLAPSED,
    LAST_SIGNAL
};
static guint splitter_signals[LAST_SIGNAL] = { 0 };



/* GmVerticalSplitterClass methods */
static void     gm_vertical_splitter_initialize_class (GmVerticalSplitterClass *vertical_splitter_class);
static void     gm_vertical_splitter_initialize       (GmVerticalSplitter      *vertical_splitter);
static gboolean gm_vertical_splitter_button_press     (GtkWidget                       *widget,
        GdkEventButton                  *event);
static gboolean gm_vertical_splitter_button_release   (GtkWidget                       *widget,
        GdkEventButton                  *event);
static gboolean gm_vertical_splitter_motion           (GtkWidget      		       *widget,
        GdkEventMotion 		       *event);

/* GtkObjectClass methods */
static void     gm_vertical_splitter_destroy          (GtkObject                       *object);

/* GtkWidgetClass methods */
static void     gm_vertical_splitter_draw             (GtkWidget                       *widget,
        GdkRectangle                    *area);

GM_DEFINE_CLASS_BOILERPLATE (GmVerticalSplitter,
                             gm_vertical_splitter,
                             E_TYPE_VPANED)

/* GtkObjectClass methods */
static void
gm_vertical_splitter_initialize_class (GmVerticalSplitterClass *vertical_splitter_class)
{
    GtkObjectClass *object_class;
    GtkWidgetClass *widget_class;

    object_class = GTK_OBJECT_CLASS (vertical_splitter_class);
    widget_class = GTK_WIDGET_CLASS (vertical_splitter_class);

    /* GtkObjectClass */
    object_class->destroy = gm_vertical_splitter_destroy;

    /* GtkWidgetClass */
    widget_class->draw = gm_vertical_splitter_draw;
    widget_class->button_press_event = gm_vertical_splitter_button_press;
    widget_class->button_release_event = gm_vertical_splitter_button_release;
    widget_class->motion_notify_event = gm_vertical_splitter_motion;




    splitter_signals[COLLAPSED] =
        gtk_signal_new ("collapsed",
                        GTK_RUN_FIRST,
                        object_class->type,
                        GTK_SIGNAL_OFFSET (GmVerticalSplitterClass, collapsed),
                        gtk_marshal_NONE__INT,
                        GTK_TYPE_NONE, 1,
                        GTK_TYPE_INT);



    gtk_object_class_add_signals (object_class, splitter_signals, LAST_SIGNAL);





}

static void
gm_vertical_splitter_initialize (GmVerticalSplitter *vertical_splitter)
{
    vertical_splitter->details = g_new0 (GmVerticalSplitterDetails, 1);
    e_paned_set_handle_size (E_PANED (vertical_splitter), BAR_HEIGHT);
}

/* GtkObjectClass methods */
static void
gm_vertical_splitter_destroy (GtkObject *object)
{
    GmVerticalSplitter *vertical_splitter;

    vertical_splitter = GM_VERTICAL_SPLITTER (object);

    g_free (vertical_splitter->details);

    /* Chain */
    GM_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
}

static void
draw_resize_bar (GtkWidget		*widget,
                 GdkWindow		*window,
                 const GdkRectangle	*area)
{

    GtkStyle	*style;
    GdkGC		*gcs[BAR_HEIGHT];
    guint		i;
    EPaned		*paned;





    g_assert (widget != NULL);
    g_assert (window != NULL);
    g_assert (area != NULL);
    g_assert (area->height == BAR_HEIGHT);

    style = widget->style;
    paned = E_PANED(widget);

    g_assert (style != NULL);

    for (i = 0; i < BAR_HEIGHT; i++)
        gcs[i] = style->bg_gc[GTK_STATE_NORMAL];

    gcs[0] = style->dark_gc[GTK_STATE_NORMAL];
    gcs[1] = style->mid_gc[GTK_STATE_ACTIVE];
    gcs[2] = style->light_gc[GTK_STATE_ACTIVE];
    gcs[3] = style->bg_gc[GTK_STATE_NORMAL];
    gcs[4] = style->mid_gc[GTK_STATE_ACTIVE];
    gcs[5] = style->dark_gc[GTK_STATE_NORMAL];



    for (i = 0; i < BAR_HEIGHT; i++)
    {
        gdk_draw_line (window,
                       gcs[i],
                       area->x,
                       area->y + i,
                       area->x  + area->width -1,
                       area->y + i);
    }
    i = 0;
    gdk_draw_line (window,
                   gcs[i],
                   area->x,
                   area->y + i,
                   area->x  + area->width -1,
                   area->y + i);

    i = 5;
    gdk_draw_line (window,
                   gcs[i],
                   area->x,
                   area->y + i,
                   area->x  + area->width -1,
                   area->y + i);
}

/* Number of ridges in the thumb currently hardcoded to 8 */
#define NUM_RIDGES 8

/* These control the layout of the ridges */
#define RIDGE_WIDTH 2
#define RIDGE_EDGE_OFFSET 2
#define BETWEEN_RIDGE_OFFSET 1


static void
draw_resize_bar_thumb (GtkWidget		*widget,
                       GdkWindow		*window,
                       const GdkRectangle	*area)
{
    EPaned		*paned;
    GtkStyle	*style;
    GdkGC		*light_gc;
    GdkGC		*dark_gc;
    guint		total_thumb_width;

    g_assert (widget != NULL);
    g_assert (window != NULL);
    g_assert (area != NULL);
    g_assert (area->height == BAR_HEIGHT);

    style = widget->style;
    g_assert (style != NULL);

    paned = E_PANED (widget);

    light_gc = style->light_gc[GTK_STATE_ACTIVE];
    dark_gc = style->fg_gc[GTK_STATE_SELECTED];



    total_thumb_width = (NUM_RIDGES * RIDGE_WIDTH) + ((NUM_RIDGES - 1) * BETWEEN_RIDGE_OFFSET);

    /* Make sure the thumb aint bigger than the handle */
    if (total_thumb_width > paned->handle_width)
    {
        total_thumb_width = paned->handle_width;
    }

    /* Draw the thumb only if we have enough space for at least one ridge */
    if (total_thumb_width > RIDGE_WIDTH)
    {
        gint	x = paned->handle_xpos + (paned->handle_width - total_thumb_width) / 2;
        guint	i;


        for (i = 0; i < NUM_RIDGES; i++)
        {


            gint y1 = area->y + RIDGE_EDGE_OFFSET;
            gint y2 = area->y + BAR_HEIGHT - RIDGE_EDGE_OFFSET;
            gdk_draw_line (window,
                           dark_gc,
                           y1,
                           x,
                           y2,
                           x-2);

            x += BETWEEN_RIDGE_OFFSET;

            gdk_draw_line (window,
                           light_gc,
                           y1,
                           x,
                           y2,
                           x-2);

            x += RIDGE_WIDTH;
        }
    }
}




/* GtkWidgetClass methods */
static void
gm_vertical_splitter_draw (GtkWidget    *widget,
                           GdkRectangle *area)
{
    EPaned *paned;
    GdkRectangle handle_area, child_area;
    guint16 border_width;

    g_return_if_fail (widget != NULL);
    g_return_if_fail (E_IS_PANED (widget));
    if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget)) {
        paned = E_PANED (widget);
        border_width = GTK_CONTAINER (paned)->border_width;

        gdk_window_clear_area (widget->window,
                               area->x, area->y, area->width,
                               area->height);

        if (e_paned_handle_shown(paned)) {
            handle_area.x = paned->handle_xpos;
            handle_area.y = paned->handle_ypos;
            handle_area.width = paned->handle_width;

            handle_area.height = paned->handle_height;
            if (gdk_rectangle_intersect (&handle_area, area, &child_area)) {
                child_area.x -= paned->handle_xpos;
                child_area.y -= paned->handle_ypos;

                /* Simply things by always drawing the full height of the bar. */
                child_area.y = 0;
                child_area.height = BAR_HEIGHT;
                draw_resize_bar (widget, paned->handle, &child_area);
                draw_resize_bar_thumb (widget, paned->handle, &child_area);
            }
        }

        /* Redraw the children
         */
        if (paned->child1 && gtk_widget_intersect (paned->child1, area, &child_area)) {
            gtk_widget_draw (paned->child1, &child_area);
        }

        if (paned->child2 && gtk_widget_intersect (paned->child2, area, &child_area)) {
            gtk_widget_draw (paned->child2, &child_area);
        }
    }
}

void
gm_vertical_splitter_expand (GmVerticalSplitter *splitter)
{
    int position;

    g_return_if_fail (GM_IS_VERTICAL_SPLITTER (splitter));

    position = e_paned_get_position (E_PANED (splitter));


    if (position >= CLOSED_THRESHOLD && position <= (E_PANED (splitter)->max_position - CLOSED_THRESHOLD)) {
        return;
    }


    position = E_PANED(splitter)->saved_position;

    if (position < CLOSED_THRESHOLD || position > (E_PANED (splitter)->max_position - CLOSED_THRESHOLD) ) {
        position = NOMINAL_SIZE;
    }
    e_paned_set_position (E_PANED (splitter), position);
    gtk_signal_emit (GTK_OBJECT (splitter), splitter_signals[COLLAPSED], SPLITTER_OPENED);
}

static  void        gm_vertical_splitter_set_saved_position (GmVerticalSplitter * splitter, gint position)
{
    gm_spliter_set_saved_position (E_PANED (splitter), position);
    GE_CONFIG(global_app)->vertical_paned_position = position;
}

void
gm_vertical_splitter_collapse	(GmVerticalSplitter *splitter, gint direction)
{
    int position;

    g_return_if_fail (GM_IS_VERTICAL_SPLITTER (splitter));

    position = e_paned_get_position (E_PANED (splitter));

    if (position < CLOSED_THRESHOLD) {
        return;
    }
    gm_vertical_splitter_set_saved_position (splitter, position);


    switch (direction) {
    case GO_UP:
        {
            e_paned_set_position (E_PANED (splitter), 0);
            break;
        }
    case GO_DOWN :
        {
            e_paned_set_position (E_PANED (splitter), E_PANED (splitter)->max_position);
        }
    }
    gtk_signal_emit (GTK_OBJECT (splitter), splitter_signals[COLLAPSED], SPLITTER_CLOSED);
}

gboolean gm_vertical_splitter_is_collapsed (GmVerticalSplitter * splitter)
{
    EPaned       *paned;
    g_return_val_if_fail (GM_IS_VERTICAL_SPLITTER (splitter), FALSE);


    paned = E_PANED (splitter);
    return gm_splitter_is_collapsed(paned);
}



/* routine to toggle the open/closed state of the splitter */
void
gm_vertical_splitter_toggle_position (GmVerticalSplitter *splitter, gint direction)
{
    gint position;

    g_return_if_fail (GM_IS_VERTICAL_SPLITTER (splitter));
    position = e_paned_get_position (E_PANED (splitter));
    if (position >= CLOSED_THRESHOLD && position <= (E_PANED(splitter)->max_position - CLOSED_THRESHOLD)) {
        gm_vertical_splitter_collapse (splitter,  direction);
    } else {
        gm_vertical_splitter_expand (splitter);
    }
}

/* GmVerticalSplitter public methods */
GtkWidget *
gm_vertical_splitter_new (void)
{
    return gtk_widget_new (gm_vertical_splitter_get_type (), NULL);
}

/* handle mouse downs by remembering the position and the time */
static gboolean
gm_vertical_splitter_button_press (GtkWidget *widget, GdkEventButton *event)
{
    GmVerticalSplitter *splitter;

    splitter = GM_VERTICAL_SPLITTER (widget);

    if (event->window != E_PANED (widget)->handle) {
        splitter->details->down = FALSE;
    } else {
        splitter->details->down = TRUE;
        splitter->details->down_position = event->y;
        splitter->details->down_time = event->time;
    }

    return GM_CALL_PARENT_WITH_RETURN_VALUE
           (GTK_WIDGET_CLASS, button_press_event, (widget, event));
}


static void
splitter_xor_line (EPaned *paned)
{
    GtkWidget *widget;
    GdkGCValues values;
    guint16 ypos, half_width;
    gint8 dash_list[2];

    widget = GTK_WIDGET(paned);

    if (!paned->xor_gc) {
        values.function = GDK_INVERT;
        values.subwindow_mode = GDK_INCLUDE_INFERIORS;
        paned->xor_gc = gdk_gc_new_with_values (widget->window, &values,
                                                GDK_GC_FUNCTION | GDK_GC_SUBWINDOW);
    }

    gdk_gc_set_line_attributes (paned->xor_gc, 1, GDK_LINE_ON_OFF_DASH,
                                GDK_CAP_NOT_LAST, GDK_JOIN_BEVEL);

    /* Make line appear as every other pixel dash */
    dash_list[0] = 1;
    dash_list[1] = 1;
    gdk_gc_set_dashes (paned->xor_gc, 1, dash_list, 2);

    ypos = paned->child1_size + GTK_CONTAINER (paned)->border_width + paned->handle_size / 2;
    half_width = paned->handle_size / 2;

    gdk_draw_line (widget->window, paned->xor_gc, 0, ypos - half_width, widget->allocation.width -1, ypos - half_width);


    gdk_draw_line (widget->window, paned->xor_gc, 0,ypos + half_width,  widget->allocation.width - 1, ypos + half_width);



}


/* handle mouse ups by seeing if it was a tap and toggling the open state accordingly */
static gboolean
gm_vertical_splitter_button_release (GtkWidget *widget, GdkEventButton *event)
{
    GmVerticalSplitter *splitter;
    int delta, delta_time;
    EPaned *paned;
    gboolean priority;

    paned = E_PANED (widget);

    splitter = GM_VERTICAL_SPLITTER (widget);
    priority = e_paned_get_priority (paned);

    if (event->window == E_PANED (widget)->handle
            && splitter->details->down) {
        delta = abs (event->y - splitter->details->down_position);
        delta_time = abs (splitter->details->down_time - event->time);
        if (delta < SPLITTER_CLICK_SLOP && delta_time < SPLITTER_CLICK_TIMEOUT)  {
            if (!priority){
                gm_vertical_splitter_toggle_position (splitter, GO_DOWN);
            }else {
                gm_vertical_splitter_toggle_position (splitter, GO_UP);
            }

        }else {
            gint position;

            position = e_paned_get_position (paned);
            gm_vertical_splitter_set_saved_position (splitter, position);
        }

    }

    splitter->details->down = FALSE;

    if (paned->in_drag && (event->button == 1)) {
        splitter_xor_line (paned);
        paned->in_drag = FALSE;
        paned->position_set = TRUE;
        gdk_pointer_ungrab (event->time);
        gtk_widget_queue_resize (GTK_WIDGET (paned));
        return TRUE;
    }

    return FALSE;
}

static gboolean
gm_vertical_splitter_motion (GtkWidget *widget, GdkEventMotion *event)
{
    EPaned *paned;
    gint y;

    g_return_val_if_fail (widget != NULL, FALSE);
    g_return_val_if_fail (E_IS_PANED (widget), FALSE);

    paned = E_PANED (widget);

    if (event->is_hint || event->window != widget->window) {
        gtk_widget_get_pointer(widget, NULL, &y);
    } else {
        y = event->y;
    }

    if (paned->in_drag) {
        gint size = y - GTK_CONTAINER (paned)->border_width - paned->handle_size / 2;
        splitter_xor_line (paned);
        paned->child1_size = CLAMP (e_paned_quantized_size (paned, size), paned->min_position, paned->max_position);
        splitter_xor_line (paned);
    }

    return TRUE;
}


